@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,13 +18,12 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6151 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-09 12:31:11 +0200 (Mon, 09 Jul 2012) $
+$Revision: 6954 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-05 10:22:00 +0200 (Fr, 05. Apr 2013) $
 
 ********************************************************************/
 
-
 #include "commands.h"
 #include "debugdialog.h"
 #include "sketch/sketchwidget.h"
@@ -34,16 +33,42 @@ $Date: 2012-07-09 12:31:11 +0200 (Mon, 09 Jul 2012) $
 #include "items/moduleidnames.h"
 #include "utils/bezier.h"
 
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+CommandProgress::CommandProgress() {
+    m_active = false;
+}
+
+void CommandProgress::setActive(bool active) {
+    m_active = active;
+}
+
+bool CommandProgress::active() {
+    return m_active;
+}
+
+void CommandProgress::emitUndo() {
+    emit incUndo();
+}
+
+void CommandProgress::emitRedo() {
+    emit incRedo();
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+
 int SelectItemCommand::selectItemCommandID = 3;
 int ChangeNoteTextCommand::changeNoteTextCommandID = 5;
 int BaseCommand::nextIndex = 0;
+CommandProgress BaseCommand::m_commandProgress;
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 BaseCommand::BaseCommand(BaseCommand::CrossViewType crossViewType, SketchWidget* sketchWidget, QUndoCommand *parent)
 	: QUndoCommand(parent)
 {
-	m_undoOnly = m_redoOnly = false;
+	m_skipFirstRedo = m_undoOnly = m_redoOnly = false;
 	m_crossViewType = crossViewType;
 	m_sketchWidget = sketchWidget;
 	m_parentCommand = parent;
@@ -82,6 +107,10 @@ void BaseCommand::setRedoOnly()
 	m_redoOnly = true;
 }
 
+void BaseCommand::setSkipFirstRedo() {
+    m_skipFirstRedo = true;
+}
+
 QString BaseCommand::getParamString() const {
 	return QString("%1 %2")
 		.arg(m_sketchWidget->viewName())
@@ -141,9 +170,35 @@ int BaseCommand::index() const {
 	return m_index;
 }
 
+void BaseCommand::undo() {
+    if (m_commandProgress.active()) m_commandProgress.emitUndo();
+}
+
+void BaseCommand::redo() {
+    if (m_commandProgress.active()) m_commandProgress.emitRedo();
+}
+
+CommandProgress * BaseCommand::initProgress() {
+    m_commandProgress.setActive(true);
+    return &m_commandProgress;
+}
+
+void BaseCommand::clearProgress() {
+    m_commandProgress.setActive(false);
+}
+
+int BaseCommand::totalChildCount(const QUndoCommand * command) {
+    int cc = command->childCount();
+    int tcc = cc;
+    for (int i = 0; i < cc; i++) {
+        tcc += totalChildCount(command->child(i));
+    }
+    return tcc;
+}
+
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-AddDeleteItemCommand::AddDeleteItemCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, QString moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, ViewGeometry & viewGeometry, qint64 id, long modelIndex, QUndoCommand *parent)
+AddDeleteItemCommand::AddDeleteItemCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, QString moduleID, ViewLayer::ViewLayerPlacement viewLayerPlacement, ViewGeometry & viewGeometry, qint64 id, long modelIndex, QUndoCommand *parent)
     : BaseCommand(crossViewType, sketchWidget, parent)
 {
  	m_dropOrigin = NULL;
@@ -151,7 +206,7 @@ AddDeleteItemCommand::AddDeleteItemCommand(SketchWidget* sketchWidget, BaseComma
     m_viewGeometry = viewGeometry;
     m_itemID = id;
 	m_modelIndex = modelIndex;
-	m_viewLayerSpec = viewLayerSpec;
+	m_viewLayerPlacement = viewLayerPlacement;
 }
 
 QString AddDeleteItemCommand::getParamString() const {
@@ -177,10 +232,9 @@ SketchWidget * AddDeleteItemCommand::dropOrigin() {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-AddItemCommand::AddItemCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, QString moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, ViewGeometry & viewGeometry, qint64 id, bool updateInfoView, long modelIndex, QUndoCommand *parent)
-    : AddDeleteItemCommand(sketchWidget, crossViewType, moduleID, viewLayerSpec, viewGeometry, id, modelIndex, parent)
+AddItemCommand::AddItemCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, QString moduleID, ViewLayer::ViewLayerPlacement viewLayerPlacement, ViewGeometry & viewGeometry, qint64 id, bool updateInfoView, long modelIndex, QUndoCommand *parent)
+    : AddDeleteItemCommand(sketchWidget, crossViewType, moduleID, viewLayerPlacement, viewGeometry, id, modelIndex, parent)
 {
-	m_doFirstRedo = m_firstRedo = true;
 	m_module = false;
 	m_updateInfoView = updateInfoView;
 }
@@ -188,18 +242,16 @@ AddItemCommand::AddItemCommand(SketchWidget* sketchWidget, BaseCommand::CrossVie
 void AddItemCommand::undo()
 {
     m_sketchWidget->deleteItem(m_itemID, true, true, false);
+    BaseCommand::undo();
 }
 
 void AddItemCommand::redo()
 {
-	if (!m_firstRedo || m_doFirstRedo) {
-		m_sketchWidget->addItem(m_moduleID, m_viewLayerSpec, m_crossViewType, m_viewGeometry, m_itemID, m_modelIndex, this);
+	if (!m_skipFirstRedo) {
+		m_sketchWidget->addItem(m_moduleID, m_viewLayerPlacement, m_crossViewType, m_viewGeometry, m_itemID, m_modelIndex, this);
 	}
-	m_firstRedo = false;
-}
-
-void AddItemCommand::turnOffFirstRedo() {
-	m_doFirstRedo = false;
+	m_skipFirstRedo = false;
+    BaseCommand::redo();
 }
 
 QString AddItemCommand::getParamString() const {
@@ -213,19 +265,21 @@ QString AddItemCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-DeleteItemCommand::DeleteItemCommand(SketchWidget* sketchWidget,BaseCommand::CrossViewType crossViewType,  QString moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, ViewGeometry & viewGeometry, qint64 id, long modelIndex, QUndoCommand *parent)
-    : AddDeleteItemCommand(sketchWidget, crossViewType, moduleID, viewLayerSpec, viewGeometry, id, modelIndex, parent)
+DeleteItemCommand::DeleteItemCommand(SketchWidget* sketchWidget,BaseCommand::CrossViewType crossViewType,  QString moduleID, ViewLayer::ViewLayerPlacement viewLayerPlacement, ViewGeometry & viewGeometry, qint64 id, long modelIndex, QUndoCommand *parent)
+    : AddDeleteItemCommand(sketchWidget, crossViewType, moduleID, viewLayerPlacement, viewGeometry, id, modelIndex, parent)
 {
 }
 
 void DeleteItemCommand::undo()
 {
-    m_sketchWidget->addItem(m_moduleID, m_viewLayerSpec, m_crossViewType, m_viewGeometry, m_itemID, m_modelIndex, this);
+    m_sketchWidget->addItem(m_moduleID, m_viewLayerPlacement, m_crossViewType, m_viewGeometry, m_itemID, m_modelIndex, this);
+    BaseCommand::undo();
 }
 
 void DeleteItemCommand::redo()
 {
 	m_sketchWidget->deleteItem(m_itemID, true, m_crossViewType == BaseCommand::CrossView, false);
+    BaseCommand::redo();
 }
 
 QString DeleteItemCommand::getParamString() const {
@@ -246,11 +300,13 @@ MoveItemCommand::MoveItemCommand(SketchWidget* sketchWidget, long itemID, ViewGe
 void MoveItemCommand::undo()
 {
     m_sketchWidget->moveItem(m_itemID, m_old, m_updateRatsnest);
+    BaseCommand::undo();
 }
 
 void MoveItemCommand::redo()
 {
     m_sketchWidget->moveItem(m_itemID, m_new, m_updateRatsnest);
+    BaseCommand::redo();
 }
 
 QString MoveItemCommand::getParamString() const {
@@ -282,11 +338,13 @@ SimpleMoveItemCommand::SimpleMoveItemCommand(SketchWidget* sketchWidget, long it
 void SimpleMoveItemCommand::undo()
 {
     m_sketchWidget->simpleMoveItem(m_itemID, m_old);
+    BaseCommand::undo();
 }
 
 void SimpleMoveItemCommand::redo()
 {
     m_sketchWidget->simpleMoveItem(m_itemID, m_new);
+    BaseCommand::redo();
 }
 
 QString SimpleMoveItemCommand::getParamString() const {
@@ -317,6 +375,7 @@ void MoveItemsCommand::undo()
 	foreach (long id, m_wires.keys()) {
 		m_sketchWidget->updateWire(id, m_wires.value(id), m_updateRatsnest);
 	}
+    BaseCommand::undo();
 }
 
 void MoveItemsCommand::redo()
@@ -327,6 +386,7 @@ void MoveItemsCommand::redo()
 	foreach (long id, m_wires.keys()) {
 		m_sketchWidget->updateWire(id, m_wires.value(id), m_updateRatsnest);
 	}
+    BaseCommand::redo();
 }
 
 void MoveItemsCommand::addWire(long id, const QString & connectorID)
@@ -364,11 +424,13 @@ RotateItemCommand::RotateItemCommand(SketchWidget* sketchWidget, long itemID, do
 void RotateItemCommand::undo()
 {
     m_sketchWidget->rotateItem(m_itemID, -m_degrees);
+    BaseCommand::undo();
 }
 
 void RotateItemCommand::redo()
 {
     m_sketchWidget->rotateItem(m_itemID, m_degrees);
+    BaseCommand::redo();
 }
 
 QString RotateItemCommand::getParamString() const {
@@ -391,11 +453,13 @@ FlipItemCommand::FlipItemCommand(SketchWidget* sketchWidget, long itemID, Qt::Or
 void FlipItemCommand::undo()
 {
     redo();
+    BaseCommand::undo();
 }
 
 void FlipItemCommand::redo()
 {
     m_sketchWidget->flipItem(m_itemID, m_orientation);
+    BaseCommand::redo();
 }
 
 
@@ -412,34 +476,44 @@ QString FlipItemCommand::getParamString() const {
 ChangeConnectionCommand::ChangeConnectionCommand(SketchWidget * sketchWidget, BaseCommand::CrossViewType crossView,
 												 long fromID, const QString & fromConnectorID,
 												 long toID, const QString & toConnectorID,
-												 ViewLayer::ViewLayerSpec viewLayerSpec,
+												 ViewLayer::ViewLayerPlacement viewLayerPlacement,
 												 bool connect, QUndoCommand * parent)
 : BaseCommand(crossView, sketchWidget, parent)
 {
 	//DebugDialog::debug(QString("ccc: from %1 %2; to %3 %4").arg(fromID).arg(fromConnectorID).arg(toID).arg(toConnectorID) );
+    m_enabled = true;
     m_fromID = fromID;
     m_fromConnectorID = fromConnectorID;
     m_toID = toID;
     m_toConnectorID = toConnectorID;
 	m_connect = connect;
 	m_updateConnections = true;
-	m_viewLayerSpec = viewLayerSpec;
+	m_viewLayerPlacement = viewLayerPlacement;
 }
 
 void ChangeConnectionCommand::undo()
 {
-    m_sketchWidget->changeConnection(m_fromID, m_fromConnectorID, m_toID, m_toConnectorID, m_viewLayerSpec, !m_connect,  m_crossViewType == CrossView,  m_updateConnections);
+    if (m_enabled) {
+        m_sketchWidget->changeConnection(m_fromID, m_fromConnectorID, m_toID, m_toConnectorID, m_viewLayerPlacement, !m_connect,  m_crossViewType == CrossView,  m_updateConnections);
+        BaseCommand::undo();
+    }
 }
 
 void ChangeConnectionCommand::redo()
 {
-    m_sketchWidget->changeConnection(m_fromID, m_fromConnectorID, m_toID, m_toConnectorID, m_viewLayerSpec, m_connect,  m_crossViewType == CrossView, m_updateConnections);
+    if (m_enabled) {
+        m_sketchWidget->changeConnection(m_fromID, m_fromConnectorID, m_toID, m_toConnectorID, m_viewLayerPlacement, m_connect,  m_crossViewType == CrossView, m_updateConnections);
+        BaseCommand::redo();
+    }
 }
 
 void ChangeConnectionCommand::setUpdateConnections(bool updatem) {
 	m_updateConnections = updatem;
 }
 
+void ChangeConnectionCommand::disable() {
+	m_enabled = false;
+}
 
 QString ChangeConnectionCommand::getParamString() const {
 	return QString("ChangeConnectionCommand ") 
@@ -449,7 +523,7 @@ QString ChangeConnectionCommand::getParamString() const {
 		.arg(m_fromConnectorID)
 		.arg(m_toID)
 		.arg(m_toConnectorID)
-		.arg(m_viewLayerSpec)
+		.arg(m_viewLayerPlacement)
 		.arg(m_connect);
 }
 
@@ -476,6 +550,7 @@ void ChangeWireCommand::undo()
 	if (!m_redoOnly) {
 		m_sketchWidget->changeWire(m_fromID, m_oldLine, m_oldPos, m_updateConnections, m_updateRatsnest);
 	}
+    BaseCommand::undo();
 }
 
 void ChangeWireCommand::redo()
@@ -483,6 +558,7 @@ void ChangeWireCommand::redo()
 	if (!m_undoOnly) {
 		m_sketchWidget->changeWire(m_fromID, m_newLine, m_newPos, m_updateConnections, m_updateRatsnest);
 	}
+    BaseCommand::redo();
 }
 
 QString ChangeWireCommand::getParamString() const {
@@ -500,12 +576,12 @@ QString ChangeWireCommand::getParamString() const {
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 ChangeWireCurveCommand::ChangeWireCurveCommand(SketchWidget* sketchWidget, long fromID,
-									 const Bezier * oldBezier, const Bezier * newBezier, 
+									 const Bezier * oldBezier, const Bezier * newBezier, bool wasAutoroutable,
 									 QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
-	m_firstTime = false;
     m_fromID = fromID;
+    m_wasAutoroutable = wasAutoroutable;
 	m_oldBezier = m_newBezier = NULL;
 	if (oldBezier) {
 		m_oldBezier = new Bezier;
@@ -520,24 +596,22 @@ ChangeWireCurveCommand::ChangeWireCurveCommand(SketchWidget* sketchWidget, long
 void ChangeWireCurveCommand::undo()
 {
 	if (!m_redoOnly) {
-		m_sketchWidget->changeWireCurve(m_fromID, m_oldBezier);
+		m_sketchWidget->changeWireCurve(m_fromID, m_oldBezier, m_wasAutoroutable);
 	}
+    BaseCommand::undo();
 }
 
 void ChangeWireCurveCommand::redo()
 {
 	if (!m_undoOnly) {
-		if (m_firstTime) {
-			m_firstTime = false;
+		if (m_skipFirstRedo) {
+			m_skipFirstRedo = false;
 		}
 		else {
-			m_sketchWidget->changeWireCurve(m_fromID, m_newBezier);
+			m_sketchWidget->changeWireCurve(m_fromID, m_newBezier, false);
 		}
 	}
-}
-
-void ChangeWireCurveCommand::setFirstTime() {
-	m_firstTime = true;
+    BaseCommand::redo();
 }
 
 QString ChangeWireCurveCommand::getParamString() const {
@@ -583,6 +657,7 @@ void ChangeLegCommand::undo()
 	if (!m_redoOnly) {
 		m_sketchWidget->changeLeg(m_fromID, m_fromConnectorID, m_oldLeg, m_relative, m_why);
 	}
+    BaseCommand::undo();
 }
 
 void ChangeLegCommand::setSimple()
@@ -600,6 +675,7 @@ void ChangeLegCommand::redo()
 			m_sketchWidget->recalcLeg(m_fromID, m_fromConnectorID, m_newLeg, m_relative, m_active, m_why);
 		}
 	}
+    BaseCommand::redo();
 }
 
 QString ChangeLegCommand::getParamString() const {
@@ -641,6 +717,7 @@ void MoveLegBendpointCommand::undo()
 	if (!m_redoOnly) {
 		m_sketchWidget->moveLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_oldPos);
 	}
+    BaseCommand::undo();
 }
 
 void MoveLegBendpointCommand::redo()
@@ -648,6 +725,7 @@ void MoveLegBendpointCommand::redo()
 	if (!m_undoOnly) {
 		m_sketchWidget->moveLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_newPos);
 	}
+    BaseCommand::redo();
 }
 
 QString MoveLegBendpointCommand::getParamString() const {
@@ -671,7 +749,6 @@ ChangeLegCurveCommand::ChangeLegCurveCommand(SketchWidget* sketchWidget, long fr
 									 const Bezier * oldBezier, const Bezier * newBezier, QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
-	m_firstTime = false;
     m_fromID = fromID;
 	m_oldBezier = m_newBezier = NULL;
 	if (oldBezier) {
@@ -690,20 +767,18 @@ ChangeLegCurveCommand::ChangeLegCurveCommand(SketchWidget* sketchWidget, long fr
 void ChangeLegCurveCommand::undo()
 {
     m_sketchWidget->changeLegCurve(m_fromID, m_fromConnectorID, m_index,  m_oldBezier);
+    BaseCommand::undo();
 }
 
 void ChangeLegCurveCommand::redo()
 {
-	if (m_firstTime) {
-		m_firstTime = false;
+	if (m_skipFirstRedo) {
+		m_skipFirstRedo = false;
 	}
 	else if (!m_undoOnly) {
 		m_sketchWidget->changeLegCurve(m_fromID, m_fromConnectorID, m_index, m_newBezier);
 	}
-}
-
-void ChangeLegCurveCommand::setFirstTime() {
-	m_firstTime = true;
+    BaseCommand::redo();
 }
 
 QString ChangeLegCurveCommand::getParamString() const {
@@ -734,7 +809,6 @@ ChangeLegBendpointCommand::ChangeLegBendpointCommand(SketchWidget* sketchWidget,
 									const Bezier * bezier0, const Bezier * bezier1, const Bezier * bezier2, QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
-	m_firstTime = false;
     m_fromID = fromID;
 	m_bezier0 = m_bezier1 = m_bezier2 = NULL;
 	if (bezier0 != NULL) {
@@ -759,18 +833,19 @@ ChangeLegBendpointCommand::ChangeLegBendpointCommand(SketchWidget* sketchWidget,
 
 void ChangeLegBendpointCommand::undo()
 {
-		if (m_newCount < m_oldCount) {
-			m_sketchWidget->addLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_pos, m_bezier0, m_bezier1);
-		}
-		else {
-			m_sketchWidget->removeLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_bezier0);
-		}
+	if (m_newCount < m_oldCount) {
+		m_sketchWidget->addLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_pos, m_bezier0, m_bezier1);
+	}
+	else {
+		m_sketchWidget->removeLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_bezier0);
+	}
+    BaseCommand::undo();
 }
 
 void ChangeLegBendpointCommand::redo()
 {
-	if (m_firstTime) {
-		m_firstTime = false;
+	if (m_skipFirstRedo) {
+		m_skipFirstRedo = false;
 	}
 	else {
 		if (m_newCount > m_oldCount) {
@@ -780,10 +855,7 @@ void ChangeLegBendpointCommand::redo()
 			m_sketchWidget->removeLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_bezier2);
 		}
 	}
-}
-
-void ChangeLegBendpointCommand::setFirstTime() {
-	m_firstTime = true;
+    BaseCommand::redo();
 }
 
 QString ChangeLegBendpointCommand::getParamString() const {
@@ -815,11 +887,13 @@ RotateLegCommand::RotateLegCommand(SketchWidget* sketchWidget, long fromID, cons
 
 void RotateLegCommand::undo()
 {
+    BaseCommand::undo();
 }
 
 void RotateLegCommand::redo()
 {
 	m_sketchWidget->rotateLeg(m_fromID, m_fromConnectorID, m_oldLeg, m_active);
+    BaseCommand::redo();
 }
 
 QString RotateLegCommand::getParamString() const {
@@ -855,11 +929,13 @@ ChangeLayerCommand::ChangeLayerCommand(SketchWidget *sketchWidget, long fromID,
 void ChangeLayerCommand::undo()
 {
     m_sketchWidget->changeLayer(m_fromID, m_oldZ, m_oldLayer);
+    BaseCommand::undo();
 }
 
 void ChangeLayerCommand::redo()
 {
     m_sketchWidget->changeLayer(m_fromID, m_newZ, m_newLayer);
+    BaseCommand::redo();
 }
 
 QString ChangeLayerCommand::getParamString() const {
@@ -936,6 +1012,7 @@ bool SelectItemCommand::mergeWith(const QUndoCommand *other)
 void SelectItemCommand::undo()
 {
 	selectAllFromStack(m_undoIDs, true, true);
+    BaseCommand::undo();
 }
 
 void SelectItemCommand::redo()
@@ -954,6 +1031,7 @@ void SelectItemCommand::redo()
 			m_sketchWidget->selectAllItems(false, m_crossViewType == BaseCommand::CrossView); 
 			break;
 	}
+    BaseCommand::redo();
 }
 
 void SelectItemCommand::selectAllFromStack(QList<long> & stack, bool select, bool updateInfoView) {
@@ -1011,11 +1089,13 @@ void ChangeZCommand::addTriplet(long id, double oldZ, double newZ) {
 void ChangeZCommand::undo()
 {
    m_sketchWidget->changeZ(m_triplets, first);
+    BaseCommand::undo();
 }
 
 void ChangeZCommand::redo()
 {
    m_sketchWidget->changeZ(m_triplets, second);
+    BaseCommand::redo();
 }
 
 double ChangeZCommand::first(RealPair * pair) {
@@ -1037,7 +1117,7 @@ CheckStickyCommand::CheckStickyCommand(SketchWidget* sketchWidget, BaseCommand::
 : BaseCommand(crossViewType, sketchWidget, parent)
 {
 	m_itemID = itemID;
-	m_firstTime = true;
+	m_skipFirstRedo = true;
 	m_checkType = checkType;
 	m_checkCurrent = checkCurrent;
 }
@@ -1062,21 +1142,23 @@ void CheckStickyCommand::undo()
 			stickyThing->sketchWidget->stickem(stickyThing->fromID, stickyThing->toID, stickyThing->stickem);
 		}
 	}
+    BaseCommand::undo();
 }
 
 void CheckStickyCommand::redo()
 {
 	if (m_checkType == UndoOnly) return;
 
-	if (m_firstTime) {
+	if (m_skipFirstRedo) {
 		m_sketchWidget->checkSticky(m_itemID, m_crossViewType == BaseCommand::CrossView, m_checkCurrent, this);
-		m_firstTime = false;
+		m_skipFirstRedo = false;
 	}
 	else {
 		foreach (StickyThing * stickyThing, m_stickyList) {
 			stickyThing->sketchWidget->stickem(stickyThing->fromID, stickyThing->toID, stickyThing->stickem);
 		}
 	}
+    BaseCommand::redo();
 }
 
 QString CheckStickyCommand::getParamString() const {
@@ -1087,7 +1169,6 @@ QString CheckStickyCommand::getParamString() const {
 }
 
 void CheckStickyCommand::stick(SketchWidget * sketchWidget, long fromID, long toID, bool stickem) {
-	m_firstTime = false;
 	StickyThing * stickyThing = new StickyThing;
 	stickyThing->sketchWidget = sketchWidget;
 	stickyThing->fromID = fromID;
@@ -1117,6 +1198,7 @@ void CleanUpWiresCommand::undo()
 	if (m_direction == UndoOnly) {
 		m_sketchWidget->cleanUpWires(m_crossViewType == BaseCommand::CrossView, NULL);  
 	}
+    BaseCommand::undo();
 }
 
 void CleanUpWiresCommand::redo()
@@ -1132,6 +1214,7 @@ void CleanUpWiresCommand::redo()
 	if (m_direction == RedoOnly) {
 		m_sketchWidget->cleanUpWires(m_crossViewType == BaseCommand::CrossView, this);  
 	}
+    BaseCommand::redo();
 }
 
 void CleanUpWiresCommand::addRatsnestConnect(long id, const QString & connectorID, bool connect)
@@ -1190,7 +1273,7 @@ void CleanUpWiresCommand::addTrace(SketchWidget * sketchWidget, Wire * wire)
 							false, NULL));
 	}
 
-	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::CrossView, ModuleIDNames::WireModuleIDName, wire->viewLayerSpec(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
+	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::CrossView, ModuleIDNames::WireModuleIDName, wire->viewLayerPlacement(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
 }
 
 bool CleanUpWiresCommand::hasTraces(SketchWidget * sketchWidget) {
@@ -1205,6 +1288,37 @@ QString CleanUpWiresCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+CleanUpRatsnestsCommand::CleanUpRatsnestsCommand(SketchWidget* sketchWidget, CleanUpWiresCommand::Direction direction, QUndoCommand *parent)
+: BaseCommand(BaseCommand::CrossView, sketchWidget, parent)
+{
+	if (direction == CleanUpWiresCommand::UndoOnly) m_undoOnly = true;
+	if (direction == CleanUpWiresCommand::RedoOnly) m_redoOnly = true;
+}
+
+void CleanUpRatsnestsCommand::undo()
+{
+	if (m_undoOnly) {
+		m_sketchWidget->cleanupRatsnests(true);  
+	}
+    BaseCommand::undo();
+}
+
+void CleanUpRatsnestsCommand::redo()
+{
+	if (m_redoOnly) {
+		m_sketchWidget->cleanupRatsnests(true);  
+	}
+    BaseCommand::redo();
+}
+
+QString CleanUpRatsnestsCommand::getParamString() const {
+	return QString("CleanUpRatsnestsCommand ") 
+		+ BaseCommand::getParamString()
+    ;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
 WireColorChangeCommand::WireColorChangeCommand(SketchWidget* sketchWidget, long wireId, const QString &oldColor, const QString &newColor, double oldOpacity, double newOpacity, QUndoCommand *parent)
 : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
@@ -1217,10 +1331,12 @@ WireColorChangeCommand::WireColorChangeCommand(SketchWidget* sketchWidget, long
 
 void WireColorChangeCommand::undo() {
 	m_sketchWidget->changeWireColor(m_wireId, m_oldColor, m_oldOpacity);
+    BaseCommand::undo();
 }
 
 void WireColorChangeCommand::redo() {
 	m_sketchWidget->changeWireColor(m_wireId, m_newColor, m_newOpacity);
+    BaseCommand::redo();
 }
 
 QString WireColorChangeCommand::getParamString() const {
@@ -1243,10 +1359,12 @@ WireWidthChangeCommand::WireWidthChangeCommand(SketchWidget* sketchWidget, long
 
 void WireWidthChangeCommand::undo() {
 	m_sketchWidget->changeWireWidth(m_wireId, m_oldWidth);
+    BaseCommand::undo();
 }
 
 void WireWidthChangeCommand::redo() {
 	m_sketchWidget->changeWireWidth(m_wireId, m_newWidth);
+    BaseCommand::redo();
 }
 
 
@@ -1269,10 +1387,12 @@ RoutingStatusCommand::RoutingStatusCommand(SketchWidget * sketchWidget, const Ro
 
 void RoutingStatusCommand::undo() {
 	m_sketchWidget->forwardRoutingStatus(m_oldRoutingStatus);
+    BaseCommand::undo();
 }
 
 void RoutingStatusCommand::redo() {
 	m_sketchWidget->forwardRoutingStatus(m_newRoutingStatus);
+    BaseCommand::redo();
 }
 
 QString RoutingStatusCommand::getParamString() const {
@@ -1296,11 +1416,13 @@ ShowLabelFirstTimeCommand::ShowLabelFirstTimeCommand(SketchWidget *sketchWidget,
 
 void ShowLabelFirstTimeCommand::undo()
 {
+    BaseCommand::undo();
 }
 
 void ShowLabelFirstTimeCommand::redo()
 {
     m_sketchWidget->showLabelFirstTime(m_itemID, m_newVis, true);
+    BaseCommand::redo();
 }
 
 QString ShowLabelFirstTimeCommand::getParamString() const {
@@ -1323,11 +1445,13 @@ RestoreLabelCommand::RestoreLabelCommand(SketchWidget *sketchWidget,long id, QDo
 
 void RestoreLabelCommand::undo()
 {
+    BaseCommand::undo();
 }
 
 void RestoreLabelCommand::redo()
 {
     m_sketchWidget->restorePartLabel(m_itemID, m_element);
+    BaseCommand::redo();
 }
 
 QString RestoreLabelCommand::getParamString() const {
@@ -1353,11 +1477,13 @@ MoveLabelCommand::MoveLabelCommand(SketchWidget *sketchWidget, long id, QPointF
 void MoveLabelCommand::undo()
 {
     m_sketchWidget->movePartLabel(m_itemID, m_oldPos, m_oldOffset);
+    BaseCommand::undo();
 }
 
 void MoveLabelCommand::redo()
 {
     m_sketchWidget->movePartLabel(m_itemID, m_newPos, m_newOffset);
+    BaseCommand::redo();
 }
 
 
@@ -1382,11 +1508,13 @@ MoveLockCommand::MoveLockCommand(SketchWidget *sketchWidget, long id, bool oldLo
 void MoveLockCommand::undo()
 {
     m_sketchWidget->setMoveLock(m_itemID, m_oldLock);
+    BaseCommand::undo();
 }
 
 void MoveLockCommand::redo()
 {
     m_sketchWidget->setMoveLock(m_itemID, m_newLock);
+    BaseCommand::redo();
 }
 
 
@@ -1414,10 +1542,12 @@ ChangeLabelTextCommand::ChangeLabelTextCommand(SketchWidget *sketchWidget, long
 
 void ChangeLabelTextCommand::undo() {
 	m_sketchWidget->setInstanceTitle(m_itemID, m_newText, m_oldText, false, true);
+    BaseCommand::undo();
 }
 
 void ChangeLabelTextCommand::redo() {
     m_sketchWidget->setInstanceTitle(m_itemID, m_oldText, m_newText, false, true);
+    BaseCommand::redo();
 }
 
 QString ChangeLabelTextCommand::getParamString() const {
@@ -1434,17 +1564,18 @@ IncLabelTextCommand::IncLabelTextCommand(SketchWidget *sketchWidget, long id,  Q
 	: BaseCommand(BaseCommand::CrossView, sketchWidget, parent)
 {
     m_itemID = id;
-	m_firstTime = true;
 }
 
 void IncLabelTextCommand::undo() { 
 	// only used when creating new parts via paste
+    BaseCommand::undo();
 }
 
 void IncLabelTextCommand::redo() {
-	if (m_firstTime) {
+	if (!m_skipFirstRedo) {
 		m_sketchWidget->incInstanceTitle(m_itemID);
 	}
+    BaseCommand::redo();
 }
 
 QString IncLabelTextCommand::getParamString() const {
@@ -1465,7 +1596,6 @@ ChangeNoteTextCommand::ChangeNoteTextCommand(SketchWidget *sketchWidget, long id
     m_newText = newText;
 	m_oldSize = oldSize;
 	m_newSize = newSize;
-	m_firstTime = true;
 }
 
 void ChangeNoteTextCommand::undo() {
@@ -1473,11 +1603,12 @@ void ChangeNoteTextCommand::undo() {
 	if (m_oldSize != m_newSize) {
 		m_sketchWidget->resizeNote(m_itemID, m_oldSize);
 	}
+    BaseCommand::undo();
 }
 
 void ChangeNoteTextCommand::redo() {
-	if (m_firstTime) {
-		m_firstTime = false;
+	if (m_skipFirstRedo) {
+		m_skipFirstRedo = false;
 		return;
 	}
 
@@ -1485,11 +1616,7 @@ void ChangeNoteTextCommand::redo() {
 	if (m_oldSize != m_newSize) {
 		m_sketchWidget->resizeNote(m_itemID, m_newSize);
 	}
-
-}
-
-void ChangeNoteTextCommand::setFirstTime(bool f) {
-	m_firstTime = f;
+    BaseCommand::redo();
 }
 
 int ChangeNoteTextCommand::id() const {
@@ -1538,11 +1665,13 @@ RotateFlipLabelCommand::RotateFlipLabelCommand(SketchWidget* sketchWidget, long
 void RotateFlipLabelCommand::undo()
 {
     m_sketchWidget->rotateFlipPartLabel(m_itemID, -m_degrees, m_orientation);
+    BaseCommand::undo();
 }
 
 void RotateFlipLabelCommand::redo()
 {
     m_sketchWidget->rotateFlipPartLabel(m_itemID, m_degrees, m_orientation);
+    BaseCommand::redo();
 }
 
 QString RotateFlipLabelCommand::getParamString() const {
@@ -1566,11 +1695,13 @@ ResizeNoteCommand::ResizeNoteCommand(SketchWidget* sketchWidget, long itemID, co
 void ResizeNoteCommand::undo()
 {
     m_sketchWidget->resizeNote(m_itemID, m_oldSize);
+    BaseCommand::undo();
 }
 
 void ResizeNoteCommand::redo()
 {
     m_sketchWidget->resizeNote(m_itemID, m_newSize);
+    BaseCommand::redo();
 }
 
 QString ResizeNoteCommand::getParamString() const {
@@ -1598,12 +1729,14 @@ void ResizeBoardCommand::undo() {
     if (!m_redoOnly) {
 	    m_sketchWidget->resizeBoard(m_itemID, m_oldWidth, m_oldHeight);
     }
+    BaseCommand::undo();
 }
 
 void ResizeBoardCommand::redo() {
     if (!m_undoOnly) {
 	    m_sketchWidget->resizeBoard(m_itemID, m_newWidth, m_newHeight);
     }
+    BaseCommand::redo();
 }
 
 QString ResizeBoardCommand::getParamString() const {
@@ -1631,11 +1764,13 @@ TransformItemCommand::TransformItemCommand(SketchWidget *sketchWidget, long id,
 void TransformItemCommand::undo()
 {
     m_sketchWidget->transformItem(m_itemID, m_oldMatrix);
+    BaseCommand::undo();
 }
 
 void TransformItemCommand::redo()
 {
     m_sketchWidget->transformItem(m_itemID, m_newMatrix);
+    BaseCommand::redo();
 }
 
 QString TransformItemCommand::getParamString() const {
@@ -1659,10 +1794,12 @@ SetResistanceCommand::SetResistanceCommand(SketchWidget * sketchWidget, long ite
 
 void SetResistanceCommand::undo() {
 	m_sketchWidget->setResistance(m_itemID, m_oldResistance, m_oldPinSpacing, true);
+    BaseCommand::undo();
 }
 
 void SetResistanceCommand::redo() {
 	m_sketchWidget->setResistance(m_itemID, m_newResistance, m_newPinSpacing, true);
+    BaseCommand::redo();
 }
 
 QString SetResistanceCommand::getParamString() const {
@@ -1689,10 +1826,12 @@ SetPropCommand::SetPropCommand(SketchWidget * sketchWidget, long itemID, QString
 
 void SetPropCommand::undo() {
 	m_sketchWidget->setProp(m_itemID, m_prop, m_oldValue, m_redraw, true);
+    BaseCommand::undo();
 }
 
 void SetPropCommand::redo() {
 	m_sketchWidget->setProp(m_itemID, m_prop, m_newValue, m_redraw, true);
+    BaseCommand::redo();
 }
 
 QString SetPropCommand::getParamString() const {
@@ -1722,10 +1861,12 @@ ResizeJumperItemCommand::ResizeJumperItemCommand(SketchWidget * sketchWidget, lo
 
 void ResizeJumperItemCommand::undo() {
 	m_sketchWidget->resizeJumperItem(m_itemID, m_oldPos, m_oldC0, m_oldC1);
+    BaseCommand::undo();
 }
 
 void ResizeJumperItemCommand::redo() {
 	m_sketchWidget->resizeJumperItem(m_itemID, m_newPos, m_newC0, m_newC1);
+    BaseCommand::redo();
 }
 
 QString ResizeJumperItemCommand::getParamString() const {
@@ -1755,6 +1896,7 @@ void ShowLabelCommand::undo()
 	foreach (long id, m_idStates.keys()) {
 		m_sketchWidget->showPartLabel(id, (m_idStates.value(id) & 2) != 0);
 	}
+    BaseCommand::undo();
 }
 
 void ShowLabelCommand::redo()
@@ -1762,6 +1904,7 @@ void ShowLabelCommand::redo()
 	foreach (long id, m_idStates.keys()) {
 		m_sketchWidget->showPartLabel(id, (m_idStates.value(id) & 1) != 0);
 	}
+    BaseCommand::redo();
 }
 
 void ShowLabelCommand::add(long id, bool prev, bool post)
@@ -1798,6 +1941,7 @@ void LoadLogoImageCommand::undo() {
     if (!m_redoOnly) {
 	    m_sketchWidget->loadLogoImage(m_itemID, m_oldSvg, m_oldAspectRatio, m_oldFilename);
     }
+    BaseCommand::undo();
 }
 
 void LoadLogoImageCommand::redo() {
@@ -1806,6 +1950,7 @@ void LoadLogoImageCommand::redo() {
 	else if (!m_undoOnly) {
 		m_sketchWidget->loadLogoImage(m_itemID, m_newFilename, m_addName);
 	}
+    BaseCommand::redo();
 }
 
 QString LoadLogoImageCommand::getParamString() const {
@@ -1830,6 +1975,7 @@ void ChangeBoardLayersCommand::undo() {
 	for (int i = childCount() - 1; i >= 0; i--) {
 		((QUndoCommand *) child(i))->undo();
 	}
+    BaseCommand::undo();
 }
 
 void ChangeBoardLayersCommand::redo() {
@@ -1837,6 +1983,7 @@ void ChangeBoardLayersCommand::redo() {
 	for (int i = 0; i < childCount(); i++) {
 		((QUndoCommand *) child(i))->redo();
 	}
+    BaseCommand::redo();
 }
 
 QString ChangeBoardLayersCommand::getParamString() const {
@@ -1859,10 +2006,12 @@ SetDropOffsetCommand::SetDropOffsetCommand(SketchWidget *sketchWidget, long id,
 
 void SetDropOffsetCommand::undo() { 
 	// only used when creating new parts
+    BaseCommand::undo();
 }
 
 void SetDropOffsetCommand::redo() {
 	m_sketchWidget->setItemDropOffset(m_itemID, m_dropOffset);
+    BaseCommand::redo();
 
 }
 
@@ -1887,10 +2036,12 @@ RenamePinsCommand::RenamePinsCommand(SketchWidget *sketchWidget, long id, const
 
 void RenamePinsCommand::undo() {
 	m_sketchWidget->renamePins(m_itemID, m_oldLabels, m_singleRow);
+    BaseCommand::undo();
 }
 
 void RenamePinsCommand::redo() {
 	m_sketchWidget->renamePins(m_itemID, m_newLabels, m_singleRow);
+    BaseCommand::redo();
 }
 
 QString RenamePinsCommand::getParamString() const {
@@ -1914,6 +2065,7 @@ void GroundFillSeedCommand::undo()
 	foreach(GFSThing gfsThing, m_items) {
 		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, !gfsThing.seed);
 	}
+    BaseCommand::undo();
 }
 
 void GroundFillSeedCommand::redo()
@@ -1921,6 +2073,7 @@ void GroundFillSeedCommand::redo()
 	foreach(GFSThing gfsThing, m_items) {
 		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, gfsThing.seed);
 	}
+    BaseCommand::redo();
 }
 
 void GroundFillSeedCommand::addItem(long id, const QString & connectorID, bool seed)
@@ -1958,6 +2111,7 @@ void WireExtrasCommand::undo()
 	if (!m_redoOnly) {
 		m_sketchWidget->setWireExtras(m_fromID, m_oldExtras);
 	}
+    BaseCommand::undo();
 }
 
 void WireExtrasCommand::redo()
@@ -1965,6 +2119,7 @@ void WireExtrasCommand::redo()
 	if (!m_undoOnly) {
 		m_sketchWidget->setWireExtras(m_fromID, m_newExtras);
 	}
+    BaseCommand::redo();
 }
 
 QString WireExtrasCommand::getParamString() const {
@@ -1975,8 +2130,78 @@ QString WireExtrasCommand::getParamString() const {
 		;
 }
 
-////////////////////////////////////
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+HidePartLayerCommand::HidePartLayerCommand(SketchWidget *sketchWidget, long fromID,
+									ViewLayer::ViewLayerID layerID, bool wasHidden, bool isHidden,
+    								QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_fromID = fromID;
+	m_wasHidden = wasHidden;
+    m_isHidden = isHidden;
+    m_layerID = layerID;
+}
+
+void HidePartLayerCommand::undo()
+{
+    m_sketchWidget->hidePartLayer(m_fromID, m_layerID, m_wasHidden);
+    BaseCommand::undo();
+}
+
+void HidePartLayerCommand::redo()
+{
+    m_sketchWidget->hidePartLayer(m_fromID, m_layerID, m_isHidden);
+    BaseCommand::redo();
+}
+
+QString HidePartLayerCommand::getParamString() const {
+	return QString("HidePartLayerCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 l:%2 was:%3 is:%4")
+		.arg(m_fromID)
+		.arg(m_layerID)
+		.arg(m_wasHidden)
+		.arg(m_isHidden)				
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+AddSubpartCommand::AddSubpartCommand(SketchWidget *sketchWidget,  CrossViewType crossView, long id, long subpartID, QUndoCommand *parent)
+    : BaseCommand(crossView, sketchWidget, parent)
+{
+    m_itemID = id;
+    m_subpartItemID = subpartID;
+
+}
+
+void AddSubpartCommand::undo()
+{
+    if (!m_redoOnly) {
+        m_sketchWidget->addSubpart(m_itemID, m_subpartItemID, true);
+    }
+    BaseCommand::undo();
+}
+
+void AddSubpartCommand::redo()
+{
+    if (!m_undoOnly) {
+        m_sketchWidget->addSubpart(m_itemID, m_subpartItemID, true);
+    }
+    BaseCommand::redo();
+}
+
+QString AddSubpartCommand::getParamString() const {
+	return QString("AddSubpartCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" id:%1 subpart id:%2")
+		.arg(m_itemID)
+		.arg(m_subpartItemID)	
+		;
+}
+
+////////////////////////////////////
 
 TemporaryCommand::TemporaryCommand(const QString & text) : QUndoCommand(text) {
     m_enabled = true;
