@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify\
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6954 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-05 10:22:00 +0200 (Fr, 05. Apr 2013) $
 
 ********************************************************************/
 
@@ -39,6 +39,27 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 /////////////////////////////////////////////
 
+class CommandProgress : public QObject {
+    Q_OBJECT
+
+public:
+    CommandProgress();
+
+    void setActive(bool);
+    bool active();
+    void emitUndo();
+    void emitRedo();
+
+signals:
+    void incUndo();
+    void incRedo();
+
+protected:
+    bool m_active;
+};
+
+/////////////////////////////////////////////
+
 class BaseCommand : public QUndoCommand
 {
 public:
@@ -66,6 +87,13 @@ public:
 	int index() const;
 	void setUndoOnly();
 	void setRedoOnly();
+    void setSkipFirstRedo();
+    void undo();
+    void redo();
+
+    static int totalChildCount(const QUndoCommand *);
+    static CommandProgress * initProgress();
+    static void clearProgress();
 
 protected:
 	virtual QString getParamString() const;
@@ -80,6 +108,9 @@ protected:
 	int m_index;
 	bool m_undoOnly;
 	bool m_redoOnly;
+    bool m_skipFirstRedo;
+
+    static CommandProgress m_commandProgress;
 };
 
 /////////////////////////////////////////////
@@ -87,7 +118,7 @@ protected:
 class AddDeleteItemCommand : public BaseCommand
 {
 public:
-	AddDeleteItemCommand(class SketchWidget * sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerSpec, ViewGeometry &, qint64 id, long modelIndex, QUndoCommand *parent);
+	AddDeleteItemCommand(class SketchWidget * sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerPlacement, ViewGeometry &, qint64 id, long modelIndex, QUndoCommand *parent);
 
 	long itemID() const;
 	void setDropOrigin(class SketchWidget *);
@@ -102,7 +133,7 @@ protected:
     ViewGeometry m_viewGeometry;
 	long m_modelIndex;
 	class SketchWidget * m_dropOrigin;
-	ViewLayer::ViewLayerSpec m_viewLayerSpec;
+	ViewLayer::ViewLayerPlacement m_viewLayerPlacement;
 };
 
 /////////////////////////////////////////////
@@ -110,10 +141,9 @@ protected:
 class AddItemCommand : public AddDeleteItemCommand
 {
 public:
-    AddItemCommand(class SketchWidget *sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerSpec, ViewGeometry &, qint64 id, bool updateInfoView, long modelIndex, QUndoCommand *parent);
+    AddItemCommand(class SketchWidget *sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerPlacement, ViewGeometry &, qint64 id, bool updateInfoView, long modelIndex, QUndoCommand *parent);
     void undo();
     void redo();
-	void turnOffFirstRedo();
 	void addRestoreIndexesCommand(class RestoreIndexesCommand *);
 
 protected:
@@ -121,8 +151,6 @@ protected:
 
 protected:
 	bool m_updateInfoView;
-	bool m_firstRedo;
-	bool m_doFirstRedo;
 	bool m_module;
 	RestoreIndexesCommand * m_restoreIndexesCommand;
 };
@@ -132,7 +160,7 @@ protected:
 class DeleteItemCommand : public AddDeleteItemCommand
 {
 public:
-	DeleteItemCommand(class SketchWidget *sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerSpec, ViewGeometry &, qint64 id, long modelIndex, QUndoCommand *parent);
+	DeleteItemCommand(class SketchWidget *sketchWidget, BaseCommand::CrossViewType, QString moduleID, ViewLayer::ViewLayerPlacement, ViewGeometry &, qint64 id, long modelIndex, QUndoCommand *parent);
     void undo();
     void redo();
 
@@ -266,11 +294,12 @@ public:
 	ChangeConnectionCommand(class SketchWidget * sketchWidget, BaseCommand::CrossViewType,
 							long fromID, const QString & fromConnectorID,
 							long toID, const QString & toConnectorID,
-							ViewLayer::ViewLayerSpec,
+							ViewLayer::ViewLayerPlacement,
 							bool connect, QUndoCommand * parent);
 	void undo();
 	void redo();
 	void setUpdateConnections(bool updatem);
+    void disable();
 
 protected:
 	QString getParamString() const;
@@ -282,7 +311,8 @@ protected:
     QString m_toConnectorID;
 	bool m_connect;
 	bool m_updateConnections;
-	ViewLayer::ViewLayerSpec m_viewLayerSpec;
+	ViewLayer::ViewLayerPlacement m_viewLayerPlacement;
+    bool m_enabled;
 
 };
 
@@ -317,11 +347,10 @@ class ChangeWireCurveCommand : public BaseCommand
 {
 public:
     ChangeWireCurveCommand(class SketchWidget *sketchWidget, long fromID,
-    					const class Bezier * oldBezier, const class Bezier * newBezier,
+    					const class Bezier * oldBezier, const class Bezier * newBezier, bool wasAutoroutable,
     					QUndoCommand *parent);
     void undo();
     void redo();
-	void setFirstTime();
 
 protected:
 	QString getParamString() const;
@@ -330,7 +359,7 @@ protected:
     long m_fromID;
     class Bezier * m_newBezier;
     class Bezier * m_oldBezier;
-    bool m_firstTime;
+    bool m_wasAutoroutable;
 };
 
 /////////////////////////////////////////////
@@ -389,7 +418,6 @@ public:
     					QUndoCommand *parent);
     void undo();
     void redo();
-	void setFirstTime();
 
 protected:
 	QString getParamString() const;
@@ -398,7 +426,6 @@ protected:
     long m_fromID;
     class Bezier * m_newBezier;
     class Bezier * m_oldBezier;
-    bool m_firstTime;
 	QString m_fromConnectorID;
 	int m_index;
 };
@@ -413,7 +440,6 @@ public:
 						const class Bezier *, const class Bezier *, const class Bezier *, QUndoCommand *parent);
     void undo();
     void redo();
-	void setFirstTime();
 
 protected:
 	QString getParamString() const;
@@ -423,7 +449,6 @@ protected:
     class Bezier * m_bezier0;
     class Bezier * m_bezier1;
     class Bezier * m_bezier2;
-    bool m_firstTime;
 	QString m_fromConnectorID;
 	int m_index;
 	int m_oldCount;
@@ -572,7 +597,6 @@ protected:
 protected:
 	long m_itemID;
 	QList<StickyThing *> m_stickyList;
-	bool m_firstTime;
 	bool m_checkCurrent;
 	CheckType m_checkType;
 };
@@ -675,7 +699,21 @@ protected:
 protected:
 	QSet<SketchWidget *> m_sketchWidgets;
 	QList<RatsnestConnectThing> m_ratsnestConnectThings;
-	Direction m_direction;
+	CleanUpWiresCommand::Direction m_direction;
+};
+
+/////////////////////////////////////////////
+
+class CleanUpRatsnestsCommand : public BaseCommand
+{
+public:
+	CleanUpRatsnestsCommand(class SketchWidget * sketchWidget, CleanUpWiresCommand::Direction, QUndoCommand * parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
 };
 
 /////////////////////////////////////////////
@@ -765,7 +803,6 @@ protected:
 
 protected:
     long m_itemID;
-	bool m_firstTime;
 };
 
 /////////////////////////////////////////////
@@ -794,7 +831,6 @@ public:
     ChangeNoteTextCommand(class SketchWidget *sketchWidget, long id, const QString & oldText, const QString & newText, QSizeF oldSize, QSizeF newSize, QUndoCommand *parent);
     void undo();
     void redo();
-	void setFirstTime(bool);
 	int id() const;
 	bool mergeWith(const QUndoCommand *other);
 
@@ -807,7 +843,6 @@ protected:
     QString m_newText;
 	QSizeF m_oldSize;
 	QSizeF m_newSize;
-	bool m_firstTime;
 
 	static int changeNoteTextCommandID;
 };
@@ -1069,6 +1104,28 @@ protected:
 
 /////////////////////////////////////////////
 
+class HidePartLayerCommand : public BaseCommand
+{
+public:
+    HidePartLayerCommand(class SketchWidget *sketchWidget, long fromID,
+					  ViewLayer::ViewLayerID layerID, bool wasHidden, bool isHidden,
+    				  QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_fromID;
+    double m_wasHidden;
+    double m_isHidden;
+	ViewLayer::ViewLayerID m_layerID;
+    ViewLayer::ViewLayerID m_oldLayer;
+};
+
+/////////////////////////////////////////////
+
 class TemporaryCommand : public QUndoCommand
 {
 
@@ -1084,4 +1141,23 @@ protected:
     bool m_enabled;
 };
 
+/////////////////////////////////////////////
+
+class AddSubpartCommand : public BaseCommand
+{
+public:
+    AddSubpartCommand(class SketchWidget *sketchWidget, CrossViewType crossView, long id, long subpartID, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_itemID;
+    long m_subpartItemID;
+};
+
+/////////////////////////////////////////////
+
 #endif // COMMANDS_H
