@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6291 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-14 05:12:01 +0200 (Tue, 14 Aug 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -322,7 +322,6 @@ ConnectorItem::ConnectorItem( Connector * connector, ItemBase * attachedTo )
 	m_connectorDetectT = m_connectorDrawT = 0;
 	m_draggingCurve = m_draggingLeg = m_rubberBandLeg = m_bigDot = m_hybrid = false;
 	m_marked = false;
-	m_checkedEffectively = false;
 	m_hoverEnterSpaceBarWasPressed = m_spaceBarWasPressed = false;
 	m_overConnectorItem = NULL;
 	m_connectorHovering = false;
@@ -355,6 +354,7 @@ ConnectorItem::~ConnectorItem() {
 
 void ConnectorItem::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 
+    //debugInfo("connector hoverEnter");
 	/*
 	QRectF sbr = this->sceneBoundingRect();
 	QPointF p = event->scenePos();
@@ -529,7 +529,7 @@ void ConnectorItem::restoreColor(bool doBuses, int busConnectionCount, bool doCr
 		.arg(this->connectorSharedName())
 		.arg(this->attachedToID())
 		.arg(this->attachedToInstanceTitle())
-		.arg(this->attachedToViewIdentifier())
+		.arg(this->attachedToViewID())
 		.arg(this->attachedToViewLayerID())
 		);
 		*/
@@ -552,7 +552,7 @@ void ConnectorItem::restoreColor(bool doBuses, int busConnectionCount, bool doCr
 		Bus * b = bus();
 		if (b != NULL) {
 			QList<ConnectorItem *> busConnectedItems;
-			attachedTo()->busConnectorItems(b, busConnectedItems);
+			attachedTo()->busConnectorItems(b, this, busConnectedItems);
 			busConnectedItems.removeOne(this);
 			foreach (ConnectorItem * busConnectorItem, busConnectedItems) {
 				busConnectorItem->restoreColor(false, connectedToCount, true);
@@ -592,7 +592,7 @@ void ConnectorItem::restoreColor(bool doBuses, int busConnectionCount, bool doCr
 		.arg(this->connectorSharedName())
 		.arg(this->attachedToID())
 		.arg(this->attachedToInstanceTitle())
-		.arg(this->attachedToViewIdentifier())
+		.arg(this->attachedToViewID())
 		.arg(this->attachedToViewLayerID())
 		.arg(how)
 	);
@@ -844,7 +844,7 @@ void ConnectorItem::mousePressEvent(QGraphicsSceneMouseEvent *event) {
 		connectorItem->showEqualPotential(true);
 	}
 	
-	if (m_rubberBandLeg) {
+	if (m_rubberBandLeg && this->m_attachedTo != NULL && m_attachedTo->acceptsMousePressLegEvent(this, event)) {
 		if (legMousePressEvent(event)) return;
 	}
 
@@ -860,7 +860,7 @@ int ConnectorItem::connectionsCount() {
 	return m_connectedTo.count();
 }
 
-void ConnectorItem::attachedMoved() {
+void ConnectorItem::attachedMoved(bool includeRatsnest, QList<ConnectorItem *> & already) {
 	//DebugDialog::debug("attached moved");
     if (!this->isVisible()) return;
 
@@ -881,11 +881,11 @@ void ConnectorItem::attachedMoved() {
 			//itemBase->debugInfo("    ");
 			continue;
 		}
-        if (itemBase->getRatsnest()) {
+        if (itemBase->getRatsnest() && !includeRatsnest) {
             continue;
         }
 
-        toConnectorItem->attachedTo()->connectedMoved(this, toConnectorItem);
+        toConnectorItem->attachedTo()->connectedMoved(this, toConnectorItem, already);
     }
 }
 
@@ -1007,7 +1007,7 @@ void ConnectorItem::setInactive(bool inactivate) {
 }
 
 void ConnectorItem::setHiddenOrInactive() {
-	if (m_hidden || m_inactive || m_hybrid) {
+	if (m_hidden || m_inactive || m_hybrid || m_layerHidden) {
 		this->setAcceptedMouseButtons(Qt::NoButton);
 		this->unsetCursor();
 		setAcceptHoverEvents(false);
@@ -1035,6 +1035,12 @@ const QString & ConnectorItem::connectorSharedID() {
 	return m_connector->connectorSharedID();
 }
 
+const QString & ConnectorItem::connectorSharedReplacedby() {
+	if (m_connector == NULL) return ___emptyString___;
+
+	return m_connector->connectorSharedReplacedby();
+}
+
 ErcData * ConnectorItem::connectorSharedErcData() {
 	if (m_connector == NULL) return NULL;
 
@@ -1077,28 +1083,22 @@ Bus * ConnectorItem::bus() {
 	return m_connector->bus();
 }
 
-int ConnectorItem::attachedToItemType() {
-	if (m_attachedTo == NULL) return ModelPart::Unknown;
-
-	return m_attachedTo->itemType();
-}
-
 ViewLayer::ViewLayerID ConnectorItem::attachedToViewLayerID() {
 	if (m_attachedTo == NULL) return ViewLayer::UnknownLayer;
 
 	return m_attachedTo->viewLayerID();
 }
 
-ViewLayer::ViewLayerSpec ConnectorItem::attachedToViewLayerSpec() {
-	if (m_attachedTo == NULL) return ViewLayer::UnknownSpec;
+ViewLayer::ViewLayerPlacement ConnectorItem::attachedToViewLayerPlacement() {
+	if (m_attachedTo == NULL) return ViewLayer::UnknownPlacement;
 
-	return m_attachedTo->viewLayerSpec();
+	return m_attachedTo->viewLayerPlacement();
 }
 
-ViewIdentifierClass::ViewIdentifier ConnectorItem::attachedToViewIdentifier() {
-	if (m_attachedTo == NULL) return ViewIdentifierClass::UnknownView;
+ViewLayer::ViewID ConnectorItem::attachedToViewID() {
+	if (m_attachedTo == NULL) return ViewLayer::UnknownView;
 
-	return m_attachedTo->viewIdentifier();
+	return m_attachedTo->viewID();
 }
 
 Connector::ConnectorType ConnectorItem::connectorType() {
@@ -1261,7 +1261,7 @@ bool ConnectorItem::isConnectedToPart() {
 	QList<ConnectorItem *> busConnectedItems;
 	Bus * b = bus();
 	if (b != NULL) {
-		attachedTo()->busConnectorItems(b, busConnectedItems);
+		attachedTo()->busConnectorItems(b, this, busConnectedItems);
 	}
 
 	for (int i = 0; i < tempItems.count(); i++) {
@@ -1273,6 +1273,7 @@ bool ConnectorItem::isConnectedToPart() {
 		{
 			switch (connectorItem->attachedToItemType()) {
 				case ModelPart::Symbol:
+				case ModelPart::SchematicSubpart:
 				case ModelPart::Jumper:
 				case ModelPart::Part:
 				case ModelPart::CopperFill:
@@ -1305,7 +1306,7 @@ bool ConnectorItem::isConnectedToPart() {
 		Bus * bus = connectorItem->bus();
 		if (bus != NULL) {
 			QList<ConnectorItem *> busConnectedItems;
-			connectorItem->attachedTo()->busConnectorItems(bus, busConnectedItems);
+			connectorItem->attachedTo()->busConnectorItems(bus, connectorItem, busConnectedItems);
 			foreach (ConnectorItem * busConnectedItem, busConnectedItems) {
 				if (!tempItems.contains(busConnectedItem)) {
 					tempItems.append(busConnectedItem);
@@ -1350,6 +1351,7 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 
 		// this one's a keeper
 		connectorItems.append(connectorItem);
+        //connectorItem->debugInfo("collect");
 
 		foreach (ConnectorItem * cto, connectorItem->connectedToItems()) {
 			if (tempItems.contains(cto)) continue;
@@ -1365,7 +1367,7 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 		Bus * bus = connectorItem->bus();
 		if (bus != NULL) {
 			QList<ConnectorItem *> busConnectedItems;
-			connectorItem->attachedTo()->busConnectorItems(bus, busConnectedItems);
+			connectorItem->attachedTo()->busConnectorItems(bus, connectorItem, busConnectedItems);
 #ifndef QT_NO_DEBUG
 			if (connectorItem->attachedToItemType() == ModelPart::Wire && busConnectedItems.count() != 2) {
 				connectorItem->debugInfo("bus is missing");
@@ -1382,19 +1384,19 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 	}
 }
 
-void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerSpec viewLayerSpec)
+void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerPlacement viewLayerPlacement)
 {
 	if (connectorItems.count() == 0) return;
 
 	//DebugDialog::debug("___________________________");
-	switch (viewLayerSpec) {
-		case ViewLayer::Top:
-		case ViewLayer::Bottom:
-		case ViewLayer::TopAndBottom:
+	switch (viewLayerPlacement) {
+		case ViewLayer::NewTop:
+		case ViewLayer::NewBottom:
+		case ViewLayer::NewTopAndBottom:
 			break;
 		default:
-			DebugDialog::debug(QString("collect parts unknown spec %1").arg(viewLayerSpec));
-			viewLayerSpec = ViewLayer::TopAndBottom;
+			DebugDialog::debug(QString("collect parts unknown spec %1").arg(viewLayerPlacement));
+			viewLayerPlacement = ViewLayer::NewTopAndBottom;
 			break;
 	}
 	
@@ -1406,6 +1408,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 		ItemBase * candidate = connectorItem->attachedTo();
 		switch (candidate->itemType()) {
 			case ModelPart::Symbol:
+			case ModelPart::SchematicSubpart:
 				if (!includeSymbols) break;
 			case ModelPart::Jumper:
 			case ModelPart::Part:
@@ -1413,7 +1416,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 			case ModelPart::Board:
 			case ModelPart::ResizableBoard:
 			case ModelPart::Via:
-				collectPart(connectorItem, partsConnectors, viewLayerSpec);
+				collectPart(connectorItem, partsConnectors, viewLayerPlacement);
 				break;
 			default:
 				break;
@@ -1421,7 +1424,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 	}
 }
 
-void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerSpec viewLayerSpec) {
+void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	if (partsConnectors.contains(connectorItem)) return;
 				
 	ConnectorItem * crossConnectorItem = connectorItem->getCrossLayerConnectorItem();
@@ -1430,7 +1433,7 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 			return;
 		}
 		
-		if (viewLayerSpec == ViewLayer::TopAndBottom) {
+		if (viewLayerPlacement == ViewLayer::NewTopAndBottom) {
 			partsConnectors.append(crossConnectorItem);
 			
 			/*
@@ -1442,14 +1445,14 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 			*/
 				
 		}
-		else if (viewLayerSpec == ViewLayer::Top) {
+		else if (viewLayerPlacement == ViewLayer::NewTop) {
 			if (connectorItem->attachedToViewLayerID() == ViewLayer::Copper1) {
 			}
 			else {
 				connectorItem = crossConnectorItem;
 			}
 		}
-		else if (viewLayerSpec == ViewLayer::Bottom) {
+		else if (viewLayerPlacement == ViewLayer::NewBottom) {
 			if (connectorItem->attachedToViewLayerID() == ViewLayer::Copper0) {
 			}
 			else {
@@ -1470,11 +1473,39 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 }
 
 void ConnectorItem::updateTooltip() {
-	QList<ConnectorItem *> connectors;
-	if (!attachedToItemType() == ModelPart::Wire) {
-		connectors.append(this);
+	if (attachedToItemType() != ModelPart::Wire) {
+		QString name = connectorSharedName();
+		QString descr = connectorSharedDescription();
+        if (name == descr) {
+            descr = "";
+        }
+        else {
+            descr = ":" + descr;
+        }
+        QString id = connectorSharedID();
+        int ix = IntegerFinder.indexIn(id);
+        if (ix < 0) {
+            id = "";
+        }
+        else {
+            if (attachedTo()->modelPart()->hasZeroConnector()) {
+                id = QString::number(IntegerFinder.cap(0).toInt() + 1) + " ";
+            }
+            else {
+                id = IntegerFinder.cap(0) + " ";
+            }
+        }
+       
+        QString tt = QString("%1<b>%2</b>%3<br /><span style='font-size:small;'>%4</span>")
+                .arg(id)
+                .arg(name)
+                .arg(descr)
+                .arg(attachedToTitle());
+		setToolTip(tt);
+		return;
 	}
 
+	QList<ConnectorItem *> connectors;
 	foreach(ConnectorItem * toConnectorItem, m_connectedTo) {
 		if (!toConnectorItem->attachedToItemType() == ModelPart::Wire) {
 			connectors.append(toConnectorItem);
@@ -1486,19 +1517,6 @@ void ConnectorItem::updateTooltip() {
 		return;
 	}
 
-	if (connectors.count() == 1) {
-		QString name = connectors[0]->connectorSharedName();
-		QString descr = connectors[0]->connectorSharedDescription();
-		if (name.compare(descr) == 0) {
-			name = connectors[0]->connectorSharedID();
-		}
-        QString tt = QString("<b>%1</b>: %2<br /><span style='font-size:small;'>%3</span>")
-                .arg(name)
-                .arg(descr)
-                .arg(connectors[0]->attachedToTitle());
-		setToolTip(tt);
-		return;
-	}
 
 	QString connections = QString("<ul style='margin-left:0;padding-left:0;'>");
 	foreach(ConnectorItem * connectorItem, connectors) {
@@ -1582,21 +1600,21 @@ bool ConnectorItem::isGrounded() {
 ConnectorItem * ConnectorItem::getCrossLayerConnectorItem() {
 	if (m_connector == NULL) return NULL;
 	if (m_attachedTo == NULL) return NULL;
-	if (m_attachedTo->viewIdentifier() != ViewIdentifierClass::PCBView) return NULL;
+	if (m_attachedTo->viewID() != ViewLayer::PCBView) return NULL;
 
 	ViewLayer::ViewLayerID viewLayerID = attachedToViewLayerID();
 	if (viewLayerID == ViewLayer::Copper0) {
-		return m_connector->connectorItemByViewLayerID(this->attachedToViewIdentifier(), ViewLayer::Copper1);
+		return m_connector->connectorItemByViewLayerID(this->attachedToViewID(), ViewLayer::Copper1);
 	}
 	if (viewLayerID == ViewLayer::Copper1) {
-		return m_connector->connectorItemByViewLayerID(this->attachedToViewIdentifier(), ViewLayer::Copper0);
+		return m_connector->connectorItemByViewLayerID(this->attachedToViewID(), ViewLayer::Copper0);
 	}
 
 	return NULL;
 }
 
-bool ConnectorItem::isInLayers(ViewLayer::ViewLayerSpec viewLayerSpec) {
-	return ViewLayer::copperLayers(viewLayerSpec).contains(attachedToViewLayerID());
+bool ConnectorItem::isInLayers(ViewLayer::ViewLayerPlacement viewLayerPlacement) {
+	return ViewLayer::copperLayers(viewLayerPlacement).contains(attachedToViewLayerID());
 }
 
 bool ConnectorItem::isCrossLayerConnectorItem(ConnectorItem * candidate) {
@@ -1632,7 +1650,7 @@ void ConnectorItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * o
 
 	if (m_effectively == EffectivelyUnknown) {
 		if (!m_circular && m_shape.isEmpty()) {
-			if (this->attachedTo()->viewIdentifier() == ViewIdentifierClass::PCBView) {
+			if (this->attachedTo()->viewID() == ViewLayer::PCBView) {
 				QRectF r = rect();
 				if (qAbs(r.width() - r.height()) < 0.01) m_effectively = EffectivelyCircular;
 				else m_effectively = EffectivelyRectangular;
@@ -1739,20 +1757,20 @@ void ConnectorItem::paintLeg(QPainter * painter, bool hasCurves)
 }
 
 
-ConnectorItem * ConnectorItem::chooseFromSpec(ViewLayer::ViewLayerSpec viewLayerSpec) {
+ConnectorItem * ConnectorItem::chooseFromSpec(ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	ConnectorItem * crossConnectorItem = getCrossLayerConnectorItem();
 	if (crossConnectorItem == NULL) return this;
 
 	ViewLayer::ViewLayerID basis = ViewLayer::Copper0;
-	switch (viewLayerSpec) {
-		case ViewLayer::Top:
+	switch (viewLayerPlacement) {
+		case ViewLayer::NewTop:
 			basis = ViewLayer::Copper1;
 			break;
-		case ViewLayer::Bottom:
+		case ViewLayer::NewBottom:
 			basis = ViewLayer::Copper0;
 			break;
 		default:
-			DebugDialog::debug(QString("unusual viewLayerSpec %1").arg(viewLayerSpec));
+			DebugDialog::debug(QString("unusual viewLayerPlacement %1").arg(viewLayerPlacement));
 			basis = ViewLayer::Copper0;
 			break;
 	}
@@ -1829,7 +1847,7 @@ void ConnectorItem::displayRatsnest(QList<ConnectorItem *> & partConnectorItems,
 
 	foreach (ConnectorItem * key, result.uniqueKeys()) {
 		foreach (ConnectorItem * value, result.values(key)) {
-			VirtualWire * vw = infoGraphicsView->makeOneRatsnestWire(key, value, false, color);
+			VirtualWire * vw = infoGraphicsView->makeOneRatsnestWire(key, value, false, color, false);
 			if (vw) {
 				vw->setColorWasNamed(colorWasNamed);
 			}
@@ -1864,7 +1882,7 @@ void ConnectorItem::clearRatsnestDisplay(QList<ConnectorItem *> & connectorItems
 			c2->tempRemove(vw->connector1(), false);
 		}
 
-		vw->debugInfo("removing rat 1");
+		//vw->debugInfo("removing rat 1");
 		vw->scene()->removeItem(vw);
 		delete vw;
 	}
@@ -1910,17 +1928,17 @@ void ConnectorItem::debugInfo(const QString & msg)
 {
 
 #ifndef QT_NO_DEBUG
-	QString s = QString("%1 cid:%2 %3 %4 id:%5 %6 vlid:%7 vid:%8 spec:%9 flg:%10 hy:%11 bus:%12 r:%13 sw:%14")
-	//QString s = QString("%1 %2 %3 %4 id:%5 %6 vlid:%7 vid:%8")
+	QString s = QString("%1 cid:%2 cname:%3 title:%4 id:%5 type:%6 inst:%7 vlid:%8 vid:%9 spec:%10 flg:%11 hy:%12 bus:%13 r:%14 sw:%15")
 			.arg(msg)
 			.arg(this->connectorSharedID())
 			.arg(this->connectorSharedName())
 			.arg(this->attachedToTitle())
 			.arg(this->attachedToID())
+			.arg(this->attachedToItemType())
 			.arg(this->attachedToInstanceTitle())
 			.arg(this->attachedToViewLayerID())
-			.arg(this->attachedToViewIdentifier())
-			.arg(this->attachedToViewLayerSpec())
+			.arg(this->attachedToViewID())
+			.arg(this->attachedToViewLayerPlacement())
 			.arg(this->attachedTo()->wireFlags()) 
 			.arg(this->m_hybrid)
 			.arg((long) this->bus(), 0, 16)
@@ -2334,7 +2352,7 @@ double ConnectorItem::findT(Bezier * bezier, double blen, double length)
 	return t;
 }
 
-const QString & ConnectorItem::legID(ViewIdentifierClass::ViewIdentifier viewID, ViewLayer::ViewLayerID viewLayerID) {
+const QString & ConnectorItem::legID(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
 	if (m_connector) return m_connector->legID(viewID, viewLayerID);
 
 	return ___emptyString___;
