@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -64,7 +64,7 @@ public:
 	void clearConnectorHover();
 	void connectTo(ConnectorItem *);
 	int connectionsCount();
-	void attachedMoved();
+	void attachedMoved(bool includeRatsnest, QList<ConnectorItem *> & already);
 	ConnectorItem * removeConnection(ItemBase *);
 	void removeConnection(ConnectorItem *, bool emitChange);
 	ConnectorItem * firstConnectedToIsh();
@@ -78,13 +78,13 @@ public:
 	void setInactive(bool inactivate);
 	ConnectorItem * overConnectorItem();
 	void setOverConnectorItem(ConnectorItem *);
-	int attachedToItemType();
 	ViewLayer::ViewLayerID attachedToViewLayerID();
-	ViewLayer::ViewLayerSpec attachedToViewLayerSpec();
-	ViewIdentifierClass::ViewIdentifier attachedToViewIdentifier();
+	ViewLayer::ViewLayerPlacement attachedToViewLayerPlacement();
+	ViewLayer::ViewID attachedToViewID();
 	const QString & connectorSharedID();
 	const QString & connectorSharedName();
 	const QString & connectorSharedDescription();
+	const QString & connectorSharedReplacedby();
 	class ErcData * connectorSharedErcData();
 	const QString & busID();
 	ModelPartShared * modelPartShared();
@@ -103,11 +103,11 @@ public:
 	void showEqualPotential(bool show);
 	void setHoverColor();
 	bool isGrounded();
-	ConnectorItem * chooseFromSpec(ViewLayer::ViewLayerSpec);
+	ConnectorItem * chooseFromSpec(ViewLayer::ViewLayerPlacement);
 	bool connectedToWires();
 	bool isCrossLayerConnectorItem(ConnectorItem * candidate);
 	bool isCrossLayerFrom(ConnectorItem * candidate);
-	bool isInLayers(ViewLayer::ViewLayerSpec);
+	bool isInLayers(ViewLayer::ViewLayerPlacement);
 	ConnectorItem * getCrossLayerConnectorItem();
 	void displayRatsnest(QList<ConnectorItem *> & partsConnectorItems, ViewGeometry::WireFlags myFlag);
 	void clearRatsnestDisplay(QList<ConnectorItem *> & connectorItems);
@@ -141,7 +141,7 @@ public:
 	void moveDone(int & index0, QPointF & oldPos0, QPointF & newPos0, int & index1, QPointF & oldPos1, QPointF & newPos1);
 	void killRubberBandLeg();  // hack; see caller
 	QRectF boundingRect() const;
-	const QString & legID(ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID);
+	const QString & legID(ViewLayer::ViewID, ViewLayer::ViewLayerID);
 	QPainterPath shape() const;
 	QPainterPath hoverShape() const;
 	void changeLegCurve(int index, const class Bezier *);
@@ -214,7 +214,6 @@ protected:
 	bool m_connectorHovering;
 	bool m_spaceBarWasPressed;
 	bool m_hoverEnterSpaceBarWasPressed;
-	bool m_checkedEffectively;
 	bool m_marked;
 	bool m_hybrid;
 	bool m_bigDot;
@@ -241,11 +240,11 @@ protected:
 	static QList<ConnectorItem *>  m_equalPotentialDisplayItems;
 
 protected:
-	static void collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerSpec);
+	static void collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerPlacement);
 
 public:
 	static void collectEqualPotential(QList<ConnectorItem *> & connectorItems, bool crossLayers, ViewGeometry::WireFlags skipFlags);
-	static void collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerSpec);
+	static void collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerPlacement);
 	static void clearEqualPotentialDisplay();
 	static bool isGrounded(ConnectorItem * c1, ConnectorItem * c2);
 	static void collectConnectorNames(QList<ConnectorItem *> & connectorItems, QStringList & connectorNames);
