@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6904 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
 
 ********************************************************************/
 
@@ -40,21 +40,24 @@ TranslatorListModel::TranslatorListModel(QFileInfoList & fileInfoList, QObject*
         m_languages.insert("french", tr("French - %1").arg("Franais"));
         m_languages.insert("german", tr("German - %1").arg("Deutsch"));
         m_languages.insert("spanish", tr("Spanish - %1").arg("Espaol"));
-
-		ushort t1[] = { 0x65e5, 0x672c, 0x8a9e, 0 };
-		m_languages.insert("japanese", tr("Japanese - %1").arg(QString::fromUtf16(t1)));
-
         m_languages.insert("portuguese_portugal", tr("Portuguese (European)- %1").arg("Portugus (Europeu)"));
         m_languages.insert("portuguese_brazil", tr("Portuguese (Brazilian) - %1").arg("Portugus (do Brasil)"));
         m_languages.insert("hungarian", tr("Hungarian - %1").arg("Magyar"));
         m_languages.insert("estonian", tr("Estonian - %1").arg("Eesti keel"));
         m_languages.insert("dutch", tr("Dutch - %1").arg("Nederlands"));
+        m_languages.insert("italian", tr("Italian - %1").arg("Italiano"));
+        m_languages.insert("polish", tr("Polish - %1").arg("Polski"));
+		m_languages.insert("turkish", tr("Turkish - %1").arg("Trke"));
+		m_languages.insert("swedish", tr("Swedish - %1").arg("Svenska"));
+		m_languages.insert("galician", tr("Galician - %1").arg("Galego"));
+		m_languages.insert("indonesian", tr("Indonesian - %1").arg("Bahasa Indonesia"));
+
+		ushort t1[] = { 0x65e5, 0x672c, 0x8a9e, 0 };
+		m_languages.insert("japanese", tr("Japanese - %1").arg(QString::fromUtf16(t1)));
 
 		ushort t2[] = { 0x0420, 0x0443, 0x0441, 0x0441, 0x043a, 0x0438, 0x0439, 0 };
         m_languages.insert("russian", tr("Russian - %1").arg(QString::fromUtf16(t2)));
 
-        m_languages.insert("italian", tr("Italian - %1").arg("Italiano"));
-
  		ushort t3[] = { 0x05e2, 0x05d1, 0x05e8, 0x05d9, 0x05ea, 0 };
         m_languages.insert("hebrew", tr("Hebrew - %1").arg(QString::fromUtf16(t3)));
 
@@ -70,14 +73,9 @@ TranslatorListModel::TranslatorListModel(QFileInfoList & fileInfoList, QObject*
         ushort t7[] = { 0x6b63, 0x9ad4, 0x4e2d, 0x6587, 0x0020, 0x0028, 0x7e41, 0x9ad4, 0x0029, 0 };
         m_languages.insert("chinese_taiwan", tr("Chinese (Traditional) - %1").arg(QString::fromUtf16(t7)));
 
-        m_languages.insert("polish", tr("Polish - %1").arg("Polski"));
-
 		ushort t8[] = { 0x010C, 0x65, 0x0161, 0x74, 0x69, 0x6E, 0x61, 0 };
         m_languages.insert("czech", tr("Czech - %1").arg(QString::fromUtf16(t8)));
 
-		m_languages.insert("turkish", tr("Turkish - %1").arg("Trke"));
-		m_languages.insert("swedish", tr("Swedish - %1").arg("Svenska"));
-
 		ushort t9[] = { 0x52,  0x6F, 0x6D, 0xE2, 0x6E, 0x0103, 0 };
 		m_languages.insert("romanian", tr("Romanian - %1").arg(QString::fromUtf16(t9)));
 
@@ -90,17 +88,24 @@ TranslatorListModel::TranslatorListModel(QFileInfoList & fileInfoList, QObject*
 		ushort t12[] = { 0x0411, 0x044A, 0x043B, 0x0433, 0x0430, 0x0440, 0x0441, 0x043A, 0x0438, 0 };
 		m_languages.insert("bulgarian", tr("Bulgarian - %1").arg(QString::fromUtf16(t12)));
 
-
-		m_languages.insert("galician", tr("Galician - %1").arg("Galego"));
-
 		ushort t13[] = { 0xD55C,  0xAD6D,  0xC5B4, 0};
 		m_languages.insert("korean", tr("Korean - %1").arg(QString::fromUtf16(t13)));
 
-		m_languages.insert("indonesian", tr("Indonesian - %1").arg("Bahasa Indonesia"));
-
 		ushort t14[] = { 0x53, 0x6C, 0x6F, 0x76, 0x65, 0x6E, 0x010D, 0x69, 0x6E, 0x61, 0};
 		m_languages.insert("slovak", tr("Slovak - %1").arg(QString::fromUtf16(t14)));
 
+        ushort t15[] = { 0x09AC,  0x09BE,   0x0982,   0x09B2,   0x09BE, 0 };
+        m_languages.insert("bengali", tr("Bengali - %1").arg(QString::fromUtf16(t15)));
+
+        ushort t16[] = { 0x0641, 0x0627, 0x0631, 0x0633, 0x06CC, 0 };
+        m_languages.insert("persian", tr("Persian - %1").arg(QString::fromUtf16(t16)));
+
+        ushort t17[] = {0x53, 0x6C, 0x6F, 0x76, 0x65, 0x6E, 0x0161, 0x010D, 0x69, 0x6E, 0x61, 0 };
+        m_languages.insert("slovenian", tr("Slovenian - %1").arg(QString::fromUtf16(t17)));
+
+        ushort t18[] = { 0x092E,  0x0930,  0x093E,  0x0920,  0x0940, 0 };
+        m_languages.insert("marathi", tr("Marathi - %1").arg(QString::fromUtf16(t18)));
+
 
         // More languages written in their own language can be found
         // at http://www.mozilla.com/en-US/firefox/all.html 
@@ -119,7 +124,7 @@ TranslatorListModel::TranslatorListModel(QFileInfoList & fileInfoList, QObject*
 
 	if (m_localeList.count() == 0) {
 		foreach (QFileInfo fileinfo, fileInfoList) {
-			QString name = fileinfo.baseName();
+			QString name = fileinfo.completeBaseName();
 			name.replace("fritzing_", "");
 			QStringList names = name.split("_");
 			if (names.count() > 1) {
