@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,26 +18,25 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6289 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
+$Revision: 6975 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-19 12:51:22 +0200 (Fr, 19. Apr 2013) $
 
 ********************************************************************/
 
 #include "fapplication.h"
 #include "debugdialog.h"
 #include "utils/misc.h"
-#include "mainwindow.h"
+#include "mainwindow/mainwindow.h"
 #include "fsplashscreen.h"
 #include "version/version.h"
 #include "dialogs/prefsdialog.h"
 #include "help/helper.h"
-#include "partseditor/partseditormainwindow.h"
 #include "fsvgrenderer.h"
 #include "version/versionchecker.h"
 #include "version/updatedialog.h"
 #include "itemdrag.h"
-#include "viewswitcher/viewswitcher.h"
+#include "dock/viewswitcher.h"
 #include "items/wire.h"
 #include "partsbinpalette/binmanager/binmanager.h"
 #include "help/tipsandtricks.h"
@@ -52,6 +51,7 @@ $Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 #include "utils/ratsnestcolors.h"
 #include "utils/cursormaster.h"
 #include "utils/textutils.h"
+#include "utils/graphicsutils.h"
 #include "infoview/htmlinfoview.h"
 #include "svg/gedaelement2svg.h"
 #include "svg/kicadmodule2svg.h"
@@ -62,9 +62,10 @@ $Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 #include "items/partfactory.h"
 #include "items/propertydef.h"
 #include "dialogs/recoverydialog.h"
-#include "lib/qtsysteminfo/QtSystemInfo.h"
 #include "processeventblocker.h"
-#include "autoroute/cmrouter/panelizer.h"
+#include "autoroute/panelizer.h"
+#include "sketch/sketchwidget.h"
+#include "sketch/pcbsketchwidget.h"
 
 // dependency injection :P
 #include "referencemodel/sqlitereferencemodel.h"
@@ -85,8 +86,6 @@ $Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 #include <QNetworkRequest>
 #include <QMultiHash>
 
-static QNetworkAccessManager * NetworkAccessManager = NULL;
-
 #ifdef LINUX_32
 #define PLATFORM_NAME "linux-32bit"
 #endif
@@ -110,23 +109,185 @@ static QNetworkAccessManager * NetworkAccessManager = NULL;
 #endif
 #endif
 
-int FApplication::RestartNeeded = 9999;
-
-QSet<QString> InstalledFonts::InstalledFontsList;
-QMultiHash<QString, QString> InstalledFonts::InstalledFontsNameMapper;   // family name to filename; SVG files seem to have to use filename
-
 static const double LoadProgressStart = 0.085;
 static const double LoadProgressEnd = 0.6;
 
-//////////////////////////
+////////////////////////////////////////////////////
+
+FServer::FServer(QObject *parent)
+    : QTcpServer(parent)
+{
+}
+
+void FServer::incomingConnection(int socketDescriptor)
+{
+    emit newConnection(socketDescriptor);
+}
+
+////////////////////////////////////////////////////
+
+QMutex FServerThread::m_busy;
+
+FServerThread::FServerThread(int socketDescriptor, QObject *parent) : QThread(parent), m_socketDescriptor(socketDescriptor)
+{
+}
+
+ void FServerThread::run()
+ {
+    QTcpSocket * socket = new QTcpSocket();
+    if (!socket->setSocketDescriptor(m_socketDescriptor)) {
+        emit error(socket->error());
+        DebugDialog::debug(QString("Socket error %1 %2").arg(socket->error()).arg(socket->errorString()));
+        socket->deleteLater();
+        return;
+    }
+
+    socket->waitForReadyRead();
+    QString header;
+    while (socket->canReadLine()) {
+        header += socket->readLine();
+    }
+
+    DebugDialog::debug("header " + header);
+
+    QStringList tokens = header.split(QRegExp("[ \r\n][ \r\n]*"), QString::SkipEmptyParts);
+    if (tokens.count() <= 0) {
+        writeResponse(socket, 400, "Bad Request", "", "");
+        return;
+    }
+
+    if (tokens[0] != "GET") {
+        writeResponse(socket, 405, "Method Not Allowed", "", "");
+        return;
+    }
+
+    if (tokens.count() < 2) {
+        writeResponse(socket, 400, "Bad Request", "", "");
+        return;
+    }
+
+    QStringList params = tokens.at(1).split("/", QString::SkipEmptyParts);
+    QString command = params.takeFirst();
+    if (params.count() == 0) {
+        writeResponse(socket, 400, "Bad Request", "", "");
+        return;
+    }
+
+    QString subFolder = params.join("/");
+	bool fixSubFolder = false;
+    QString mimeType;
+    if (command == "svg") {
+		 mimeType = "image/svg+xml";
+    }
+    else if (command == "gerber") {
+    }
+    else if (command == "svg-tcp") {
+		fixSubFolder = true;
+    }
+    else if (command == "gerber-tcp") {
+		fixSubFolder = true;
+    }
+    else {
+        writeResponse(socket, 400, "Bad Request", "", "");
+        return;
+    }
+
+	if (fixSubFolder) {
+		// replace "/" that was removed from "http:/blah" above
+		int ix = subFolder.indexOf(":/");
+		if (ix >= 0) {
+			subFolder.insert(ix + 1, "/");
+		}
+	}
+
+    int waitInterval = 100;     // 100ms to wait
+    int timeoutSeconds = 2 * 60;    // timeout after 2 minutes
+    int attempts = timeoutSeconds * 1000 / waitInterval;  // timeout a
+    bool gotLock = false;
+    for (int i = 0; i < attempts; i++) {
+        if (m_busy.tryLock()) {
+            gotLock = true;
+            break;
+        }
+    }
+
+    if (!gotLock) {
+        writeResponse(socket, 503, "Service Unavailable", "", "Server busy.");
+        return;
+    }
+
+    DebugDialog::debug(QString("emitting do command %1 %2").arg(command).arg(subFolder));
+    QString result;
+	int status;
+    emit doCommand(command, subFolder, result, status);
+
+    m_busy.unlock();
+
+	if (status != 200) {
+		writeResponse(socket, status, "failed", "", result);
+	}
+	else if (command.endsWith("tcp")) {
+		QString filename = result;
+		mimeType = "application/zip";
+		QFile file(filename);
+		if (file.open(QFile::ReadOnly)) {
+			QString response = QString("HTTP/1.0 %1 %2\r\n").arg(200).arg("ok");
+			response += QString("Content-Type: %1; charset=\"utf-8\"\r\n").arg(mimeType);
+			response += QString("Content-Length: %1\r\n").arg(file.size());
+			response += QString("\r\n");    
+			socket->write(response.toUtf8());
+			int buffersize = 8192;
+			long remaining = file.size();
+			while (remaining >= buffersize) {
+				QByteArray bytes = file.read(buffersize);
+				socket->write(bytes);
+				remaining -= buffersize;
+			}
+			if (remaining > 0) {
+				QByteArray bytes = file.read(remaining);
+				socket->write(bytes);
+			}
+			socket->disconnectFromHost();
+			socket->waitForDisconnected();
+			socket->deleteLater();
+			file.close();
+		}
+		else {
+			writeResponse(socket, 500, "failed", "", "local zip failure (2)");
+		}
+
+		QFileInfo info(filename);
+		QDir dir = info.dir();
+		FolderUtils::rmdir(dir);
+	}
+	else {
+		writeResponse(socket, 200, "Ok", mimeType, result);
+	}
+}
+
+void FServerThread::writeResponse(QTcpSocket * socket, int code, const QString & codeString, const QString & mimeType, const QString & message) 
+{
+    QString type = mimeType;
+    if (type.isEmpty()) type = "text/plain";
+    QString response = QString("HTTP/1.0 %1 %2\r\n").arg(code).arg(codeString);
+    response += QString("Content-Type: %1; charset=\"utf-8\"\r\n").arg(type);
+    response += QString("Content-Length: %1\r\n").arg(message.count());
+    response += QString("\r\n%1").arg(message);
+       
+    socket->write(response.toUtf8());
+    socket->disconnectFromHost();
+    socket->waitForDisconnected();
+    socket->deleteLater();
+}
+
+////////////////////////////////////////////////////
 
 FApplication::FApplication( int & argc, char ** argv) : QApplication(argc, argv)
 {
-	MainWindow::RestartNeeded = FApplication::RestartNeeded;
+    m_fServer = NULL;
 	m_spaceBarIsPressed = false;
 	m_mousePressed = false;
 	m_referenceModel = NULL;
-	m_paletteBinModel = NULL;
 	m_started = false;
 	m_updateDialog = NULL;
 	m_lastTopmostWindow = NULL;
@@ -187,6 +348,21 @@ bool FApplication::init() {
 			toRemove << i << i + 1;
 		}
 
+		if ((m_arguments[i].compare("-drc", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("--drc", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = DRCService;
+			m_outputFolder = m_arguments[i + 1];
+			toRemove << i << i + 1;
+		}
+
+		if ((m_arguments[i].compare("-db", Qt::CaseInsensitive) == 0) ||
+            (m_arguments[i].compare("-database", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("--database", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = DatabaseService;
+			m_outputFolder = m_arguments[i + 1];
+			toRemove << i << i + 1;
+		}
+
 		if ((m_arguments[i].compare("-kicad", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--kicad", Qt::CaseInsensitive) == 0)) {
 			m_serviceType = KicadFootprintService;
@@ -201,6 +377,38 @@ bool FApplication::init() {
 			toRemove << i << i + 1;
 		}
 
+		if ((m_arguments[i].compare("-svg", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("--svg", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = SvgService;
+            DebugDialog::setEnabled(true);
+			m_outputFolder = m_arguments[i + 1];
+			toRemove << i << i + 1;
+		}
+
+		if ((m_arguments[i].compare("-port", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("--port", Qt::CaseInsensitive) == 0)) {
+            DebugDialog::setEnabled(true);
+            bool ok;
+            int p = m_arguments[i + 1].toInt(&ok);
+            if (ok) {
+                m_portNumber = p;
+            }
+
+            toRemove << i << i + 1;
+
+            if (i + 2 < m_arguments.count()) {
+                if (ok) {
+                    m_portRootFolder = m_arguments[i + 2];
+                    m_serviceType = PortService;
+                }
+                toRemove << i + 2;
+            }
+
+			m_outputFolder = m_arguments[i + 1];
+		}
+
+
+
 		if ((m_arguments[i].compare("-g", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("-gerber", Qt::CaseInsensitive) == 0)||
 			(m_arguments[i].compare("--gerber", Qt::CaseInsensitive) == 0)) {
@@ -216,6 +424,15 @@ bool FApplication::init() {
 			m_serviceType = PanelizerService;
 			m_panelFilename = m_arguments[i + 1];
 			m_outputFolder = " ";					// otherwise program will bail out
+            m_panelizerCustom = false;
+			toRemove << i << i + 1;
+		}
+
+		if ((m_arguments[i].compare("-pc", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = PanelizerService;
+			m_panelFilename = m_arguments[i + 1];
+			m_outputFolder = " ";					// otherwise program will bail out
+            m_panelizerCustom = true;
 			toRemove << i << i + 1;
 		}
 
@@ -285,14 +502,13 @@ bool FApplication::init() {
 
 	MainWindow::initNames();
 	FSvgRenderer::initNames();
-	ViewIdentifierClass::initNames();
+	ViewLayer::initNames();
 	RatsnestColors::initNames();
 	Wire::initNames();
 	ItemBase::initNames();
 	ViewLayer::initNames();
 	Connector::initNames();
 	Helper::initText();
-	PartsEditorMainWindow::initText();
 	BinManager::initNames();
 	PaletteModel::initNames();
 	SvgIconWidget::initNames();
@@ -313,7 +529,7 @@ FApplication::~FApplication(void)
 	}
 
 	FSvgRenderer::cleanup();
-	ViewIdentifierClass::cleanup();
+	ViewLayer::cleanup();
 	ViewLayer::cleanup();
 	ItemBase::cleanup();
 	Wire::cleanup();
@@ -337,10 +553,6 @@ FApplication::~FApplication(void)
 }
 
 void FApplication::clearModels() {
-	if (m_paletteBinModel) {
-		m_paletteBinModel->clearPartHash();
-		delete m_paletteBinModel;
-	}
 	if (m_referenceModel) {
 		m_referenceModel->clearPartHash();
 		delete m_referenceModel;
@@ -355,6 +567,7 @@ bool FApplication::spaceBarIsPressed() {
 bool FApplication::eventFilter(QObject *obj, QEvent *event)
 {
 	// check whether the space bar is down.
+    //qDebug() << "event" << event->type();
 
 	Q_UNUSED(obj);
 
@@ -440,7 +653,8 @@ bool FApplication::findTranslator(const QString & translationsPath) {
 		QLocale::setDefault(QLocale(suffix));
 	}
 
-    bool loaded = m_translator.load(QString("fritzing_") + suffix, translationsPath);
+    bool loaded = m_translator.load(QString("fritzing_") + suffix.toLower(), translationsPath);
+    DebugDialog::debug(QString("translation %1 loaded %2 from %3").arg(suffix).arg(loaded).arg(translationsPath));
 	if (loaded) {
 		QApplication::installTranslator(&m_translator);
 	}
@@ -452,8 +666,7 @@ void FApplication::registerFonts() {
 	registerFont(":/resources/fonts/DroidSans.ttf", true);
 	registerFont(":/resources/fonts/DroidSans-Bold.ttf", false);
 	registerFont(":/resources/fonts/DroidSansMono.ttf", false);
-	//registerFont(":/resources/fonts/ocra10.ttf", true);
-	registerFont(":/resources/fonts/OCRA.otf", true);
+	registerFont(":/resources/fonts/OCRA.ttf", true);
 
 	/*	
 		QFontDatabase database;
@@ -464,29 +677,41 @@ void FApplication::registerFonts() {
 	*/	
 }
 
-ReferenceModel * FApplication::loadReferenceModel() {
+ReferenceModel * FApplication::loadReferenceModel(const QString & databaseName, bool fullLoad) {
 	m_referenceModel = new CurrentReferenceModel();	
 	ItemBase::setReferenceModel(m_referenceModel);
 	connect(m_referenceModel, SIGNAL(loadedPart(int, int)), this, SLOT(loadedPart(int, int)));
-	m_referenceModel->loadAll(true);								// this is very slow
-	m_paletteBinModel = new PaletteModel(true, false, false);
-    m_paletteBinModel->setReferenceModel(m_referenceModel);
-	//DebugDialog::debug("after new palette model");
+
+    bool dbExists = false;
+    QDir * dir = FolderUtils::getApplicationSubFolder("parts");
+    QString dbPath;
+    if (dir) {
+        dbPath = dir->absoluteFilePath("parts.db");
+        QFileInfo info(dbPath);
+        dbExists = info.exists();
+    }
+
+	bool ok = m_referenceModel->loadAll(databaseName, fullLoad, dbExists);		// loads local parts, resource parts, and any other parts in files not in the db--these part override db parts with the same moduleID
+    if (ok && databaseName.isEmpty()) {
+        if (dir == NULL) {
+        }
+        else {
+            QFile file(dir->absoluteFilePath("parts.db"));
+            if (file.exists()) {
+                m_referenceModel->loadFromDB(dbPath);
+            }
+        }
+    }
+    delete dir;
+
 	return m_referenceModel;
 }
 
-MainWindow * FApplication::loadWindows(int & loaded, bool lockFiles) {
+MainWindow * FApplication::openWindowForService(bool lockFiles) {
 	// our MainWindows use WA_DeleteOnClose so this has to be added to the heap (via new) rather than the stack (for local vars)
-	MainWindow * mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", false, lockFiles);   // this is also slow
+	MainWindow * mainWindow = MainWindow::newMainWindow(m_referenceModel, "", false, lockFiles);   // this is also slow
 	mainWindow->setReportMissingModules(false);
-
-	loaded = 0;
-	initFilesToLoad();
-	foreach (QString file, m_filesToLoad) {
-		loadOne(mainWindow, file, loaded++);
-	}
-
-	//DebugDialog::debug("after argc");
+    mainWindow->noBackup();
 
 	return mainWindow;
 }
@@ -502,6 +727,14 @@ int FApplication::serviceStartup() {
 			runGedaService();
 			return 0;
 	
+		case DRCService:
+			runDRCService();
+			return 0;
+	
+		case DatabaseService:
+			runDatabaseService();
+			return 0;
+	
 		case KicadFootprintService:
 			runKicadFootprintService();
 			return 0;
@@ -514,6 +747,10 @@ int FApplication::serviceStartup() {
 			runGerberService();
 			return 0;
 
+		case SvgService:
+			runSvgService();
+			return 0;
+
 		case PanelizerService:
 			runPanelizerService();
 			return 0;
@@ -532,14 +769,14 @@ int FApplication::serviceStartup() {
 	}
 }
 
-
 void FApplication::runGerberService()
 {
-	createUserDataStoreFolderStructure();
-
-	registerFonts();
-	loadReferenceModel();
+	initService();
+    runGerberServiceAux();
+}
 
+void FApplication::runGerberServiceAux()
+{
 	QDir dir(m_outputFolder);
 	QString s = dir.absolutePath();
 	QStringList filters;
@@ -547,14 +784,13 @@ void FApplication::runGerberService()
 	QStringList filenames = dir.entryList(filters, QDir::Files);
 	foreach (QString filename, filenames) {
 		QString filepath = dir.absoluteFilePath(filename);
-		int loaded = 0;
-		MainWindow * mainWindow = loadWindows(loaded, false);
-		mainWindow->noBackup();
+		MainWindow * mainWindow = openWindowForService(false);
 		m_started = true;
-
+        
 		FolderUtils::setOpenSaveFolderAux(m_outputFolder);
 		if (mainWindow->loadWhich(filepath, false, false, "")) {
-			mainWindow->exportToGerber(m_outputFolder);
+            QFileInfo info(filepath);
+            GerberGenerator::exportToGerber(info.completeBaseName(), m_outputFolder, NULL, mainWindow->pcbView(), false);
 		}
 
 		mainWindow->setCloseSilently(true);
@@ -562,6 +798,78 @@ void FApplication::runGerberService()
 	}
 }
 
+void FApplication::initService() 
+{
+	createUserDataStoreFolderStructure();
+	registerFonts();
+	loadReferenceModel("", false);
+}
+
+void FApplication::runSvgService()
+{
+    initService();
+    runSvgServiceAux();
+}
+
+void FApplication::runSvgServiceAux()
+{
+	QDir dir(m_outputFolder);
+	QString s = dir.absolutePath();
+	QStringList filters;
+	filters << "*" + FritzingBundleExtension;
+	QStringList filenames = dir.entryList(filters, QDir::Files);
+	foreach (QString filename, filenames) {
+		QString filepath = dir.absoluteFilePath(filename);
+		MainWindow * mainWindow = openWindowForService(false);
+		m_started = true;
+        
+		FolderUtils::setOpenSaveFolderAux(m_outputFolder);
+		if (mainWindow->loadWhich(filepath, false, false, "")) {
+            QFileInfo info(filepath);
+            QList<ViewLayer::ViewID> ids;
+            ids << ViewLayer::BreadboardView << ViewLayer::SchematicView << ViewLayer::PCBView;
+            foreach (ViewLayer::ViewID id, ids) {
+                QString fn = QString("%1_%2.svg").arg(info.completeBaseName()).arg(ViewLayer::viewIDNaturalName(id));
+                QString svgPath = dir.absoluteFilePath(fn);
+                mainWindow->setCurrentView(id);
+                mainWindow->exportSvg(GraphicsUtils::StandardFritzingDPI, false, false, svgPath);
+            }
+		}
+
+		mainWindow->setCloseSilently(true);
+		mainWindow->close();
+	}
+}
+
+void FApplication::runDatabaseService()
+{
+	createUserDataStoreFolderStructure();
+
+    DebugDialog::setEnabled(true);
+    QDir * parent = FolderUtils::getApplicationSubFolder("pdb");
+    QFileInfoList dirs = parent->entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
+    delete parent;
+
+    QStringList nameFilters;
+    nameFilters << ("*" + FritzingPartExtension);
+    foreach (QFileInfo dirInfo, dirs) {
+        if (!dirInfo.isDir()) continue;
+
+        QDir dir(dirInfo.absoluteFilePath());
+        QFileInfoList files = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+        foreach (QFileInfo fileInfo, files) {
+            QString path = fileInfo.absoluteFilePath();
+            QString newPath = path;
+            newPath.replace("/pdb/", "/parts/");
+            QFile::rename(path, newPath);
+        }
+    }
+
+
+    QFile::remove(m_outputFolder);
+	loadReferenceModel(m_outputFolder, true);  // m_outputFolder is actually a full path ending in ".db"
+}
+
 void FApplication::runGedaService() {
 	try {
 		QDir dir(m_outputFolder);
@@ -585,6 +893,62 @@ void FApplication::runGedaService() {
 	}
 }
 
+
+void FApplication::runDRCService() {
+    m_started = true;
+    initService();
+    DebugDialog::setEnabled(true);
+
+    try {
+		QDir dir(m_outputFolder);
+		QStringList filters;
+		filters << "*.fzz";
+		QStringList filenames = dir.entryList(filters, QDir::Files);
+		foreach (QString filename, filenames) {
+			QString filepath = dir.absoluteFilePath(filename);
+	        MainWindow * mainWindow = openWindowForService(false);
+            if (mainWindow == NULL) continue;
+
+            mainWindow->setCloseSilently(true);
+
+	        if (!mainWindow->loadWhich(filepath, false, false, "")) {
+		        DebugDialog::debug(QString("failed to load '%1'").arg(filepath));
+		        mainWindow->close();
+                delete mainWindow;
+                continue;
+	        }
+
+	        mainWindow->showPCBView();
+
+            int moved = mainWindow->pcbView()->checkLoadedTraces();
+            if (moved > 0) {
+                QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("%1 wires moved from their saved position in %2.").arg(moved).arg(filepath));
+                DebugDialog::debug(QString("\ncheckloadedtraces %1\n").arg(filepath)); 
+            }
+	
+
+            Panelizer::checkDonuts(mainWindow, true);
+            Panelizer::checkText(mainWindow, true);
+
+            QList<ItemBase *> boards = mainWindow->pcbView()->findBoard();	
+	        foreach (ItemBase * boardItem, boards) {
+                mainWindow->pcbView()->selectAllItems(false, false);
+                boardItem->setSelected(true);
+                mainWindow->newDesignRulesCheck(false);
+            }
+
+            mainWindow->close();
+            delete mainWindow;
+		}
+	}
+	catch (const QString & msg) {
+		DebugDialog::debug(msg);
+	}
+	catch (...) {
+		DebugDialog::debug("who knows");
+	}
+}
+
 void FApplication::runKicadFootprintService() {
 	QDir dir(m_outputFolder);
 	QStringList filters;
@@ -662,7 +1026,7 @@ void FApplication::runKicadSchematicService() {
 	}
 }
 
-int FApplication::startup(bool firstRun)
+int FApplication::startup()
 {
 	//DebugDialog::setEnabled(true);
 
@@ -686,38 +1050,32 @@ int FApplication::startup(bool firstRun)
 
 	// DebugDialog::debug("Data Location: "+QDesktopServices::storageLocation(QDesktopServices::DataLocation));
 
-	if(firstRun) {		
-		registerFonts();
+	registerFonts();
 
-		if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressStart);
-		ProcessEventBlocker::processEvents();
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressStart);
+	ProcessEventBlocker::processEvents();
 
-		#ifdef Q_WS_WIN
-			// associate .fz file with fritzing app on windows (xp only--vista is different)
-			// TODO: don't change settings if they're already set?
-			// TODO: only do this at install time?
-			QSettings settings1("HKEY_CLASSES_ROOT\\Fritzing", QSettings::NativeFormat);
-			settings1.setValue(".", "Fritzing Application");
-			foreach (QString extension, fritzingExtensions()) {
-				QSettings settings2("HKEY_CLASSES_ROOT\\" + extension, QSettings::NativeFormat);
-				settings2.setValue(".", "Fritzing");
-			}
-			QSettings settings3("HKEY_CLASSES_ROOT\\Fritzing\\shell\\open\\command", QSettings::NativeFormat);
-			settings3.setValue(".", QString("\"%1\" \"%2\"")
-							   .arg(QDir::toNativeSeparators(QApplication::applicationFilePath()))
-							   .arg("%1") );
-		#endif
+	#ifdef Q_WS_WIN
+		// associate .fz file with fritzing app on windows (xp only--vista is different)
+		// TODO: don't change settings if they're already set?
+		// TODO: only do this at install time?
+		QSettings settings1("HKEY_CLASSES_ROOT\\Fritzing", QSettings::NativeFormat);
+		settings1.setValue(".", "Fritzing Application");
+		foreach (QString extension, fritzingExtensions()) {
+			QSettings settings2("HKEY_CLASSES_ROOT\\" + extension, QSettings::NativeFormat);
+			settings2.setValue(".", "Fritzing");
+		}
+		QSettings settings3("HKEY_CLASSES_ROOT\\Fritzing\\shell\\open\\command", QSettings::NativeFormat);
+		settings3.setValue(".", QString("\"%1\" \"%2\"")
+							.arg(QDir::toNativeSeparators(QApplication::applicationFilePath()))
+							.arg("%1") );
+	#endif
 
+	cleanFzzs();
 
-		cleanFzzs();
-	} 
-	else 
-	{
-		clearModels();
-		FSvgRenderer::cleanup();
-	}
+    createUserDataStoreFolderStructure();
 
-	loadReferenceModel();
+	loadReferenceModel("", false);
 
 	QString prevVersion;
 	{
@@ -727,24 +1085,27 @@ int FApplication::startup(bool firstRun)
 		QString currVersion = Version::versionString();
 		if(prevVersion != currVersion) {
 			QVariant pid = settings.value("pid");
+            QVariant language = settings.value("language");
 			settings.clear();
 			if (!pid.isNull()) {
 				settings.setValue("pid", pid);
 			}
+			if (!language.isNull()) {
+				settings.setValue("language", language);
+			}
 		}
 	}
 
 	//bool fabEnabled = settings.value(ORDERFABENABLED, QVariant(false)).toBool();
 	//if (!fabEnabled) {
-		NetworkAccessManager = new QNetworkAccessManager(this);
-		connect(NetworkAccessManager, SIGNAL(finished(QNetworkReply *)), this, SLOT(gotOrderFab(QNetworkReply *)));
-
-		NetworkAccessManager->get(QNetworkRequest(QUrl(QString("http://fab.fritzing.org/launched%1").arg(makeRequestParamsString()))));
+		QNetworkAccessManager * manager = new QNetworkAccessManager(this);
+		connect(manager, SIGNAL(finished(QNetworkReply *)), this, SLOT(gotOrderFab(QNetworkReply *)));
+		manager->get(QNetworkRequest(QUrl(QString("http://fab.fritzing.org/launched%1").arg(Version::makeRequestParamsString(true)))));
 	//}
 
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressEnd);
 
-	createUserDataStoreFolderStructure();
+
 
 	//DebugDialog::debug("after createUserDataStoreFolderStructure");
 
@@ -761,7 +1122,7 @@ int FApplication::startup(bool firstRun)
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.875);
 
 	DebugDialog::debug("load something");
-	loadSomething(firstRun, prevVersion);
+	loadSomething(prevVersion);
 	m_started = true;
 
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.99);
@@ -777,7 +1138,7 @@ void FApplication::registerFont(const QString &fontFile, bool reallyRegister) {
 		QStringList familyNames = QFontDatabase::applicationFontFamilies(id);
 		QFileInfo finfo(fontFile);
 		foreach (QString family, familyNames) {
-			InstalledFonts::InstalledFontsNameMapper.insert(family, finfo.baseName());
+			InstalledFonts::InstalledFontsNameMapper.insert(family, finfo.completeBaseName());
 			InstalledFonts::InstalledFontsList << family;
 			//DebugDialog::debug("registering font family: "+family);
 		}
@@ -792,7 +1153,7 @@ void FApplication::finish()
 }
 
 void FApplication::loadNew(QString path) {
-	MainWindow * mw = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, path, true, true);
+	MainWindow * mw = MainWindow::newMainWindow(m_referenceModel, path, true, true);
 	if (!mw->loadWhich(path, false, true, "")) {
 		mw->close();
 	}
@@ -936,7 +1297,7 @@ void FApplication::initSplash(FSplashScreen & splash) {
 
 struct Thing {
         QString moduleID;
-        ViewIdentifierClass::ViewIdentifier viewIdentifier;
+        ViewLayer::ViewID viewID;
         ViewLayer::ViewLayerID viewLayerID;
 };
 
@@ -969,7 +1330,7 @@ void FApplication::checkForUpdates(bool atUserRequest)
 
     QString atom = QString("http://fritzing.org/download/feed/atom/%1/%2")
 		.arg(PLATFORM_NAME)
-		.arg(makeRequestParamsString());
+		.arg(Version::makeRequestParamsString(true));
     DebugDialog::debug(atom);
     versionChecker->setUrl(atom);
 	m_updateDialog->setAtUserRequest(atUserRequest);
@@ -982,14 +1343,14 @@ void FApplication::checkForUpdates(bool atUserRequest)
 
 void FApplication::enableCheckUpdates(bool enabled)
 {
-	DebugDialog::debug("before enable check updates");
+	//DebugDialog::debug("before enable check updates");
     foreach (QWidget *widget, QApplication::topLevelWidgets()) {
         MainWindow *mainWindow = qobject_cast<MainWindow *>(widget);
 		if (mainWindow) {
 			mainWindow->enableCheckUpdates(enabled);
 		}
 	}
-	DebugDialog::debug("after enable check updates");
+	//DebugDialog::debug("after enable check updates");
 }
 
 
@@ -1125,7 +1486,12 @@ which is really not intended for hundreds of widgets.
 }
 
 bool FApplication::runAsService() {
-	return ((FApplication *) qApp)->m_serviceType != NoService;
+    if (m_serviceType == PortService) {
+        initServer();
+        return false;
+    }
+
+	return m_serviceType != NoService;
 }
 
 void FApplication::loadedPart(int loaded, int total) {
@@ -1161,12 +1527,11 @@ bool FApplication::notify(QObject *receiver, QEvent *e)
     return false;
 }
 
-void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
+void FApplication::loadSomething(const QString & prevVersion) {
     // At this point we're trying to determine what sketches to load which are from one of the following sources:
     // Only one of these sources will actually provide sketches to load and they're listed in order of priority:
 
     //		We found sketch backups to recover
-	//		it's a restart (!firstRun)
 	//		there's a previous version (open an empty sketch)
 	//		files were double-clicked
     //		The last opened sketch
@@ -1186,20 +1551,13 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 
 	DebugDialog::debug(QString("load previous %1").arg(loadPrevious));
 
-	if (!loadPrevious && sketchesToLoad.isEmpty()) {
-		if (!firstRun) {
-			DebugDialog::debug(QString("not first run"));
-			sketchesToLoad = loadLastOpenSketch();
-		}
-	}
-
 	if (!loadPrevious && sketchesToLoad.isEmpty()) {
 		// Check for double-clicked files to load
 		DebugDialog::debug(QString("check files to load %1").arg(m_filesToLoad.count()));
 
         foreach (QString filename, m_filesToLoad) {
             DebugDialog::debug(QString("Loading non-service file %1").arg(filename));
-            MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, filename, true, true);
+            MainWindow *mainWindow = MainWindow::newMainWindow(m_referenceModel, filename, true, true);
             mainWindow->loadWhich(filename, true, true, "");
             sketchesToLoad << mainWindow;
         }
@@ -1214,7 +1572,7 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 	MainWindow * newBlankSketch = NULL;
 	if (sketchesToLoad.isEmpty()) {
 		DebugDialog::debug(QString("create empty sketch"));
-		newBlankSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", true, true);
+		newBlankSketch = MainWindow::newMainWindow(m_referenceModel, "", true, true);
 		if (newBlankSketch) {
 			// make sure to start an empty sketch with a board
 			newBlankSketch->addDefaultParts();   // do this before call to show()
@@ -1251,7 +1609,7 @@ QList<MainWindow *> FApplication::loadLastOpenSketch() {
 
     DebugDialog::debug(QString("Loading last open sketch %1").arg(lastSketchPath));
     settings.remove("lastOpenSketch");				// clear the preference, in case the load crashes
-    MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, lastSketchPath, true, true);
+    MainWindow *mainWindow = MainWindow::newMainWindow(m_referenceModel, lastSketchPath, true, true);
     mainWindow->loadWhich(lastSketchPath, true, true, "");
     sketches << mainWindow;
     settings.setValue("lastOpenSketch", lastSketchPath);	// the load works, so restore the preference
@@ -1309,7 +1667,7 @@ QList<MainWindow *> FApplication::recoverBackups()
 			QString fileExt;
 			QString bundledFileName = FolderUtils::getSaveFileName(NULL, tr("Please specify an .fzz file name to save to (cancel will delete the backup)"), originalPath, tr("Fritzing (*%1)").arg(FritzingBundleExtension), &fileExt);
 			if (!bundledFileName.isEmpty()) {
-				MainWindow *currentRecoveredSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, originalBaseName, true, true);
+				MainWindow *currentRecoveredSketch = MainWindow::newMainWindow(m_referenceModel, originalBaseName, true, true);
     			currentRecoveredSketch->mainLoad(backupName, bundledFileName);
 				currentRecoveredSketch->saveAsShareable(bundledFileName, true);
 				currentRecoveredSketch->setCurrentFile(bundledFileName, true, true);
@@ -1363,42 +1721,26 @@ void FApplication::gotOrderFab(QNetworkReply * networkReply) {
 		QSettings settings;
 		settings.setValue(ORDERFABENABLED, QVariant(true));
 	}
-}
-
-QString FApplication::makeRequestParamsString() {
-	QSettings settings;
-	if (settings.value("pid").isNull()) {
-		settings.setValue("pid", FolderUtils::getRandText());
-	}
-
-	QtSystemInfo systemInfo(this);
-	QString siVersion(QUrl::toPercentEncoding(Version::versionString()));
-	QString siSystemName(QUrl::toPercentEncoding(systemInfo.systemName()));
-	QString siSystemVersion(QUrl::toPercentEncoding(systemInfo.systemVersion()));
-	QString siKernelName(QUrl::toPercentEncoding(systemInfo.kernelName()));
-	QString siKernelVersion(QUrl::toPercentEncoding(systemInfo.kernelVersion()));
-	QString siArchitecture(QUrl::toPercentEncoding(systemInfo.architectureName()));
-    QString string = QString("?pid=%1&version=%2&sysname=%3&kernname=%4&kernversion=%5arch=%6&sysversion=%7")
-		.arg(settings.value("pid").toString())
-		.arg(siVersion)
-		.arg(siSystemName)
-		.arg(siKernelName)
-		.arg(siKernelVersion)
-		.arg(siArchitecture)
-		.arg(siSystemVersion);
-	return string;
+    networkReply->manager()->deleteLater();
+    networkReply->deleteLater();
 }
 
 void FApplication::runPanelizerService()
 {	
 	m_started = true;
-	Panelizer::panelize(this, m_panelFilename);
+	Panelizer::panelize(this, m_panelFilename, m_panelizerCustom);
 }
 
 void FApplication::runInscriptionService()
 {	
 	m_started = true;
-	Panelizer::inscribe(this, m_panelFilename);
+    bool drc = false;
+    foreach (QString arg, m_arguments) {
+        if (arg.compare("-drc", Qt::CaseInsensitive) == 0) {
+            drc = true;
+        }
+    }
+	Panelizer::inscribe(this, m_panelFilename, drc);
 }
 
 QList<MainWindow *> FApplication::orderedTopLevelMainWindows() {
@@ -1414,9 +1756,7 @@ void FApplication::runExampleService()
 {	
 	m_started = true;
 
-	createUserDataStoreFolderStructure();
-	registerFonts();
-	loadReferenceModel();
+	initService();
 
 	QDir sketchesDir(FolderUtils::getApplicationSubFolderPath("sketches"));
 	runExampleService(sketchesDir);
@@ -1430,12 +1770,9 @@ void FApplication::runExampleService(QDir & dir) {
 		QString path = fileInfo.absoluteFilePath();
 		DebugDialog::debug("sketch file " + path);
 
-		int loaded = 0;
-		MainWindow * mainWindow = loadWindows(loaded, false);
+		MainWindow * mainWindow = openWindowForService(false);
 		if (mainWindow == NULL) continue;
 
-		mainWindow->noBackup();
-
 		FolderUtils::setOpenSaveFolderAux(dir.absolutePath());
 
 		if (!mainWindow->loadWhich(path, false, false, "")) {
@@ -1465,3 +1802,130 @@ void FApplication::cleanFzzs() {
 	LockManager::checkLockedFiles("fzz", backupList, lockedFiles, true, LockManager::SlowTime);
 	LockManager::releaseLockedFiles(folder, lockedFiles);
 }
+
+void FApplication::initServer() {
+    m_fServer = new FServer(this);
+    connect(m_fServer, SIGNAL(newConnection(int)), this, SLOT(newConnection(int)));
+    m_fServer->listen(QHostAddress::Any, m_portNumber);
+}
+
+void FApplication::newConnection(int socketDescription) {
+     FServerThread *thread = new FServerThread(socketDescription, this);
+     connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
+     connect(thread, SIGNAL(doCommand(const QString &, const QString &, QString &, int &)), 
+			this, SLOT(doCommand(const QString &, const QString &, QString &, int &)), Qt::BlockingQueuedConnection);
+     thread->start();
+}
+
+
+void FApplication::doCommand(const QString & command, const QString & params, QString & result, int & status) {
+	status = 200;
+	result = "";
+
+    QDir dir(m_portRootFolder);
+
+	QString subfolder = params;
+	if (command.endsWith("tcp")) {
+		subfolder = TextUtils::getRandText();
+		dir.mkdir(subfolder);
+	}
+
+    dir.cd(subfolder);
+    m_outputFolder = dir.absolutePath();
+
+	if (command.endsWith("tcp")) {
+		QEventLoop loop;
+		QNetworkAccessManager networkManager;
+		QObject::connect(&networkManager, SIGNAL(finished(QNetworkReply*)), &loop, SLOT(quit()));
+
+		QUrl url = params;
+		QNetworkReply* reply = networkManager.get(QNetworkRequest(url));
+
+		loop.exec();
+
+		status = 404;
+
+		if (reply->error() == QNetworkReply::NoError) {
+			if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toUInt() == 200) {
+				if (reply->isReadable()) {
+					int buffersize = 8192;
+					QStringList components = params.split("/");
+					QString filename = components.last();
+					QFile file(dir.absoluteFilePath(filename));
+					if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+						status = 200;
+						while (reply->bytesAvailable() >= buffersize) {
+							QByteArray bytes = reply->read(buffersize);
+							file.write(bytes);
+						}
+						if (reply->bytesAvailable() > 0) {
+							file.write(reply->readAll());
+						}
+						file.close();
+					}
+					else {
+						result = "unable to save to local file";
+					}
+				}
+				else {
+					result = "response unreadable";
+				}
+			}
+			else {
+				result = QString("bad response from url server %1").arg(reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toUInt());
+			}
+		}
+		else {
+			result = QString("url get failed %1").arg(reply->error());
+		}
+
+		reply->deleteLater();
+	}
+
+	if (!result.isEmpty()) {
+		// network problem
+		return;
+	}
+
+	if (command.startsWith("svg")) {
+        runSvgServiceAux();
+	    QStringList nameFilters;
+	    nameFilters << ("*.svg");
+	    QFileInfoList fileList = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+        if (fileList.count() > 0) {
+            QFile file(fileList.at(0).absoluteFilePath());
+            if (file.open(QFile::ReadOnly)) {
+                result = file.readAll();
+            }
+        }
+    }
+    else if (command.startsWith("gerber")) {
+        runGerberServiceAux();
+	    QStringList nameFilters;
+	    nameFilters << ("*.txt");
+	    QFileInfoList fileList = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+        if (fileList.count() > 0) {
+            QFile file(fileList.at(0).absoluteFilePath());
+            if (file.open(QFile::ReadOnly)) {
+                result = file.readAll();
+            }
+        }
+    }
+
+
+	if (command.endsWith("tcp")) {
+		QStringList skipSuffixes(".zip");
+		skipSuffixes << ".fzz";
+		QString filename = dir.absoluteFilePath(subfolder + ".zip");
+		if (FolderUtils::createZipAndSaveTo(dir, filename, skipSuffixes)) {
+			status = 200;
+			result = filename;
+		}
+		else {
+			status = 500;
+			result = "local zip failure";
+		}
+	}
+}
+
+
