@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6565 $:
+$Author: irascibl@gmail.com $:
+$Date: 2012-10-15 12:10:48 +0200 (Mo, 15. Okt 2012) $
 
 ********************************************************************/
 
@@ -34,7 +34,7 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 #include <QObject>
 
 #include "sketchmainhelp.h"
-#include "../mainwindow.h"
+#include "../mainwindow/mainwindow.h"
 
 class Helper : public QObject {
 	Q_OBJECT
@@ -51,7 +51,7 @@ class Helper : public QObject {
 		static void initText();
 
 	protected slots:
-		void somethingDroppedIntoView(const QPoint &, ModelPart *);
+		void somethingDroppedIntoView(const QPoint &);
 		void viewSwitched();
 
 	protected:
