@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6140 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-04 15:38:22 +0200 (Wed, 04 Jul 2012) $
+$Revision: 6609 $:
+$Author: irascibl@gmail.com $:
+$Date: 2012-10-30 14:21:08 +0100 (Di, 30. Okt 2012) $
 
 ********************************************************************/
 
@@ -38,6 +38,7 @@ $Date: 2012-07-04 15:38:22 +0200 (Wed, 04 Jul 2012) $
 #include <QVBoxLayout>
 #include <QComboBox>
 #include <QCheckBox>
+#include <QDoubleSpinBox>
 
 #include "../items/itembase.h"
 #include "../items/wire.h"
@@ -76,6 +77,8 @@ public:
 	void reloadContent(class InfoGraphicsView *);
 
 	void viewItemInfo(class InfoGraphicsView *, ItemBase* item, bool swappingEnabled);
+    void updateLocation(ItemBase *);
+    void updateRotation(ItemBase *);
 
 	void hoverEnterItem(class InfoGraphicsView *, QGraphicsSceneHoverEvent * event, ItemBase * item, bool swappingEnabled);
 	void hoverLeaveItem(class InfoGraphicsView *, QGraphicsSceneHoverEvent * event, ItemBase * item);
@@ -86,6 +89,8 @@ public:
 	void unregisterCurrentItem();
 	void unregisterCurrentItemIf(long id);
 
+    void init(bool tinyMode);
+
 public:
 	static const int STANDARD_ICON_IMG_WIDTH;
 	static const int STANDARD_ICON_IMG_HEIGHT;
@@ -105,6 +110,9 @@ protected slots:
 	void changeLock(bool);
 	void changeSticky(bool);
     void clickObsolete(const QString &);
+    void xyEntry();
+    void unitsClicked();
+    void rotEntry();
 
 protected:
 	void appendStuff(ItemBase* item, bool swappingEnabled); //finds out if it's a wire or something else
@@ -124,6 +132,12 @@ protected:
 	void clearPropThingPlugin(PropThing * propThing);
 	void clearPropThingPlugin(PropThing * propThing, QWidget * plugin);
 	void viewConnectorItemInfo(QGraphicsSceneHoverEvent * event, ConnectorItem* item);
+    void showLayers(bool show, ItemBase *, const QString & family, const QString & value, bool swappingEnabled);
+    void makeLockFrame();
+    void makeLocationFrame();
+    void makeRotationFrame();
+    void setLocation(ItemBase *);
+    void setRotation(ItemBase *);
 
 protected:
 	QPointer<ItemBase> m_currentItem;
@@ -144,18 +158,35 @@ protected:
 	QLabel * m_connDescr;
 	QLabel * m_connName;
 	QLabel * m_connType;
-	QLabel * m_proplabel;
-	QLabel * m_taglabel;
+	QLabel * m_propLabel;
+	QLabel * m_placementLabel;
+	QLabel * m_tagLabel;
 	QLabel * m_connLabel;
+    QLabel * m_layerLabel;
+    QLabel * m_lockLabel;
+    QLabel * m_locationLabel;
+    QLabel * m_rotationLabel;
 	QFrame * m_connFrame;
 	QFrame * m_propFrame;
+	QFrame * m_placementFrame;
+    QFrame * m_layerFrame;
+    QFrame * m_lockFrame;
+    QFrame * m_locationFrame;
+    QFrame * m_rotationFrame;
 	QCheckBox * m_lockCheckbox;
 	QCheckBox * m_stickyCheckbox;
 	QGridLayout * m_propLayout;
+	QGridLayout * m_placementLayout;
+    QVBoxLayout * m_layerLayout;
 	QList <PropThing *> m_propThings;
     QPointer<class InfoGraphicsView> m_pendingInfoGraphicsView;
 	QPointer<ItemBase> m_pendingItemBase;
 	bool m_pendingSwappingEnabled;
+    QWidget * m_layerWidget;
+    QDoubleSpinBox * m_xEdit;
+    QDoubleSpinBox * m_yEdit;
+    QLabel * m_unitsLabel;
+    QDoubleSpinBox * m_rotEdit;
 
 	// note: these m_last items should only be checked for equality and not otherwise accessed
 	ItemBase * m_lastTitleItemBase;
@@ -168,9 +199,7 @@ protected:
 	ModelPart * m_lastPropsModelPart;
 	ItemBase * m_lastPropsItemBase;
 	bool m_lastPropsSwappingEnabled;
-
-
-
+    bool m_tinyMode;
 };
 
 #endif
