@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6912 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 
 ********************************************************************/
 
@@ -139,10 +139,9 @@ register int	line_flag ;
 
 /////////////////////////////////////////////////////////
 
-ClipableWire::ClipableWire( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool initLabel ) 
-	: Wire(modelPart, viewIdentifier,  viewGeometry,  id, itemMenu, initLabel)
+ClipableWire::ClipableWire( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool initLabel ) 
+	: Wire(modelPart, viewID,  viewGeometry,  id, itemMenu, initLabel)
 {
-	m_canHaveCurve = false;
 	m_clipEnds = false;
 	m_trackHoverItem = NULL;
 	m_justFilteredEvent = NULL;
@@ -253,7 +252,7 @@ bool ClipableWire::filterMousePressConnectorEvent(ConnectorItem * connectorItem,
 	m_justFilteredEvent = NULL;
 
 	if (!m_clipEnds) return false;
-	if (m_viewIdentifier != ViewIdentifierClass::PCBView) return false;
+	if (m_viewID != ViewLayer::PCBView) return false;
 
 	ConnectorItem * to = NULL;
 	foreach (ConnectorItem * toConnectorItem, connectorItem->connectedToItems()) {
