@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6244 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-04 01:13:27 +0200 (Sat, 04 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -53,8 +53,8 @@ static HoleClassThing TheHoleThing;
 
 //////////////////////////////////////////////////
 
-Hole::Hole( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Hole::Hole( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: PaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	PaletteItem::setUpHoleSizes("hole", TheHoleThing);
 }
@@ -77,7 +77,7 @@ QString Hole::holeSize() {
 
 
 void Hole::setHoleSize(QString newSize, bool force) {
-	//DebugDialog::debug(QString("old holesize %1").arg(viewIdentifier()) + holeSize(), sceneBoundingRect());
+	//DebugDialog::debug(QString("old holesize %1").arg(viewID()) + holeSize(), sceneBoundingRect());
 	//foreach (QGraphicsItem * childItem, childItems()) {
 	//	DebugDialog::debug(QString("   child"), childItem->sceneBoundingRect());
 	//}
@@ -88,7 +88,7 @@ void Hole::setHoleSize(QString newSize, bool force) {
 
 		if (m_partLabel) m_partLabel->displayTextsIf();	
 	}
-	//DebugDialog::debug(QString("new holesize %1 ").arg(viewIdentifier()) + holeSize(), sceneBoundingRect());
+	//DebugDialog::debug(QString("new holesize %1 ").arg(viewID()) + holeSize(), sceneBoundingRect());
 	//foreach (QGraphicsItem * childItem, childItems()) {
 	//	DebugDialog::debug(QString("   child"), childItem->sceneBoundingRect());
 	//}
@@ -107,12 +107,12 @@ QRectF Hole::getRect(const QString & newSize) {
 }
 
 void Hole::setBoth(const QString & holeDiameter, const QString & ringThickness) {
-	if (this->m_viewIdentifier != ViewIdentifierClass::PCBView) return;
+	if (this->m_viewID != ViewLayer::PCBView) return;
 
 	ItemBase * otherLayer = setBothSvg(holeDiameter, ringThickness);
 
 	// there's only one NonConnectorItem
-	foreach (SvgIdLayer * svgIdLayer, fsvgRenderer()->setUpNonConnectors()) {
+	foreach (SvgIdLayer * svgIdLayer, fsvgRenderer()->setUpNonConnectors(viewLayerPlacement())) {
 		if (svgIdLayer == NULL) continue;
 
 		setBothNonConnectors(this, svgIdLayer);
@@ -126,7 +126,7 @@ void Hole::setBoth(const QString & holeDiameter, const QString & ringThickness)
 
 ItemBase * Hole::setBothSvg(const QString & holeDiameter, const QString & ringThickness) 
 {
-	QString svg = makeSvg(holeDiameter, ringThickness, m_viewLayerID);
+	QString svg = makeSvg(holeDiameter, ringThickness, m_viewLayerID, true);
 	resetRenderer(svg);
 	//DebugDialog::debug("both");
 	//DebugDialog::debug(svg);
@@ -144,7 +144,7 @@ ItemBase * Hole::setBothSvg(const QString & holeDiameter, const QString & ringTh
 	}
 
 	if (otherLayer) {
-		osvg = makeSvg(holeDiameter, ringThickness, otherLayer->viewLayerID());
+		osvg = makeSvg(holeDiameter, ringThickness, otherLayer->viewLayerID(), true);
 		//DebugDialog::debug(osvg);
 		otherLayer->resetRenderer(osvg);
 	}
@@ -159,15 +159,15 @@ void Hole::setBothNonConnectors(ItemBase * itemBase, SvgIdLayer * svgIdLayer) {
 		NonConnectorItem * nonConnectorItem = dynamic_cast<NonConnectorItem *>(child);
 		if (nonConnectorItem == NULL) continue;
 
-		//DebugDialog::debug(QString("hole set rect %1").arg(m_viewIdentifier), svgIdLayer->m_rect);
-		nonConnectorItem->setRect(svgIdLayer->m_rect);
+		//DebugDialog::debug(QString("hole set rect %1").arg(m_viewID), svgIdLayer->m_rect);
+		nonConnectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
 		nonConnectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
 		break;
 	}
 }
 
 
-QString Hole::makeSvg(const QString & holeDiameter, const QString & ringThickness, ViewLayer::ViewLayerID viewLayerID) 
+QString Hole::makeSvg(const QString & holeDiameter, const QString & ringThickness, ViewLayer::ViewLayerID viewLayerID, bool includeHole) 
 {
 	double offsetDPI = OffsetPixels / GraphicsUtils::SVGDPI;
 	double hd = TextUtils::convertToInches(holeDiameter);
@@ -199,15 +199,19 @@ QString Hole::makeSvg(const QString & holeDiameter, const QString & ringThicknes
 					.arg(id);
 	}
 	else {
-		svg += QString("<circle fill='none' cx='%1' cy='%1' r='%2' stroke-width='%3' stroke='%4' id='%5' />")
+		svg += QString("<circle fill='%6' cx='%1' cy='%1' r='%2' stroke-width='%3' stroke='%4' id='%5' />")
 			.arg((hd / 2) + rt + offsetDPI)
 			.arg((hd / 2) + (rt / 2))
 			.arg(rt)
 			.arg(setColor)
-			.arg(id);
-		svg += QString("<circle drill='0' fill='black' cx='%1' cy='%1' r='%2' stroke-width='0'  />")   // set the drill attribute to 0 for gerber translation
-			.arg((hd / 2) + rt + offsetDPI)
-			.arg(hd / 2);
+			.arg(id)
+            .arg(rt == 0 ? "black" : "none")
+            ;
+        if (includeHole) {
+		    svg += QString("<circle drill='0' fill='black' cx='%1' cy='%1' r='%2' stroke-width='0'  />")   // set the drill attribute to 0 for gerber translation
+			    .arg((hd / 2) + rt + offsetDPI)
+			    .arg(hd / 2);
+        }
 	}
   		
 	svg += "</g></svg>";
@@ -237,8 +241,8 @@ void Hole::addedToScene(bool temporary)
 }
 
 bool Hole::hasCustomSVG() {
-	switch (m_viewIdentifier) {
-		case ViewIdentifierClass::PCBView:
+	switch (m_viewID) {
+		case ViewLayer::PCBView:
 			return true;
 		default:
 			return ItemBase::hasCustomSVG();
@@ -249,42 +253,32 @@ ItemBase::PluralType Hole::isPlural() {
 	return Plural;
 }
 
-QString Hole::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
+QString Hole::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
 {
-	if (m_viewIdentifier != ViewIdentifierClass::PCBView || 
+	if (m_viewID != ViewLayer::PCBView || 
 		(viewLayerID != ViewLayer::Copper0 && viewLayerID != ViewLayer::Copper1)) 
 	{
-		return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+		return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 	}
 
 	QStringList holeSize = m_modelPart->localProp("hole size").toString().split(",");
 	if (holeSize.length() == 2) {
-		QString svg = makeSvg(holeSize.at(0), holeSize.at(1), viewLayerID);
+		QString svg = makeSvg(holeSize.at(0), holeSize.at(1), viewLayerID, false);
 		if (!svg.isEmpty()) {
-			QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
-			SvgFileSplitter splitter;
-			bool result = splitter.splitString(svg, xmlName);
-			if (!result) {
-				return "";
-			}
-			result = splitter.normalize(dpi, xmlName, blackOnly);
-			if (!result) {
-				return "";
-			}
-			return splitter.elementString(xmlName);
+            return PaletteItemBase::normalizeSvg(svg, viewLayerID, blackOnly, dpi, factor);
 		}
 	}
 
-	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 }
 
-bool Hole::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+bool Hole::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (prop.compare("hole size", Qt::CaseInsensitive) == 0) {
         return collectHoleSizeInfo(TheHoleThing.holeSizeValue, parent, swappingEnabled, returnProp, returnValue, returnWidget);
 	}
 
-	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 void Hole::changeHoleSize(const QString & newSize) {
@@ -317,13 +311,13 @@ bool Hole::canFindConnectorsUnder() {
 	return false;
 }
 
-ViewIdentifierClass::ViewIdentifier Hole::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier vid, bool) 
+ViewLayer::ViewID Hole::useViewIDForPixmap(ViewLayer::ViewID vid, bool) 
 {
-    if (vid == ViewIdentifierClass::PCBView) {
-        return ViewIdentifierClass::IconView;
+    if (vid == ViewLayer::PCBView) {
+        return ViewLayer::IconView;
     }
 
-    return ViewIdentifierClass::UnknownView;
+    return ViewLayer::UnknownView;
 }
 
 void Hole::changeUnits(bool) 
