@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -31,6 +31,7 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 #include <QPointF>
 #include <QSize>
 #include <QHash>
+#include <QList>
 #include <QGraphicsSceneHoverEvent>
 #include <QGraphicsSvgItem>
 #include <QPointer>
@@ -41,7 +42,6 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 
 #include "../viewgeometry.h"
 #include "../viewlayer.h"
-#include "../viewidentifierclass.h"
 #include "../utils/misc.h"
 
 class ConnectorItem;
@@ -62,7 +62,7 @@ public:
 	};
 
 public:
-	ItemBase(class ModelPart*, ViewIdentifierClass::ViewIdentifier, const ViewGeometry &, long id, QMenu * itemMenu);
+	ItemBase(class ModelPart*, ViewLayer::ViewID, const ViewGeometry &, long id, QMenu * itemMenu);
 	virtual ~ItemBase();
 
 	qint64 id() const;
@@ -82,13 +82,13 @@ public:
 	virtual void writeGeometry(QXmlStreamWriter &);
 	virtual void moveItem(ViewGeometry &) = 0;
 	virtual void setItemPos(QPointF & pos);
-	virtual void rotateItem(double degrees);
+	virtual void rotateItem(double degrees, bool includeRatsnest);
 	virtual void flipItem(Qt::Orientations orientation);
-	void transformItem(const QTransform &);
+	virtual void transformItem(const QTransform &, bool includeRatsnest);
 	virtual void transformItem2(const QMatrix &);
 	virtual void removeLayerKin();
-	ViewIdentifierClass::ViewIdentifier viewIdentifier();
-	QString & viewIdentifierName();
+	ViewLayer::ViewID viewID();
+	QString & viewIDName();
 	ViewLayer::ViewLayerID viewLayerID() const;
 	void setViewLayerID(ViewLayer::ViewLayerID, const LayerHash & viewLayers);
 	void setViewLayerID(const QString & layerName, const LayerHash & viewLayers);
@@ -96,17 +96,20 @@ public:
 
 	void collectConnectors(ConnectorPairHash & connectorHash, SkipCheckFunction);
 
-	virtual void busConnectorItems(class Bus * bus, QList<ConnectorItem *> & items);
+	virtual void busConnectorItems(class Bus * bus, ConnectorItem *, QList<ConnectorItem *> & items);
 	virtual void setHidden(bool hidden);
+	virtual void setLayerHidden(bool hidden);
 	bool hidden();
+	bool layerHidden();
 	virtual void setInactive(bool inactivate);
 	bool inactive();
-	ConnectorItem * findConnectorItemWithSharedID(const QString & connectorID, ViewLayer::ViewLayerSpec);
-	void updateConnections(ConnectorItem *);
-	virtual void updateConnections();
+	ConnectorItem * findConnectorItemWithSharedID(const QString & connectorID, ViewLayer::ViewLayerPlacement);
+	ConnectorItem * findConnectorItemWithSharedID(const QString & connectorID);
+	void updateConnections(ConnectorItem *, bool includeRatsnest, QList<ConnectorItem *> & already);
+	virtual void updateConnections(bool includeRatsnest, QList<ConnectorItem *> & already);
 	virtual const QString & title() const;
 	bool getRatsnest();
-	const QHash<QString, QPointer<class Bus> > & buses();
+	QList<class Bus *> buses();
 	int itemType() const;					// wanted this to return ModelPart::ItemType but couldn't figure out how to get it to compile
 	virtual bool isSticky();
 	virtual bool isBaseSticky();
@@ -146,24 +149,24 @@ public:
 	QString makePartLabelSvg(bool blackOnly, double dpi, double printerScale);
 	QPointF partLabelScenePos();
 	QRectF partLabelSceneBoundingRect();
-	bool isSwappable();
+	virtual bool isSwappable();
+    virtual void setSwappable(bool);
 	void mousePressEvent(QGraphicsSceneMouseEvent *event);
 	virtual void collectWireConnectees(QSet<class Wire *> & wires);
 	virtual bool collectFemaleConnectees(QSet<ItemBase *> & items);
 	void prepareGeometryChange();
 	virtual void resetID();
-	void updateConnectionsAux();
-	virtual ItemBase * lowerConnectorLayerVisible(ItemBase *);
+	void updateConnectionsAux(bool includeRatsnest, QList<ConnectorItem *> & already);
 	void hoverEnterEvent( QGraphicsSceneHoverEvent * event );
 	void hoverLeaveEvent( QGraphicsSceneHoverEvent * event );
     void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
 	virtual void figureHover();
-	virtual QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi);
+	virtual QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
 	virtual void slamZ(double newZ);
 	bool isEverVisible();
 	void setEverVisible(bool);
 	virtual bool connectionIsAllowed(ConnectorItem *);
-	virtual bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget);
+	virtual bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
 	virtual QString getProperty(const QString & key);
 	ConnectorItem * rightClickedConnector();
 	virtual bool canEditPart();
@@ -171,13 +174,13 @@ public:
 	virtual void setProp(const QString & prop, const QString & value);
 	QString prop(const QString & p);
 	bool isObsolete();
-	void prepareProps();
+    virtual QHash<QString, QString> prepareProps(ModelPart *, bool wantDebug, QStringList & keys);
 	void resetValues(const QString & family, const QString & prop);
 	const QString & filename();
 	void setFilename(const QString &);
 	virtual PluralType isPlural();
-	ViewLayer::ViewLayerSpec viewLayerSpec() const;
-	void setViewLayerSpec(ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerPlacement viewLayerPlacement() const;
+	void setViewLayerPlacement(ViewLayer::ViewLayerPlacement);
 	virtual void calcRotation(QTransform & rotation, QPointF center, ViewGeometry &);
     void updateConnectors();
 	const QString & moduleID();
@@ -190,6 +193,7 @@ public:
 	void collectPropsMap(QString & family, QMap<QString, QString> &);
 	virtual bool rotationAllowed();
 	virtual bool rotation45Allowed();
+	virtual bool freeRotationAllowed();
 	void ensureUniqueTitle(const QString &title, bool force);
 	virtual void setDropOffset(QPointF offset);
 	bool hasRubberBandLeg() const;
@@ -213,9 +217,21 @@ public:
 	bool resetRenderer(const QString & svg);
 	bool resetRenderer(const QString & svg, QString & newSvg);
     void getPixmaps(QPixmap * &, QPixmap * &, QPixmap * &, bool swappingEnabled, QSize);
-    class FSvgRenderer * setUpImage(class ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, class LayerAttributes &, QString & error);
-	class FSvgRenderer * setUpImage(class ModelPart * modelPart, QDomDocument *, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, class LayerAttributes &, QString & error);
-
+    class FSvgRenderer * setUpImage(class ModelPart * modelPart, class LayerAttributes &);
+	void showConnectors(const QStringList &);
+	void setItemIsSelectable(bool selectable);
+	virtual bool inRotation();
+	virtual void setInRotation(bool);
+	const QString & spice() const;
+	const QString & spiceModel() const;
+    void addSubpart(ItemBase *);
+    void setSuperpart(ItemBase *);
+    ItemBase * superpart();
+    ItemBase * findSubpart(const QString & connectorID, ViewLayer::ViewLayerPlacement);
+    const QList< QPointer<ItemBase> > & subparts();
+    void setSquashShape(bool);
+    const QPainterPath & selectionShape();
+    virtual void setTransform2(const QTransform &);
 
 public:
 	virtual void getConnectedColor(ConnectorItem *, QBrush * &, QPen * &, double & opacity, double & negativePenWidth, bool & negativeOffsetRect);
@@ -263,18 +279,20 @@ public:
 	virtual void mouseMoveConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
 	virtual void mouseReleaseConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
 	virtual bool acceptsMousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
+	virtual bool acceptsMousePressLegEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
+	virtual void setAcceptsMousePressLegEvent(bool);
 	virtual bool acceptsMouseDoubleClickConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
 	virtual bool acceptsMouseMoveConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
 	virtual bool acceptsMouseReleaseConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
 	virtual void connectionChange(ConnectorItem * onMe, ConnectorItem * onIt, bool connect);
-	virtual void connectedMoved(ConnectorItem * from, ConnectorItem * to);
+	virtual void connectedMoved(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & already);
 	virtual ItemBase * layerKinChief();
 	virtual const QList<ItemBase *> & layerKin();
 	virtual void findConnectorsUnder() = 0;
 	virtual ConnectorItem* newConnectorItem(class Connector *connector);
 	virtual ConnectorItem* newConnectorItem(ItemBase * layerkin, Connector *connector);
 
-	virtual void setInstanceTitle(const QString &title);
+	virtual void setInstanceTitle(const QString &title, bool initial);
 	void updatePartLabelInstanceTitle();
 
 public slots:
@@ -287,7 +305,6 @@ public:
 	static bool zLessThan(ItemBase * & p1, ItemBase * & p2);
 	static qint64 getNextID();
 	static qint64 getNextID(qint64 fromIndex);
-	static QString getSvgFilename(ModelPart *, const QString & baseName); 
 
 protected:
 	void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
@@ -303,27 +320,30 @@ protected:
 
 	void setInstanceTitleTooltip(const QString& text);
 	virtual void setDefaultTooltip();
-	void setInstanceTitleAux(const QString & title);
+	void setInstanceTitleAux(const QString & title, bool initial);
 	void saveLocAndTransform(QXmlStreamWriter & streamWriter);
-    QPixmap * getPixmap(ViewIdentifierClass::ViewIdentifier, bool swappingEnabled, QSize size);
-    virtual ViewIdentifierClass::ViewIdentifier useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier, bool swappingEnabled);
-    virtual void makeLocalMods(QByteArray & svg, const QString & filename);
+    QPixmap * getPixmap(ViewLayer::ViewID, bool swappingEnabled, QSize size);
+    virtual ViewLayer::ViewID useViewIDForPixmap(ViewLayer::ViewID, bool swappingEnabled);
+    virtual bool makeLocalModifications(QByteArray & svg, const QString & filename);
+    void updateHidden();
+    void createShape(LayerAttributes & layerAttributes);
 
 protected:
-	static bool getFlipDoc(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec, QDomDocument &);
-	static bool fixCopper1(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec, QDomDocument &);
+	static bool getFlipDoc(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement, QDomDocument &, Qt::Orientations);
+	static bool fixCopper1(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement, QDomDocument &);
 
 protected:
  	QSizeF m_size;
 	qint64 m_id;
 	ViewGeometry m_viewGeometry;
 	QPointer<ModelPart> m_modelPart;
-	ViewIdentifierClass::ViewIdentifier m_viewIdentifier;
+	ViewLayer::ViewID m_viewID;
 	ViewLayer::ViewLayerID m_viewLayerID;
 	int m_connectorHoverCount;
 	int m_connectorHoverCount2;
 	int m_hoverCount;
 	bool m_hidden;
+	bool m_layerHidden;
 	bool m_inactive;
 	bool m_sticky;
 	QHash< long, QPointer<ItemBase> > m_stickyList;
@@ -338,23 +358,29 @@ protected:
 	ConnectorItem * m_rightClickedConnector;
 	QMap<QString, QString> m_propsMap;
 	QString m_filename;
-	ViewLayer::ViewLayerSpec m_viewLayerSpec;
+	ViewLayer::ViewLayerPlacement m_viewLayerPlacement;
 	bool m_moveLock;
 	bool m_hasRubberBandLeg;
 	QList<ConnectorItem *> m_cachedConnectorItems;
 	QGraphicsSvgItem * m_moveLockItem;
 	QGraphicsSvgItem * m_stickyItem;
     FSvgRenderer * m_fsvgRenderer;
-       
+	bool m_acceptsMousePressLegEvent;
+    bool m_swappable;
+ 	bool m_inRotation;
+    QPointer<ItemBase> m_superpart;
+    QList< QPointer<ItemBase> > m_subparts;
+    bool m_squashShape;
+    QPainterPath m_selectionShape;
+      
  protected:
 	static long nextID;
-	static QPointer<class ReferenceModel> referenceModel;
+	static QPointer<class ReferenceModel> TheReferenceModel;
 
 public:
 	static const QString ITEMBASE_FONT_PREFIX;
 	static const QString ITEMBASE_FONT_SUFFIX;
 	static QHash<QString, QString> TranslatedPropertyNames;
-	static QString SvgFilesDir;
 	const static QColor hoverColor;
 	const static double hoverOpacity;
 	const static QColor connectorHoverColor;
