@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6218 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -31,6 +31,8 @@ $Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 #include "../utils/graphicsutils.h"
 #include "../svg/svgfilesplitter.h"
 #include "../sketch/infographicsview.h"
+#include "../debugdialog.h"
+#include "../layerattributes.h"
 
 static QString Copper0LayerTemplate = "";
 static QString JumperWireLayerTemplate = "";
@@ -68,10 +70,11 @@ void shorten(QRectF r0, QPointF r0c, QPointF r1c, double & r0x, double & r0y, do
 
 /////////////////////////////////////////////////////////
 
-JumperItem::JumperItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel) 
-	: PaletteItem(modelPart, viewIdentifier,  viewGeometry,  id, itemMenu, doLabel)
+JumperItem::JumperItem( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel) 
+	: PaletteItem(modelPart, viewID,  viewGeometry,  id, itemMenu, doLabel)
 {
-	if (Colors.isEmpty()) {
+    m_originalClickItem = NULL;
+    if (Colors.isEmpty()) {
 		Colors.insert(ViewLayer::Copper0, ViewLayer::Copper0Color);
 		Colors.insert(ViewLayer::Copper1, ViewLayer::Copper1Color);
 		Colors.insert(ViewLayer::PartImage, JumperColor);
@@ -100,7 +103,7 @@ JumperItem::~JumperItem() {
 
 QRectF JumperItem::boundingRect() const
 {
-    if (m_viewIdentifier != ViewIdentifierClass::PCBView) {
+    if (m_viewID != ViewLayer::PCBView) {
         return PaletteItem::boundingRect();
     }
 
@@ -109,7 +112,7 @@ QRectF JumperItem::boundingRect() const
 
 void JumperItem::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-    if (m_viewIdentifier != ViewIdentifierClass::PCBView) {
+    if (m_viewID != ViewLayer::PCBView) {
        PaletteItem::paintHover(painter, option, widget);
 	   return;
     }
@@ -119,7 +122,7 @@ void JumperItem::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *o
 
 void JumperItem::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-    if (m_viewIdentifier != ViewIdentifierClass::PCBView) {
+    if (m_viewID != ViewLayer::PCBView) {
        PaletteItem::paintSelected(painter, option, widget);
 	   return;
     }
@@ -129,7 +132,7 @@ void JumperItem::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem
 
 QPainterPath JumperItem::hoverShape() const
 {
-    if (m_viewIdentifier != ViewIdentifierClass::PCBView) {
+    if (m_viewID != ViewLayer::PCBView) {
         return PaletteItem::hoverShape();
      }
 
@@ -154,11 +157,11 @@ QPainterPath JumperItem::shape() const
 	return hoverShape();
 }
 
-bool JumperItem::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
+bool JumperItem::setUpImage(ModelPart * modelPart, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
-	bool result = PaletteItem::setUpImage(modelPart, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
+	bool result = PaletteItem::setUpImage(modelPart, viewLayers, layerAttributes);
 
-	if (doConnectors) {
+	if (layerAttributes.doConnectors) {
 		foreach (ConnectorItem * item, cachedConnectorItems()) {
 			item->setCircular(true);
 			if (item->connectorSharedName().contains('0')) {
@@ -171,14 +174,14 @@ bool JumperItem::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIden
 			}
 		}
 
-		initialResize(viewIdentifier);
+		initialResize(layerAttributes.viewID);
 	}
 
 	return result;
 }
 
-void JumperItem::initialResize(ViewIdentifierClass::ViewIdentifier viewIdentifier) {
-	if (viewIdentifier != ViewIdentifierClass::PCBView) return;
+void JumperItem::initialResize(ViewLayer::ViewID viewID) {
+	if (viewID != ViewLayer::PCBView) return;
 
 	bool ok;
 	double r0x = m_modelPart->localProp("r0x").toDouble(&ok);
@@ -200,8 +203,29 @@ void JumperItem::initialResize(ViewIdentifierClass::ViewIdentifier viewIdentifie
 
 }
 
+bool JumperItem::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
+{
+    m_originalClickItem = originalItem;
+    mousePressEvent(event);
+    return (m_dragItem != NULL);
+}
+
+void JumperItem::mouseMoveEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
+{
+    Q_UNUSED(originalItem);
+    mouseMoveEvent(event);
+}
+
+void JumperItem::mouseReleaseEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
+{
+    Q_UNUSED(originalItem);
+    mouseReleaseEvent(event);
+}
+
 void JumperItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
+    PaletteItemBase * originalItem = m_originalClickItem;
+    m_originalClickItem = NULL;
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infographics != NULL && infographics->spaceBarIsPressed()) { 
 		event->ignore();
@@ -227,6 +251,7 @@ void JumperItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 			m_otherItem = m_connector0;
 		}
 		else {
+            if (originalItem) return ItemBase::mousePressEvent(event);
 			return PaletteItem::mousePressEvent(event);
 		}
 	}
@@ -264,7 +289,8 @@ void JumperItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 	if (cross) cross->setRect(r);
 
 	resize();
-	ItemBase::updateConnections(m_dragItem);	
+    QList<ConnectorItem *> already;
+	ItemBase::updateConnections(m_dragItem, true, already);	
 }
 
 void JumperItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
@@ -335,7 +361,7 @@ void JumperItem::resize(QPointF p0, QPointF p1) {
 }
 
 void JumperItem::resize() {
-	if (m_viewIdentifier != ViewIdentifierClass::PCBView) return;
+	if (m_viewID != ViewLayer::PCBView) return;
 
 	if (m_connector0 == NULL) return;
 	if (m_connector1 == NULL) return;
@@ -411,7 +437,7 @@ QSizeF JumperItem::footprintSize() {
 	return r0.size();
 }
 
-QString JumperItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
+QString JumperItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
 {
 	QString xml = "";
 	switch (viewLayerID) {
@@ -426,21 +452,10 @@ QString JumperItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStrin
 	}
 
 	if (!xml.isEmpty()) {
-		QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
-		SvgFileSplitter splitter;
-		bool result = splitter.splitString(xml, xmlName);
-		if (!result) {
-			return ___emptyString___;
-		}
-		result = splitter.normalize(dpi, xmlName, blackOnly);
-		if (!result) {
-			return ___emptyString___;
-		}
-		return splitter.elementString(xmlName);
+        return PaletteItemBase::normalizeSvg(xml, viewLayerID, blackOnly, dpi, factor);
 	}
 
-
-	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 }
 
 void JumperItem::setAutoroutable(bool ar) {
@@ -460,8 +475,8 @@ ConnectorItem * JumperItem::connector1() {
 }
 
 bool JumperItem::hasCustomSVG() {
-	switch (m_viewIdentifier) {
-		case ViewIdentifierClass::PCBView:
+	switch (m_viewID) {
+		case ViewLayer::PCBView:
 			return true;
 		default:
 			return ItemBase::hasCustomSVG();
@@ -472,8 +487,8 @@ bool JumperItem::inDrag() {
 	return m_dragItem != NULL;
 }
 
-void JumperItem::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerSpec viewLayerSpec) {
-	PaletteItem::loadLayerKin(viewLayers, viewLayerSpec);
+void JumperItem::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
+	PaletteItem::loadLayerKin(viewLayers, viewLayerPlacement);
 	resize();
 }
 
@@ -498,7 +513,8 @@ void JumperItem::addedToScene(bool temporary) {
 	PaletteItem::addedToScene(temporary);
 }
 
-void JumperItem::rotateItem(double degrees) {
+void JumperItem::rotateItem(double degrees, bool updateRatsnest) {
+    Q_UNUSED(updateRatsnest);
 	QPointF tc0, tc1;
 	QTransform rotation;
 	rotation.rotate(degrees);
@@ -555,11 +571,11 @@ bool JumperItem::hasPartNumberProperty()
 	return false;
 }
 
-ViewIdentifierClass::ViewIdentifier JumperItem::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier vid, bool) 
+ViewLayer::ViewID JumperItem::useViewIDForPixmap(ViewLayer::ViewID vid, bool) 
 {
-    if (vid == ViewIdentifierClass::PCBView) {
-        return ViewIdentifierClass::IconView;
+    if (vid == ViewLayer::PCBView) {
+        return ViewLayer::IconView;
     }
 
-    return ViewIdentifierClass::UnknownView;
+    return ViewLayer::UnknownView;
 }
