@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,15 +35,14 @@ class LayerKinPaletteItem : public PaletteItemBase
 Q_OBJECT
 
 public:       
-	LayerKinPaletteItem(PaletteItemBase * chief, ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);
+	LayerKinPaletteItem(PaletteItemBase * chief, ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);
 	void setOffset(double x, double y);
 	ItemBase * layerKinChief();
 	bool ok();
 	void setHidden(bool hidden);
 	void setInactive(bool inactivate);
 	void clearModelPart();
-	ItemBase * lowerConnectorLayerVisible(ItemBase *);
-	void init(ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, const LayerHash &viewLayers);
+	void init(LayerAttributes &, const LayerHash &viewLayers);
 	bool isSticky();
 	bool isBaseSticky();
 	void setSticky(bool);
@@ -53,19 +52,60 @@ public:
 	bool alreadySticking(ItemBase * itemBase);
 	bool stickyEnabled();
 	void resetID();
-	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi);
+	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
+	bool isSwappable();
+    void setSwappable(bool);
+	bool inRotation();
+	void setInRotation(bool);
 
 protected:
 	QVariant itemChange(GraphicsItemChange change, const QVariant &value);
-	void updateConnections();
+	void updateConnections(bool includeRatsnest, QList<ConnectorItem *> & already);
 	void mousePressEvent(QGraphicsSceneMouseEvent *event);
-	//void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
-	//void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
+	void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
+	void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
 	ConnectorItem* newConnectorItem(class Connector *connector);
 
 protected:
 	QPointer<PaletteItemBase> m_layerKinChief;
 	bool m_ok;
+    bool m_passMouseEvents;
 };
 
+struct TextThing {
+    int minX;
+    int maxX;
+    int minY;
+    int maxY;
+    QRectF newRect;
+    QMatrix flipMatrix;
+};
+
+class SchematicTextLayerKinPaletteItem : public LayerKinPaletteItem
+{
+Q_OBJECT
+
+public:       
+	SchematicTextLayerKinPaletteItem(PaletteItemBase * chief, ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);
+
+    void transformItem(const QTransform &, bool includeRatsnest);
+ 	bool setUpImage(ModelPart* modelPart, const LayerHash & viewLayers, LayerAttributes &);
+    void clearTextThings();
+    void setTransform2(const QTransform &);
+
+public:
+    static void renderText(QImage &, QDomElement & text, int & minX, int & minY, int & maxX, int & maxY, QMatrix &, QRectF & viewBox); 
+
+protected:
+    QString makeFlipTextSvg();
+    void positionTexts(QList<QDomElement> & texts);
+    void initTextThings();
+    QString vflip(const QString & svg, bool isFlipped);
+
+protected:
+    bool m_flipped;
+    QList<TextThing> m_textThings;
+};
+
+
 #endif
