@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6218 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,6 +35,7 @@ $Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 #include "moduleidnames.h"
 #include "../svg/groundplanegenerator.h"
 #include "../utils/cursormaster.h"
+#include "../debugdialog.h"
 
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -47,17 +48,21 @@ $Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 #include <QImage>
 #include <QLineEdit>
 #include <QApplication>
+#include <QColor>
+#include <QColorDialog>
 
 static QStringList LogoImageNames;
+static QStringList Logo0ImageNames;
 static QStringList NewLogoImageNames;
 static QStringList Copper0ImageNames;
 static QStringList Copper1ImageNames;
 
-LogoItem::LogoItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: ResizableBoard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+LogoItem::LogoItem( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: ResizableBoard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	if (LogoImageNames.count() == 0) {
 		LogoImageNames << "Made with Fritzing" << "Fritzing icon" << "OHANDA logo" << "OSHW logo";
+		Logo0ImageNames << "Made with Fritzing 0" << "Fritzing icon 0" << "OHANDA logo 0" << "OSHW logo 0";
 	}
 
     m_svgOnly = false;
@@ -65,7 +70,7 @@ LogoItem::LogoItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier v
 	m_inLogoEntry = QTime::currentTime().addSecs(-10);
 	m_aspectRatioCheck = NULL;
 	m_keepAspectRatio = true;
-	m_hasLogo = (modelPart->moduleID() == ModuleIDNames::LogoTextModuleIDName);
+	m_hasLogo = (modelPart->moduleID().endsWith(ModuleIDNames::LogoTextModuleIDName));
 	m_logo = modelPart->localProp("logo").toString();
 	if (m_hasLogo && m_logo.isEmpty()) {
 		m_logo = modelPart->properties().value("logo", "logo");
@@ -88,8 +93,19 @@ void LogoItem::addedToScene(bool temporary)
 		m_aspectRatio.setHeight(this->boundingRect().height());
 		m_originalFilename = filename();
 		QString svg = prop("shape");
-		if (!svg.isEmpty()) {					
-			m_aspectRatio = modelPart()->localProp("aspectratio").toSizeF();
+		if (!svg.isEmpty()) {
+            QString arString = modelPart()->localProp("aspectratio").toString();
+            if (!arString.isEmpty()) {
+			    m_aspectRatio =  modelPart()->localProp("aspectratio").toSizeF();
+            }
+            else {
+                bool okw, okh;
+                double w = m_modelPart->localProp("width").toDouble(&okw);
+		        double h = m_modelPart->localProp("height").toDouble(&okh);
+                if (okw && okh) {
+                    m_aspectRatio = QSizeF(w, h);
+                }
+            }
 			if (resetRenderer(getShapeForRenderer(svg))) {
 			}
 			else {
@@ -116,14 +132,14 @@ bool LogoItem::canRetrieveLayer(ViewLayer::ViewLayerID viewLayerID) {
 }
 
 QStringList & LogoItem::getImageNames() {
-	return LogoImageNames;
+	return isBottom() ? Logo0ImageNames : LogoImageNames;
 }
 
 QStringList & LogoItem::getNewImageNames() {
 	return NewLogoImageNames;
 }
 
-QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
+QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	if (canRetrieveLayer(viewLayerID)) {
 		QString svg = prop("shape");
@@ -160,7 +176,7 @@ QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString,
 				}
 			}
 
-			result = splitter.normalize(dpi, xmlName, blackOnly);
+			result = splitter.normalize(dpi, xmlName, blackOnly, factor);
 			if (!result) {
 				return "";
 			}
@@ -175,10 +191,32 @@ QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString,
 		}
 	}
 
-	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	return PaletteItemBase::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 }
 
-bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+QStringList LogoItem::collectValues(const QString & family, const QString & prop, QString & value) {
+    QStringList values = ResizableBoard::collectValues(family, prop, value);
+
+    bool copperTopOK = true;
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+    if (infoGraphicsView == NULL) copperTopOK = false;
+    else if (infoGraphicsView->boardLayers() == 1) copperTopOK = false;
+
+    QStringList newValues;
+	if (prop.compare("layer", Qt::CaseInsensitive) == 0) {
+        foreach (QString xmlName, values) {
+            if (xmlName == ViewLayer::viewLayerXmlNameFromID(ViewLayer::Copper1) && copperTopOK == false) continue;
+
+            newValues << Board::convertFromXmlName(xmlName);
+        }
+        value = Board::convertFromXmlName(value);
+        return newValues;
+    }
+
+    return values;
+}
+
+bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (m_hasLogo) {
 		if (prop.compare("logo", Qt::CaseInsensitive) == 0) {
@@ -204,12 +242,13 @@ bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const
 	}
 
 	if (prop.compare("shape", Qt::CaseInsensitive) == 0) {
-		returnWidget = setUpDimEntry(true, !m_hasLogo, returnWidget);
+		returnWidget = setUpDimEntry(true, !m_hasLogo, false, returnWidget);
+        returnWidget->setEnabled(swappingEnabled);
 		returnProp = tr("shape");
 		return true;
 	}
 
-	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 bool LogoItem::checkImage(const QString & filename) {
@@ -225,8 +264,15 @@ void LogoItem::prepLoadImageAux(const QString & fileName, bool addName)
 	}
 }
 
-bool LogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName) 
+bool LogoItem::reloadImage(const QString & incomingSvg, const QSizeF & aspectRatio, const QString & fileName, bool addName) 
 {
+    QString svg = incomingSvg;
+	if (isBottom()) {
+		if (!svg.contains("_flipped_")) {
+			svg = flipSvg(svg);
+		}
+	}
+
     QString shape = getShapeForRenderer(svg);
 	bool result = resetRenderer(shape);
 	if (result) {
@@ -291,10 +337,13 @@ void LogoItem::loadImage(const QString & fileName, bool addName)
 			return;
 		}
 
-		TextUtils::cleanSodipodi(svg);
+		if (svg.contains("coreldraw", Qt::CaseInsensitive) && svg.contains("cdata", Qt::CaseInsensitive)) {
+			unableToLoad(fileName, tr("because this appears to be a SVG file exported from CorelDRAW without choosing the 'presentation attributes' setting"));
+			return;
+		}
+
+        TextUtils::fixMuch(svg, true);
 		TextUtils::fixPixelDimensionsIn(svg);
-		TextUtils::fixViewboxOrigin(svg);
-		TextUtils::tspanRemove(svg);
 
 		QString errorStr;
 		int errorLine;
@@ -326,20 +375,9 @@ void LogoItem::loadImage(const QString & fileName, bool addName)
             // todo: change opacity?
         }
 
-		bool isIllustrator = TextUtils::isIllustratorDoc(domDocument);
-
-		QString viewBox = root.attribute("viewBox");
-		if (viewBox.isEmpty()) {
-			bool ok1, ok2;
-			double w = TextUtils::convertToInches(root.attribute("width"), &ok1, isIllustrator) * GraphicsUtils::SVGDPI;
-			double h = TextUtils::convertToInches(root.attribute("height"), &ok2, isIllustrator) * GraphicsUtils::SVGDPI;
-			if (!ok1 || !ok2) {
-				unableToLoad(fileName, tr("because of an improper width or height attribute"));
-				return;
-			}
-
-			root.setAttribute("viewBox", QString("0 0 %1 %2").arg(w).arg(h));
-		}
+        QRectF vb;
+        double w, h;
+		TextUtils::ensureViewBox(domDocument, GraphicsUtils::SVGDPI, vb, true, w, h, false);
 
         QDomElement layerElement = TextUtils::findElementWithAttribute(root, "id", layerName());
         if (layerElement.isNull()) {
@@ -463,12 +501,9 @@ void LogoItem::setProp(const QString & prop, const QString & value) {
 void LogoItem::setLogo(QString logo, bool force) {
 	if (!force && m_logo.compare(logo) == 0) return;
 
-	switch (this->m_viewIdentifier) {
-		case ViewIdentifierClass::PCBView:
-			break;
-		default:
-			return;
-	}
+    if (!isEverVisible()) {
+        return;
+    }
 
 	QString svg;
 	QFile f(m_originalFilename);
@@ -580,6 +615,11 @@ QString LogoItem::hackSvg(const QString & svg, const QString & logo)
 	coords[2] = QString::number(logo.length() * 10);
 	root.setAttribute("viewBox", coords.join(" "));
 
+    QStringList exceptions;
+	exceptions << "none" << "";
+	QString toColor(colorString());
+	SvgFileSplitter::changeColors(root, toColor, exceptions);
+
 	QDomNodeList domNodeList = root.elementsByTagName("text");
 	for (int i = 0; i < domNodeList.count(); i++) {
 		QDomElement node = domNodeList.item(i).toElement();
@@ -599,12 +639,15 @@ QString LogoItem::hackSvg(const QString & svg, const QString & logo)
 				QString h = root.attribute("height");
 				bool ok;
 				modelPart()->setLocalProp("height", TextUtils::convertToInches(h, &ok, false) * 25.4);
-				return doc.toString();
+                if (!isBottom()) return  doc.toString();
+                return flipSvg(doc.toString());
 			}
 		}
 	}
 
-	return svg;
+    if (!isBottom()) return svg;
+
+	return flipSvg(svg);
 }
 
 void LogoItem::widthEntry() {
@@ -663,11 +706,11 @@ ItemBase::PluralType LogoItem::isPlural() {
 }
 
 ViewLayer::ViewLayerID LogoItem::layer() {
-	return  ViewLayer::Silkscreen1;
+	return  isBottom() ? ViewLayer::Silkscreen0 : ViewLayer::Silkscreen1;
 }
 
 QString LogoItem::colorString() {
-	return ViewLayer::Silkscreen1Color;
+    return isBottom() ? ViewLayer::Silkscreen0Color :  ViewLayer::Silkscreen1Color;
 }
 
 QString LogoItem::layerName() 
@@ -676,11 +719,25 @@ QString LogoItem::layerName()
 }
 
 double LogoItem::minWidth() {
-	return ResizableBoard::CornerHandleSize * 2;
+    double zoom = 1;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		zoom = infoGraphicsView->currentZoom() / 100;
+        if (zoom == 0) zoom = 1;
+	}
+    
+	return ResizableBoard::CornerHandleSize * 2 / zoom;
 }
 
 double LogoItem::minHeight() {
-	return ResizableBoard::CornerHandleSize * 2;
+    double zoom = 1;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		zoom = infoGraphicsView->currentZoom() / 100;
+        if (zoom == 0) zoom = 1;
+	}
+
+	return ResizableBoard::CornerHandleSize * 2 / zoom;
 }
 
 bool LogoItem::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
@@ -691,6 +748,10 @@ bool LogoItem::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
 	return true;
 }
 
+bool LogoItem::freeRotationAllowed() {
+	return true;
+}
+
 ResizableBoard::Corner LogoItem::findCorner(QPointF scenePos, Qt::KeyboardModifiers modifiers) {
 	ResizableBoard::Corner corner = ResizableBoard::findCorner(scenePos, modifiers);
 	if (corner == ResizableBoard::NO_CORNER) return corner;
@@ -709,10 +770,130 @@ void LogoItem::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *opt
 	PaletteItem::paintHover(painter, option, widget);
 }
 
+bool LogoItem::isBottom() {
+    QString layer = modelPart()->properties().value("layer");
+	return layer.contains("0") || layer.contains("bottom");
+}
+
+QString LogoItem::flipSvg(const QString & svg)
+{
+	QString newSvg = svg;
+	newSvg.replace(ViewLayer::viewLayerXmlNameFromID(ViewLayer::Silkscreen1), ViewLayer::viewLayerXmlNameFromID(ViewLayer::Silkscreen0));
+	newSvg.replace(ViewLayer::Silkscreen1Color, ViewLayer::Silkscreen0Color, Qt::CaseInsensitive);
+    return flipSvgAux(newSvg);
+}
+
+QString LogoItem::flipSvgAux(QString & newSvg)
+{
+	QMatrix m;
+	QSvgRenderer renderer(newSvg.toUtf8());
+	QRectF bounds = renderer.viewBoxF();
+	m.translate(bounds.center().x(), bounds.center().y());
+	QMatrix mMinus = m.inverted();
+    QMatrix cm = mMinus * QMatrix().scale(-1, 1) * m;
+	int gix = newSvg.indexOf("<g");
+	newSvg.replace(gix, 2, "<g _flipped_='1' transform='" + TextUtils::svgMatrix(cm) + "'");
+	return newSvg;
+}
+
 ///////////////////////////////////////////////////////////////////////
 
-CopperLogoItem::CopperLogoItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: LogoItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+SchematicLogoItem::SchematicLogoItem( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: LogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
+{
+}
+
+SchematicLogoItem::~SchematicLogoItem() {
+}
+
+ViewLayer::ViewLayerID SchematicLogoItem::layer() {
+	return (m_hasLogo ? ViewLayer::SchematicText : ViewLayer::Schematic);
+}
+
+QString SchematicLogoItem::colorString() {
+    return "black";
+}
+
+bool SchematicLogoItem::isBottom() {
+    return false;
+}
+
+///////////////////////////////////////////////////////////////////////
+
+BreadboardLogoItem::BreadboardLogoItem( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: LogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
+{
+    m_color = "#787878";
+	QString color = modelPart->localProp("color").toString();
+    if (color.isEmpty()) {
+	    color = modelPart->properties().value("color");
+        if (color.isEmpty()) color = m_color;
+		modelPart->setLocalProp("color", color);
+	}
+    m_color = color;
+}
+
+BreadboardLogoItem::~BreadboardLogoItem() {
+}
+
+ViewLayer::ViewLayerID BreadboardLogoItem::layer() {
+	return ViewLayer::Breadboard;
+}
+
+QString BreadboardLogoItem::colorString() {
+    return m_color;
+}
+
+bool BreadboardLogoItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
+{
+	if (m_hasLogo) {
+		if (prop.compare("color", Qt::CaseInsensitive) == 0) {
+			returnProp = tr("color");
+			returnValue = m_color;
+
+		    QPushButton * button = new QPushButton(tr("Set text color"));
+		    button->setObjectName("infoViewButton");
+		    connect(button, SIGNAL(pressed()), this, SLOT(changeTextColor()));
+
+			returnWidget = button;
+			return true;
+		}
+	}
+
+	return LogoItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
+}
+
+void BreadboardLogoItem::changeTextColor() {
+    QColor color(m_color);
+    QColor newColor = QColorDialog::getColor(color, NULL, tr("Select text color"));
+    if (!newColor.isValid()) return;
+    if (newColor.name().compare(m_color, Qt::CaseInsensitive) == 0) return;
+
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->setProp(this, "color", tr("color"), m_color, newColor.name(), true);
+	}
+}
+
+void BreadboardLogoItem::setProp(const QString & prop, const QString & value) {
+	if (m_hasLogo && prop.compare("color", Qt::CaseInsensitive) == 0) {
+        m_color = value;
+        modelPart()->setLocalProp("color", m_color);
+		setLogo(m_logo, true);
+		return;
+	}
+
+	LogoItem::setProp(prop, value);
+}
+
+bool BreadboardLogoItem::isBottom() {
+    return false;
+}
+
+///////////////////////////////////////////////////////////////////////
+
+CopperLogoItem::CopperLogoItem( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: LogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	if (Copper1ImageNames.count() == 0) {
 		Copper1ImageNames << "Fritzing icon copper1";
@@ -734,53 +915,23 @@ CopperLogoItem::~CopperLogoItem() {
 }
 
 ViewLayer::ViewLayerID CopperLogoItem::layer() {
-	return isCopper0() ? ViewLayer::Copper0 :  ViewLayer::Copper1;
+	return isBottom() ? ViewLayer::Copper0 :  ViewLayer::Copper1;
 }
 
 QString CopperLogoItem::colorString() {
-	return isCopper0() ? ViewLayer::Copper0Color :  ViewLayer::Copper1Color;
+	return isBottom() ? ViewLayer::Copper0Color :  ViewLayer::Copper1Color;
 }
 
 QStringList & CopperLogoItem::getImageNames() {
-	return isCopper0() ? Copper0ImageNames :  Copper1ImageNames;
-}
-
-QString CopperLogoItem::hackSvg(const QString & svg, const QString & logo) {
-	QString newSvg = LogoItem::hackSvg(svg, logo);
-	if (!isCopper0()) return newSvg;
-
-	return flipSvg(newSvg);
+	return isBottom() ? Copper0ImageNames : Copper1ImageNames;
 }
 
 QString CopperLogoItem::flipSvg(const QString & svg)
 {
 	QString newSvg = svg;
-	newSvg.replace("copper1", "copper0");
+	newSvg.replace(ViewLayer::viewLayerXmlNameFromID(ViewLayer::Copper1), ViewLayer::viewLayerXmlNameFromID(ViewLayer::Copper0));
 	newSvg.replace(ViewLayer::Copper1Color, ViewLayer::Copper0Color, Qt::CaseInsensitive);
-	QMatrix m;
-	QSvgRenderer renderer(newSvg.toUtf8());
-	QRectF bounds = renderer.viewBoxF();
-	m.translate(bounds.center().x(), bounds.center().y());
-	QMatrix mMinus = m.inverted();
-    QMatrix cm = mMinus * QMatrix().scale(-1, 1) * m;
-	int gix = newSvg.indexOf("<g");
-	newSvg.replace(gix, 2, "<g _flipped_='1' transform='" + TextUtils::svgMatrix(cm) + "'");
-	return newSvg;
-}
-
-bool CopperLogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName) 
-{
-	if (isCopper0()) {
-		if (!svg.contains("_flipped_")) {
-			return LogoItem::reloadImage(flipSvg(svg), aspectRatio, fileName, addName);
-		}
-	}
-
-	return LogoItem::reloadImage(svg, aspectRatio, fileName, addName);
-}
-
-bool CopperLogoItem::isCopper0() {
-	return modelPart()->properties().value("layer").contains("0");
+    return flipSvgAux(newSvg);
 }
 
 ////////////////////////////////////////////
@@ -789,8 +940,8 @@ bool CopperLogoItem::isCopper0() {
 //
 // don't mess with colors at all?
 
-BoardLogoItem::BoardLogoItem(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel) 
-    : LogoItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+BoardLogoItem::BoardLogoItem(ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel) 
+    : LogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
     m_hasLogo = false;
     m_svgOnly = true;
@@ -836,15 +987,14 @@ bool BoardLogoItem::resizeMM(double mmW, double mmH, const LayerHash & viewLayer
 void BoardLogoItem::reloadLayerKin(double mmW, double mmH)
 {
 	foreach (ItemBase * itemBase, m_layerKin) {
-        if (itemBase->viewLayerID() == LogoItem::layer()) {
-		    QString svg = ResizableBoard::getShapeForRenderer(prop("shape"), LogoItem::layer());
+        if (itemBase->viewLayerID() == ViewLayer::Silkscreen1 || itemBase->viewLayerID() == ViewLayer::Silkscreen0) {
+		    QString svg = ResizableBoard::getShapeForRenderer(prop("shape"), itemBase->viewLayerID());
 		    if (!svg.isEmpty()) {
 			    itemBase->prepareGeometryChange();
 			    if (itemBase->resetRenderer(svg)) {
 				    itemBase->modelPart()->setLocalProp("width", mmW);
 				    itemBase->modelPart()->setLocalProp("height", mmH);
 			    }
-			    break;
 		    }
 	    }
     }
@@ -870,78 +1020,18 @@ bool BoardLogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio,
     return result;
 }
 
-
-QString BoardLogoItem::setBoardOutline(const QString & svg) {
-    QString errorStr;
-	int errorLine;
-	int errorColumn;
-
-	QDomDocument domDocument;
-	if (!domDocument.setContent(svg, true, &errorStr, &errorLine, &errorColumn)) {
-		return svg;
-	}
-
-    QDomElement root = domDocument.documentElement();
-	if (root.tagName() != "svg") {
-		return svg;
-	}
-
-    QDomElement board = TextUtils::findElementWithAttribute(root, "id", "board");
-    if (board.isNull()) {
-        board = root;
-    }
-
-    QList<QDomElement> leaves;
-    TextUtils::collectLeaves(board, leaves);
-
-    if (leaves.count() == 1) {
-        QDomElement leaf = leaves.at(0);
-        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
-        return TextUtils::removeXMLEntities(domDocument.toString());
-    }
-
-    int ix = 0;
-    QStringList ids;
-    foreach (QDomElement leaf, leaves) {
-        ids.append(leaf.attribute("id", ""));
-        leaf.setAttribute("id", QString("____%1____").arg(ix++));
-    }
-
-    QSvgRenderer renderer(domDocument.toByteArray());
-
-    ix = 0;
-    foreach (QDomElement leaf, leaves) {    
-        leaf.setAttribute("id", ids.at(ix++));
-    }
-
-    double maxArea = 0;
-    int maxIndex = -1;
-    for (int i = 0; i < leaves.count(); i++) {
-        QRectF r = renderer.boundsOnElement(QString("____%1____").arg(i));
-        if (r.width() * r.height() > maxArea) {
-            maxArea = r.width() * r.height();
-            maxIndex = i;
-        }
-    }
-
-    if (maxIndex >= 0) {
-        QDomElement leaf = leaves.at(maxIndex);
-        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
-        return TextUtils::removeXMLEntities(domDocument.toString());
-    }
-
-    return svg;
-}
-
 bool BoardLogoItem::checkImage(const QString & filename) {
+    // override logoitem
     return ResizableBoard::checkImage(filename);
 }
 
-bool BoardLogoItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+bool BoardLogoItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (prop.compare("shape", Qt::CaseInsensitive) == 0) {
-        Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
-		returnWidget = setUpDimEntry(true, true,  returnWidget);   
+        Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
+		returnWidget = setUpDimEntry(true, true, false, returnWidget);  
+        returnWidget->setEnabled(swappingEnabled);
+
         m_aspectRatioCheck->setEnabled(false);
         m_aspectRatioCheck->setChecked(true);
         m_aspectRatioCheck->setVisible(false);
@@ -950,6 +1040,11 @@ bool BoardLogoItem::collectExtraInfo(QWidget * parent, const QString & family, c
 		return true;
 	}
 
-    return LogoItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+    return LogoItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
+bool BoardLogoItem::isBottom() {
+    return false;
+}
+
+
