@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -42,11 +42,11 @@ class LogoItem : public ResizableBoard
 	Q_OBJECT
 
 public:
-	LogoItem(ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	LogoItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
 	~LogoItem();
 
-	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi);
-	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget);
+	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
+	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
 	bool resizeMM(double w, double h, const LayerHash & viewLayers);
 	QString getProperty(const QString & key);
 	void setLogo(QString logo, bool force);
@@ -78,12 +78,17 @@ protected:
 	double minWidth();
 	double minHeight();
 	bool freeRotationAllowed(Qt::KeyboardModifiers modifiers);
+	bool freeRotationAllowed();
 	ResizableBoard::Corner findCorner(QPointF p, Qt::KeyboardModifiers);
     virtual QString getShapeForRenderer(const QString & svg);
     virtual bool canRetrieveLayer(ViewLayer::ViewLayerID viewLayerID);
     bool checkImage(const QString & filename);
 	QStringList & getImageNames();
 	QStringList & getNewImageNames();
+	virtual bool isBottom();
+	virtual QString flipSvg(const QString & svg);
+	QString flipSvgAux(QString & newSvg);
+	QStringList collectValues(const QString & family, const QString & prop, QString & value);
 
 protected:
 	QString m_logo;
@@ -98,30 +103,63 @@ class CopperLogoItem : public LogoItem
 Q_OBJECT
 	
 public:
-	CopperLogoItem(ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	CopperLogoItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
 	~CopperLogoItem();
 
-	bool reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName);
-
 protected:
 	ViewLayer::ViewLayerID layer();
 	QString colorString();
 	QStringList & getImageNames();
-	QString hackSvg(const QString & svg, const QString & logo);
 	QString flipSvg(const QString & svg);
-	bool isCopper0();
+};
+
+class SchematicLogoItem : public LogoItem
+{
+Q_OBJECT
+	
+public:
+	SchematicLogoItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	~SchematicLogoItem();
+
+protected:
+	ViewLayer::ViewLayerID layer();
+	QString colorString();
+    bool isBottom();
+};
+
+class BreadboardLogoItem : public LogoItem
+{
+Q_OBJECT
+	
+public:
+	BreadboardLogoItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	~BreadboardLogoItem();
+
+    bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
+	void setProp(const QString & prop, const QString & value);
+
+public slots:
+    void changeTextColor();
+
+protected:
+	ViewLayer::ViewLayerID layer();
+	QString colorString();
+    bool isBottom();
+
+protected:
+    QString m_color;
 };
 
 class BoardLogoItem : public LogoItem
 {
 Q_OBJECT
 public:
-	BoardLogoItem(ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	BoardLogoItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
 	~BoardLogoItem();
 
     bool resizeMM(double w, double h, const LayerHash & viewLayers);
 	bool reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName);
-	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget);
+	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
 
 protected:
 	ViewLayer::ViewLayerID layer();
@@ -131,9 +169,8 @@ protected:
     QString getShapeForRenderer(const QString & svg);
     bool canRetrieveLayer(ViewLayer::ViewLayerID viewLayerID);
     void reloadLayerKin(double mmW, double mmH);
-    QString setBoardOutline(const QString & svg);
     bool checkImage(const QString & filename);
-
+    bool isBottom();
 };
 
 
