@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6147 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-06 10:36:40 +0200 (Fri, 06 Jul 2012) $
+$Revision: 6947 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-03 06:45:22 +0200 (Mi, 03. Apr 2013) $
 
 ********************************************************************/
 
@@ -44,6 +44,10 @@ const QString ModuleIDNames::JumperModuleIDName = "JumperModuleID";
 const QString ModuleIDNames::GroundPlaneModuleIDName = "GroundPlaneModuleID";
 const QString ModuleIDNames::GroundModuleIDName = "GroundModuleID";
 const QString ModuleIDNames::PowerModuleIDName = "PowerModuleID";
+const QString ModuleIDNames::TwoPowerModuleIDName = "2PowerModuleID";
+const QString ModuleIDNames::NetLabelModuleIDName = "NetLabelModuleID";
+const QString ModuleIDNames::PowerLabelModuleIDName = "PowerLabelModuleID";
+const QString ModuleIDNames::LeftNetLabelModuleIDName = "LeftNetLabelModuleID";
 const QString ModuleIDNames::JustPowerModuleIDName = "JustPowerModuleID";
 const QString ModuleIDNames::ResistorModuleIDName = "ResistorModuleID";
 const QString ModuleIDNames::LogoTextModuleIDName = "LogoTextModuleID";
@@ -52,6 +56,10 @@ const QString ModuleIDNames::Copper1LogoTextModuleIDName = "Copper1LogoTextModul
 const QString ModuleIDNames::Copper1LogoImageModuleIDName = "Copper1LogoImageModuleID";
 const QString ModuleIDNames::Copper0LogoTextModuleIDName = "Copper0LogoTextModuleID";
 const QString ModuleIDNames::Copper0LogoImageModuleIDName = "Copper0LogoImageModuleID";
+const QString ModuleIDNames::SchematicLogoTextModuleIDName = "SchematicLogoTextModuleID";
+const QString ModuleIDNames::SchematicLogoImageModuleIDName = "SchematicLogoImageModuleID";
+const QString ModuleIDNames::BreadboardLogoTextModuleIDName = "BreadboardLogoTextModuleID";
+const QString ModuleIDNames::BreadboardLogoImageModuleIDName = "BreadboardLogoImageModuleID";
 const QString ModuleIDNames::BoardLogoImageModuleIDName = "BoardLogoImageModuleID";
 const QString ModuleIDNames::OneLayerBoardLogoImageModuleIDName = "OneLayerBoardLogoImageModuleID";
 const QString ModuleIDNames::HoleModuleIDName = "HoleModuleID";
