@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6147 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-06 10:36:40 +0200 (Fri, 06 Jul 2012) $
+$Revision: 6947 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-03 06:45:22 +0200 (Mi, 03. Apr 2013) $
 
 ********************************************************************/
 
@@ -46,6 +46,10 @@ public:
 	static const QString GroundPlaneModuleIDName;
 	static const QString GroundModuleIDName;
 	static const QString PowerModuleIDName;
+	static const QString TwoPowerModuleIDName;
+	static const QString NetLabelModuleIDName;	
+    static const QString PowerLabelModuleIDName;
+	static const QString LeftNetLabelModuleIDName;
 	static const QString JustPowerModuleIDName;
 	static const QString ResistorModuleIDName;
 	static const QString LogoTextModuleIDName;
@@ -54,6 +58,10 @@ public:
 	static const QString Copper0LogoImageModuleIDName;
 	static const QString Copper1LogoTextModuleIDName;
 	static const QString Copper1LogoImageModuleIDName;
+	static const QString SchematicLogoTextModuleIDName;
+	static const QString SchematicLogoImageModuleIDName;
+	static const QString BreadboardLogoTextModuleIDName;
+	static const QString BreadboardLogoImageModuleIDName;
 	static const QString BoardLogoImageModuleIDName;
 	static const QString OneLayerBoardLogoImageModuleIDName;
 	static const QString HoleModuleIDName;
