@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6218 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
+$Revision: 6980 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 01:45:43 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -209,8 +209,8 @@ QString LinkDialog::url() {
 
 /////////////////////////////////////////////
 
-Note::Note( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,  const ViewGeometry & viewGeometry, long id, QMenu* itemMenu)
-	: ItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
+Note::Note( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry & viewGeometry, long id, QMenu* itemMenu)
+	: ItemBase(modelPart, viewID, viewGeometry, id, itemMenu)
 {
 	m_charsAdded = 0;
 	if (initialTextString.isEmpty()) {
@@ -240,6 +240,8 @@ Note::Note( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdent
 	QPixmap pixmap(":/resources/images/icons/noteResizeGrip.png");
 	m_resizeGrip = new ResizeHandle(pixmap, Qt::SizeFDiagCursor, this);
 	connect(m_resizeGrip, SIGNAL(mousePressSignal(QGraphicsSceneMouseEvent *, ResizeHandle *)), this, SLOT(handleMousePressSlot(QGraphicsSceneMouseEvent *, ResizeHandle *)));
+	connect(m_resizeGrip, SIGNAL(mouseMoveSignal(QGraphicsSceneMouseEvent *, ResizeHandle *)), this, SLOT(handleMouseMoveSlot(QGraphicsSceneMouseEvent *, ResizeHandle *)));
+	connect(m_resizeGrip, SIGNAL(mouseReleaseSignal(QGraphicsSceneMouseEvent *, ResizeHandle *)), this, SLOT(handleMouseReleaseSlot(QGraphicsSceneMouseEvent *, ResizeHandle *)));
 	connect(m_resizeGrip, SIGNAL(zoomChangedSignal(double)), this, SLOT(handleZoomChangedSlot(double)));
 
 	m_graphicsTextItem = new NoteGraphicsTextItem();
@@ -356,38 +358,6 @@ void Note::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 		return;
 	}
 
-	if (m_inResize) {
-		double minWidth = emptyMinWidth;
-		double minHeight = emptyMinHeight;
-		QSizeF gripSize = m_resizeGrip->boundingRect().size();
-		QSizeF minSize = m_graphicsTextItem->document()->size() + gripSize + gripSize;
-		if (minSize.height() > minHeight) minHeight = minSize.height();
-
-		QRectF rect = boundingRect();
-		rect.moveTopLeft(this->pos());
-
-		double oldX1 = rect.x();
-		double oldY1 = rect.y();
-		double newX = event->scenePos().x() + m_inResize->resizeOffset().x();
-		double newY = event->scenePos().y() + m_inResize->resizeOffset().y();
-		QRectF newR;
-
-		if (newX - oldX1 < minWidth) {
-			newX = oldX1 + minWidth;
-		}
-		if (newY - oldY1 < minHeight) {
-			newY = oldY1 + minHeight;
-		}	
-		newR.setRect(0, 0, newX - oldX1, newY - oldY1);
-
-		prepareGeometryChange();
-		m_rect = newR;
-		positionGrip();
-		event->accept();
-		
-		return;
-	}
-
 	ItemBase::mouseMoveEvent(event);
 }
 
@@ -397,17 +367,6 @@ void Note::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {
 		return;
 	}
 
-	if (m_inResize) {
-		this->ungrabMouse();
-		m_inResize = NULL;
-		InfoGraphicsView *infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-		if (infoGraphicsView != NULL) {
-			infoGraphicsView->noteSizeChanged(this, m_viewGeometry.rect(), m_rect);
-		}
-		event->accept();
-		return;
-	}
-
 	ItemBase::mouseReleaseEvent(event);
 }
 
@@ -461,6 +420,8 @@ void Note::forceFormat(int position, int charsAdded) {
 }
 
 void Note::contentsChangedSlot() {
+
+    //DebugDialog::debug(QString("contents changed ") + m_graphicsTextItem->document()->toPlainText());
 	if (m_charsAdded > 0) {
 		forceFormat(m_charsPosition, m_charsAdded);
 	}
@@ -711,8 +672,52 @@ void Note::handleMousePressSlot(QGraphicsSceneMouseEvent * event, ResizeHandle *
 	resizeHandle->setResizeOffset(this->pos() + QPointF(sz.width(), sz.height()) - event->scenePos());
 
 	m_inResize = resizeHandle;
-	this->grabMouse();
+}
 
+void Note::handleMouseMoveSlot(QGraphicsSceneMouseEvent * event, ResizeHandle * resizeHandle) {
+    Q_UNUSED(resizeHandle);
+
+    if (!m_inResize) return;
+
+	double minWidth = emptyMinWidth;
+	double minHeight = emptyMinHeight;
+	QSizeF gripSize = m_resizeGrip->boundingRect().size();
+	QSizeF minSize = m_graphicsTextItem->document()->size() + gripSize + gripSize;
+	if (minSize.height() > minHeight) minHeight = minSize.height();
+
+	QRectF rect = boundingRect();
+	rect.moveTopLeft(this->pos());
+
+	double oldX1 = rect.x();
+	double oldY1 = rect.y();
+	double newX = event->scenePos().x() + m_inResize->resizeOffset().x();
+	double newY = event->scenePos().y() + m_inResize->resizeOffset().y();
+	QRectF newR;
+
+	if (newX - oldX1 < minWidth) {
+		newX = oldX1 + minWidth;
+	}
+	if (newY - oldY1 < minHeight) {
+		newY = oldY1 + minHeight;
+	}	
+	newR.setRect(0, 0, newX - oldX1, newY - oldY1);
+
+	prepareGeometryChange();
+	m_rect = newR;
+	positionGrip();	
+}
+
+void Note::handleMouseReleaseSlot(QGraphicsSceneMouseEvent * event, ResizeHandle * resizeHandle) {
+    Q_UNUSED(resizeHandle);
+    Q_UNUSED(event);
+
+	if (!m_inResize) return;
+
+	m_inResize = NULL;
+	InfoGraphicsView *infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->noteSizeChanged(this, m_viewGeometry.rect(), m_rect);
+	}
 }
 
 bool Note::hasPartLabel() {
@@ -737,22 +742,23 @@ bool Note::rotation45Allowed() {
 	return false;
 }
 
-QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
+QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	Q_UNUSED(svgHash);
+    factor = 1;
 
 	switch (viewLayerID) {
 		case ViewLayer::BreadboardNote:
-			if (viewIdentifier() != ViewIdentifierClass::BreadboardView) return ___emptyString___;
+			if (viewID() != ViewLayer::BreadboardView) return "";
 			break;
 		case ViewLayer::PcbNote:
-			if (viewIdentifier() != ViewIdentifierClass::PCBView) return ___emptyString___;
+			if (viewID() != ViewLayer::PCBView) return "";
 			break;
 		case ViewLayer::SchematicNote:
-			if (viewIdentifier() != ViewIdentifierClass::SchematicView) return ___emptyString___;
+			if (viewID() != ViewLayer::SchematicView) return "";
 			break;
 		default:
-			return ___emptyString___;
+			return "";
 	}
 
 	QString svg = "<g>";
@@ -832,7 +838,7 @@ QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QSt
 	return svg;	
 }
 
-ViewIdentifierClass::ViewIdentifier Note::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier, bool) 
+ViewLayer::ViewID Note::useViewIDForPixmap(ViewLayer::ViewID, bool) 
 {
-    return ViewIdentifierClass::IconView;
+    return ViewLayer::IconView;
 }
