@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6289 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -42,6 +42,8 @@ $Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/familypropertycombobox.h"
+#include "../svg/svgfilesplitter.h"
+#include "wire.h"
 
 #include <QGraphicsSceneMouseEvent>
 #include <QSvgRenderer>
@@ -70,8 +72,9 @@ QString HoleSettings::holeSize() {
 
 /////////////////////////////////////////////////
 
+static QRegExp LabelFinder("id=['|\"]label['|\"]");
+
 static bool ByIDParseSuccessful = true;
-static QRegExp IntegerFinder("\\d+");
 
 static QRegExp ConnectorFinder("connector\\d+pin");
 const QString PaletteItem::HoleSizePrefix("_hs_");
@@ -117,9 +120,10 @@ bool byID(Connector * c1, Connector * c2)
 
 /////////////////////////////////////////////////
 
-PaletteItem::PaletteItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: PaletteItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
+PaletteItem::PaletteItem( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: PaletteItemBase(modelPart, viewID, viewGeometry, id, itemMenu)
 {
+    m_flipCount = 0;
 	if(doLabel) {
 		m_partLabel = new PartLabel(this, NULL);
 		m_partLabel->setVisible(false);
@@ -134,89 +138,84 @@ PaletteItem::~PaletteItem() {
 	}
 }
 
-bool PaletteItem::renderImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, bool doConnectors, QString & error) {
+bool PaletteItem::renderImage(ModelPart * modelPart, ViewLayer::ViewID viewID, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, bool doConnectors, QString & error) {
 	LayerAttributes layerAttributes; 
-	bool result = setUpImage(modelPart, viewIdentifier, viewLayers, viewLayerID, this->viewLayerSpec(), doConnectors, layerAttributes, error);
+    initLayerAttributes(layerAttributes, viewID, viewLayerID, viewLayerPlacement(), doConnectors, true);
+	bool result = setUpImage(modelPart, viewLayers, layerAttributes);
+    error = layerAttributes.error;
 
 	m_syncMoved = this->pos();
 	return result;
 }
 
-void PaletteItem::loadLayerKin(const LayerHash & viewLayers, ViewLayer::ViewLayerSpec viewLayerSpec) {
+void PaletteItem::loadLayerKin(const LayerHash & viewLayers, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 
 	if (m_modelPart == NULL) return;
 
 	ModelPartShared * modelPartShared = m_modelPart->modelPartShared();
 	if (modelPartShared == NULL) return;
-	if (modelPartShared->domDocument() == NULL) return;
 
 	qint64 id = m_id + 1;
 	ViewGeometry viewGeometry = m_viewGeometry;
 	viewGeometry.setZ(-1);
 
-	LayerList notLayers;
-	switch (viewLayerSpec) {
-		case ViewLayer::ThroughHoleThroughTop_OneLayer:
-			if (m_modelPart->flippedSMD()) {
-				notLayers << ViewLayer::Copper1 << ViewLayer::Copper1Trace << ViewLayer::Silkscreen1 << ViewLayer::Silkscreen1Label;
-			}
-			//notLayers << ViewLayer::Silkscreen0 << ViewLayer::Silkscreen0Label << ViewLayer::Copper1 << ViewLayer:: Copper1Trace;
-			break;
-		case ViewLayer::ThroughHoleThroughTop_TwoLayers:
-			if (m_modelPart->flippedSMD()) {
-				notLayers << ViewLayer::Copper0 << ViewLayer::Copper0Trace << ViewLayer::Silkscreen0 << ViewLayer::Silkscreen0Label;
-			}
-			//notLayers << ViewLayer::Silkscreen0 << ViewLayer::Silkscreen0Label;
-			break;
-		case ViewLayer::ThroughHoleThroughBottom_TwoLayers:
-			//notLayers << ViewLayer::Silkscreen << ViewLayer::SilkscreenLabel;
-			break;
-
-		case ViewLayer::GroundPlane_Top:
-			notLayers << ViewLayer::GroundPlane0;
-			break;
-		case ViewLayer::GroundPlane_Bottom:
-			notLayers << ViewLayer::GroundPlane1;
-			break;
-
-		// not sure these ever get used...
-		case ViewLayer::SMDOnTop_TwoLayers:
-			notLayers << ViewLayer::Copper0 << ViewLayer::Copper0Trace << ViewLayer::Silkscreen0 << ViewLayer::Silkscreen0Label;
-			break;
-		case ViewLayer::SMDOnBottom_TwoLayers:
-		case ViewLayer::SMDOnBottom_OneLayer:
-			notLayers << ViewLayer::Copper1 << ViewLayer::Copper1Trace << ViewLayer::Silkscreen1 << ViewLayer::Silkscreen1Label;
-			break;
-
-                case ViewLayer::Top:
-                case ViewLayer::Bottom:
-                case ViewLayer::TopAndBottom:
-		case ViewLayer::WireOnTop_TwoLayers:
-		case ViewLayer::WireOnBottom_OneLayer:
-                case ViewLayer::WireOnBottom_TwoLayers:
-                case ViewLayer::UnknownSpec:
-			DebugDialog::debug("bad view spec in LoadLayerKin");
-			break;
-	}
 
 	foreach (ViewLayer::ViewLayerID viewLayerID, viewLayers.keys()) {
 		if (viewLayerID == m_viewLayerID) continue;
-		if (notLayers.contains(viewLayerID)) continue;
-		if (!m_modelPart->hasViewFor(m_viewIdentifier, viewLayerID)) continue;
+		if (!m_modelPart->hasViewFor(m_viewID, viewLayerID)) continue;
 
-		LayerKinPaletteItem * lkpi = newLayerKinPaletteItem(this, m_modelPart, m_viewIdentifier, viewGeometry, id, viewLayerID, viewLayerSpec, m_itemMenu, viewLayers);
-		if (lkpi->ok()) {
-			//DebugDialog::debug(QString("adding layer kin %1 %2 %3").arg(id).arg(m_viewIdentifier).arg(viewLayerID) );
-			lkpi->setViewLayerSpec(viewLayerSpec);
-			addLayerKin(lkpi);
-			id++;
-		}
-		else {
-			delete lkpi;
-		}
+        if (m_modelPart->itemType() == ModelPart::CopperFill) {
+            if (viewLayerPlacement == ViewLayer::NewTop) {
+                if (ViewLayer::bottomLayers().contains(viewLayerID)) continue;
+            }
+            else {
+                if (ViewLayer::topLayers().contains(viewLayerID)) continue;
+            }
+        }
+        else if (m_modelPart->flippedSMD()) {
+            if (viewLayerPlacement == ViewLayer::NewTop) {
+                if (ViewLayer::bottomLayers().contains(viewLayerID)) continue;
+            }
+            else {
+                if (ViewLayer::topLayers().contains(viewLayerID)) continue;
+            }
+        }
+        else if (m_modelPart->itemType() == ModelPart::Part) {
+            // through hole part
+            if (ViewLayer::silkLayers().contains(viewLayerID)) {
+                if (viewLayerPlacement == ViewLayer::NewTop) {
+                    if (ViewLayer::bottomLayers().contains(viewLayerID)) continue;
+                }
+                else {
+                    if (ViewLayer::topLayers().contains(viewLayerID)) continue;
+                }
+            }
+
+        }
+
+        makeOneKin(id, viewLayerID, viewLayerPlacement, viewGeometry, viewLayers);
+	}
+
+}
+
+void PaletteItem::makeOneKin(qint64 & id, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement viewLayerPlacement, ViewGeometry & viewGeometry, const LayerHash & viewLayers) {
+    LayerAttributes layerAttributes;
+    initLayerAttributes(layerAttributes, m_viewID, viewLayerID, viewLayerPlacement, true, true);
+    LayerKinPaletteItem * lkpi = newLayerKinPaletteItem(this, m_modelPart, viewGeometry, id, m_itemMenu, viewLayers, layerAttributes);
+	if (lkpi->ok()) {
+		DebugDialog::debug(QString("adding layer kin %1 %2 %3 %4")
+            .arg(id).arg(m_viewID).arg(viewLayerID) 
+            .arg((long) lkpi, 0, 16)
+        );
+		addLayerKin(lkpi);
+		id++;
+	}
+	else {
+		delete lkpi;
 	}
 }
 
+
 void PaletteItem::addLayerKin(LayerKinPaletteItem * lkpi) {
 	m_layerKin.append(lkpi);
 }
@@ -287,17 +286,17 @@ const QList<class ItemBase *> & PaletteItem::layerKin()
 	return m_layerKin;
 }
 
-void PaletteItem::rotateItem(double degrees) {
-	PaletteItemBase::rotateItem(degrees);
+void PaletteItem::rotateItem(double degrees, bool includeRatsnest) {
+	PaletteItemBase::rotateItem(degrees, includeRatsnest);
 	for (int i = 0; i < m_layerKin.count(); i++) {
-		m_layerKin[i]->rotateItem(degrees);
+		m_layerKin[i]->rotateItem(degrees, includeRatsnest);
 	}
 }
 
 void PaletteItem::flipItem(Qt::Orientations orientation) {
 	PaletteItemBase::flipItem(orientation);
 	foreach (ItemBase * lkpi, m_layerKin) {
-		lkpi->flipItem(orientation);
+        lkpi->flipItem(orientation);
 	}
 }
 
@@ -309,9 +308,12 @@ void PaletteItem::transformItem2(const QMatrix & matrix) {
 }
 
 void PaletteItem::setTransforms() {
-	setTransform(getViewGeometry().transform());
+    QTransform transform = getViewGeometry().transform();
+    if (transform.isIdentity()) return;
+
+	setTransform(transform);
 	for (int i = 0; i < m_layerKin.count(); i++) {
-		m_layerKin[i]->setTransform(m_layerKin[i]->getViewGeometry().transform());
+		m_layerKin[i]->setTransform2(m_layerKin[i]->getViewGeometry().transform());
 	}
 }
 
@@ -329,10 +331,10 @@ void PaletteItem::setItemPos(QPointF & loc) {
 	}
 }
 
-void PaletteItem::updateConnections() {
-	updateConnectionsAux();
+void PaletteItem::updateConnections(bool includeRatsnest, QList<ConnectorItem *> & already) {
+	updateConnectionsAux(includeRatsnest, already);
 	foreach (ItemBase * lkpi, m_layerKin) {
-		lkpi->updateConnectionsAux();
+		lkpi->updateConnectionsAux(includeRatsnest, already);
 	}
 }
 
@@ -353,7 +355,7 @@ void PaletteItem::collectWireConnectees(QSet<Wire *> & wires) {
 	}
 }
 
-void PaletteItem::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event) {
+bool PaletteItem::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event) {
 	//DebugDialog::debug("layerkinchief got mouse press event");
 	/*
 
@@ -363,7 +365,7 @@ void PaletteItem::mousePressEvent(PaletteItemBase * originalItem, QGraphicsScene
 	}
 	*/
 
-	PaletteItemBase::mousePressEvent(originalItem, event);
+	return PaletteItemBase::mousePressEvent(originalItem, event);
 }
 
 void PaletteItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
@@ -374,12 +376,6 @@ void PaletteItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 		return;
 	}
 
-	if (lowerConnectorLayerVisible(this)) {
-		DebugDialog::debug("PaletteItem::mousePressEvent isn't obsolete");
-		event->ignore();
-		return;
-	}
-
 	mousePressEvent(this, event);
 }
 
@@ -397,10 +393,10 @@ void PaletteItem::syncKinMoved(QPointF offset, QPointF newPos) {
 	//}
 }
 
-void PaletteItem::setInstanceTitle(const QString& title) {
-	ItemBase::setInstanceTitle(title);
+void PaletteItem::setInstanceTitle(const QString& title, bool initial) {
+	ItemBase::setInstanceTitle(title, initial);
 	foreach (ItemBase * lkpi, m_layerKin) {
-		lkpi->setInstanceTitle(title);
+		lkpi->setInstanceTitle(title, initial);
 	}
 }
 
@@ -415,41 +411,11 @@ void PaletteItem::setInactive(bool inactivate) {
 }
 
 void PaletteItem::figureHover() {
-	// if a layer contains connectors, make it the one that accepts hover events
-	// if you make all layers accept hover events, then the topmost layer will get the event
-	// and lower layers won't
-
-	// note: this affects which layer responds to tooltips: see FGraphicsScene::helpEvent()
-
-	QList<ItemBase *> allKin;
-	allKin.append(this);
+    setAcceptHoverEvents(true);
+    setAcceptedMouseButtons(ALLMOUSEBUTTONS);
 	foreach(ItemBase * lkpi, m_layerKin) {
-		allKin.append(lkpi);
-	}
-
-	qSort(allKin.begin(), allKin.end(), ItemBase::zLessThan);
-	foreach (ItemBase * base, allKin) {
-		base->setAcceptHoverEvents(false);
-		base->setAcceptedMouseButtons(Qt::NoButton);
-	}
-
-	int ix = 0;
-	foreach (ItemBase * base, allKin) {
-		if (!(base->hidden() || base->inactive()) && base->hasConnectors()) {
-			base->setAcceptHoverEvents(true);
-			base->setAcceptedMouseButtons(ALLMOUSEBUTTONS);
-			break;
-		}
-		ix++;
-	}
-
-	for (int i = 0; i < ix; i++) {
-		ItemBase * base = allKin[i];
-		if (!(base->hidden() || base->inactive())) {
-			base->setAcceptHoverEvents(true);
-			base->setAcceptedMouseButtons(ALLMOUSEBUTTONS);
-			return;
-		}
+		lkpi->setAcceptHoverEvents(true);
+		lkpi->setAcceptedMouseButtons(ALLMOUSEBUTTONS);
 	}
 }
 
@@ -460,32 +426,6 @@ void PaletteItem::clearModelPart() {
 	ItemBase::clearModelPart();
 }
 
-ItemBase * PaletteItem::lowerConnectorLayerVisible(ItemBase * itemBase) {
-	if (m_layerKin.count() == 0) return NULL;
-
-	if ((itemBase != this) 
-		&& this->isVisible() 
-		&& (!this->hidden()) && (!this->inactive()) && (this->zValue() < itemBase->zValue())
-		&& this->hasConnectors()) 
-	{
-		return this;
-	}
-
-	foreach (ItemBase * lkpi, m_layerKin) {
-		if (lkpi == itemBase) continue;
-
-		if (lkpi->isVisible() 
-			&& (!lkpi->hidden()) && (!lkpi->inactive()) 
-			&& (lkpi->zValue() < itemBase->zValue()) 
-			&& lkpi->hasConnectors() ) 
-		{
-			return lkpi;
-		}
-	}
-
-	return NULL;
-}
-
 void PaletteItem::resetID() {
 	ItemBase::resetID();
 	foreach (ItemBase * lkpi, m_layerKin) {
@@ -500,28 +440,28 @@ void PaletteItem::slamZ(double z) {
 	}
 }
 
-void PaletteItem::resetImage(InfoGraphicsView * infoGraphicsView, QDomDocument * domDocument) {
+void PaletteItem::resetImage(InfoGraphicsView * infoGraphicsView) {
 	foreach (Connector * connector, modelPart()->connectors()) {
-		connector->unprocess(this->viewIdentifier(), this->viewLayerID());
+		connector->unprocess(this->viewID(), this->viewLayerID());
 	}
-
-	QString error;
+    
 	LayerAttributes layerAttributes;
-	this->setUpImage(modelPart(), domDocument, this->viewIdentifier(), infoGraphicsView->viewLayers(), this->viewLayerID(), this->viewLayerSpec(), true, layerAttributes, error);
+    initLayerAttributes(layerAttributes, viewID(), viewLayerID(), viewLayerPlacement(), true, !m_selectionShape.isEmpty());
+	this->setUpImage(modelPart(), infoGraphicsView->viewLayers(),layerAttributes);
 	
 	foreach (ItemBase * layerKin, m_layerKin) {
-		resetKinImage(layerKin, infoGraphicsView, domDocument);
+		resetKinImage(layerKin, infoGraphicsView);
 	}
 }
 
-void PaletteItem::resetKinImage(ItemBase * layerKin, InfoGraphicsView * infoGraphicsView, QDomDocument * domDocument) 
+void PaletteItem::resetKinImage(ItemBase * layerKin, InfoGraphicsView * infoGraphicsView) 
 {
 	foreach (Connector * connector, modelPart()->connectors()) {
-		connector->unprocess(layerKin->viewIdentifier(), layerKin->viewLayerID());
+		connector->unprocess(layerKin->viewID(), layerKin->viewLayerID());
 	}
-	QString error;
 	LayerAttributes layerAttributes;
-	qobject_cast<PaletteItemBase *>(layerKin)->setUpImage(modelPart(), domDocument, layerKin->viewIdentifier(), infoGraphicsView->viewLayers(), layerKin->viewLayerID(), layerKin->viewLayerSpec(), true, layerAttributes, error);
+    initLayerAttributes(layerAttributes, layerKin->viewID(), layerKin->viewLayerID(), layerKin->viewLayerPlacement(), true, !layerKin->selectionShape().isEmpty());
+	qobject_cast<PaletteItemBase *>(layerKin)->setUpImage(modelPart(), infoGraphicsView->viewLayers(), layerAttributes);
 }
 
 QString PaletteItem::genFZP(const QString & moduleid, const QString & templateName, int minPins, int maxPins, int steps, bool smd)
@@ -569,7 +509,7 @@ QString PaletteItem::genFZP(const QString & moduleid, const QString & templateNa
 	return FzpTemplate.arg(count).arg(middle);
 }
 
-bool PaletteItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool PaletteItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare("editable pin labels", Qt::CaseInsensitive) == 0 && value.compare("true") == 0) {
 		returnProp = "";
@@ -585,9 +525,59 @@ bool PaletteItem::collectExtraInfo(QWidget * parent, const QString & family, con
 		return true;
 	}
 
-	return PaletteItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	bool result = PaletteItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
+
+    if (prop.compare("layer") == 0 && modelPart()->flippedSMD() && returnWidget != NULL) {
+        bool disabled = true;
+        InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+        if (infoGraphicsView && infoGraphicsView->boardLayers() == 2) disabled = false;
+        returnWidget->setDisabled(disabled);
+    }
+
+    return result;
 }
 
+QStringList PaletteItem::collectValues(const QString & family, const QString & prop, QString & value) {
+    if (prop.compare("layer") == 0) {
+        if (modelPart()->flippedSMD()) {
+            QStringList values = PaletteItemBase::collectValues(family, prop, value);
+            if (values.count() == 0) {
+                ItemBase * itemBase = modelPart()->viewItem(ViewLayer::PCBView);
+                if (itemBase) {
+                    values << TranslatedPropertyNames.value("bottom") << TranslatedPropertyNames.value("top");
+                    if (itemBase->viewLayerID() == ViewLayer::Copper0) {
+                        value = values.at(0);
+                    }
+                    else {
+                        value = values.at(1);
+                    }
+                }
+            }
+            return values;
+        }
+
+        if (modelPart()->itemType() == ModelPart::Part) {
+            QStringList values = PaletteItemBase::collectValues(family, prop, value);
+            if (values.count() == 0) {
+                ItemBase * itemBase = modelPart()->viewItem(ViewLayer::PCBView);
+                if (itemBase) {
+                    values << TranslatedPropertyNames.value("bottom") << TranslatedPropertyNames.value("top");
+                    if (itemBase->viewLayerPlacement() == ViewLayer::NewBottom) {
+                        value = values.at(0);
+                    }
+                    else {
+                        value = values.at(1);
+                    }
+                }
+            }
+            return values;
+        }
+    }
+
+   return PaletteItemBase::collectValues(family, prop, value);
+}
+
+
 void PaletteItem::openPinLabelDialog() {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView == NULL) {
@@ -694,7 +684,7 @@ QList<Connector *> PaletteItem::sortConnectors() {
 bool PaletteItem::changePinLabels(bool singleRow, bool sip) {
 	Q_UNUSED(singleRow);
 	Q_UNUSED(sip);
-	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return true;
+	if (m_viewID != ViewLayer::SchematicView) return true;
 
 	return false;
 }
@@ -716,12 +706,12 @@ QStringList PaletteItem::getPinLabels(bool & hasLocal) {
 }
 
 void PaletteItem::resetConnectors() {
-	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return;
+	if (m_viewID != ViewLayer::SchematicView) return;
 
 	QSizeF size = fsvgRenderer()->defaultSizeF();   // pixels
 	QRectF viewBox = fsvgRenderer()->viewBoxF();
 	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
-		SvgIdLayer * svgIdLayer = connectorItem->connector()->fullPinInfo(m_viewIdentifier, m_viewLayerID);
+		SvgIdLayer * svgIdLayer = connectorItem->connector()->fullPinInfo(m_viewID, m_viewLayerID);
 		if (svgIdLayer == NULL) continue;
 
 		QRectF bounds = fsvgRenderer()->boundsOnElement(svgIdLayer->m_svgId);
@@ -733,17 +723,17 @@ void PaletteItem::resetConnectors() {
 }
 
 
-void PaletteItem::resetConnectors(ItemBase * otherLayer,FSvgRenderer * otherLayerRenderer)
+void PaletteItem::resetConnectors(ItemBase * otherLayer, FSvgRenderer * otherLayerRenderer)
 {
 	// there's only one connector
 	foreach (Connector * connector, m_modelPart->connectors().values()) {
 		if (connector == NULL) continue;
 
-		connector->unprocess(m_viewIdentifier, m_viewLayerID);
-		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, m_viewLayerID);
+		connector->unprocess(m_viewID, m_viewLayerID);
+		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewID, m_viewLayerID);
 		if (svgIdLayer == NULL) continue;
 
-		bool result = fsvgRenderer()->setUpConnector(svgIdLayer, false);
+		bool result = fsvgRenderer()->setUpConnector(svgIdLayer, false, viewLayerPlacement());
 		if (!result) continue;
 
 		resetConnector(this, svgIdLayer);
@@ -753,11 +743,11 @@ void PaletteItem::resetConnectors(ItemBase * otherLayer,FSvgRenderer * otherLaye
 		foreach (Connector * connector, m_modelPart->connectors().values()) {
 			if (connector == NULL) continue;
 
-			connector->unprocess(m_viewIdentifier, otherLayer->viewLayerID());
-			SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, otherLayer->viewLayerID());
+			connector->unprocess(m_viewID, otherLayer->viewLayerID());
+			SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewID, otherLayer->viewLayerID());
 			if (svgIdLayer == NULL) continue;
 
-			bool result = otherLayerRenderer->setUpConnector(svgIdLayer, false);
+			bool result = otherLayerRenderer->setUpConnector(svgIdLayer, false, viewLayerPlacement());
 			if (!result) continue;
 
 			resetConnector(otherLayer, svgIdLayer);
@@ -769,13 +759,15 @@ void PaletteItem::resetConnectors(ItemBase * otherLayer,FSvgRenderer * otherLaye
 
 void PaletteItem::resetConnector(ItemBase * itemBase, SvgIdLayer * svgIdLayer) 
 {
+    QList<ConnectorItem *> already;
 	foreach (ConnectorItem * connectorItem, itemBase->cachedConnectorItems()) {
-		//DebugDialog::debug(QString("via set rect %1").arg(itemBase->viewIdentifier()), svgIdLayer->m_rect);
+		//DebugDialog::debug(QString("via set rect %1").arg(itemBase->viewID()), svgIdLayer->m_rect);
 
-		connectorItem->setRect(svgIdLayer->m_rect);
-		connectorItem->setTerminalPoint(svgIdLayer->m_point);
+		connectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
+		connectorItem->setTerminalPoint(svgIdLayer->point(viewLayerPlacement()));
 		connectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
-		connectorItem->attachedMoved();
+        connectorItem->setIsPath(svgIdLayer->m_path);
+		connectorItem->attachedMoved(false, already);
 		break;
 	}
 }
@@ -916,6 +908,8 @@ QWidget * PaletteItem::createHoleSettings(QWidget * parent, HoleSettings & holeS
 	vBoxLayout->addWidget(holeSettings.sizesComboBox);
 
     if (advanced) {
+        vBoxLayout->addSpacing(4);
+
         QFrame * hFrame = new QFrame(frame);
         QHBoxLayout * hLayout = new QHBoxLayout(hFrame);
 	    hLayout->setMargin(0);
@@ -1117,8 +1111,8 @@ QStringList PaletteItem::getSizes(QString & holeSize, HoleSettings & holeSetting
 }
 
 void PaletteItem::changeHoleSize(const QString & newSize) {
-    if (this->m_viewIdentifier != ViewIdentifierClass::PCBView) {
-        PaletteItem * paletteItem = qobject_cast<PaletteItem *>(modelPart()->viewItem(ViewIdentifierClass::PCBView));
+    if (this->m_viewID != ViewLayer::PCBView) {
+        PaletteItem * paletteItem = qobject_cast<PaletteItem *>(modelPart()->viewItem(ViewLayer::PCBView));
         if (paletteItem == NULL) return;
 
         paletteItem->changeHoleSize(newSize);
@@ -1159,15 +1153,30 @@ void PaletteItem::changeHoleSize(const QString & newSize) {
 
 QString PaletteItem::hackFzpHoleSize(const QString & fzp, const QString & moduleid, int hsix) 
 {
+    QString errorStr;
+    int errorLine;
+    int errorColumn;
     QDomDocument document;
-    document.setContent(fzp);
+    bool result = document.setContent(fzp, &errorStr, &errorLine, &errorColumn);
+    if (!result) {
+        DebugDialog::debug(QString("bad fzp in %1:%2").arg(moduleid).arg(fzp));
+    }
     QStringList strings = moduleid.mid(hsix).split("_");
     return hackFzpHoleSize(document, moduleid, "pcb/" + moduleid + ".svg", strings.at(2) + "," + strings.at(3)); 
 }
 
 
 QString PaletteItem::hackFzpHoleSize(const QString & newModuleID, const QString & pcbFilename, const QString & newSize) {
-    QDomDocument document = modelPart()->domDocument()->cloneNode(true).toDocument();
+    QFile file(modelPart()->path());
+    QString errorStr;
+    int errorLine;
+    int errorColumn;
+    QDomDocument document;
+    bool result = document.setContent(&file, &errorStr, &errorLine, &errorColumn);    
+    if (!result) {
+        DebugDialog::debug(QString("bad doc fzp in %1:%2 %3 %4").arg(newModuleID).arg(errorStr).arg(errorLine).arg(errorColumn));
+    }
+
     return hackFzpHoleSize(document, newModuleID, pcbFilename, newSize);
 }
 
@@ -1190,7 +1199,7 @@ QString PaletteItem::hackFzpHoleSize(QDomDocument & document, const QString & ne
         QString name = prop.attribute("name");
         if (name.compare("hole size", Qt::CaseInsensitive) == 0) {
             gotProp = true;
-            TextUtils::replaceChildText(document, prop, newSize);
+            TextUtils::replaceChildText(prop, newSize);
             break;
         }
 
@@ -1311,22 +1320,31 @@ void PaletteItem::generateSwap(const QString & text, GenModuleID genModuleID, Ge
         QDomDocument doc;
         doc.setContent(fzp);
 
-        QString bbName = LayerAttributes::getSvgElementLayers(&doc, ViewIdentifierClass::BreadboardView).attribute("image");
-        QString schName = LayerAttributes::getSvgElementLayers(&doc, ViewIdentifierClass::SchematicView).attribute("image");
-        QString pcbName = LayerAttributes::getSvgElementLayers(&doc, ViewIdentifierClass::PCBView).attribute("image");
+        QHash<QString, QString> viewNames;
 
-        if (!PartFactory::svgFileExists(bbName, path)) {
-            QString svg = makeBreadboardSvg(bbName);
+        QDomElement root = doc.documentElement();
+	    QDomElement views = root.firstChildElement("views");
+        QDomElement view = views.firstChildElement();
+	    while (!view.isNull()) {
+            viewNames.insert(view.tagName(), view.attribute("image", ""));
+            view = view.nextSiblingElement();
+        }
+
+        QString name = viewNames.value("breadboardView", "");
+        if (!PartFactory::svgFileExists(name, path)) {
+            QString svg = makeBreadboardSvg(name);
 	        TextUtils::writeUtf8(path, svg);
         }
 
-        if (!PartFactory::svgFileExists(schName, path)) {
-            QString svg = makeSchematicSvg(schName);
+        name = viewNames.value("schematicView", "");
+        if (!PartFactory::svgFileExists(name, path)) {
+            QString svg = makeSchematicSvg(name);
 	        TextUtils::writeUtf8(path, svg);
         }
 
-        if (!PartFactory::svgFileExists(pcbName, path)) {
-            QString svg = makePcbSvg(pcbName);
+        name = viewNames.value("pcbView", "");
+        if (!PartFactory::svgFileExists(name, path)) {
+            QString svg = makePcbSvg(name);
 	        TextUtils::writeUtf8(path, svg);
         } 
     }
@@ -1404,42 +1422,66 @@ bool PaletteItem::changeDiameter(HoleSettings & holeSettings, QObject * sender)
 	return (newValue != oldValue);
 }
 
-void PaletteItem::makeLocalMods(QByteArray & svg, const QString & filename) {
-    // at the moment, this is only for chip labels and pin labels for saved-as-new-part parts (i.e. that are no longer MysteryParts)
-    switch (m_viewIdentifier) {
-        case ViewIdentifierClass::IconView:
-        case ViewIdentifierClass::PCBView:
-            return;
+bool PaletteItem::makeLocalModifications(QByteArray & svg, const QString & filename) {
+    // a bottleneck for modifying part svg xml at setupImage time
+
+    // for saved-as-new-part parts (i.e. that are no longer MysteryParts) that still have a chip-label or custom pin names
+    // also handles adding a title if there is a label id in the 
+    switch (m_viewID) {
+        case ViewLayer::PCBView:
+            return false;
             
         default:
-            if (itemType() != ModelPart::Part) return;
+            if (itemType() != ModelPart::Part) return false;
+            if (filename.startsWith("icon")) return false;
 
-            if (filename.contains("mystery") || filename.contains("sip") || filename.contains("dip")) {
-                break;
-            }
-
-            return;
+            break;
     }
 
-
+    bool gotChipLabel = false;
     QString chipLabel = modelPart()->properties().value("chip label", ""); 
     if (!chipLabel.isEmpty()) {
         svg = TextUtils::replaceTextElement(svg, "label", chipLabel);
+        gotChipLabel = true;
     }
 
-    if (m_viewIdentifier == ViewIdentifierClass::SchematicView) {
+    bool modified = false;
+    if (m_viewID == ViewLayer::SchematicView) {
         QString value = modelPart()->properties().value("editable pin labels", "");
         if (value.compare("true") == 0) {
             bool hasLayout, sip;
-            QStringList labels = sipOrDipOr(hasLayout, sip);
+            QStringList labels = sipOrDipOrLabels(hasLayout, sip);
             if (labels.count() > 0) {
-                svg = PartFactory::makeSipOrDipOr(labels, hasLayout, sip).toUtf8();
+                svg = PartFactory::makeSchematicSipOrDipOr(labels, hasLayout, sip).toUtf8();
+                modified = true;
+            }
+        }
+        gotChipLabel = true;
+    }
+
+    if (gotChipLabel) return modified;
+
+    int rix = svg.indexOf("label");
+    if (rix >= 0) {
+        rix = qMax(0, rix - 4);     // backup for id="
+        int ix = svg.indexOf("id=\"label\"", rix);
+        if (ix < 0) {
+            ix = svg.indexOf("id='label'", rix);
+        }
+        if (ix >= 0) {
+            int tix = svg.lastIndexOf("<text", ix);
+            int lix = svg.lastIndexOf("<", ix);
+            if (tix == lix) {
+                svg = TextUtils::replaceTextElement(svg, "label", modelPart()->title());
+                modified = true;
             }
         }
     }
+
+    return modified;
 }
 
-QStringList PaletteItem::sipOrDipOr(bool & hasLayout, bool & sip) {
+QStringList PaletteItem::sipOrDipOrLabels(bool & hasLayout, bool & sip) {
 	hasLayout = sip = false;
     bool hasLocal = false;
 	QStringList labels = getPinLabels(hasLocal);
@@ -1464,3 +1506,55 @@ QStringList PaletteItem::sipOrDipOr(bool & hasLayout, bool & sip) {
     return labels;
 }
 
+void PaletteItem::resetLayerKin(const QString & svg) {
+    QString svgNoText = SvgFileSplitter::hideText3(svg);
+
+	resetRenderer(svgNoText);
+
+    foreach (ItemBase * lkpi, layerKin()) {
+        if (lkpi->viewLayerID() == ViewLayer::SchematicText) {
+            bool hasText;
+	        QString svgText = SvgFileSplitter::showText3(svg, hasText);
+	        lkpi->resetRenderer(svgText);
+            lkpi->setProperty("textSvg", svgText);
+            qobject_cast<SchematicTextLayerKinPaletteItem *>(lkpi)->clearTextThings();
+            break;
+        }
+    }
+}
+
+QTransform PaletteItem::untransform() {
+    QTransform chiefTransform = this->transform();
+    chiefTransform.setMatrix(chiefTransform.m11(), chiefTransform.m12(), chiefTransform.m13(), chiefTransform.m21(), chiefTransform.m22(), chiefTransform.m23(), 0, 0, chiefTransform.m33()); 
+    bool identity = chiefTransform.isIdentity();
+    if (!identity) {    
+        QTransform invert = chiefTransform.inverted();
+        transformItem(invert, false);
+        foreach (ItemBase * lkpi, layerKin()) {    
+            lkpi->transformItem(invert, false);
+        }
+    }
+
+    return chiefTransform;
+}
+
+void PaletteItem::retransform(const QTransform & chiefTransform) {
+    if (!chiefTransform.isIdentity()) {
+        transformItem(chiefTransform, false);
+        foreach (ItemBase * lkpi, layerKin()) {    
+            lkpi->transformItem(chiefTransform, false);
+        }
+    }
+}
+
+void PaletteItem::initLayerAttributes(LayerAttributes & layerAttributes, ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement viewLayerPlacement, bool doConnectors, bool doCreateShape) {
+    layerAttributes.viewID = viewID;
+    layerAttributes.viewLayerID = viewLayerID;
+    layerAttributes.viewLayerPlacement = viewLayerPlacement;
+    layerAttributes.doConnectors = doConnectors;
+    layerAttributes.createShape = doCreateShape;
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		layerAttributes.orientation = infoGraphicsView->smdOrientation();
+	}
+}
