@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,7 +35,6 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 #include "../connectors/svgidlayer.h"
 #include "wire.h"
 #include "partlabel.h"
-#include "../utils/focusoutcombobox.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/cursormaster.h"
@@ -55,10 +54,10 @@ static QPointF RotationAxis;
 static QTransform OriginalTransform;
 
 
-PaletteItemBase::PaletteItemBase(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu ) :
-	ItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
+PaletteItemBase::PaletteItemBase(ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu ) :
+	ItemBase(modelPart, viewID, viewGeometry, id, itemMenu)
 {
-	m_inRotation = m_syncSelected = false;
+	m_syncSelected = false;
 	m_offset.setX(0);
 	m_offset.setY(0);
  	m_blockItemSelectedChange = false;
@@ -115,6 +114,8 @@ QPainterPath PaletteItemBase::shape() const
 {
 	// TODO: figure out real shape of svg
     QPainterPath path;
+    if (m_squashShape) return path;
+
     path.addRect(0, 0, m_size.width(), m_size.height());
 
 	if (!hasRubberBandLeg()) return path;
@@ -141,7 +142,8 @@ bool PaletteItemBase::itemMoved() {
 
 void PaletteItemBase::moveItem(ViewGeometry & viewGeometry) {
 	this->setPos(viewGeometry.loc());
-	updateConnections();
+    QList<ConnectorItem *> already;
+	updateConnections(false, already);
 }
 
 void PaletteItemBase::saveInstanceLocation(QXmlStreamWriter & streamWriter)
@@ -175,14 +177,14 @@ bool PaletteItemBase::syncSelected() {
 
 void PaletteItemBase::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-	if (m_hidden) return;
+	if (m_hidden || m_layerHidden) return;
 
 	ItemBase::paint(painter, option, widget);
 }
 
 void PaletteItemBase::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-	if (m_hidden) return;
+	if (m_hidden || m_layerHidden) return;
 
 	if (!hasRubberBandLeg()) {
 		ItemBase::paintHover(painter, option, widget);
@@ -196,7 +198,7 @@ void PaletteItemBase::paintHover(QPainter *painter, const QStyleOptionGraphicsIt
 
 void PaletteItemBase::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-	if (m_hidden) return;
+	if (m_hidden || m_layerHidden) return;
 
 	if (!hasRubberBandLeg()) {
 		ItemBase::paintSelected(painter, option, widget);
@@ -217,22 +219,22 @@ void PaletteItemBase::mousePressConnectorEvent(ConnectorItem * connectorItem, QG
 bool PaletteItemBase::acceptsMousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent * event) {
 	Q_UNUSED(event);
 	
-	//if (m_viewIdentifier != ViewIdentifierClass::PCBView) {
+	//if (m_viewID != ViewLayer::PCBView) {
 		return true;
 	//}
 }
 
 
-void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
+bool PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
 {
 	Q_UNUSED(originalItem);
 
-	m_inRotation = false;
+	setInRotation(false);
 
 	QPointF corner;
 	if (freeRotationAllowed(event->modifiers()) && inRotationLocation(event->scenePos(), event->modifiers(), corner)) {
 		this->saveGeometry();
-		m_inRotation = true;
+		setInRotation(true);
 		RotationCenter = mapToScene(this->boundingRectWithoutLegs().center());
 		RotationAxis = event->scenePos(); 
 		OriginalTransform = this->transform();
@@ -248,7 +250,7 @@ void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 		this->debugInfo("in rotation");
 		InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 		if (infoGraphicsView) infoGraphicsView->setAnyInRotation();
-		return;
+		return false;
 	}
 
 	ItemBase::mousePressEvent(event);
@@ -257,11 +259,21 @@ void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 			connectorItem->setOverConnectorItem(NULL);
 		}
 	}
+
+    return false;
+}
+
+void PaletteItemBase::mouseMoveEvent(PaletteItemBase *, QGraphicsSceneMouseEvent *)
+{
+}
+
+void PaletteItemBase::mouseReleaseEvent(PaletteItemBase *, QGraphicsSceneMouseEvent *)
+{
 }
 
 void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
-	if (!m_inRotation) {
+	if (!inRotation()) {
 		ItemBase::mouseMoveEvent(event);
 		return;
 	}
@@ -274,9 +286,9 @@ void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
 	double deltaAngle = (a2 - a1) * 180 / M_PI;
 	//DebugDialog::debug(QString("original:%1 delta:%2").arg(originalAngle).arg(deltaAngle));
-	switch (m_viewIdentifier) {
-		case ViewIdentifierClass::BreadboardView:
-		case ViewIdentifierClass::PCBView:
+	switch (m_viewID) {
+		case ViewLayer::BreadboardView:
+		case ViewLayer::PCBView:
 			{
 				double nearest = qRound((originalAngle + deltaAngle) / 45) * 45;
 				if (qAbs(originalAngle + deltaAngle - nearest) < 6) {
@@ -285,7 +297,7 @@ void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 				}
 			}
 			break;
-		case ViewIdentifierClass::SchematicView:
+		case ViewLayer::SchematicView:
 			{
 				double nearest = qRound((originalAngle + deltaAngle) / 90) * 90;
 				deltaAngle = nearest - originalAngle;
@@ -303,24 +315,27 @@ void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 	}
 
 	//DebugDialog::debug(QString("rotating item %1 da:%2 oa:%3 %4").arg(QTime::currentTime().toString("HH:mm:ss.zzz")).arg(deltaAngle).arg(originalAngle).arg((long) this, 0, 16));
-	chief->rotateItem(deltaAngle);
+	chief->rotateItem(deltaAngle, true);
+
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(chief);
+    if (infoGraphicsView) infoGraphicsView->updateRotation(chief);
 }
 
 void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-	if (!m_inRotation) {
+	if (!inRotation()) {
 		ItemBase::mouseReleaseEvent(event);
 		return;
 	}
 
-	m_inRotation = false;
+	setInRotation(false);
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView) {
 		// TODO: doesn't account for scaling
 		// see: http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
 		double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
 		double currentAngle = atan2(transform().m12(), transform().m11()) * 180 / M_PI;
-		rotateItem(originalAngle - currentAngle);		// put it back; undo command will redo it
+		this->layerKinChief()->rotateItem(originalAngle - currentAngle, true);		// put it back; undo command will redo it
 		saveGeometry();
 		infoGraphicsView->triggerRotate(this->layerKinChief(), currentAngle - originalAngle);
 	}
@@ -378,34 +393,29 @@ void PaletteItemBase::collectWireConnectees(QSet<Wire *> & wires) {
 		}
 	}
 }
-bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
-{
-    return setUpImage(modelPart, NULL, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
-}
 
-
-bool PaletteItemBase::setUpImage(ModelPart * modelPart, QDomDocument * domDocument, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
+bool PaletteItemBase::setUpImage(ModelPart * modelPart, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
-	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, domDocument, viewIdentifier, viewLayerID, viewLayerSpec, layerAttributes, error);
+	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, layerAttributes);
 	if (renderer == NULL) {
 		return false;
 	}
 
-	m_canFlipVertical = layerAttributes.canFlipVertical();
-	m_canFlipHorizontal = layerAttributes.canFlipHorizontal();
+	m_canFlipVertical = modelPart->canFlipVertical(layerAttributes.viewID);
+	m_canFlipHorizontal = modelPart->canFlipHorizontal(layerAttributes.viewID);
 	m_filename = layerAttributes.filename();
 	//DebugDialog::debug(QString("filename %1").arg(m_filename) );
-	setSticky(layerAttributes.isSticky());
-	QString elementID = layerAttributes.layerName();
+	setSticky(modelPart->anySticky(layerAttributes.viewID));
+	QString elementID = ViewLayer::viewLayerXmlNameFromID(layerAttributes.viewLayerID);
 	setViewLayerID(elementID, viewLayers);
 
-	//DebugDialog::debug(QString("setting layer %1 view:%2 z:%3").arg(modelPart->title()).arg(viewIdentifier).arg(this->z()) );
+	//DebugDialog::debug(QString("setting layer %1 view:%2 z:%3").arg(modelPart->title()).arg(viewID).arg(this->z()) );
 	this->setZValue(this->z());
 	this->setSharedRendererEx(renderer);
 
 	m_svg = true;
 
-	if (doConnectors) {
+	if (layerAttributes.doConnectors) {
 		setUpConnectors(renderer, modelPart->ignoreTerminalPoints());
 	}
 
@@ -420,10 +430,10 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, QDomDocument * domDocume
 void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTerminalPoints) {
 	clearConnectorItemCache();
 
-	if (m_viewIdentifier == ViewIdentifierClass::PCBView && ViewLayer::isNonCopperLayer(m_viewLayerID)) {
+	if (m_viewID == ViewLayer::PCBView && ViewLayer::isNonCopperLayer(m_viewLayerID)) {
 		//DebugDialog::debug(QString("skip connectors: %1 vid:%2 vlid:%3")
 		//				   .arg(this->title())
-		//				   .arg(m_viewIdentifier) 
+		//				   .arg(m_viewID) 
 		//				   .arg(m_viewLayerID)
 		//	);
 		// don't waste time
@@ -435,20 +445,30 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 
 		//DebugDialog::debug(QString("id:%1 vid:%2 vlid:%3")
 		//				   .arg(connector->connectorSharedID())
-		//				   .arg(m_viewIdentifier) 
+		//				   .arg(m_viewID) 
 		//				   .arg(m_viewLayerID)
 		//	);
 
 
-		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, m_viewLayerID);
+		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewID, m_viewLayerID);
 		if (svgIdLayer == NULL) {
-			DebugDialog::debug(QString("svgidlayer fail %1 vid:%2 vlid:%3").arg(connector->connectorSharedID(), m_viewIdentifier, m_viewLayerID));
+			DebugDialog::debug(QString("svgidlayer fail %1 vid:%2 vlid:%3 %4")
+                .arg(connector->connectorSharedID())
+                .arg(m_viewID)
+                .arg(m_viewLayerID)
+                .arg(m_modelPart->path())
+                );
 			continue;
 		}
 
-		bool result = renderer->setUpConnector(svgIdLayer, ignoreTerminalPoints);
+		bool result = renderer->setUpConnector(svgIdLayer, ignoreTerminalPoints, viewLayerPlacement());
 		if (!result) {
-			DebugDialog::debug(QString("setup connector fail %1 vid:%2 vlid:%3").arg(connector->connectorSharedID(), m_viewIdentifier, m_viewLayerID));
+			DebugDialog::debug(QString("setup connector fail %1 vid:%2 vlid:%3 %4")
+                .arg(connector->connectorSharedID())
+                .arg(m_viewID)
+                .arg(m_viewLayerID)
+                .arg(m_modelPart->path())
+                );
 			continue;
 		}
 
@@ -456,9 +476,10 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 		ConnectorItem * connectorItem = newConnectorItem(connector);
 
 		connectorItem->setHybrid(svgIdLayer->m_hybrid);
-		connectorItem->setRect(svgIdLayer->m_rect);
-		connectorItem->setTerminalPoint(svgIdLayer->m_point);
+		connectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
+		connectorItem->setTerminalPoint(svgIdLayer->point(viewLayerPlacement()));
 		connectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
+        connectorItem->setIsPath(svgIdLayer->m_path);
 		if (!svgIdLayer->m_legId.isEmpty()) {
 			m_hasRubberBandLeg = true;
 			connectorItem->setRubberBandLeg(QColor(svgIdLayer->m_legColor), svgIdLayer->m_legStrokeWidth, svgIdLayer->m_legLine);
@@ -468,7 +489,7 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 
 	}
 
-	foreach (SvgIdLayer * svgIdLayer, renderer->setUpNonConnectors()) {
+	foreach (SvgIdLayer * svgIdLayer, renderer->setUpNonConnectors(viewLayerPlacement())) {
 		if (svgIdLayer == NULL) continue;
 
 		NonConnectorItem * nonConnectorItem = new NonConnectorItem(this);
@@ -477,19 +498,20 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 			//.arg(ViewLayer::viewLayerNameFromID(m_viewLayerID))
 			//.arg(this->zValue()) );
 
-		nonConnectorItem->setRect(svgIdLayer->m_rect);
+		nonConnectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
 		nonConnectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
+        nonConnectorItem->setIsPath(svgIdLayer->m_path);
 		//DebugDialog::debug(QString("terminal point %1 %2").arg(terminalPoint.x()).arg(terminalPoint.y()) );
 
 		delete svgIdLayer;
 	}
 }
 
-void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
+void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to,  QList<ConnectorItem *> & already) {
 	if (from->connectorType() != Connector::Female) return;
 
 	// female connectors really only operate in breadboard view
-	if (m_viewIdentifier != ViewIdentifierClass::BreadboardView) return;
+	if (m_viewID != ViewLayer::BreadboardView) return;
 
 	QTransform t = this->transform();
 	if (t.isRotating()) {
@@ -507,16 +529,11 @@ void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
 	if (fromTerminalPoint == toTerminalPoint) return;
 
 	this->setPos(this->pos() + fromTerminalPoint - toTerminalPoint);
-	updateConnections();
+	updateConnections(false, already);
 }
 
 void PaletteItemBase::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
-	if (lowerConnectorLayerVisible(this)) {
-		DebugDialog::debug("PaletteItemBase::hoverEnterEvent isn't obsolete");
-		event->ignore();
-		return;
-	}
-
+	// debugInfo("hover enter paletteitembase");
 	ItemBase::hoverEnterEvent(event);
 	if (event->isAccepted()) {
 		if (hasRubberBandLeg()) {
@@ -569,39 +586,38 @@ void PaletteItemBase::cursorKeyEvent(Qt::KeyboardModifiers modifiers)
 }
 
 void PaletteItemBase::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
-	if (lowerConnectorLayerVisible(this)) {
-		// TODO: I think this is obsolete
-		DebugDialog::debug("PaletteItemBase::contextMenuEvent isn't obsolete");
-		event->ignore();
-		return;
-	}
-
 	ItemBase::contextMenuEvent(event);
 }
 
 
 LayerKinPaletteItem *PaletteItemBase::newLayerKinPaletteItem(PaletteItemBase * chief, ModelPart * modelPart, 
-															 ViewIdentifierClass::ViewIdentifier viewIdentifier,
 															 const ViewGeometry & viewGeometry, long id,
-															 ViewLayer::ViewLayerID viewLayerID, 
-															 ViewLayer::ViewLayerSpec viewLayerSpec, 
-															 QMenu* itemMenu, const LayerHash & viewLayers)
+															 QMenu* itemMenu, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
-	LayerKinPaletteItem *lk = new
-                LayerKinPaletteItem(chief, modelPart, viewIdentifier, viewGeometry, id, itemMenu);
-	lk->init(viewLayerID, viewLayerSpec, viewLayers);
+	LayerKinPaletteItem *lk = NULL;
+    if (layerAttributes.viewLayerID == ViewLayer::SchematicText) {
+        lk = new SchematicTextLayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
+    }
+    else {
+        lk = new LayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
+    }
+	lk->init(layerAttributes, viewLayers);
 	return lk;
 }
 
-QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
+QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
 {
 	QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
 	QString path = filename();
 
+    Qt::Orientations orientation = Qt::Vertical;
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+    if (infoGraphicsView) {
+        orientation = infoGraphicsView->smdOrientation();
+    }
+
 	QDomDocument flipDoc;
-	if (!getFlipDoc(modelPart(), path, viewLayerID, m_viewLayerSpec, flipDoc)) {
-		fixCopper1(modelPart(), path, viewLayerID, m_viewLayerSpec, flipDoc);
-	}
+    getFlipDoc(modelPart(), path, viewLayerID, m_viewLayerPlacement, flipDoc, orientation);
 	
 	//DebugDialog::debug(QString("path: %1").arg(path));
 
@@ -627,11 +643,11 @@ QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<Q
 		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
 			if (!connectorItem->hasRubberBandLeg()) continue;
 
-			splitter.gReplace(connectorItem->legID(m_viewIdentifier, m_viewLayerID));
+			splitter.gReplace(connectorItem->legID(m_viewID, m_viewLayerID));
 		}
 	}
 
-	result = splitter.normalize(dpi, xmlName, blackOnly);
+	result = splitter.normalize(dpi, xmlName, blackOnly, factor);
 	if (!result) {
 		return "";
 	}
@@ -646,7 +662,7 @@ bool PaletteItemBase::canEditPart() {
 	return ItemBase::canEditPart();
 }
 
-bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare(ModelPartShared::PartNumberPropertyName, Qt::CaseInsensitive) == 0) {
 		returnProp = TranslatedPropertyNames.value(prop);
@@ -662,7 +678,7 @@ bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family,
 		return true;
 	}
 
-	return ItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return ItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 void PaletteItemBase::setProp(const QString & prop, const QString & value) 
@@ -712,10 +728,16 @@ bool PaletteItemBase::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
 	return rotation45Allowed();
 }
 
+bool PaletteItemBase::freeRotationAllowed() {
+    if (viewID() == ViewLayer::SchematicView) return false;
+
+	return rotation45Allowed();
+}
+
 bool PaletteItemBase::inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers modifiers, QPointF & returnPoint)
 {
 	if (!freeRotationAllowed(modifiers)) return false;
-	if (m_viewIdentifier == ViewIdentifierClass::SchematicView) return false;
+	if (m_viewID == ViewLayer::SchematicView) return false;
 
 	QRectF r = this->boundingRectWithoutLegs();
 	QPolygonF polygon;
@@ -734,10 +756,6 @@ bool PaletteItemBase::inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers
 	return false;
 }
 
-bool PaletteItemBase::inRotation() {
-	return m_inRotation;
-}
-
 void PaletteItemBase::checkFreeRotation(Qt::KeyboardModifiers modifiers, QPointF scenePos)
 {
 	if (!freeRotationAllowed(modifiers)) return;
@@ -753,6 +771,22 @@ void PaletteItemBase::checkFreeRotation(Qt::KeyboardModifiers modifiers, QPointF
 	}
 }
 
+QString PaletteItemBase::normalizeSvg(QString & svg, ViewLayer::ViewLayerID viewLayerID, bool blackOnly, double dpi, double & factor)
+{
+	QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
+	SvgFileSplitter splitter;
+	bool result = splitter.splitString(svg, xmlName);
+	if (!result) {
+		return "";
+	}
+	result = splitter.normalize(dpi, xmlName, blackOnly, factor);
+	if (!result) {
+		return "";
+	}
+	return splitter.elementString(xmlName);
+}
+
+
 /*
 
 void PaletteItemBase::setPos(const QPointF & pos) {
