@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -42,7 +42,7 @@ class PaletteItemBase : public ItemBase, public CursorKeyListener
 	Q_OBJECT
 
 public:
-	PaletteItemBase(ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);
+	PaletteItemBase(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu);
 
 	void saveGeometry();
 	bool itemMoved();
@@ -55,52 +55,52 @@ public:
  	QPointF syncMoved();
 	void mousePressConnectorEvent(class ConnectorItem *, QGraphicsSceneMouseEvent *);
 	bool acceptsMousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
- 	virtual bool setUpImage(ModelPart* modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, bool doConnectors, LayerAttributes &, QString & error);
- 	virtual bool setUpImage(ModelPart* modelPart, QDomDocument *, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, bool doConnectors, LayerAttributes &, QString & error);
-	void connectedMoved(ConnectorItem * from, ConnectorItem * to);
+ 	virtual bool setUpImage(ModelPart* modelPart, const LayerHash & viewLayers, LayerAttributes &);
+	void connectedMoved(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & already);
 	bool collectFemaleConnectees(QSet<ItemBase *> & items);
 	void collectWireConnectees(QSet<class Wire *> & wires);
-	virtual void mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event);
+	virtual bool mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event);
+	virtual void mouseMoveEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event);
+	virtual void mouseReleaseEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event);
 	void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
 	void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
+	void hoverEnterEvent(QGraphicsSceneHoverEvent * event );
+	void hoverLeaveEvent(QGraphicsSceneHoverEvent * event );
+	void hoverMoveEvent(QGraphicsSceneHoverEvent * event );
+	void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
 	//bool isBuriedConnectorHit(QGraphicsSceneMouseEvent *event);
 	QRectF boundingRect() const;
 	QRectF boundingRectWithoutLegs() const;
-	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi);
+	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
 	virtual bool canEditPart();
 	QPainterPath shape() const;
 	QPainterPath hoverShape() const;
 	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
 	void paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
 	void paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
-	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget);
+	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
 	void setProp(const QString & prop, const QString & value);
 	const QCursor * getCursor(Qt::KeyboardModifiers);
 	void cursorKeyEvent(Qt::KeyboardModifiers modifiers);
-	bool inRotation();
 
 	/*
 	// for debugging
 	void setPos(const QPointF & pos);
 	void setPos(double x, double y);
 	*/
+protected:
+    static QString normalizeSvg(QString & svg, ViewLayer::ViewLayerID viewLayerID, bool blackOnly, double dpi, double & factor);
 
 protected:
     void setUpConnectors(FSvgRenderer *, bool ignoreTerminalPoints);
 	void findConnectorsUnder();
-	void hoverEnterEvent(QGraphicsSceneHoverEvent * event );
-	void hoverLeaveEvent(QGraphicsSceneHoverEvent * event );
-	void hoverMoveEvent(QGraphicsSceneHoverEvent * event );
-	void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
 	virtual bool canFindConnectorsUnder();
 	bool inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers modifiers, QPointF & returnPoint);
 	virtual bool freeRotationAllowed(Qt::KeyboardModifiers modifiers);
+	bool freeRotationAllowed();
 	void checkFreeRotation(Qt::KeyboardModifiers modifiers, QPointF scenePos);
 
-	virtual LayerKinPaletteItem * newLayerKinPaletteItem(
-		PaletteItemBase * chief, ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,
-		const ViewGeometry & viewGeometry, long id,ViewLayer::ViewLayerID, ViewLayer::ViewLayerSpec, QMenu* itemMenu, const LayerHash & viewLayers
-	);
+	virtual LayerKinPaletteItem * newLayerKinPaletteItem(PaletteItemBase * chief, ModelPart *, const ViewGeometry &, long id, QMenu* itemMenu, const LayerHash &, LayerAttributes &);
 
 protected slots:
 	void partPropertyEntry();
@@ -112,7 +112,6 @@ protected:
  	bool m_syncSelected;
  	QPointF m_syncMoved;
  	bool m_svg;
-	bool m_inRotation;
 };
 
 
