@@ -18,13 +18,14 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6962 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-14 00:08:36 +0200 (So, 14. Apr 2013) $
 
 ********************************************************************/
 
 #include "partfactory.h"
+#include "../debugdialog.h"
 #include "../viewgeometry.h"
 #include "../model/modelpart.h"
 #include "../model/modelbase.h"
@@ -54,141 +55,249 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 #include "breadboard.h"
 #include "stripboard.h"
 #include "led.h"
+#include "schematicsubpart.h"
+#include "layerkinpaletteitem.h"
 #include "../utils/folderutils.h"
 #include "../utils/lockmanager.h"
 #include "../utils/textutils.h"
+#include "../utils/graphicsutils.h"
+
+#include <qmath.h>
 
 static QString PartFactoryFolderPath;
 static QHash<QString, LockedFile *> LockedFiles;
+static QString SvgFilesDir = "svg";
+static QHash<QString, QPointF> SubpartOffsets;
 
-ItemBase * PartFactory::createPart( ModelPart * modelPart, ViewLayer::ViewLayerSpec viewLayerSpec, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, QMenu * wireMenu, bool doLabel)
+ItemBase * PartFactory::createPart( ModelPart * modelPart, ViewLayer::ViewLayerPlacement viewLayerPlacement, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, QMenu * wireMenu, bool doLabel)
 {
 	modelPart->setModelIndexFromMultiplied(id);			// make sure the model index is synched with the id; this is not always the case when parts are first created.
-	ItemBase * itemBase = createPartAux(modelPart, viewIdentifier, viewGeometry, id, itemMenu, wireMenu, doLabel);
+	ItemBase * itemBase = createPartAux(modelPart, viewID, viewGeometry, id, itemMenu, wireMenu, doLabel);
 	if (itemBase) {
-		itemBase->setViewLayerSpec(viewLayerSpec);
+		itemBase->setViewLayerPlacement(viewLayerPlacement);
 	}
 	return itemBase;
 }
 
-ItemBase * PartFactory::createPartAux( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, QMenu * wireMenu, bool doLabel)
+ItemBase * PartFactory::createPartAux( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, QMenu * wireMenu, bool doLabel)
 {
 	switch (modelPart->itemType()) {
 		case ModelPart::Wire:
 		{
 			bool ratsnest = viewGeometry.getRatsnest();
 			if (ratsnest) {
-				return new VirtualWire(modelPart, viewIdentifier, viewGeometry, id, wireMenu);		
+				return new VirtualWire(modelPart, viewID, viewGeometry, id, wireMenu);		
 			}
 			if (viewGeometry.getAnyTrace()) {
-				TraceWire * traceWire = new TraceWire(modelPart, viewIdentifier, viewGeometry, id, wireMenu);
+				TraceWire * traceWire = new TraceWire(modelPart, viewID, viewGeometry, id, wireMenu);
 				return traceWire;
 			}
-			return new Wire(modelPart, viewIdentifier, viewGeometry, id, wireMenu, false);
+			return new Wire(modelPart, viewID, viewGeometry, id, wireMenu, false);
 
 		}
 		case ModelPart::Note:
-			return new Note(modelPart, viewIdentifier, viewGeometry, id, NULL);
+			return new Note(modelPart, viewID, viewGeometry, id, NULL);
 		case ModelPart::CopperFill:
-			return new GroundPlane(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new GroundPlane(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Jumper:
-			return new JumperItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new JumperItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::ResizableBoard:
-			return new ResizableBoard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new ResizableBoard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Board:
-			return new Board(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new Board(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Logo:
 			if (modelPart->moduleID().startsWith("copper", Qt::CaseInsensitive)) {
-				return new CopperLogoItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+				return new CopperLogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 			}
-            else if (modelPart->moduleID().contains("board", Qt::CaseInsensitive)) {
-                return new BoardLogoItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+            else if (modelPart->moduleID().startsWith("Schematic", Qt::CaseInsensitive)) {
+                return new SchematicLogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
             }
-			return new LogoItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+            else if (modelPart->moduleID().startsWith("Breadboard", Qt::CaseInsensitive)) {
+                return new BreadboardLogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
+            }
+            // make sure "board" match comes after "breadboard" match
+            else if (modelPart->moduleID().contains("board", Qt::CaseInsensitive)) {
+                return new BoardLogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
+            }
+			return new LogoItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Ruler:
-			return new Ruler(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new Ruler(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Symbol:
-			return new SymbolPaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new SymbolPaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Via:
-			return new Via(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new Via(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		case ModelPart::Hole:
-			return new Hole(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+			return new Hole(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
+        case ModelPart::SchematicSubpart:
+            return new SchematicSubpart(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 		default:
 			{
 				QString moduleID = modelPart->moduleID();
 				if (moduleID.endsWith(ModuleIDNames::ModuleIDNameSuffix)) {
 					if (moduleID.endsWith(ModuleIDNames::ResistorModuleIDName)) {
-						return new Resistor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Resistor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::CapacitorModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::CrystalModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::ThermistorModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::ZenerDiodeModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::PotentiometerModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::InductorModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
+					}
+					if (moduleID.endsWith(ModuleIDNames::TwoPowerModuleIDName)) {
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::ColorLEDModuleIDName)) {
-						return new LED(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new LED(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::ColorFluxLEDModuleIDName)) {
-						return new LED(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new LED(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::LEDModuleIDName)) {
-						return new Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::PerfboardModuleIDName)) {
-						return new Perfboard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Perfboard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::StripboardModuleIDName)) {
-						return new Stripboard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Stripboard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::SchematicFrameModuleIDName)) {
-						return new SchematicFrame(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new SchematicFrame(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::PadModuleIDName)) {
-						return new Pad(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Pad(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					if (moduleID.endsWith(ModuleIDNames::CopperBlockerModuleIDName)) {
-						return new CopperBlocker(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new CopperBlocker(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 					// must get the subclasses first
 					if (modelPart->itemType() == ModelPart::Breadboard) {
-						return new Breadboard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+						return new Breadboard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 					}
 
 				}
 				QString family = modelPart->properties().value("family", "");
 				if (family.compare("mystery part", Qt::CaseInsensitive) == 0) {
-					return new MysteryPart(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+					return new MysteryPart(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 				}
 				if (family.compare("screw terminal", Qt::CaseInsensitive) == 0) {
-					return new ScrewTerminal(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+					return new ScrewTerminal(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 				}
 				if (family.compare("pin header", Qt::CaseInsensitive) == 0) {
-					return new PinHeader(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+					return new PinHeader(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 				}
 				if (family.compare("generic IC", Qt::CaseInsensitive) == 0) {
-					return new Dip(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+					return new Dip(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 				}
-				return new PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel);
+				return new PaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel);
 
 			}
 	}
 }
 
-QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & expectedFileName) {
+QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & baseName, bool generate, bool handleSubparts) 
+{
+	QStringList tempPaths;
+	QString postfix = "/"+ SvgFilesDir +"/%1/"+ baseName;
+    QString userStore = FolderUtils::getUserDataStorePath("parts")+postfix;
+    QString pfPath = PartFactory::folderPath() + postfix;
+	if(!modelPart->path().isEmpty()) {
+		QDir dir(modelPart->path());			// is a path to a filename
+		dir.cdUp();									// lop off the filename
+		dir.cdUp();									// parts root
+		tempPaths << dir.absolutePath() + postfix;
+		tempPaths << FolderUtils::getApplicationSubFolderPath("parts")+postfix;    // some svgs may still be in the fritzing parts folder, though the other svgs are in the user folder
+        if (tempPaths.at(0).compare(userStore) != 0) {
+            tempPaths << userStore;
+        }
+        if (tempPaths.at(0).compare(pfPath) != 0) {
+            tempPaths << pfPath;
+        }
+        //DebugDialog::debug("temp path");
+        //foreach (QString tempPath, tempPaths) {
+        //    DebugDialog::debug(tempPath);
+        //}
+    } 
+	else {
+        DebugDialog::debug("modelPart with no path--this shouldn't happen");
+		tempPaths << FolderUtils::getApplicationSubFolderPath("parts")+postfix;
+		tempPaths << userStore;
+	}
+	tempPaths << ":resources/parts/svg/%1/" + baseName;
 
+		//DebugDialog::debug(QString("got tempPath %1").arg(tempPath));
+
+	QString filename;
+    bool exists = false;
+	foreach (QString tempPath, tempPaths) {
+		foreach (QString possibleFolder, ModelPart::possibleFolders()) {
+			filename = tempPath.arg(possibleFolder);
+			if (QFileInfo(filename).exists()) {
+                exists = true;
+				if (possibleFolder == "obsolete") {
+					DebugDialog::debug(QString("module %1:%2 obsolete svg %3").arg(modelPart->title()).arg(modelPart->moduleID()).arg(filename));
+				}
+				break;
+			} 
+		}
+        if (exists) break;
+	}
+
+    if (!exists && generate) {
+	    filename = PartFactory::getSvgFilename(modelPart, baseName);
+    }
+
+    if (handleSubparts && modelPart->modelPartShared() && modelPart->modelPartShared()->hasSubparts())
+    {
+        ModelPartShared * superpart = modelPart->modelPartShared();
+        QString schematicName = superpart->imageFileName(ViewLayer::SchematicView);
+        QString originalPath = getSvgFilename(modelPart, schematicName, true, false);
+        foreach (ModelPartShared * mps, superpart->subparts()) {
+            QString schematicFileName = mps->imageFileName(ViewLayer::SchematicView);
+            if (schematicFileName.isEmpty()) continue;
+
+            QString path = partPath() + schematicFileName;
+            QFileInfo info(path);
+            if (info.exists()) {
+                mps->setSubpartOffset(SubpartOffsets.value(path, QPointF(0, 0)));
+                continue;
+            }
+
+            QFile file(originalPath);
+	        QString errorStr;
+	        int errorLine;
+	        int errorColumn;
+	        QDomDocument doc;
+	        if (!doc.setContent(&file, &errorStr, &errorLine, &errorColumn)) {
+                DebugDialog::debug(QString("xml failure %1 %2 %3").arg(errorStr).arg(errorLine).arg(errorColumn));
+                continue;
+            }
+
+            QDomElement root = doc.documentElement();
+            QDomElement top = showSubpart(root, mps->subpartID());
+            fixSubpartBounds(top, mps);
+            SubpartOffsets.insert(path, mps->subpartOffset());
+            TextUtils::writeUtf8(path, doc.toString(4));
+        }
+    }
+
+    return filename;
+}
+
+QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & expectedFileName) 
+{
 	if (expectedFileName.startsWith("pcb/dip_", Qt::CaseInsensitive)) {
 		return getSvgFilenameAux(expectedFileName, modelPart, &MysteryPart::makePcbDipSvg);
 	}
@@ -228,6 +337,10 @@ QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & expec
 		return getSvgFilenameAux(expectedFileName, modelPart, &MysteryPart::makeSchematicSvg);
 	}
 
+	if (expectedFileName.startsWith("pcb/generic_sip_", Qt::CaseInsensitive)) {
+		return getSvgFilenameAux(expectedFileName, modelPart, &PinHeader::makePcbSvg);
+	}
+
 	if (expectedFileName.startsWith("schematic/generic_ic_dip_", Qt::CaseInsensitive)) {
 		return getSvgFilenameAux(expectedFileName, modelPart, &Dip::makeSchematicSvg);
 	}
@@ -236,6 +349,10 @@ QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & expec
 		return getSvgFilenameAux(expectedFileName, modelPart, &Dip::makeBreadboardSvg);
 	}
 
+	if (expectedFileName.startsWith("pcb/generic_ic_dip_", Qt::CaseInsensitive)) {
+		return getSvgFilenameAux(expectedFileName, modelPart, &MysteryPart::makePcbDipSvg);
+	}
+
 	if (expectedFileName.startsWith("pcb/nsjumper_", Qt::CaseInsensitive)) {
 		return getSvgFilenameAux(expectedFileName, modelPart, &PinHeader::makePcbSvg);
 	}
@@ -286,7 +403,7 @@ QString PartFactory::getSvgFilename(ModelPart * modelPart, const QString & expec
 }
 
 bool PartFactory::svgFileExists(const QString & expectedFileName, QString & path) {
-	QString p = FolderUtils::getApplicationSubFolderPath("parts") + "/"+ ItemBase::SvgFilesDir + "/core/";
+	QString p = FolderUtils::getApplicationSubFolderPath("parts") + "/"+ SvgFilesDir + "/core/";
 	if (QFileInfo(p + expectedFileName).exists()) {
         path = expectedFileName;
         return true;
@@ -437,7 +554,7 @@ QString PartFactory::partPath() {
     return PartFactoryFolderPath + "/svg/core/";
 }
 
-QString PartFactory::makeSipOrDipOr(const QStringList & labels, bool hasLayout, bool sip) 
+QString PartFactory::makeSchematicSipOrDipOr(const QStringList & labels, bool hasLayout, bool sip) 
 {		
 	if (hasLayout) {
 		return MysteryPart::makeSchematicSvg(labels, false);
@@ -450,3 +567,112 @@ QString PartFactory::makeSipOrDipOr(const QStringList & labels, bool hasLayout,
 	return Dip::makeSchematicSvg(labels);
 }
 
+QDomElement PartFactory::showSubpart(QDomElement & root, const QString & subpart)
+{    
+    if (subpart.isEmpty()) return ___emptyElement___;
+
+    QString id = root.attribute("id");
+    if (id == subpart) {
+        return root;
+    }
+
+    QDomElement top;
+    QDomElement child = root.firstChildElement();
+    while (!child.isNull()) {
+        QDomElement candidate = showSubpart(child, subpart);
+        if (!candidate.isNull()) {
+            top = candidate;
+        }
+        child = child.nextSiblingElement();
+    }
+
+    if (root.tagName() != "g" && root.tagName() != "svg") {
+        root.setTagName("g");
+    }
+    
+    return top;
+}
+
+void PartFactory::fixSubpartBounds(QDomElement & top, ModelPartShared * mps)
+{    
+    if (top.isNull()) return;
+
+    QString transform = top.attribute("transform", "");
+    top.removeAttribute("transform");
+    QDomElement g = top.ownerDocument().createElement("g");
+    if (!transform.isEmpty()) g.setAttribute("transform", transform);
+    QDomElement child = top.firstChildElement();
+    while (!child.isNull()) {
+        g.appendChild(child);
+        child = top.firstChildElement();
+    }
+    top.appendChild(g);
+
+    QSvgRenderer renderer;
+    QByteArray byteArray = top.ownerDocument().toByteArray();
+    bool loaded = renderer.load(byteArray);
+    if (!loaded) return;
+
+    QRectF viewBox;
+    double wignore, hignore;
+    TextUtils::ensureViewBox(top.ownerDocument(), 1, viewBox, false, wignore, hignore, false);
+    double sWidth, sHeight, vbWidth, vbHeight;
+    QDomDocument doc = top.ownerDocument();
+    TextUtils::getSvgSizes(doc, sWidth, sHeight, vbWidth, vbHeight);
+
+	QMatrix m = renderer.matrixForElement(mps->subpartID());
+    QRectF elementBounds = renderer.boundsOnElement(mps->subpartID());
+	QRectF bounds = m.mapRect(elementBounds);                                   // bounds is in terms of the whole svg
+
+    // unfortunately, QSvgRenderer doesn't deal with text bounds
+
+    int w = qCeil(sWidth * GraphicsUtils::SVGDPI);
+    int h = qCeil(sWidth * GraphicsUtils::SVGDPI);
+    QImage image(w,h, QImage::Format_Mono);
+
+    QDomNodeList nodeList = top.elementsByTagName("text");
+    QList<QDomElement> texts;
+    for (int i = 0; i < nodeList.count(); i++) {
+        texts.append(nodeList.at(i).toElement());
+    }
+
+    int ix = 0;
+    foreach (QDomElement text, texts) {
+        text.setTagName("g");
+    }
+
+    ix = 0;
+    foreach (QDomElement text, texts) {
+        int minX, minY, maxX, maxY;
+        QMatrix matrix;
+        QRectF viewBox2;
+        SchematicTextLayerKinPaletteItem::renderText(image, text, minX, minY, maxX, maxY, matrix, viewBox2);  
+        QRectF r(minX * viewBox.width() / image.width(), 
+                minY * viewBox.height() / image.height(), 
+                (maxX - minX) * viewBox.width() / image.width(),
+                (maxY - minY) * viewBox.height() / image.height());
+        bounds |= r;
+    }
+
+    foreach (QDomElement text, texts) {
+        text.setTagName("text");
+    }
+
+    QDomElement root = top.ownerDocument().documentElement();
+    double newW = sWidth * bounds.width() / vbWidth;
+    double newH = sHeight * bounds.height() / vbHeight;
+    root.setAttribute("width", QString("%1in").arg(newW));
+    root.setAttribute("height", QString("%1in").arg(newH));
+    root.setAttribute("viewBox", QString("0 0 %1 %2").arg(bounds.width()).arg(bounds.height()));
+
+    if (bounds.left() != 0 || bounds.top() != 0) {
+        QDomElement g2 = top.ownerDocument().createElement("g");
+        g2.appendChild(g);
+        top.appendChild(g2);
+        g2.setAttribute("transform", QString("translate(%1,%2)").arg(-bounds.left()).arg(-bounds.top()));
+    }
+
+    mps->setSubpartOffset(QPointF(bounds.left() * sWidth / vbWidth, bounds.top() * sHeight / vbHeight));
+}
+
+
