@@ -18,16 +18,15 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
 #include "perfboard.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/textutils.h"
-#include "../utils/focusoutcombobox.h"
 #include "../utils/boundedregexpvalidator.h"
 #include "../fsvgrenderer.h"
 #include "../sketch/infographicsview.h"
@@ -57,8 +56,8 @@ bool Perfboard::m_gotWarning = false;
 
 /////////////////////////////////////////////////////////////////////
 
-Perfboard::Perfboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Perfboard::Perfboard( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	m_size = modelPart->localProp("size").toString();
 	if (m_size.isEmpty()) {
@@ -76,8 +75,8 @@ void Perfboard::setProp(const QString & prop, const QString & value)
 		Capacitor::setProp(prop, value);
 		return;
 	}
-	switch (this->m_viewIdentifier) {
-		case ViewIdentifierClass::BreadboardView:
+	switch (this->m_viewID) {
+		case ViewLayer::BreadboardView:
 			if (value.compare(m_size) != 0) {
                 QString temp = value;
 				QString svg = makeBreadboardSvg(temp);
@@ -186,7 +185,7 @@ QString Perfboard::genFZP(const QString & moduleid)
 	return FzpTemplate.arg(x).arg(y).arg(middle);
 }
 
-bool Perfboard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool Perfboard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare("size", Qt::CaseInsensitive) == 0) {
 		returnProp = tr("size");
@@ -269,7 +268,7 @@ bool Perfboard::collectExtraInfo(QWidget * parent, const QString & family, const
 		return true;
 	}
 
-	return Capacitor::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return Capacitor::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 
