@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6272 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 13:28:16 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -29,7 +29,6 @@ $Date: 2012-08-09 13:28:16 +0200 (Thu, 09 Aug 2012) $
 #include "../fsvgrenderer.h"
 #include "../commands.h"
 #include "../utils/textutils.h"
-#include "../layerattributes.h"
 #include "partlabel.h"
 #include "partfactory.h"
 #include "../sketch/infographicsview.h"
@@ -62,8 +61,8 @@ static QString ShroudedSpacing;
 
 static HoleClassThing TheHoleThing;
 
-PinHeader::PinHeader( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+PinHeader::PinHeader( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: PaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 
     setUpHoleSizes("pinheader", TheHoleThing);
@@ -449,6 +448,9 @@ QString PinHeader::makePcbSvg(const QString & originalExpectedFileName)
     int pins = TextUtils::getPinsAndSpacing(expectedFileName, spacingString);
     if (pins == 0) return "";
 
+    // only one spacing for mystery parts.
+    if (expectedFileName.contains("mystery")) spacingString = "100mil";
+
     QString svg;
 
 	if (expectedFileName.contains("shrouded")) {
@@ -1012,7 +1014,7 @@ QString PinHeader::makePcbSMDSvg(const QString & expectedFileName)
 	return svg;
 }
 
-bool PinHeader::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+bool PinHeader::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (prop.compare("hole size", Qt::CaseInsensitive) == 0) {
         if (moduleID().contains("smd", Qt::CaseInsensitive)) {
@@ -1024,7 +1026,7 @@ bool PinHeader::collectExtraInfo(QWidget * parent, const QString & family, const
         }
 	}
 
-	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 void PinHeader::swapEntry(const QString & text) {
