@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6998 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -36,6 +36,7 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 #include "../layerattributes.h"
 #include "moduleidnames.h"
 #include "partlabel.h"
+#include "../debugdialog.h"
 
 #include <qmath.h>
 #include <QRegExpValidator>
@@ -51,8 +52,8 @@ static QHash<QString, QColor> Tolerances;
 //	save into parts bin
 //	other manifestations of "220"?
 
-Resistor::Resistor( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: Capacitor(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Resistor::Resistor( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: Capacitor(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	m_changingPinSpacing = false;
 	if (Resistances.count() == 0) {
@@ -133,15 +134,15 @@ void Resistor::setResistance(QString resistance, QString pinSpacing, bool force)
 
 	modelPart()->setLocalTitle(resistance + " " + OhmSymbol + " " + tr("Resistor"));
 
-	switch (this->m_viewIdentifier) {
-		case ViewIdentifierClass::BreadboardView:
+	switch (this->m_viewID) {
+		case ViewLayer::BreadboardView:
 			if (force || resistance.compare(m_ohms) != 0) {
 				QString svg = makeSvg(resistance, m_viewLayerID);
 				//DebugDialog::debug(svg);
 				reloadRenderer(svg, false);
 			}
 			break;
-		case ViewIdentifierClass::PCBView:
+		case ViewLayer::PCBView:
 			if (force || pinSpacing.compare(m_pinSpacing) != 0) {
 
 				InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
@@ -149,20 +150,18 @@ void Resistor::setResistance(QString resistance, QString pinSpacing, bool force)
 
 				if (modelPart()->properties().value("package").compare("tht", Qt::CaseInsensitive) == 0) 
 				{
-					// pinspacing is irrelevant for SMD resistors
-					QDomElement element = LayerAttributes::getSvgElementLayers(modelPart()->domDocument(), m_viewIdentifier);
-					if (element.isNull()) break;
-
-					// hack the dom element and call setUpImage
+                    // pinspacing is irrelevant for SMD resistors
 					QString filename = PinSpacings.value(pinSpacing, "");
 					if (filename.isEmpty()) break;
 
-					element.setAttribute("image", filename);
+                    QString original = modelPart()->imageFileName(m_viewID);
+                    modelPart()->setImageFileName(m_viewID, filename);
 					m_changingPinSpacing = true;
-					resetImage(infoGraphicsView, NULL);
+					resetImage(infoGraphicsView);
 					m_changingPinSpacing = false;
-
-					updateConnections();
+                    modelPart()->setImageFileName(m_viewID, original);
+                    QList<ConnectorItem *> already;
+					updateConnections(false, already);
 				}
 
 			}
@@ -181,29 +180,18 @@ void Resistor::setResistance(QString resistance, QString pinSpacing, bool force)
     if (m_partLabel) m_partLabel->displayTextsIf();
 }
 
-QString Resistor::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
+QString Resistor::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
 {
 	switch (viewLayerID) {
 		case ViewLayer::Breadboard:
 		case ViewLayer::Icon:
 			break;
 		default:
-			return Capacitor::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+			return Capacitor::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 	}
 
 	QString svg = makeSvg(m_ohms, viewLayerID);
-
-	QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
-	SvgFileSplitter splitter;
-	bool result = splitter.splitString(svg, xmlName);
-	if (!result) {
-		return "";
-	}
-	result = splitter.normalize(dpi, xmlName, blackOnly);
-	if (!result) {
-		return "";
-	}
-	return splitter.elementString(xmlName);
+    return PaletteItemBase::normalizeSvg(svg, viewLayerID, blackOnly, dpi, factor);
 }
 
 QString Resistor::makeSvg(const QString & resistance, ViewLayer::ViewLayerID viewLayerID) {
@@ -268,7 +256,7 @@ void Resistor::setBands(QDomElement & element, int firstband, int secondband, in
 	}
 }
 
-bool Resistor::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool Resistor::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare("resistance", Qt::CaseInsensitive) == 0) {
 		returnProp = tr("resistance");
@@ -283,11 +271,12 @@ bool Resistor::collectExtraInfo(QWidget * parent, const QString & family, const
 		validator->setSymbol(OhmSymbol);
 		validator->setConverter(TextUtils::convertFromPowerPrefix);
 		validator->setBounds(0, 9900000000.0);
-		validator->setRegExp(QRegExp("((\\d{1,3})|(\\d{1,3}\\.)|(\\d{1,3}\\.\\d))[kMG]{0,1}[\\x03A9]{0,1}"));
+		validator->setRegExp(QRegExp(QString("((\\d{1,3})|(\\d{1,3}\\.)|(\\d{1,3}\\.\\d))[\\x%1umkMG]{0,1}[\\x03A9]{0,1}").arg(TextUtils::MicroSymbolCode, 4, 16, QChar('0'))));
 		focusOutComboBox->setValidator(validator);
 		connect(focusOutComboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(resistanceEntry(const QString &)));
 
-		focusOutComboBox->setObjectName("infoViewComboBox");		
+		focusOutComboBox->setObjectName("infoViewComboBox");
+        focusOutComboBox->setToolTip(tr("You can either type in a resistance value, or select one from the drop down. Format nnn.dP where P is one of 'umkMG'"));
 
 		returnValue = current;			
 		returnWidget = focusOutComboBox;	
@@ -295,7 +284,7 @@ bool Resistor::collectExtraInfo(QWidget * parent, const QString & family, const
 		return true;
 	}
 
-	return Capacitor::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return Capacitor::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 QString Resistor::getProperty(const QString & key) {
@@ -340,7 +329,7 @@ void Resistor::updateResistances(QString r) {
 
 ConnectorItem* Resistor::newConnectorItem(Connector *connector) {
 	if (m_changingPinSpacing) {
-		return connector->connectorItemByViewLayerID(viewIdentifier(), viewLayerID());
+		return connector->connectorItemByViewLayerID(viewID(), viewLayerID());
 	}
 
 	return Capacitor::newConnectorItem(connector);
@@ -348,16 +337,16 @@ ConnectorItem* Resistor::newConnectorItem(Connector *connector) {
 
 ConnectorItem* Resistor::newConnectorItem(ItemBase * layerKin, Connector *connector) {
 	if (m_changingPinSpacing) {
-		return connector->connectorItemByViewLayerID(viewIdentifier(), layerKin->viewLayerID());
+		return connector->connectorItemByViewLayerID(viewID(), layerKin->viewLayerID());
 	}
 
 	return Capacitor::newConnectorItem(layerKin, connector);
 }
 
 bool Resistor::hasCustomSVG() {
-	switch (m_viewIdentifier) {
-		case ViewIdentifierClass::BreadboardView:
-		case ViewIdentifierClass::IconView:
+	switch (m_viewID) {
+		case ViewLayer::BreadboardView:
+		case ViewLayer::IconView:
 			return true;
 		default:
 			return ItemBase::hasCustomSVG();
@@ -403,22 +392,22 @@ void Resistor::setProp(const QString & prop, const QString & value)
 	}
 }
 
-bool Resistor::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
+bool Resistor::setUpImage(ModelPart * modelPart, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
-	bool result = Capacitor::setUpImage(modelPart, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
-	if (viewLayerID == ViewLayer::Breadboard) {
+	bool result = Capacitor::setUpImage(modelPart, viewLayers, layerAttributes);
+	if (layerAttributes.viewLayerID == ViewLayer::Breadboard) {
 		if (result && m_breadboardSvgFile.isEmpty()) m_breadboardSvgFile = layerAttributes.filename();
 	}
-	else if (viewLayerID == ViewLayer::Icon) {
+	else if (layerAttributes.viewLayerID == ViewLayer::Icon) {
 		if (result && m_iconSvgFile.isEmpty()) m_iconSvgFile = layerAttributes.filename();
 	}
 	return result;
 }
 
-ViewIdentifierClass::ViewIdentifier Resistor::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier vid, bool swappingEnabled) {
-    if (swappingEnabled && vid == ViewIdentifierClass::BreadboardView) {
+ViewLayer::ViewID Resistor::useViewIDForPixmap(ViewLayer::ViewID vid, bool swappingEnabled) {
+    if (swappingEnabled && vid == ViewLayer::BreadboardView) {
         return vid;
     }
 
-    return ItemBase::useViewIdentifierForPixmap(vid, swappingEnabled);
+    return ItemBase::useViewIDForPixmap(vid, swappingEnabled);
 }
