@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6218 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,6 +35,8 @@ $Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 #include "../commands.h"
 #include "moduleidnames.h"
 #include "../layerattributes.h"
+#include "../debugdialog.h"
+#include "../svg/gerbergenerator.h"
 
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -48,8 +50,15 @@ $Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 #include <qmath.h>
 #include <qnumeric.h>
 
-QHash<QString, QString> BoardLayerTemplates;
-QHash<QString, QString> SilkscreenLayerTemplates;
+QList<QString> PaperSizeNames;
+QList<QSizeF> PaperSizeDimensions;
+
+static QHash<QString, QString> NamesToXmlNames;
+static QHash<QString, QString> XmlNamesToNames;
+
+static QHash<QString, QString> BoardLayerTemplates;
+static QHash<QString, QString> SilkscreenLayerTemplates;
+static QHash<QString, QString> Silkscreen0LayerTemplates;
 static const int LineThickness = 8;
 static const QRegExp HeightExpr("height=\\'\\d*px");
 static QString StandardCustomBoardExplanation;
@@ -64,8 +73,8 @@ static const double JND = (double) 0.01;
 QString Board::OneLayerTranslated;
 QString Board::TwoLayersTranslated;
 
-Board::Board( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Board::Board( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: PaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
     m_svgOnly = true;
 	m_fileNameComboBox = NULL;
@@ -75,15 +84,25 @@ Board::Board( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIde
         }
         if (itemType() == ModelPart::Board && !modelPart->properties().keys().contains("filename")) {
             // deal with old style custom boards
-            modelPart->modelPartShared()->setProperty("filename", "");
+            modelPart->modelPartShared()->setProperty("filename", "", false);
         }
     }
 
     if (StandardCustomBoardExplanation.isEmpty()) {
-        StandardCustomBoardExplanation = tr("\n\nA custom board svg typically has one silkscreen layer and one board layer.\n") +
+        StandardCustomBoardExplanation = tr("\n\nA custom board svg typically has one or two silkscreen layers and one board layer.\n") +
                                             tr("Have a look at the circle_pcb.svg file in your Fritzing installation folder at parts/svg/core/pcb/.\n\n");
     }
 
+    if (NamesToXmlNames.count() == 0) {
+        NamesToXmlNames.insert("copper bottom", "copper0");
+        NamesToXmlNames.insert("copper top", "copper1");
+        NamesToXmlNames.insert("silkscreen bottom", "silkscreen0");
+        NamesToXmlNames.insert("silkscreen top", "silkscreen");
+        foreach (QString key, NamesToXmlNames.keys()) {
+            XmlNamesToNames.insert(NamesToXmlNames.value(key), key);
+        }
+    }
+
 }
 
 Board::~Board() {
@@ -136,14 +155,14 @@ QStringList Board::collectValues(const QString & family, const QString & prop, Q
 
 }
 
-bool Board::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+bool Board::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (prop.compare("filename", Qt::CaseInsensitive) == 0 && isBoard(this)) {
         setupLoadImage(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
 		return true;
 	}
 
-	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 bool Board::rotation45Allowed() {
@@ -319,7 +338,8 @@ bool Board::checkImage(const QString & filename) {
     QList<QDomElement> elements;
     TextUtils::findElementsWithAttribute(root, "id", elements);
     int layers = 0;
-    int boardLayers = 0;
+    QList<QDomElement> boardElements;
+    int silk0Layers = 0;
     int silk1Layers = 0;
     bool boardHasChildren = false;
     foreach (QDomElement element, elements) {
@@ -328,25 +348,31 @@ bool Board::checkImage(const QString & filename) {
         if (viewLayerID != ViewLayer::UnknownLayer) {
             layers++;
             if (viewLayerID == ViewLayer::Board) {
-                boardLayers++;
+                boardElements << element;
                 if (element.childNodes().count() > 0) {
                     boardHasChildren = true;
                 }
             }
-            if (viewLayerID == ViewLayer::Silkscreen1) {
+            else if (viewLayerID == ViewLayer::Silkscreen1) {
                 silk1Layers++;
             }
+            else if (viewLayerID == ViewLayer::Silkscreen0) {
+                silk0Layers++;
+            }
         }
     }
 
-    if (boardLayers == 1 && !boardHasChildren) {
+    if (boardElements.count() == 1 && !boardHasChildren) {
         unableToLoad(filename, tr("the <board> element contains no shape elements") + StandardCustomBoardExplanation);
         return false;
     }
 
-    if ((boardLayers == 1) && (silk1Layers == 1)) return true;
+    if ((boardElements.count() == 1) && ((silk1Layers == 1) || (silk0Layers == 1))) {
+        moreCheckImage(filename);
+        return true;
+    }
 
-    if (boardLayers > 1) {
+    if (boardElements.count() > 1) {
         unableToLoad(filename, tr("because there are multiple <board> layers") + StandardCustomBoardExplanation);
         return false;
     }
@@ -356,7 +382,12 @@ bool Board::checkImage(const QString & filename) {
         return false;
     }
 
-    if (layers > 0 && boardLayers == 0) {
+    if (silk0Layers > 1) {
+        unableToLoad(filename, tr("because there are multiple <silkscreen0> layers") + StandardCustomBoardExplanation);
+        return false;
+    }
+
+    if (layers > 0 && boardElements.count() == 0) {
         unableToLoad(filename, tr("because there is no <board> layer") + StandardCustomBoardExplanation);
         return false;
     }
@@ -366,14 +397,122 @@ bool Board::checkImage(const QString & filename) {
         return false;
     }
 
-    if (layers == 0 || (boardLayers == 1 && silk1Layers == 0)) {
-        return canLoad(filename, tr("but the pcb itself will have no silkscreen layer") + StandardCustomBoardExplanation);
+    if (layers == 0 || (boardElements.count() == 1 && silk1Layers == 0 && silk1Layers == 0)) {
+        bool result = canLoad(filename, tr("but the pcb itself will have no silkscreen layer") + StandardCustomBoardExplanation);
+        if (result) moreCheckImage(filename);
+        return result;
     }
 
     unableToLoad(filename, tr("the svg doesn't fit the custom board format") + StandardCustomBoardExplanation);
     return false;
 }
 
+void Board::moreCheckImage(const QString & filename) {
+    QFile file(filename);
+    file.open(QFile::ReadOnly);
+    QString svg = file.readAll();
+    file.close();
+
+    QString nsvg = setBoardOutline(svg);
+
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	QDomDocument domDocument;
+	domDocument.setContent(nsvg, &errorStr, &errorLine, &errorColumn);
+    QDomElement element = TextUtils::findElementWithAttribute(domDocument.documentElement(), "id", GerberGenerator::MagicBoardOutlineID);
+
+    int subpaths = 1;
+    int mCount = 0;
+    if (element.tagName() == "path") {
+        QString originalPath = element.attribute("d", "").trimmed();
+        if (GerberGenerator::MultipleZs.indexIn(originalPath) >= 0) {
+            QStringList ds = element.attribute("d").split("z", QString::SkipEmptyParts);
+            subpaths = ds.count();
+            foreach (QString d, ds) {
+                if (d.trimmed().startsWith("m", Qt::CaseInsensitive)) mCount++;
+            }
+        }
+    }
+
+    QString msg;
+    if (subpaths == 1) {
+        msg = tr("Fritzing thinks the custom shape has no cutouts.");
+    }
+    else {
+        msg = tr("Fritzing thinks the custom shape has %n cutouts.", "", subpaths - 1);
+        if (subpaths != mCount) {
+            msg += tr("\nHowever, the cutouts may not be formatted correctly.");
+        }
+    }
+    msg +=  tr("\n\nIf you intended your custom shape to have cutouts and you did not get the expected result ");
+    msg += tr("it is because Fritzing requires that you make cutouts using a shape 'subtraction' or 'difference' operation in your vector graphics editor.\n\n");
+    msg += tr("In any case we recommend that you test your custom shape by using the 'File > Export for Production > Extended Gerber' option ");
+    msg += tr("and checking the resulting contour file with a Gerber viewer application.");
+    QMessageBox::information(NULL, "Custom Shape", msg);
+}
+
+QString Board::setBoardOutline(const QString & svg) {
+    QString errorStr;
+	int errorLine;
+	int errorColumn;
+
+	QDomDocument domDocument;
+	if (!domDocument.setContent(svg, true, &errorStr, &errorLine, &errorColumn)) {
+		return svg;
+	}
+
+    QDomElement root = domDocument.documentElement();
+	if (root.tagName() != "svg") {
+		return svg;
+	}
+
+    QDomElement board = TextUtils::findElementWithAttribute(root, "id", "board");
+    if (board.isNull()) {
+        board = root;
+    }
+
+    QList<QDomElement> leaves;
+    TextUtils::collectLeaves(board, leaves);
+
+    if (leaves.count() == 1) {
+        QDomElement leaf = leaves.at(0);
+        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
+        return TextUtils::removeXMLEntities(domDocument.toString());
+    }
+
+    int ix = 0;
+    QStringList ids;
+    foreach (QDomElement leaf, leaves) {
+        ids.append(leaf.attribute("id", ""));
+        leaf.setAttribute("id", QString("____%1____").arg(ix++));
+    }
+
+    QSvgRenderer renderer(domDocument.toByteArray());
+
+    ix = 0;
+    foreach (QDomElement leaf, leaves) {    
+        leaf.setAttribute("id", ids.at(ix++));
+    }
+
+    double maxArea = 0;
+    int maxIndex = -1;
+    for (int i = 0; i < leaves.count(); i++) {
+        QRectF r = renderer.boundsOnElement(QString("____%1____").arg(i));
+        if (r.width() * r.height() > maxArea) {
+            maxArea = r.width() * r.height();
+            maxIndex = i;
+        }
+    }
+
+    if (maxIndex >= 0) {
+        QDomElement leaf = leaves.at(maxIndex);
+        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
+        return TextUtils::removeXMLEntities(domDocument.toString());
+    }
+
+    return svg;
+}
 void Board::unableToLoad(const QString & fileName, const QString & reason) {
 	QMessageBox::information(
 		NULL,
@@ -401,19 +540,35 @@ void Board::prepLoadImageAux(const QString & fileName, bool addName)
 	}
 }
 
-ViewIdentifierClass::ViewIdentifier Board::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier vid, bool) 
+ViewLayer::ViewID Board::useViewIDForPixmap(ViewLayer::ViewID vid, bool) 
 {
-    if (vid == ViewIdentifierClass::PCBView) {
-        return ViewIdentifierClass::IconView;
+    if (vid == ViewLayer::PCBView) {
+        return ViewLayer::IconView;
     }
 
-    return ViewIdentifierClass::UnknownView;
+    return ViewLayer::UnknownView;
+}
+
+QString Board::convertToXmlName(const QString & name) {
+    foreach (QString key, ItemBase::TranslatedPropertyNames.keys()) {
+        if (name.compare(ItemBase::TranslatedPropertyNames.value(key), Qt::CaseInsensitive) == 0) {
+            return NamesToXmlNames.value(key);
+        }
+    }
+
+    return name;
+}
+
+QString Board::convertFromXmlName(const QString & xmlName) {
+    QString result = ItemBase::TranslatedPropertyNames.value(XmlNamesToNames.value(xmlName));
+    if (result.isEmpty()) return xmlName;
+    return result;
 }
 
 ///////////////////////////////////////////////////////////
 
-ResizableBoard::ResizableBoard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: Board(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+ResizableBoard::ResizableBoard( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: Board(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	fixWH();
 
@@ -422,6 +577,7 @@ ResizableBoard::ResizableBoard( ModelPart * modelPart, ViewIdentifierClass::View
     m_aspectRatioCheck = NULL;
     m_aspectRatioLabel = NULL;
     m_revertButton = NULL;
+    m_paperSizeComboBox = NULL;
 
 	m_corner = ResizableBoard::NO_CORNER;
 	m_currentScale = 1.0;
@@ -452,17 +608,17 @@ void ResizableBoard::loadTemplates() {
 
     BoardLayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Board));
     SilkscreenLayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Silkscreen1));
+    Silkscreen0LayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Silkscreen0));
 }
 
 double ResizableBoard::minWidth() {
-	return 0.5 * GraphicsUtils::SVGDPI;
+	return 0.25 * GraphicsUtils::SVGDPI;
 }
 
 double ResizableBoard::minHeight() {
-	return 0.5 * GraphicsUtils::SVGDPI;
+	return 0.25 * GraphicsUtils::SVGDPI;
 }
 
-
 void ResizableBoard::mousePressEvent(QGraphicsSceneMouseEvent * event) 
 {
 	m_corner = ResizableBoard::NO_CORNER;
@@ -527,20 +683,28 @@ void ResizableBoard::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 			size.setWidth(size.width() - ds.x());
 			size.setHeight(size.height() - ds.y());
 			break;
-                default:
-                        break;
+        default:
+            break;
 	}
 
-	if (size.width() < minWidth()) size.setWidth(minWidth());
-	if (size.height() < minHeight()) size.setHeight(minHeight());
+	if (size.width() < minWidth()) {
+        DebugDialog::debug("to min width");
+        size.setWidth(minWidth());
+    }
+	if (size.height() < minHeight()) {
+        DebugDialog::debug("to min height");
+        size.setHeight(minHeight());
+    }
 
 	if (m_keepAspectRatio) {
 		double cw = size.height() * m_aspectRatio.width() / m_aspectRatio.height();
 		double ch = size.width() * m_aspectRatio.height() / m_aspectRatio.width();
 		if (ch < minHeight()) {
+            DebugDialog::debug("from min height");
 			size.setWidth(cw);
 		}
 		else if (cw < minWidth()) {
+            DebugDialog::debug("from min width");
 			size.setHeight(ch);
 		}
 		else {
@@ -637,15 +801,8 @@ void ResizableBoard::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {
 	}
 }
 
-bool ResizableBoard::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
-{
-	bool result = Board::setUpImage(modelPart, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
-
-	return result;
-}
-
-ViewIdentifierClass::ViewIdentifier ResizableBoard::theViewIdentifier() {
-	return ViewIdentifierClass::PCBView;
+ViewLayer::ViewID ResizableBoard::theViewID() {
+	return ViewLayer::PCBView;
 }
 
 void ResizableBoard::resizePixels(double w, double h, const LayerHash & viewLayers) {
@@ -654,9 +811,12 @@ void ResizableBoard::resizePixels(double w, double h, const LayerHash & viewLaye
 
 bool ResizableBoard::resizeMM(double mmW, double mmH, const LayerHash & viewLayers) {
 	if (mmW == 0 || mmH == 0) {
-		QString error;
 		LayerAttributes layerAttributes;
-		setUpImage(modelPart(), m_viewIdentifier, viewLayers, m_viewLayerID, m_viewLayerSpec, true, layerAttributes, error);
+        layerAttributes.viewID = m_viewID;
+        layerAttributes.viewLayerID = m_viewLayerID;
+        layerAttributes.viewLayerPlacement = m_viewLayerPlacement;
+        layerAttributes.createShape = layerAttributes.doConnectors = true;
+		setUpImage(modelPart(), viewLayers, layerAttributes);
 		modelPart()->setLocalProp("height", QVariant());
 		modelPart()->setLocalProp("width", QVariant());
 		// do the layerkin
@@ -701,15 +861,15 @@ void ResizableBoard::resizeMMAux(double mmW, double mmH)
 				itemBase->modelPart()->setLocalProp("width", mmW);
 				itemBase->modelPart()->setLocalProp("height", mmH);
 			}
-			break;
 		}
 	}
+
 }
 
-void ResizableBoard::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerSpec viewLayerSpec) {
+void ResizableBoard::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 
 	loadTemplates();				
-	Board::loadLayerKin(viewLayers, viewLayerSpec);
+	Board::loadLayerKin(viewLayers, viewLayerPlacement);
 	double w =  m_modelPart->localProp("width").toDouble();
 	if (w != 0) {
 		resizeMM(w, m_modelPart->localProp("height").toDouble(), viewLayers);
@@ -726,28 +886,18 @@ void ResizableBoard::setInitialSize() {
 	}
 }
 
-QString ResizableBoard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
+QString ResizableBoard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	double w = m_modelPart->localProp("width").toDouble();
 	if (w != 0) {
 		double h = m_modelPart->localProp("height").toDouble();
 		QString xml = makeLayerSvg(viewLayerID, w, h, GraphicsUtils::mm2mils(w), GraphicsUtils::mm2mils(h));
 		if (!xml.isEmpty()) {
-			QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
-			SvgFileSplitter splitter;
-			bool result = splitter.splitString(xml, xmlName);
-			if (!result) {
-				return "";
-			}
-			result = splitter.normalize(dpi, xmlName, blackOnly);
-			if (!result) {
-				return "";
-			}
-			return splitter.elementString(xmlName);
+			return PaletteItemBase::normalizeSvg(xml, viewLayerID, blackOnly, dpi, factor);
 		}
 	}
 
-	return Board::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	return Board::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 }
 
 QSizeF ResizableBoard::getSizeMM() {
@@ -762,7 +912,8 @@ QString ResizableBoard::makeLayerSvg(ViewLayer::ViewLayerID viewLayerID, double
 		case ViewLayer::Board:
 			return makeBoardSvg(mmW, mmH, milsW, milsH);
 		case ViewLayer::Silkscreen1:
-			return makeSilkscreenSvg(mmW, mmH, milsW, milsH);
+		case ViewLayer::Silkscreen0:
+			return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
 			break;
 		default:
 			return "";
@@ -771,7 +922,8 @@ QString ResizableBoard::makeLayerSvg(ViewLayer::ViewLayerID viewLayerID, double
 
 QString ResizableBoard::makeNextLayerSvg(ViewLayer::ViewLayerID viewLayerID, double mmW, double mmH, double milsW, double milsH) {
 
-	if (viewLayerID == ViewLayer::Silkscreen1) return makeSilkscreenSvg(mmW, mmH, milsW, milsH);
+	if (viewLayerID == ViewLayer::Silkscreen1) return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
+	if (viewLayerID == ViewLayer::Silkscreen0) return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
 
 	return "";
 }
@@ -786,9 +938,10 @@ QString ResizableBoard::makeBoardSvg(double mmW, double mmH, double milsW, doubl
     return makeSvg(mmW, mmH, BoardLayerTemplates.value(moduleID()));
 }
 
-QString ResizableBoard::makeSilkscreenSvg(double mmW, double mmH, double milsW, double milsH) {
+QString ResizableBoard::makeSilkscreenSvg(ViewLayer::ViewLayerID viewLayerID, double mmW, double mmH, double milsW, double milsH) {
     Q_UNUSED(milsW);
     Q_UNUSED(milsH);
+    if (viewLayerID == ViewLayer::Silkscreen0) return makeSvg(mmW, mmH, Silkscreen0LayerTemplates.value(moduleID()));
     return makeSvg(mmW, mmH, SilkscreenLayerTemplates.value(moduleID()));
 }
 
@@ -841,12 +994,12 @@ void ResizableBoard::getParams(QPointF & p, QSizeF & s) {
 }
 
 bool ResizableBoard::hasCustomSVG() {
-	return theViewIdentifier() == m_viewIdentifier;
+	return theViewID() == m_viewID;
 }
 
-bool ResizableBoard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool ResizableBoard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
-	bool result = Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	bool result = Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 
 	if (prop.compare("shape", Qt::CaseInsensitive) == 0) {
 
@@ -886,7 +1039,8 @@ bool ResizableBoard::collectExtraInfo(QWidget * parent, const QString & family,
 			return true;
 		}
 
-		returnWidget = setUpDimEntry(false, false, returnWidget);
+		returnWidget = setUpDimEntry(false, false, false, returnWidget);
+        returnWidget->setEnabled(swappingEnabled);
 		return true;
 	}
 
@@ -897,6 +1051,18 @@ QStringList ResizableBoard::collectValues(const QString & family, const QString
 	return Board::collectValues(family, prop, value);
 }
 
+void ResizableBoard::paperSizeChanged(int index) {
+    QComboBox * comboBox = qobject_cast<QComboBox *>(sender());
+    if (comboBox == NULL) return;
+
+    QModelIndex modelIndex = comboBox->model()->index(index,0);  
+    QSizeF size = comboBox->model()->data(modelIndex, Qt::UserRole).toSizeF();
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->resizeBoard(size.width(), size.height(), true);
+	}
+}
+
 void ResizableBoard::widthEntry() {
 	QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
 	if (edit == NULL) return;
@@ -936,7 +1102,7 @@ bool ResizableBoard::hasPartNumberProperty()
 
 void ResizableBoard::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-	if (m_hidden) return;
+	if (m_hidden || m_layerHidden) return;
 
 	Board::paintSelected(painter, option, widget);
 
@@ -1003,6 +1169,15 @@ bool ResizableBoard::inResize() {
 	return m_corner != ResizableBoard::NO_CORNER;
 }
 
+void ResizableBoard::figureHover() {
+    setAcceptHoverEvents(true);
+    setAcceptedMouseButtons(ALLMOUSEBUTTONS);
+	foreach(ItemBase * lkpi, m_layerKin) {
+		lkpi->setAcceptHoverEvents(false);
+		lkpi->setAcceptedMouseButtons(Qt::NoButton);
+	}
+}
+
 void ResizableBoard::hoverEnterEvent( QGraphicsSceneHoverEvent * event ) {
 	Board::hoverEnterEvent(event);
 }
@@ -1039,6 +1214,7 @@ ResizableBoard::Corner ResizableBoard::findCorner(QPointF scenePos, Qt::Keyboard
 	Q_UNUSED(modifiers);
 		
 	if (!this->isSelected()) return ResizableBoard::NO_CORNER;
+    if (this->moveLock()) return ResizableBoard::NO_CORNER;
 
 	double d = CornerHandleSize / m_currentScale;
 	double d2 = d * d;
@@ -1081,7 +1257,7 @@ void ResizableBoard::setKinCursor(Qt::CursorShape cursor) {
 	}
 }
 
-QFrame * ResizableBoard::setUpDimEntry(bool includeAspectRatio, bool includeRevert, QWidget * & returnWidget)
+QFrame * ResizableBoard::setUpDimEntry(bool includeAspectRatio, bool includeRevert, bool includePaperSizes, QWidget * & returnWidget)
 {
 	double tens = pow(10.0, m_decimalsAfter);
 	double w = qRound(m_modelPart->localProp("width").toDouble() * tens) / tens;	// truncate to 1 decimal point
@@ -1148,38 +1324,65 @@ QFrame * ResizableBoard::setUpDimEntry(bool includeAspectRatio, bool includeReve
 	vboxLayout->addWidget(subframe1);
 	vboxLayout->addWidget(subframe2);
 
-	if (includeAspectRatio || includeRevert) {
+	if (includeAspectRatio || includeRevert || includePaperSizes) {
 		QFrame * subframe3 = new QFrame();
 		QHBoxLayout * hboxLayout3 = new QHBoxLayout();
 		hboxLayout3->setAlignment(Qt::AlignLeft);
 		hboxLayout3->setContentsMargins(0, 0, 0, 0);
 		hboxLayout3->setSpacing(0);
 	
-        if (includeAspectRatio) {
-		    QLabel * l3 = new QLabel(tr("keep aspect ratio"));
-		    l3->setMargin(0);
-		    l3->setObjectName("infoViewLabel");
-		    QCheckBox * checkBox = new QCheckBox();
-		    checkBox->setChecked(m_keepAspectRatio);
-		    checkBox->setObjectName("infoViewCheckBox");
+		if (includeAspectRatio) {
+			QLabel * l3 = new QLabel(tr("keep aspect ratio"));
+			l3->setMargin(0);
+			l3->setObjectName("infoViewLabel");
+			QCheckBox * checkBox = new QCheckBox();
+			checkBox->setChecked(m_keepAspectRatio);
+			checkBox->setObjectName("infoViewCheckBox");
+
+			hboxLayout3->addWidget(l3);
+			hboxLayout3->addWidget(checkBox);
+			connect(checkBox, SIGNAL(toggled(bool)), this, SLOT(keepAspectRatio(bool)));
+			m_aspectRatioCheck = checkBox;
+			m_aspectRatioLabel = l3;
+	    }
+	    if (includeRevert) {
+			QPushButton * pb = new QPushButton(tr("Revert"));
+			pb->setObjectName("infoViewButton");
+			hboxLayout3->addWidget(pb);
+			connect(pb, SIGNAL(clicked(bool)), this, SLOT(revertSize(bool)));
+			double w = modelPart()->localProp("width").toDouble();
+			double ow = modelPart()->localProp("originalWidth").toDouble();
+			double h = modelPart()->localProp("height").toDouble();
+			double oh = modelPart()->localProp("originalHeight").toDouble();
+			pb->setEnabled(qAbs(w - ow) > JND || qAbs(h - oh) > JND);
+			m_revertButton = pb;
+	    }
+        if (includePaperSizes) {
+            initPaperSizes();
+
+			QLabel * l3 = new QLabel(tr("size"));
+			l3->setMargin(0);
+			l3->setObjectName("infoViewLabel");
+
+            m_paperSizeComboBox = new QComboBox();
+	        m_paperSizeComboBox->setObjectName("infoViewComboBox");
+	        m_paperSizeComboBox->setEditable(false);
+            m_paperSizeComboBox->setMinimumWidth(150);
+            m_paperSizeComboBox->addItem(tr("custom"));
+            for (int i = 0; i < PaperSizeNames.count(); i++) {
+                QSizeF dim = PaperSizeDimensions.at(i);
+                m_paperSizeComboBox->addItem(PaperSizeNames.at(i), dim);
+            }
+
+            QModelIndex modelIndex = m_paperSizeComboBox->model()->index(0,0);  
+            m_paperSizeComboBox->model()->setData(modelIndex, 0, Qt::UserRole - 1);           // to make it selectable again use Qt::ItemIsSelectable | Qt::ItemIsEnabled)
+
+            updatePaperSizes(w, h);
+            connect(m_paperSizeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSizeChanged(int)));
+
+            hboxLayout3->addWidget(l3);
+            hboxLayout3->addWidget(m_paperSizeComboBox);
 
-		    hboxLayout3->addWidget(l3);
-		    hboxLayout3->addWidget(checkBox);
- 		    connect(checkBox, SIGNAL(toggled(bool)), this, SLOT(keepAspectRatio(bool)));
-		    m_aspectRatioCheck = checkBox;
-            m_aspectRatioLabel = l3;
-       }
-        if (includeRevert) {
-            QPushButton * pb = new QPushButton(tr("Revert"));
-            pb->setObjectName("infoViewButton");
-            hboxLayout3->addWidget(pb);
-            connect(pb, SIGNAL(clicked(bool)), this, SLOT(revertSize(bool)));
-            double w = modelPart()->localProp("width").toDouble();
-            double ow = modelPart()->localProp("originalWidth").toDouble();
-            double h = modelPart()->localProp("height").toDouble();
-            double oh = modelPart()->localProp("originalHeight").toDouble();
-            pb->setEnabled(qAbs(w - ow) > JND || qAbs(h - oh) > JND);
-            m_revertButton = pb;
         }
 
 		subframe3->setLayout(hboxLayout3);
@@ -1220,6 +1423,7 @@ void ResizableBoard::setWidthAndHeight(double w, double h)
 	if (m_heightEditor) {
 		m_heightEditor->setText(QString::number(h));
 	}
+    updatePaperSizes(w, h);
 }
 
 QString ResizableBoard::getShapeForRenderer(const QString & svg, ViewLayer::ViewLayerID viewLayerID) 
@@ -1264,3 +1468,30 @@ void ResizableBoard::revertSize(bool) {
 	}
 }
 
+void ResizableBoard::updatePaperSizes(double w, double h) {
+    if (m_paperSizeComboBox ==  NULL) return;
+
+    int currentIndex = 0;
+    for (int i = 0; i < PaperSizeNames.count(); i++) {
+        QSizeF dim = PaperSizeDimensions.at(i);
+        if (qAbs(w - dim.width()) < 0.1 && qAbs(h - dim.height()) < 0.1) {
+            currentIndex = i + 1;
+        }
+    }
+
+    QString custom = tr("custom");
+    if (currentIndex == 0) {
+        custom =  QString("%1x%2").arg(w).arg(h);
+    }
+    m_paperSizeComboBox->setItemText(0, custom);
+    m_paperSizeComboBox->setCurrentIndex(currentIndex);
+}
+
+void ResizableBoard::initPaperSizes() {
+    if (PaperSizeNames.count() == 0) {
+        PaperSizeNames << tr("A0 (1030x1456)") << tr("A1 (728x1030)") << tr("A2 (515x728)") << tr("A3 (364x515)") << tr("A4 (257x364)") << tr("A5 (182x257)") << tr("A6 (128x182)") 
+            << tr("Letter (8.5x11)") << tr("Legal (8.5x14)") << tr("Ledger (17x11)") << tr("Tabloid (11x17)");
+        PaperSizeDimensions << QSizeF(1030,1456) << QSizeF(728,1030) << QSizeF(515,728) << QSizeF(364,515) << QSizeF(257,364) << QSizeF(182,257) << QSizeF(128,182) 
+            << QSizeF(215.9,279.4) << QSizeF(215.9,355.6) << QSizeF(432,279) << QSizeF(279,432);
+    }
+}
\ No newline at end of file
