@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -34,6 +34,7 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 #include "../connectors/connectoritem.h"
 #include "../connectors/busshared.h"
 #include "../connectors/connectorshared.h"
+#include "../debugdialog.h"
 
 #include <QCursor>
 #include <QBitmap>
@@ -50,10 +51,12 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 static QCursor * SpotFaceCutterCursor = NULL;
 static QCursor * MagicWandCursor = NULL;
 
-static bool ShiftDown;
+static bool ShiftDown = false;
 static QPointF OriginalShiftPos;
+static bool ShiftX = false;
+static bool ShiftY = false;
 static bool SpaceBarWasPressed = false;
-
+static const double MinMouseMove = 2;
 
 /////////////////////////////////////////////////////////////////////
 
@@ -86,7 +89,7 @@ Stripbit::Stripbit(const QPainterPath & path, ConnectorItem * connectorItem, int
 	m_connectorItem = connectorItem;
 	m_inHover = m_removed = false;
 
-	setAcceptsHoverEvents(true);
+	setAcceptHoverEvents(true);
 	setAcceptedMouseButtons(Qt::LeftButton);
 	setFlag(QGraphicsItem::ItemIsMovable, true);
 	setFlag(QGraphicsItem::ItemIsSelectable, false);
@@ -133,8 +136,8 @@ void Stripbit::mousePressEvent(QGraphicsSceneMouseEvent *event)
 	}
 
 	if (event->modifiers() & Qt::ShiftModifier) {
-		DebugDialog::debug("got shift down");
 		ShiftDown = true;
+        ShiftX = ShiftY = false;
 		OriginalShiftPos = event->scenePos();
 	}
 
@@ -168,19 +171,30 @@ void Stripbit::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 	Stripbit * other = NULL;
 	QPointF p = event->scenePos();
 	if (ShiftDown) {
-		if (qAbs(p.x() - OriginalShiftPos.x()) >= qAbs(p.y() - OriginalShiftPos.y())) {
+		if (ShiftX) {
 			// moving along x, constrain y
 			p.setY(OriginalShiftPos.y());
 		}
-		else {
+		else if (ShiftY) {
 			// moving along y, constrain x
 			p.setX(OriginalShiftPos.x());
 		}
+        else {
+            double dx = qAbs(p.x() - OriginalShiftPos.x());
+            double dy = qAbs(p.y() - OriginalShiftPos.y());
+            if (dx - dy > MinMouseMove) {
+                ShiftX = true;
+            }
+            else if (dy - dx > MinMouseMove) {
+                ShiftY = true;
+            }
+        }
 	}
 
 
 	if (!ShiftDown && (event->modifiers() & Qt::ShiftModifier)) {
 		ShiftDown = true;
+        ShiftX = ShiftY = false;
 		OriginalShiftPos = event->scenePos();
 	}
 
@@ -270,10 +284,10 @@ int Stripbit::x() {
 
 /////////////////////////////////////////////////////////////////////
 
-Stripboard::Stripboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: Perfboard(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Stripboard::Stripboard( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: Perfboard(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
-	if (!viewIdentifier == ViewIdentifierClass::BreadboardView) return;
+	if (!viewID == ViewLayer::BreadboardView) return;
 
 	int x, y;
 	getXY(x, y, m_size);
@@ -296,9 +310,9 @@ Stripboard::Stripboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifi
 Stripboard::~Stripboard() {
 }
 
-QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
+QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
 {
-	QString svg = Perfboard::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	QString svg = Perfboard::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 	if (svg.isEmpty()) return svg;
 	if (!svg.contains(GerberGenerator::MagicBoardOutlineID)) return svg;
 
@@ -347,9 +361,9 @@ QString Stripboard::genFZP(const QString & moduleid)
 	return fzp;
 }
 
-bool Stripboard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool Stripboard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
-	return Perfboard::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return Perfboard::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 void Stripboard::addedToScene(bool temporary)
@@ -564,7 +578,7 @@ void Stripboard::reinitBuses(bool triggerUndo)
 
 		connectorItem->restoreColor(true, 0, true);
 		if (connectorItem->bus()) {
-			this->busConnectorItems(connectorItem->bus(), visited);
+			this->busConnectorItems(connectorItem->bus(), connectorItem, visited);
 		}
 	}
 }
