@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -30,46 +30,80 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 #include "paletteitem.h"
 
+/*
+#include <QTime>
+
+class FocusBugLineEdit : public QLineEdit {
+    Q_OBJECT
+
+public:
+	FocusBugLineEdit(QWidget * parent = NULL);
+	~FocusBugLineEdit();
+
+signals:
+	void safeEditingFinished();
+
+protected slots:
+	void editingFinishedSlot();
+	
+protected:
+	QTime m_lastEditingFinishedEmit; 					
+
+};
+*/
+
 class SymbolPaletteItem : public PaletteItem 
 {
 	Q_OBJECT
 
 public:
-	SymbolPaletteItem(ModelPart *, ViewIdentifierClass::ViewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
+	SymbolPaletteItem(ModelPart *, ViewLayer::ViewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel);
 	~SymbolPaletteItem();
 
 	ConnectorItem* newConnectorItem(class Connector *connector);
-	void busConnectorItems(class Bus * bus, QList<ConnectorItem *> & items);
+	void busConnectorItems(class Bus * bus, ConnectorItem *, QList<ConnectorItem *> & items);
 	double voltage();
 	void setProp(const QString & prop, const QString & value);
 	void setVoltage(double);
-	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget);
+	bool collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide);
 	QString getProperty(const QString & key);
 	ConnectorItem * connector0();
 	ConnectorItem * connector1();
-	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi);
+	QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
 	PluralType isPlural();
 	void addedToScene(bool temporary);
 	bool hasPartNumberProperty();
+	bool isOnlyNetLabel();
+	bool hasPartLabel();
+	bool getAutoroutable();
+	void setAutoroutable(bool);
+    void setLabel(const QString &);
+    QString getLabel();
+    QString getDirection();
 
 public:
 	static double DefaultVoltage;
 
 public slots:
 	void voltageEntry(const QString & text);
+	void labelEntry();
 
 protected:
 	void removeMeFromBus(double voltage);
 	double useVoltage(ConnectorItem * connectorItem);
-	QString makeSvg();
+	QString makeSvg(ViewLayer::ViewLayerID);
+	QString makeNetLabelSvg(ViewLayer::ViewLayerID);
 	QString replaceTextElement(QString svg);
-    ViewIdentifierClass::ViewIdentifier useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier, bool swappingEnabled);
+    ViewLayer::ViewID useViewIDForPixmap(ViewLayer::ViewID, bool swappingEnabled);
+    QString retrieveNetLabelSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
+    void resetLayerKin();
 
 protected:
 	double m_voltage;
 	QPointer<ConnectorItem> m_connector0;
 	QPointer<ConnectorItem> m_connector1;
 	bool m_voltageReference;
+	bool m_isNetLabel;
 };
 
 #endif
