@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6146 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-06 08:47:09 +0200 (Fri, 06 Jul 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -37,8 +37,8 @@ const int TraceWire::MaxTraceWidthMils = 128;
 
 /////////////////////////////////////////////////////////
 
-TraceWire::TraceWire( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu  ) 
-	: ClipableWire(modelPart, viewIdentifier,  viewGeometry,  id, itemMenu, true)
+TraceWire::TraceWire( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry & viewGeometry, long id, QMenu * itemMenu  ) 
+	: ClipableWire(modelPart, viewID,  viewGeometry,  id, itemMenu, true)
 {
 	m_canChainMultiple = true;
 	m_wireDirection = TraceWire::NoDirection;
@@ -78,7 +78,7 @@ QComboBox * TraceWire::createWidthComboBox(double m, QWidget * parent)
 }
 
 
-bool TraceWire::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool TraceWire::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare("width", Qt::CaseInsensitive) == 0) {
 		returnProp = tr("width");
@@ -93,7 +93,20 @@ bool TraceWire::collectExtraInfo(QWidget * parent, const QString & family, const
 		return true;
 	}
 
-	return ClipableWire::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	bool result =  ClipableWire::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
+    if (prop.compare("layer") == 0 && returnWidget != NULL) {
+        bool disabled = !canSwitchLayers();
+        if (!disabled) {
+            InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+            if (infoGraphicsView == NULL || infoGraphicsView->boardLayers() == 1) disabled = true;
+        }
+        returnWidget->setDisabled(disabled);
+    }
+
+    return result;
+
+
+    return result;
 }
 
 
@@ -171,3 +184,33 @@ TraceWire * TraceWire::getTrace(ConnectorItem * connectorItem)
 void TraceWire::setSchematic(bool schematic) {
 	m_viewGeometry.setSchematicTrace(schematic);
 }
+
+QHash<QString, QString> TraceWire::prepareProps(ModelPart * modelPart, bool wantDebug, QStringList & keys) 
+{
+    QHash<QString, QString> props = ClipableWire::prepareProps(modelPart, wantDebug, keys);
+
+    if (m_viewID != ViewLayer::PCBView) return props;
+    if (!m_viewGeometry.getPCBTrace()) return props;
+
+    keys.append("layer");
+    props.insert("layer", ViewLayer::topLayers().contains(m_viewLayerID) ? "top" : "bottom");
+    return props;
+}
+
+QStringList TraceWire::collectValues(const QString & family, const QString & prop, QString & value) {
+    if (prop.compare("layer") == 0) {
+        QStringList values = ClipableWire::collectValues(family, prop, value);
+        if (values.count() == 0) {
+            values << TranslatedPropertyNames.value("bottom") << TranslatedPropertyNames.value("top");
+            if (ViewLayer::bottomLayers().contains(m_viewLayerID)) {
+                value = values.at(0);
+            }
+            else {
+                value = values.at(1);
+            }
+        }
+        return values;
+    }
+
+    return ClipableWire::collectValues(family, prop, value);
+}
