@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6273 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 21:40:57 +0200 (Thu, 09 Aug 2012) $
+$Revision: 7000 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-29 07:24:08 +0200 (Mo, 29. Apr 2013) $
 
 ********************************************************************/
 
@@ -28,18 +28,6 @@ $Date: 2012-08-09 21:40:57 +0200 (Thu, 09 Aug 2012) $
 
 curvy To Do
 
-	* pixel turds
-
-	* undo/redo
-
-	* save/load
-
-	* copy/paste
-
-	* make-straight function
-
-	* export
-
 	curvy to begin with? would have to vary with some function of angle and distance
 		could convert control points to t values?
 
@@ -75,6 +63,8 @@ later:
 #include <QComboBox>
 #include <QToolTip>
 #include <QApplication>
+#include <QCheckBox>
+#include <QHBoxLayout>
 
 #include "../debugdialog.h"
 #include "../sketch/infographicsview.h"
@@ -93,6 +83,10 @@ later:
 
 #include <stdlib.h>
 
+QVector<qreal> Wire::TheDash;
+QBrush BandedBrush(QColor(255, 255, 255));
+
+
 QHash<QString, QString> Wire::colorTrans;
 QStringList Wire::colorNames;
 QHash<int, QString> Wire::widthTrans;
@@ -136,11 +130,12 @@ Wire * WireAction::wire() {
 
 /////////////////////////////////////////////////////////////
 
-Wire::Wire( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier,  const ViewGeometry & viewGeometry, long id, QMenu* itemMenu, bool initLabel)
-	: ItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
+Wire::Wire( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry & viewGeometry, long id, QMenu* itemMenu, bool initLabel)
+	: ItemBase(modelPart, viewID, viewGeometry, id, itemMenu)
 {
+    m_banded = false;
 	m_bezier = NULL;
-	m_canHaveCurve = true;
+	m_displayBendpointCursor = m_canHaveCurve = true;
 	m_hoverStrokeWidth = DefaultHoverStrokeWidth;
 	m_connector0 = m_connector1 = NULL;
 	m_partLabel = initLabel ? new PartLabel(this, NULL) : NULL;
@@ -170,7 +165,7 @@ Wire::~Wire() {
 
 FSvgRenderer * Wire::setUp(ViewLayer::ViewLayerID viewLayerID, const LayerHash &  viewLayers, InfoGraphicsView * infoGraphicsView) {
 	ItemBase::setViewLayerID(viewLayerID, viewLayers);
-	FSvgRenderer * svgRenderer = setUpConnectors(m_modelPart, m_viewIdentifier);
+	FSvgRenderer * svgRenderer = setUpConnectors(m_modelPart, m_viewID);
 	if (svgRenderer != NULL) {
 		initEnds(m_viewGeometry, svgRenderer->viewBox(), infoGraphicsView);
 	}
@@ -297,7 +292,24 @@ void Wire::paintBody(QPainter * painter, const QStyleOptionGraphicsItem * option
 		painter->restore();
 	}
 	   
-	// DebugDialog::debug(QString("pen width %1 %2").arg(m_pen.widthF()).arg(m_viewIdentifier));
+	// DebugDialog::debug(QString("pen width %1 %2").arg(m_pen.widthF()).arg(m_viewID));
+
+    if (m_banded) {
+        QBrush brush = m_pen.brush();
+        m_pen.setStyle(Qt::SolidLine);
+        m_pen.setBrush(BandedBrush);
+        painter->setPen(m_pen);
+	    if (painterPath.isEmpty()) {
+		    painter->drawLine(getPaintLine());	
+	    }
+	    else {
+		    painter->drawPath(painterPath);
+	    }
+        m_pen.setBrush(brush);
+        m_pen.setDashPattern(TheDash);
+        m_pen.setCapStyle(Qt::FlatCap);
+    }
+
 	painter->setPen(m_pen);
 	if (painterPath.isEmpty()) {
 		painter->drawLine(getPaintLine());	
@@ -305,6 +317,11 @@ void Wire::paintBody(QPainter * painter, const QStyleOptionGraphicsItem * option
 	else {
 		painter->drawPath(painterPath);
 	}
+
+    if (m_banded) {
+        m_pen.setStyle(Qt::SolidLine);
+        m_pen.setCapStyle(Qt::RoundCap);
+    }
 }
 
 void Wire::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) 
@@ -432,6 +449,7 @@ void Wire::initDragEnd(ConnectorItem * connectorItem, QPointF scenePos) {
 	saveGeometry();
 	QLineF line = this->line();
 	m_drag0 = (connectorItem == m_connector0);
+    //debugInfo("setting drag end to true");
 	m_dragEnd = true;
 	m_dragCurve = false;
 	if (m_drag0) {
@@ -487,6 +505,8 @@ void Wire::mouseMoveEventAux(QPointF eventPos, Qt::KeyboardModifiers modifiers)
 		return;
 	}
 
+    //debugInfo("dragging wire");
+
 	ConnectorItem * whichConnectorItem;
 	ConnectorItem * otherConnectorItem;
 	if (m_drag0) {
@@ -646,6 +666,7 @@ void Wire::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) {
 		return;
 	}
 
+    //debugInfo("wire release drag");
 	if (releaseDrag()) return;
 
 	ItemBase::mouseReleaseEvent(event);
@@ -665,6 +686,7 @@ bool Wire::releaseDrag() {
 		return true;
 	}
 
+    //debugInfo("clearing drag end");
 	m_dragEnd = false;
 
 	ConnectorItem * from = (m_drag0) ? m_connector0 : m_connector1;
@@ -701,6 +723,7 @@ void Wire::writeGeometry(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeAttribute("mils", QString::number(mils()));
 	streamWriter.writeAttribute("color", m_pen.brush().color().name());
 	streamWriter.writeAttribute("opacity", QString::number(m_opacity));
+	streamWriter.writeAttribute("banded", m_banded ? "1" : "0");
 	if (m_bezier) m_bezier->write(streamWriter);
 	streamWriter.writeEndElement();
 }
@@ -722,6 +745,8 @@ void Wire::setExtras(const QDomElement & element, InfoGraphicsView * infoGraphic
 		}
 	}
 
+    m_banded = (element.attribute("banded", "") == "1");
+
 	setColorFromElement(element);
 	QDomElement bElement = element.firstChildElement("bezier");
 	Bezier bezier = Bezier::fromElement(bElement);
@@ -863,7 +888,7 @@ void Wire::simpleConnectedMoved(ConnectorItem * from, ConnectorItem * to)
 
 	this->setPos(p1);
 	this->setLine(0,0, p2.x() - p1.x(), p2.y() - p1.y() );
-	//DebugDialog::debug(QString("set line %5: %1 %2, %3 %4, vis:%6 lyr:%7").arg(p1.x()).arg(p1.y()).arg(p2.x()).arg(p2.y()).arg(id()).arg(isVisible()).arg(m_viewIdentifier) );
+	//DebugDialog::debug(QString("set line %5: %1 %2, %3 %4, vis:%6 lyr:%7").arg(p1.x()).arg(p1.y()).arg(p2.x()).arg(p2.y()).arg(id()).arg(isVisible()).arg(m_viewID) );
 	setConnector1Rect();
 }
 
@@ -892,7 +917,7 @@ void Wire::calcNewLine(ConnectorItem * from, ConnectorItem * to, QPointF & p1, Q
 	}
 }
 
-void Wire::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
+void Wire::connectedMoved(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & already) {
 	// "from" is the connector on the part
 	// "to" is the connector on the wire
 
@@ -927,26 +952,29 @@ void Wire::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
 	}
 	this->setPos(p1);
 	this->setLine(0,0, p2.x() - p1.x(), p2.y() - p1.y() );
-	//DebugDialog::debug(QString("set line %5: %1 %2, %3 %4, vis:%6 lyr:%7").arg(p1.x()).arg(p1.y()).arg(p2.x()).arg(p2.y()).arg(id()).arg(isVisible()).arg(m_viewIdentifier) );
+	//DebugDialog::debug(QString("set line %5: %1 %2, %3 %4, vis:%6 lyr:%7").arg(p1.x()).arg(p1.y()).arg(p2.x()).arg(p2.y()).arg(id()).arg(isVisible()).arg(m_viewID) );
 	setConnector1Rect();
 
 	if (chained) {
 		foreach (ConnectorItem * otherEndTo, otherEnd->connectedToItems()) {
 			if (otherEndTo->attachedToItemType() == ModelPart::Wire) {
-				otherEndTo->attachedTo()->connectedMoved(otherEnd, otherEndTo);
+				otherEndTo->attachedTo()->connectedMoved(otherEnd, otherEndTo, already);
 			}
 		}
 	}
 }
 
 
-FSvgRenderer * Wire::setUpConnectors(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier) 
+FSvgRenderer * Wire::setUpConnectors(ModelPart * modelPart, ViewLayer::ViewID viewID) 
 {
 	clearConnectorItemCache();
 
-	QString error;
 	LayerAttributes layerAttributes;
-	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, viewIdentifier, m_viewLayerID, m_viewLayerSpec, layerAttributes, error);
+    layerAttributes.viewID = viewID;
+    layerAttributes.viewLayerID = m_viewLayerID;
+    layerAttributes.viewLayerPlacement = m_viewLayerPlacement;
+    layerAttributes.createShape = layerAttributes.doConnectors = false;
+	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, layerAttributes);
 	if (renderer == NULL) {
 		return NULL;
 	}
@@ -954,16 +982,16 @@ FSvgRenderer * Wire::setUpConnectors(ModelPart * modelPart, ViewIdentifierClass:
 	foreach (Connector * connector, modelPart->connectors().values()) {
 		if (connector == NULL) continue;
 
-		SvgIdLayer * svgIdLayer = connector->fullPinInfo(viewIdentifier, m_viewLayerID);
+		SvgIdLayer * svgIdLayer = connector->fullPinInfo(viewID, m_viewLayerID);
 		if (svgIdLayer == NULL) continue;
 
-		bool result = renderer->setUpConnector(svgIdLayer, false);
+		bool result = renderer->setUpConnector(svgIdLayer, false, viewLayerPlacement());
 		if (!result) continue;
 
 		ConnectorItem * connectorItem = newConnectorItem(connector);
-		connectorItem->setRect(svgIdLayer->m_rect);
-		connectorItem->setTerminalPoint(svgIdLayer->m_point);
-		m_originalConnectorRect = svgIdLayer->m_rect;
+		connectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
+		connectorItem->setTerminalPoint(svgIdLayer->point(viewLayerPlacement()));
+		m_originalConnectorRect = svgIdLayer->rect(viewLayerPlacement());
 
 		connectorItem->setCircular(true);
 		//DebugDialog::debug(QString("terminal point %1 %2").arg(terminalPoint.x()).arg(terminalPoint.y()) );
@@ -1081,10 +1109,6 @@ void Wire::setRatsnest(bool ratsnest) {
 	m_viewGeometry.setRatsnest(ratsnest);
 }
 
-bool Wire::getRatsnest() {
-	return m_viewGeometry.getRatsnest();
-}
-
 void Wire::setAutoroutable(bool ar) {
 	m_viewGeometry.setAutoroutable(ar);
 }
@@ -1148,7 +1172,7 @@ void Wire::setColorString(QString colorName, double op) {
 	// sets a color using the name (.e. "red")
 	// note: colorName is associated with a Fritzing color, not a Qt color
 
-	QString colorString = RatsnestColors::wireColor(m_viewIdentifier, colorName);     
+	QString colorString = RatsnestColors::wireColor(m_viewID, colorName);     
 	if (colorString.isEmpty()) {
 		colorString = colorName;
 	}
@@ -1158,7 +1182,7 @@ void Wire::setColorString(QString colorName, double op) {
 	setColor(c, op);
 	m_colorName = colorName;
 
-	QString shadowColorString = RatsnestColors::shadowColor(m_viewIdentifier, colorName);
+	QString shadowColorString = RatsnestColors::shadowColor(m_viewID, colorName);
 	if (shadowColorString.isEmpty()) {
 		shadowColorString = colorString;
 	}
@@ -1182,6 +1206,9 @@ QString Wire::colorString() {
 void Wire::initNames() {
 	if (colorNames.count() > 0) return;
 
+    TheDash.clear();
+    TheDash << 10 << 8;
+
 	widths << 8 << 12 << 16 << 24 << 32 << 48;
     int i = 0;
 	widthTrans.insert(widths[i++], tr("super fine (8 mil)"));
@@ -1209,6 +1236,7 @@ void Wire::initNames() {
 	colorNames.append(tr("orange"));
     colorNames.append(tr("brown"));
     colorNames.append(tr("purple"));
+    colorNames.append(tr("pink"));
 
 	// need this hash table to translate from user's language to internal color name
     colorTrans.insert(tr("blue"), "blue");
@@ -1221,6 +1249,7 @@ void Wire::initNames() {
 	colorTrans.insert(tr("orange"), "orange");
 	colorTrans.insert(tr("brown"), "brown");
     colorTrans.insert(tr("purple"), "purple");
+    colorTrans.insert(tr("pink"), "pink");
 }
 
 bool Wire::hasFlag(ViewGeometry::WireFlag flag)
@@ -1273,29 +1302,84 @@ bool Wire::canChangeColor() {
 	if (getRatsnest()) return false;
 	if (!getTrace()) return true;
 
-	return (this->m_viewIdentifier == ViewIdentifierClass::SchematicView);
+	return (this->m_viewID == ViewLayer::SchematicView);
 }
 
 void Wire::collectDirectWires(QList<Wire *> & wires) {
+    bool firstRound = false;
 	if (!wires.contains(this)) {
 		wires.append(this);
+        firstRound = true;
 	}
 
-	collectDirectWires(m_connector0, wires);
-	collectDirectWires(m_connector1, wires);
+    QList<ConnectorItem *> junctions;
+    if (firstRound) {
+        // collect up to any junction
+	    collectDirectWires(m_connector0, wires, junctions);
+	    collectDirectWires(m_connector1, wires, junctions);
+        return;
+    }
+
+    // second round: deal with junctions
+    foreach (Wire * wire, wires) {
+        junctions << wire->connector0() << wire->connector1();
+    }
+
+    int ix = 0;
+    while (ix < junctions.count()) {
+        ConnectorItem * junction = junctions.at(ix++);
+
+        QSet<Wire *> jwires;
+        foreach (ConnectorItem * toConnectorItem, junction->connectedToItems()) {
+            if (toConnectorItem->attachedToItemType() != ModelPart::Wire) break;
+
+            Wire * w = qobject_cast<Wire *>(toConnectorItem->attachedTo());
+            if (!wires.contains(w)) jwires << w;
+
+            bool onlyWiresConnected = true;
+            foreach (ConnectorItem * toToConnectorItem, toConnectorItem->connectedToItems()) {
+                if (toToConnectorItem->attachedToItemType() != ModelPart::Wire) {
+                    onlyWiresConnected = false;
+                    break;
+                }
+
+                w = qobject_cast<Wire *>(toToConnectorItem->attachedTo());
+                if (!wires.contains(w)) jwires << w;
+            }
+            if (!onlyWiresConnected) break;
+        }
+
+        if (jwires.count() == 1) {
+            // there is a junction of > 2 wires and all wires leading to it except one are already on the delete list
+            Wire * w = jwires.values().at(0);
+            wires << w;
+            w->collectDirectWires(w->connector0(), wires, junctions);
+            w->collectDirectWires(w->connector1(), wires, junctions);
+        }
+    }
 }
 
-void Wire::collectDirectWires(ConnectorItem * connectorItem, QList<Wire *> & wires) {
-	if (connectorItem->connectionsCount() != 1) return;
 
-	ConnectorItem * toConnectorItem = connectorItem->connectedToItems()[0];
+void Wire::collectDirectWires(ConnectorItem * connectorItem, QList<Wire *> & wires, QList<ConnectorItem *> & junctions) {
+	if (connectorItem->connectionsCount() == 0) return;
+    if (connectorItem->connectionsCount() > 1) {
+        if (!junctions.contains(connectorItem)) junctions.append(connectorItem);
+        return;
+    }
+
+	ConnectorItem * toConnectorItem = connectorItem->connectedToItems().at(0);
 	if (toConnectorItem->attachedToItemType() != ModelPart::Wire) return;
 
+    if (toConnectorItem->connectionsCount() != 1) {
+        if (!junctions.contains(connectorItem)) junctions.append(connectorItem);
+        return;
+    }
+
 	Wire * nextWire = qobject_cast<Wire *>(toConnectorItem->attachedTo());
 	if (wires.contains(nextWire)) return;
 
 	wires.append(nextWire);
-	nextWire->collectDirectWires(nextWire->otherConnector(toConnectorItem), wires);
+	nextWire->collectDirectWires(nextWire->otherConnector(toConnectorItem), wires, junctions);
 }
 
 QVariant Wire::itemChange(GraphicsItemChange change, const QVariant &value)
@@ -1422,7 +1506,7 @@ bool Wire::connectionIsAllowed(ConnectorItem * to) {
 
 	if (w->getRatsnest()) return false;
 
-	return m_viewIdentifier != ViewIdentifierClass::BreadboardView;
+	return m_viewID != ViewLayer::BreadboardView;
 }
 
 bool Wire::isGrounded() {
@@ -1445,7 +1529,7 @@ void Wire::setIgnoreSelectionChange(bool ignore) {
 	m_ignoreSelectionChange = ignore;
 }
 
-bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
 	if (prop.compare("width", Qt::CaseInsensitive) == 0) {
 		// don't display width property
@@ -1464,7 +1548,7 @@ bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QStr
 			QString englishCurrColor = colorString();
 			foreach(QString transColorName, Wire::colorNames) {
 				QString englishColorName = Wire::colorTrans.value(transColorName);
-				bool ok = (this->m_viewIdentifier != ViewIdentifierClass::SchematicView || englishColorName.compare("white", Qt::CaseInsensitive) != 0);
+				bool ok = (this->m_viewID != ViewLayer::SchematicView || englishColorName.compare("white", Qt::CaseInsensitive) != 0);
 				if (ok) {
 					comboBox->addItem(transColorName, QVariant(englishColorName));
 					if (englishColorName.compare(englishCurrColor, Qt::CaseInsensitive) == 0) {
@@ -1475,7 +1559,25 @@ bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QStr
 			}
 
 			connect(comboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(colorEntry(const QString &)));
-			returnWidget = comboBox;
+
+            if (this->hasShadow()) {
+                QCheckBox * checkBox = new QCheckBox(tr("Banded"));
+		        checkBox->setChecked(m_banded);
+		        checkBox->setObjectName("infoViewCheckBox");
+                connect(checkBox, SIGNAL(clicked(bool)), this, SLOT(setBandedProp(bool)));
+
+                QFrame * frame = new QFrame(parent);
+                QHBoxLayout * hboxLayout = new QHBoxLayout;
+                hboxLayout->addWidget(comboBox);
+                hboxLayout->addWidget(checkBox);
+                frame->setLayout(hboxLayout);
+
+                returnWidget = frame;
+            }
+            else {
+		        returnWidget = comboBox;
+            }
+	
 			returnValue = comboBox->currentText();
 			return true;
 		}
@@ -1486,7 +1588,7 @@ bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QStr
 		}
 	}
 
-	return ItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return ItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 void Wire::colorEntry(const QString & text) {
@@ -1554,19 +1656,7 @@ void Wire::addedToScene(bool temporary) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView == NULL) return;
 
-	bool succeeded = connect(this, SIGNAL(wireChangedSignal(Wire*, const QLineF & , const QLineF & , QPointF, QPointF, ConnectorItem *, ConnectorItem *)	),
-			infoGraphicsView, SLOT(wireChangedSlot(Wire*, const QLineF & , const QLineF & , QPointF, QPointF, ConnectorItem *, ConnectorItem *)),
-			Qt::DirectConnection);		// DirectConnection means call the slot directly like a subroutine, without waiting for a thread or queue
-	succeeded = connect(this, SIGNAL(wireChangedCurveSignal(Wire*, const Bezier *, const Bezier *, bool)),
-			infoGraphicsView, SLOT(wireChangedCurveSlot(Wire*, const Bezier *, const Bezier *, bool)),
-			Qt::DirectConnection);		// DirectConnection means call the slot directly like a subroutine, without waiting for a thread or queue
-	succeeded = succeeded && connect(this, SIGNAL(wireSplitSignal(Wire*, QPointF, QPointF, const QLineF & )),
-			infoGraphicsView, SLOT(wireSplitSlot(Wire*, QPointF, QPointF, const QLineF & )));
-	succeeded = succeeded && connect(this, SIGNAL(wireJoinSignal(Wire*, ConnectorItem *)),
-			infoGraphicsView, SLOT(wireJoinSlot(Wire*, ConnectorItem*)));
-	if (!succeeded) {
-		DebugDialog::debug("wire signal connect failed");
-	}
+	infoGraphicsView->newWire(this);
 }
 
 void Wire::setConnectorDimensions(double width, double height) 
@@ -1657,7 +1747,7 @@ void Wire::setPen(const QPen &pen)
 }
 
 bool Wire::canHaveCurve() {
-	return m_canHaveCurve && (m_viewIdentifier == ViewIdentifierClass::BreadboardView);
+	return m_canHaveCurve && !getRatsnest();
 }
 
 void Wire::dragCurve(QPointF eventPos, Qt::KeyboardModifiers)
@@ -1755,7 +1845,7 @@ void Wire::updateCursor(Qt::KeyboardModifiers modifiers)
 	else if (infoGraphicsView != NULL && infoGraphicsView->curvyWiresIndicated(modifiers)) {
 		CursorMaster::instance()->addCursor(this, *CursorMaster::MakeCurveCursor);
 	}
-	else {
+	else if (m_displayBendpointCursor) {
 		CursorMaster::instance()->addCursor(this, *CursorMaster::NewBendpointCursor);
 	}
 }
@@ -1765,11 +1855,47 @@ bool Wire::canChainMultiple()
 	return m_canChainMultiple;
 }
 
-ViewIdentifierClass::ViewIdentifier Wire::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier vid, bool) 
+ViewLayer::ViewID Wire::useViewIDForPixmap(ViewLayer::ViewID vid, bool) 
 {
-    if (vid == ViewIdentifierClass::BreadboardView) {
-        return ViewIdentifierClass::IconView;
+    if (vid == ViewLayer::BreadboardView) {
+        return ViewLayer::IconView;
     }
 
-    return ViewIdentifierClass::UnknownView;
+    return ViewLayer::UnknownView;
 }
+
+void Wire::setDisplayBendpointCursor(bool dbc) {
+	m_displayBendpointCursor = dbc;
+}
+
+bool Wire::banded() {
+    return m_banded;
+}
+
+void Wire::setBanded(bool banded) {
+    m_banded = banded;
+	QList<Wire *> chained;
+	QList<ConnectorItem *> ends;
+	collectChained(chained, ends);
+    foreach (Wire * w, chained) {
+        w->m_banded = banded;
+        w->update();
+    }
+}
+
+void Wire::setBandedProp(bool banded) {
+   	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->setProp(this, "banded", ItemBase::TranslatedPropertyNames.value("banded"), m_banded ? "Yes" : "No", banded  ? "Yes" : "No", true);
+    }
+}
+
+void Wire::setProp(const QString & prop, const QString & value) {
+	if (prop.compare("banded", Qt::CaseInsensitive) == 0) {
+		setBanded(value == "Yes");
+		return;
+	}
+
+	ItemBase::setProp(prop, value);
+}
+
