@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6954 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-05 10:22:00 +0200 (Fr, 05. Apr 2013) $
 
 ********************************************************************/
 
@@ -32,7 +32,6 @@ $Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 #include <QString>
 #include <QDomElement>
 
-#include "viewidentifierclass.h"
 #include "viewlayer.h"
 
 class LayerAttributes {
@@ -42,30 +41,22 @@ public:
 	
 	const QString & filename();
 	void setFilename(const QString &);
-	const QString & layerName();
-	bool isSticky();
-	bool multiLayer();
-	bool getSvgElementID(QDomDocument * , ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID );
-	bool canFlipHorizontal();
-	bool canFlipVertical();
 	const QByteArray & loaded();
 	void clearLoaded();
 	void setLoaded(const QByteArray &);
 
-public:
-	static QDomElement getSvgElementLayers(QDomDocument * doc, ViewIdentifierClass::ViewIdentifier viewIdentifier );
-
-protected:
-	static QDomElement getSvgElementLayer(QDomDocument *, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID, int & layerCount );
-
 protected:
 	QString m_filename;
-	QString m_layerName;
-	bool m_multiLayer;
-	bool m_sticky;
-	bool m_canFlipHorizontal;
-	bool m_canFlipVertical;
 	QByteArray m_loaded;
+
+public:
+    QString error;
+    ViewLayer::ViewID viewID;
+    ViewLayer::ViewLayerID viewLayerID;
+    ViewLayer::ViewLayerPlacement viewLayerPlacement;
+    bool doConnectors;
+    Qt::Orientations orientation;
+    bool createShape;
 };
 
 #endif
