@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6154 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-10 13:05:13 +0200 (Tue, 10 Jul 2012) $
+$Revision: 6963 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-14 10:13:52 +0200 (So, 14. Apr 2013) $
 
 ********************************************************************/
 
@@ -34,6 +34,7 @@ $Date: 2012-07-10 13:05:13 +0200 (Tue, 10 Jul 2012) $
 
 #include <QMessageBox>
 
+QList<QString> ModelBase::CoreList;
 
 /////////////////////////////////////////////////
 
@@ -50,6 +51,11 @@ ModelBase::ModelBase( bool makeRoot )
 
 ModelBase::~ModelBase() {
 	if (m_root) {
+        ModelPartShared * modelPartShared = m_root->modelPartShared();
+        if (modelPartShared) {
+            m_root->setModelPartShared(NULL);
+            delete modelPartShared;
+        }
 		delete m_root;
 	}
 }
@@ -63,8 +69,8 @@ ModelPart * ModelBase::retrieveModelPart(const QString & /* moduleID */)  {
 }
 
 // loads a model from an fz file--assumes a reference model exists with all parts
-bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<ModelPart *> & modelParts) {
-	m_referenceModel = refModel;
+bool ModelBase::loadFromFile(const QString & fileName, ModelBase * referenceModel, QList<ModelPart *> & modelParts, bool checkViews) {
+	m_referenceModel = referenceModel;
 
     QFile file(fileName);
     if (!file.open(QFile::ReadOnly | QFile::Text)) {
@@ -106,6 +112,7 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
 	bool checkForRats = true;
 	bool checkForTraces = true;
     bool checkForMysteryParts = true;
+    bool checkForObsoleteSMDOrientation = true;
 	m_fritzingVersion = root.attribute("fritzingVersion");
 	if (m_fritzingVersion.length() > 0) {
 		// with version 0.4.3 ratsnests in fz files are obsolete
@@ -125,6 +132,10 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
 		versionThingRats.minorVersion = 7;
 		versionThingRats.minorSubVersion = 5;
 		checkForMysteryParts = !Version::greaterThan(versionThingRats, versionThingFz);
+        // with version 0.8.0 flipSMD is horizontal
+	    versionThingRats.minorVersion = 7;
+	    versionThingRats.minorSubVersion = 13;
+	    checkForObsoleteSMDOrientation = !Version::greaterThan(versionThingRats, versionThingFz);
 	}
 
 	ModelPartSharedRoot * modelPartSharedRoot = this->rootModelPartShared();
@@ -198,7 +209,18 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
 		}
 	}
 
-	bool result = loadInstances(domDocument, instances, modelParts);
+    if (checkForObsoleteSMDOrientation) {
+		QDomElement instance = instances.firstChildElement("instance");
+   		while (!instance.isNull()) {
+			if (checkObsoleteOrientation(instance)) {
+                emit obsoleteSMDOrientationSignal();
+                break;
+            }
+			instance = instance.nextSiblingElement("instance");
+		}
+    }
+
+	bool result = loadInstances(domDocument, instances, modelParts, checkViews);
 	emit loadedInstances(this, instances);
 	return result;
 }
@@ -207,7 +229,7 @@ ModelPart * ModelBase::fixObsoleteModuleID(QDomDocument & domDocument, QDomEleme
 	return PartFactory::fixObsoleteModuleID(domDocument, instance, moduleIDRef, m_referenceModel);
 }
 
-bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instances, QList<ModelPart *> & modelParts)
+bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instances, QList<ModelPart *> & modelParts, bool checkViews)
 {
 	QHash<QString, QString> missingModules;
    	QDomElement instance = instances.firstChildElement("instance");
@@ -215,12 +237,30 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
    	while (!instance.isNull()) {
 		emit loadingInstance(this, instance);
 
+        if (checkViews) {
+            QDomElement views = instance.firstChildElement("views");
+            QDomElement view = views.firstChildElement();
+            if (views.isNull() || view.isNull()) {
+                // do not load a part with no views
+                //QString text;
+                //QTextStream stream(&text);
+                //instance.save(stream, 0);
+                //DebugDialog::debug(text);
+                instance = instance.nextSiblingElement("instance");
+                continue;
+            }
+        }
+
    		// for now assume all parts are in the palette
    		QString moduleIDRef = instance.attribute("moduleIdRef");
+
+        //DebugDialog::debug("loading " + moduleIDRef);
 		if (moduleIDRef.compare(ModuleIDNames::SpacerModuleIDName) == 0) {
 			ModelPart * mp = new ModelPart(ModelPart::Space);
 			mp->setInstanceText(instance.attribute("path"));
 			mp->setParent(m_root);
+            mp->modelPartShared()->setModuleID(ModuleIDNames::SpacerModuleIDName);
+            mp->modelPartShared()->setPath(instance.attribute("path"));
 			modelParts.append(mp);
 			instance = instance.nextSiblingElement("instance");
 			continue;
@@ -228,7 +268,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 
 
    		modelPart = m_referenceModel->retrieveModelPart(moduleIDRef);
-   		if (modelPart == NULL) {
+        if (modelPart == NULL) {
 			DebugDialog::debug(QString("module id %1 not found in database").arg(moduleIDRef));
 			modelPart = fixObsoleteModuleID(domDocument, instance, moduleIDRef);
 			if (modelPart == NULL) {
@@ -245,6 +285,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 			}
    		}
 
+        modelPart->setInBin(true);
    		modelPart = addModelPart(m_root, modelPart);
    		modelPart->setInstanceDomElement(instance);
 		modelParts.append(modelPart);
@@ -252,7 +293,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
    		// TODO Mariano: i think this is not the way
    		QString instanceTitle = instance.firstChildElement("title").text();
    		if(!instanceTitle.isNull() && !instanceTitle.isEmpty()) {
-   			modelPart->setInstanceTitle(instanceTitle);
+   			modelPart->setInstanceTitle(instanceTitle, false);
    		}
 
 		QDomElement localConnectors = instance.firstChildElement("localConnectors");
@@ -322,10 +363,16 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 }
 
 ModelPart * ModelBase::addModelPart(ModelPart * parent, ModelPart * copyChild) {
+
+    //if (copyChild->moduleID() == "df9d072afa2b594ac67b60b4153ff57b_29") {
+    //    DebugDialog::debug("alive in here");
+    //}
+
 	ModelPart * modelPart = new ModelPart();
 	modelPart->copyNew(copyChild);
 	modelPart->setParent(parent);
 	modelPart->initConnectors();
+    modelPart->flipSMDAnd();
 	return modelPart;
 }
 
@@ -398,9 +445,9 @@ void ModelBase::save(const QString & fileName, QXmlStreamWriter & streamWriter,
     }
 }
 
-bool ModelBase::paste(ModelBase * refModel, QByteArray & data, QList<ModelPart *> & modelParts, QHash<QString, QRectF> & boundingRects, bool preserveIndex)
+bool ModelBase::paste(ModelBase * referenceModel, QByteArray & data, QList<ModelPart *> & modelParts, QHash<QString, QRectF> & boundingRects, bool preserveIndex)
 {
-	m_referenceModel = refModel;
+	m_referenceModel = referenceModel;
 
 	QDomDocument domDocument;
 	QString errorStr;
@@ -455,7 +502,7 @@ bool ModelBase::paste(ModelBase * refModel, QByteArray & data, QList<ModelPart *
 	//file.write(domDocument.toByteArray());
 	//file.close();
 
-	return loadInstances(domDocument, instances, modelParts);
+	return loadInstances(domDocument, instances, modelParts, true);
 }
 
 void ModelBase::renewModelIndexes(QDomElement & parentElement, const QString & childName, QHash<long, long> & oldToNew)
@@ -468,6 +515,11 @@ void ModelBase::renewModelIndexes(QDomElement & parentElement, const QString & c
 		if (!views.isNull()) {
 			QDomElement view = views.firstChildElement();
 			while (!view.isNull()) {
+                bool ok;
+                int superpart = view.attribute("superpart").toLong(&ok);
+                if (ok) {
+                    view.setAttribute("superpart", QString::number(oldToNew.value(superpart)));
+                }
 				QDomElement connectors = view.firstChildElement("connectors");
 				if (!connectors.isNull()) {
 					QDomElement connector = connectors.firstChildElement("connector");
@@ -506,11 +558,11 @@ void ModelBase::setReportMissingModules(bool b) {
 	m_reportMissingModules = b;
 }
 
-ModelPart * ModelBase::genFZP(const QString & moduleID, ModelBase * refModel) {
+ModelPart * ModelBase::genFZP(const QString & moduleID, ModelBase * referenceModel) {
 	QString path = PartFactory::getFzpFilename(moduleID);
 	if (path.isEmpty()) return NULL;
 
-	ModelPart* mp = refModel->addPart(path, true, true);
+	ModelPart* mp = referenceModel->addPart(path, true, true);
 	if (mp) mp->setCore(true);
 	return mp;
 }
@@ -547,6 +599,15 @@ bool ModelBase::isRatsnest(QDomElement & instance) {
 	return false;
 }
 
+bool ModelBase::checkObsoleteOrientation(QDomElement & instance)
+{
+	QString flippedSMD = instance.attribute("flippedSMD", "");
+	if (flippedSMD != "true") return false;
+
+	QDomElement views = instance.firstChildElement("views");
+	QDomElement pcbView = views.firstChildElement("pcbView");
+    return (pcbView.attribute("layer", "") == "copper0");
+}
 
 void ModelBase::checkTraces(QDomElement & instance) {
 	QString moduleIDRef = instance.attribute("moduleIdRef");
@@ -680,4 +741,10 @@ void ModelBase::checkMystery(QDomElement & instance)
         }
         prop = prop.nextSiblingElement("property");
     }
-}
\ No newline at end of file
+}
+
+bool ModelBase::onCoreList(const QString & moduleID) {
+    // CoreList contains db entries that are (presumably) overridden by an fzp in the parts folder
+    return CoreList.contains(moduleID);
+
+}
