@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6956 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-07 12:14:50 +0200 (So, 07. Apr 2013) $
 
 ********************************************************************/
 
@@ -38,11 +38,11 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 #include <QHash>
 #include <QList>
 #include <QPointer>
+#include <QSharedPointer>
 
 #include "modelpartshared.h"
 #include "../connectors/connector.h"
 #include "../connectors/bus.h"
-#include "../utils/svgandpartfilepath.h"
 
 class ModelPart : public QObject
 {
@@ -63,6 +63,7 @@ public:
 		 Hole,
 		 Via,
 		 Ruler,
+         SchematicSubpart,
 		 Space,
 		 Unknown
 	};
@@ -72,12 +73,13 @@ public:
 		CoreFlag = 1,
 		ContribFlag = 2,
 		AlienFlag = 4,
-		FzzFlag = 8
+		FzzFlag = 8,
+        InBinFlag = 16
 	};
 	Q_DECLARE_FLAGS(LocationFlags, LocationFlag)
 
 public:
-	ModelPart(QDomDocument *, const QString& path, ItemType type);
+	ModelPart(QDomDocument &, const QString& path, ItemType type);
 	ModelPart(ItemType type = ModelPart::Unknown);
 	~ModelPart();
 
@@ -94,8 +96,9 @@ public:
 	void saveAsPart(QXmlStreamWriter & streamWriter, bool startDocument);
 	void addViewItem(class ItemBase *);
 	void removeViewItem(class ItemBase *);
+    void killViewItems();
 	class ItemBase * viewItem(QGraphicsScene * scene);
-	class ItemBase * viewItem(ViewIdentifierClass::ViewIdentifier);
+	class ItemBase * viewItem(ViewLayer::ViewID);
 	bool hasViewItems();
 	void initConnectors(bool force=false);
 	const QHash<QString, QPointer<Connector> > & connectors();
@@ -106,14 +109,18 @@ public:
 	const QDomElement & instanceDomElement();
 	Connector * getConnector(const QString & id);
 
+    const QString & fritzingVersion();
+
 	const QString & title();
 	const QString & description();
+	const QString & spice();
+	const QString & spiceModel();
 	const QString & url();
 	const QStringList & tags();
 	const QStringList & displayKeys();
 	const QHash<QString,QString> & properties() const;
 	const QHash<QString, QPointer<Bus> > & buses();
-	QDomDocument * domDocument();
+	const QString & taxonomy();
 	const QString & version();
 	const QString & path();
 	const QString & label();
@@ -122,6 +129,8 @@ public:
 	const QString & uri();
 	const QDate & date();
 	QString family();
+    void setDBID(qulonglong);
+    qulonglong dbid();
 
 	Bus * bus(const QString & busID);
 	bool ignoreTerminalPoints();
@@ -135,8 +144,18 @@ public:
 	bool isFzz(); // from "outside" 
 	void setFzz(bool alien);
 	void setLocationFlag(bool setting, LocationFlag flag);
+    bool isInBin();
+    void setInBin(bool);
 
-	bool hasViewIdentifier(ViewIdentifierClass::ViewIdentifier viewIdentifier);
+	bool hasViewID(ViewLayer::ViewID);
+	bool canFlipVertical(ViewLayer::ViewID);
+	bool canFlipHorizontal(ViewLayer::ViewID);
+	bool anySticky(ViewLayer::ViewID);
+    QString imageFileName(ViewLayer::ViewID);
+    void setImageFileName(ViewLayer::ViewID, const QString & filename);
+    QString imageFileName(ViewLayer::ViewID, ViewLayer::ViewLayerID);
+    LayerList viewLayers(ViewLayer::ViewID);
+    void setViewImage(struct ViewImage *);
 
 	QList<ModelPart*> getAllParts();
 	QList<ModelPart*> getAllNonCoreParts();
@@ -144,7 +163,7 @@ public:
 
 	const QString & instanceTitle() const;
 	const QString & instanceText();
-	void setInstanceTitle(QString);
+	void setInstanceTitle(QString, bool initial);
 	void setInstanceText(QString);
 	QString getNextTitle(const QString & candidate);
 
@@ -153,20 +172,30 @@ public:
 	QVariant localProp(const QString & name) const;
 	void setLocalProp(const char * name, const QVariant & value);
 	QVariant localProp(const char * name) const;
-
+    void setTag(const QString &tag);
+	void setProperty(const QString & key, const QString & value, bool showInLabel);
+    bool showInLabel(const QString & key);
+    
 	const QString & replacedby();
 	bool isObsolete();
 
-	void setFlippedSMD(bool);
 	bool flippedSMD();
 	bool needsCopper1();
-	bool hasViewFor(ViewIdentifierClass::ViewIdentifier);
-	bool hasViewFor(ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID);
-	QString hasBaseNameFor(ViewIdentifierClass::ViewIdentifier);
+	bool hasViewFor(ViewLayer::ViewID);
+	bool hasViewFor(ViewLayer::ViewID, ViewLayer::ViewLayerID);
+	QString hasBaseNameFor(ViewLayer::ViewID);
 	void initBuses();
 	void clearBuses();
 	void setConnectorLocalName(const QString & id, const QString & name);
+	QString connectorLocalName(const QString & id);
 	void setLocalTitle(const QString &);
+    const QList<ViewImage *> viewImages();
+    void addConnector(Connector *);
+    void flipSMDAnd();
+    void lookForZeroConnector();
+    bool hasZeroConnector();
+    bool hasSubparts();
+    void setSubpartID(const QString &);
 
 public:
 	static long nextIndex();
@@ -186,6 +215,7 @@ protected:
 	void saveInstance(QXmlStreamWriter & streamWriter);
 	QList< QPointer<ModelPart> > * ensureInstanceTitleIncrements(const QString & prefix);
 	void clearOldInstanceTitle(const QString & title);
+    bool setSubpartInstanceTitle();
 
 protected:
 	QList< QPointer<class ItemBase> > m_viewItems;
@@ -193,13 +223,11 @@ protected:
 	ItemType m_type;
 	QPointer<ModelPartShared> m_modelPartShared;
 	QHash<QString, QPointer<Connector> > m_connectorHash;
-	QList< QPointer<Connector> > m_deletedConnectors;
 	QHash<QString, QPointer<Bus> > m_busHash;
 	long m_index;						// only used at save time to identify model parts in the xml
 	QDomElement m_instanceDomElement;	// only used at load time (so far)
 
 	LocationFlags m_locationFlags;
-	bool m_originalModelPartShared;
 	bool m_indexSynched;
 
 	QString m_instanceTitle;
