@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6265 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
+$Revision: 7004 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-29 13:10:59 +0200 (Mo, 29. Apr 2013) $
 
 ********************************************************************/
 
@@ -38,19 +38,13 @@ $Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../items/moduleidnames.h"
+#include "../items/partfactory.h"
 
-bool PaletteModel::CreateAllPartsBinFile = false;  // now generating the all parts bin in advance using a python script:  [fritzing]/part-gen-scripts/misc_scripts/genAllParts.py
-
-bool PaletteModel::CreateNonCorePartsBinFile = false;
-bool PaletteModel::CreateContribPartsBinFile = true;
 bool PaletteModel::CreateTempPartsBinFile = true;
 
 static bool JustAppendAllPartsInstances = false;
 static bool FirstTime = true;
-static bool FirstTimeWrite = true;
 
-QString PaletteModel::AllPartsBinFilePath;
-QString PaletteModel::NonCorePartsBinFilePath;
 QString PaletteModel::ContribPartsBinFilePath;
 
 static QString FritzingContribPath;
@@ -64,23 +58,41 @@ const static QString InstanceTemplate(
         		"\t\t\t</views>\n"
         		"\t\t</instance>\n");
 
-PaletteModel::PaletteModel() : ModelBase(true) {
-	m_loadedFromFile = false;
-	m_loadingCore = false;
-	m_loadingContrib = false;
+
+void setFlip(ViewLayer::ViewID viewID, QXmlStreamReader & xml, QHash<ViewLayer::ViewID, ViewImage *> & viewImages)
+{
+    bool fv = xml.attributes().value("flipvertical", "").toString().compare("true") == 0;
+    bool fh = xml.attributes().value("fliphorizontal", "").toString().compare("true") == 0;
+    viewImages.value(viewID)->canFlipHorizontal = fh;
+    viewImages.value(viewID)->canFlipVertical = fv;
 }
 
-PaletteModel::PaletteModel(bool makeRoot, bool doInit, bool fastLoad) : ModelBase( makeRoot ) {
+/////////////////////////////////////
+
+PaletteModel::PaletteModel() : ModelBase(true) {
 	m_loadedFromFile = false;
-	m_loadingCore = false;
 	m_loadingContrib = false;
+    m_fullLoad = false;
+}
+
+PaletteModel::PaletteModel(bool makeRoot, bool doInit) : ModelBase( makeRoot ) {
+	m_loadedFromFile = false;
+	m_loadingContrib = false;
+    m_fullLoad = false;
 
 	if (doInit) {
-		initParts(fastLoad);
+		initParts(false);
 	}
 }
 
-void PaletteModel::initParts(bool fastLoad) {
+PaletteModel::~PaletteModel()
+{
+    foreach (ModelPart * modelPart, m_partHash.values()) {
+        delete modelPart;
+    }
+}
+
+void PaletteModel::initParts(bool dbExists) {
 	QDir * dir = FolderUtils::getApplicationSubFolder("parts");
 	if (dir == NULL) {
 	    QMessageBox::information(NULL, QObject::tr("Fritzing"),
@@ -91,7 +103,7 @@ void PaletteModel::initParts(bool fastLoad) {
 	FritzingContribPath = dir->absoluteFilePath("contrib");
 	delete dir;
 
-	loadParts(fastLoad);
+	loadParts(dbExists);
 	if (m_root == NULL) {
 	    QMessageBox::information(NULL, QObject::tr("Fritzing"),
 	                             QObject::tr("No parts found.") );
@@ -99,8 +111,6 @@ void PaletteModel::initParts(bool fastLoad) {
 }
 
 void PaletteModel::initNames() {
-	AllPartsBinFilePath = FolderUtils::getApplicationSubFolderPath("bins")+"/allParts.dbg" + FritzingBinExtension;
-	NonCorePartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/nonCoreParts" + FritzingBinExtension;
 	ContribPartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/contribParts" + FritzingBinExtension;
 }
 
@@ -119,16 +129,7 @@ bool PaletteModel::containsModelPart(const QString & moduleID) {
 	return m_partHash.contains(moduleID);
 }
 
-void PaletteModel::updateOrAddModelPart(const QString & moduleID, ModelPart *newOne) {
-	ModelPart *oldOne = m_partHash.value(moduleID, NULL);
-	if(oldOne) {
-		oldOne->copy(newOne);
-	} else {
-		m_partHash.insert(moduleID, newOne);
-	}
-}
-
-void PaletteModel::loadParts(bool fastLoad) {
+void PaletteModel::loadParts(bool dbExists) {
 	QStringList nameFilters;
 	nameFilters << "*" + FritzingPartExtension;
 
@@ -146,34 +147,37 @@ void PaletteModel::loadParts(bool fastLoad) {
 
 
 	if (FirstTime) {
-		writeCommonBinsHeader();
 	}
 
 	int totalPartCount = 0;
 	emit loadedPart(0, totalPartCount);
 
-
 	QDir * dir1 = FolderUtils::getApplicationSubFolder("parts");
 	if (dir1 != NULL) {
 		countParts(*dir1, nameFilters, totalPartCount);
 	}
 	QDir dir2(FolderUtils::getUserDataStorePath("parts"));
 	countParts(dir2, nameFilters, totalPartCount);
-	QDir dir3(":/resources/parts");
-	countParts(dir3, nameFilters, totalPartCount);
 
+	QDir dir3(":/resources/parts");
+    if (m_fullLoad || !dbExists) {
+	    countParts(dir3, nameFilters, totalPartCount);
+    }
 
 	int loadingPart = 0;
 	if (dir1 != NULL) {
-		loadPartsAux(*dir1, nameFilters, loadingPart, totalPartCount, fastLoad);
+		loadPartsAux(*dir1, nameFilters, loadingPart, totalPartCount);
 		delete dir1;
 	}
 
-	loadPartsAux(dir2, nameFilters, loadingPart, totalPartCount, fastLoad);
-	loadPartsAux(dir3, nameFilters, loadingPart, totalPartCount, fastLoad);  
+	loadPartsAux(dir2, nameFilters, loadingPart, totalPartCount);
+
+    if (m_fullLoad || !dbExists) {
+        // otherwise these will already be in the database
+	    loadPartsAux(dir3, nameFilters, loadingPart, totalPartCount);  
+    }
 
 	if (FirstTime) {
-		writeCommonBinsFooter();
 	}
 	
 	JustAppendAllPartsInstances = false;   
@@ -187,103 +191,6 @@ void PaletteModel::loadParts(bool fastLoad) {
 
 }
 
-void PaletteModel::writeCommonBinsHeader() {
-	writeCommonBinsHeaderAux(CreateAllPartsBinFile, AllPartsBinFilePath, "All Parts");
-	writeCommonBinsHeaderAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath, "All my parts");
-	writeCommonBinsHeaderAux(CreateContribPartsBinFile, ContribPartsBinFilePath, "Contributed Parts");
-}
-
-void PaletteModel::writeCommonBinsHeaderAux(bool doIt, const QString &filename, const QString &binName) {
-	QString header =
-		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + 
-		TextUtils::CreatedWithFritzingXmlComment +
-		QString("<module fritzingVersion='%1' icon='%2.png'>\n").arg(Version::versionString()).arg(binName) +
-		QString("\t<title>%1</title>\n").arg(binName) +
-		"\t<instances>\n";
-	writeCommonBinAux(header, QFile::WriteOnly, doIt, filename);
-}
-
-void PaletteModel::writeCommonBinsFooter() {
-	writeCommonBinsFooterAux(CreateAllPartsBinFile, AllPartsBinFilePath);
-	writeCommonBinsFooterAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath);
-	writeCommonBinsFooterAux(CreateContribPartsBinFile, ContribPartsBinFilePath);
-}
-
-void PaletteModel::writeCommonBinsFooterAux(bool doIt, const QString &filename) {
-	QString footer = "\t</instances>\n</module>\n";
-	writeCommonBinAux(footer, QFile::Append, doIt, filename);
-}
-
-void PaletteModel::writeCommonBinInstance(const QString &moduleID, const QString &path, bool doIt, const QString &filename) {
-	if (!doIt) return;
-	
-	QString pathAux = path;
-	pathAux.remove(FolderUtils::getApplicationSubFolderPath("")+"/");
-
-	if (JustAppendAllPartsInstances) {
-		QString instance = InstanceTemplate.arg(moduleID).arg("");
-		writeCommonBinAux(instance, QFile::Append, doIt, filename);
-	}
-	else {
-		QString errorStr;
-		int errorLine;
-		int errorColumn;
-		QDomDocument domDocument;
-
-		QFile file(filename);
-		if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
-			return;
-		}
-
-		QDomElement root = domDocument.documentElement();
-   		if (root.isNull()) {
-   			return;
-		}
-
-		if (root.tagName() != "module") {
-			return;
-		}
-
-		QDomElement instances = root.firstChildElement("instances");
-		if (instances.isNull()) return;
-
-		QDomElement instance = domDocument.createElement("instance");
-		instances.appendChild(instance);
-		instance.setAttribute("moduleIdRef", moduleID);
-		instance.setAttribute("path", "");
-		QDomElement views = domDocument.createElement("views");
-		instance.appendChild(views);
-		QDomElement iconView = domDocument.createElement("iconView");
-		views.appendChild(iconView);
-		iconView.setAttribute("layer", "icon");
-		QDomElement geometry = domDocument.createElement("geometry");
-		iconView.appendChild(geometry);
-		geometry.setAttribute("x", "-1");
-		geometry.setAttribute("y", "-1");
-		geometry.setAttribute("z", "-1");
-		writeCommonBinAux(domDocument.toString(), QFile::WriteOnly, doIt, filename);
-	}
-}
-
-void PaletteModel::writeCommonBinAux(const QString &textToWrite, QIODevice::OpenMode openMode, bool doIt, const QString &filename) {
-	if(!doIt) return;
-
-	if (FirstTimeWrite) {
-		FirstTimeWrite = false;
-		QFileInfo info(filename);
-		QDir dir = info.absoluteDir();
-		dir.mkpath(info.absolutePath());
-	}
-
-	QFile file(filename);
-	if (file.open(openMode | QFile::Text)) {
-		QTextStream out(&file);
-		out.setCodec("UTF-8");
-		out << textToWrite;
-		file.close();
-	}
-}
-
 void PaletteModel::countParts(QDir & dir, QStringList & nameFilters, int & partCount) {
     QStringList list = dir.entryList(nameFilters, QDir::Files | QDir::NoSymLinks);
 	partCount += list.size();
@@ -298,14 +205,14 @@ void PaletteModel::countParts(QDir & dir, QStringList & nameFilters, int & partC
     }
 }
 
-void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loadingPart, int totalPartCount, bool fastLoad) {
-    QString temp = dir.absolutePath();
+void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loadingPart, int totalPartCount) {
+    //QString temp = dir.absolutePath();
     QFileInfoList list = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
     for (int i = 0; i < list.size(); ++i) {
         QFileInfo fileInfo = list.at(i);
         QString path = fileInfo.absoluteFilePath ();
         //DebugDialog::debug(QString("part path:%1 core? %2").arg(path).arg(m_loadingCore? "true" : "false"));
-        loadPart(path, false, fastLoad);
+        loadPart(path, false);
 		emit loadedPart(++loadingPart, totalPartCount);
         //DebugDialog::debug("loadedok");
     }
@@ -315,17 +222,15 @@ void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loa
     	QString temp2 = dirs[i];
        	dir.cd(temp2);
 
-       	//if(temp2 == "core" || temp2=="contrib" || temp2=="user") {
-			m_loadingCore = temp2=="core";
-			m_loadingContrib = temp2=="contrib";
-       	//}
+		m_loadingContrib = (temp2 == "contrib");
 
-    	loadPartsAux(dir, nameFilters, loadingPart, totalPartCount, fastLoad);
+    	loadPartsAux(dir, nameFilters, loadingPart, totalPartCount);
     	dir.cdUp();
     }
 }
 
-ModelPart * PaletteModel::loadPart(const QString & path, bool update, bool fastLoad) {
+ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
+  
     QFile file(path);
     if (!file.open(QFile::ReadOnly | QFile::Text)) {
         QMessageBox::warning(NULL, QObject::tr("Fritzing"),
@@ -339,147 +244,49 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update, bool fastL
 
 	ModelPart::ItemType type = ModelPart::Part;
 	QString moduleID;
+    QString title;
 	QString propertiesText;
-	QDomDocument* domDocument = NULL;
-	QString title, label, date, author, description, taxonomy, replacedby, version, url;
-	QStringList tags;
-	QStringList displayKeys;
-	QHash<QString, QString> properties;
-	QMultiHash<ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerID> hasViewFor;
-	QHash<ViewIdentifierClass::ViewIdentifier, QString> hasBaseNameFor;
-
-	if (fastLoad) {
-		QXmlStreamReader xml(&file);
-		xml.setNamespaceProcessing(false);
-		ViewIdentifierClass::ViewIdentifier viewIdentifier = ViewIdentifierClass::IconView;
-		while (!xml.atEnd()) {
-			bool done = false;
-			switch (xml.readNext()) {
-				case QXmlStreamReader::StartElement:
-				{
-					QString name = xml.name().toString();
-					if (name.compare("module") == 0) {
-						moduleID = xml.attributes().value("moduleId").toString();
-					}
-					else if (name.compare("title") == 0) {
-						title = xml.readElementText();
-					}
-					else if (name.compare("tag") == 0) {
-						QString tag = xml.readElementText();
-						tags.append(tag);
-					}
-					else if (name.compare("property") == 0) {
-						QString name = xml.attributes().value("name").toString().toLower().trimmed();
-						QString showInLabel = xml.attributes().value("showInLabel").toString();
-						QString value = xml.readElementText();
-						if (value.isNull()) {
-							value = "";
-						}
-						if (!showInLabel.isEmpty()) displayKeys.append(name);
-						properties.insert(name, value);
-						propertiesText += (name + value);
-					}
-					else if (name.compare("label") == 0) {
-						label = xml.readElementText();
-					}
-					else if (name.compare("author") == 0) {
-						author = xml.readElementText();
-					}
-					else if (name.compare("version") == 0) {
-						replacedby = xml.attributes().value("replacedby").toString();
-						version = xml.readElementText();
-					}
-					else if (name.compare("description") == 0) {
-						description = xml.readElementText();
-					}
-					else if (name.compare("url") == 0) {
-						url = xml.readElementText();
-					}
-					else if (name.compare("taxonomy") == 0) {
-						taxonomy = xml.readElementText();
-					}
-					else if (name.compare("date") == 0) {
-						date = xml.readElementText();
-					}
-					else if (name.compare("breadboardView") == 0) {
-						viewIdentifier = ViewIdentifierClass::BreadboardView;
-					}
-					else if (name.compare("schematicView") == 0) {
-						viewIdentifier = ViewIdentifierClass::SchematicView;
-					}
-					else if (name.compare("pcbView") == 0) {
-						viewIdentifier = ViewIdentifierClass::PCBView;
-					}
-					else if (name.compare("iconView") == 0) {
-						viewIdentifier = ViewIdentifierClass::IconView;
-					}
-					else if (name.compare("layers") == 0) {
-						hasBaseNameFor.insert(viewIdentifier, xml.attributes().value("image").toString());
-					}
-					else if (name.compare("layer") == 0) {
-						QString layerName = xml.attributes().value("layerId").toString();
-						ViewLayer::ViewLayerID viewLayerID = ViewLayer::viewLayerIDFromXmlString(layerName);
-						if (ViewIdentifierClass::viewHasLayer(viewIdentifier, viewLayerID)) {
-							hasViewFor.insert(viewIdentifier, viewLayerID);
-						}
-						//else {
-							//DebugDialog::debug(QString("missing view layer %3: vid:%1 %4 vlid:%2").arg(viewIdentifier).arg(viewLayerID).arg(moduleID).arg(layerName));
-						//}
-					}
-					else if (name.compare("connectors") == 0) {
-						done = true;
-					}
-				}
-                break;
-            default:
-                break;
-			}
-			if (done) break;
-		}
 
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	QDomDocument domDocument;
+	if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
+		QMessageBox::information(NULL, QObject::tr("Fritzing"),
+								QObject::tr("Parse error (2) at line %1, column %2:\n%3\n%4")
+								.arg(errorLine)
+								.arg(errorColumn)
+								.arg(errorStr)
+								.arg(path));
+		return NULL;
 	}
-	else {
-		QString errorStr;
-		int errorLine;
-		int errorColumn;
-		domDocument = new QDomDocument();
 
-		if (!domDocument->setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
-			QMessageBox::information(NULL, QObject::tr("Fritzing"),
-								 QObject::tr("Parse error (2) at line %1, column %2:\n%3\n%4")
-								 .arg(errorLine)
-								 .arg(errorColumn)
-								 .arg(errorStr)
-								 .arg(path));
-			delete domDocument;
-			return NULL;
-		}
-
-		QDomElement root = domDocument->documentElement();
-   		if (root.isNull()) {
-			//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (8)."));
-   			return NULL;
-		}
-
-		if (root.tagName() != "module") {
-			//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (9)."));
-			return NULL;
-		}
-
-		moduleID = root.attribute("moduleId");
-		if (moduleID.isNull() || moduleID.isEmpty()) {
-			//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (10)."));
-			return NULL;
-		}
-
-		// check if it's a wire
-		QDomElement properties = root.firstChildElement("properties");
-		propertiesText = properties.text();
-
-		QDomElement t = root.firstChildElement("title");
-		TextUtils::findText(t, title);
+	QDomElement root = domDocument.documentElement();
+   	if (root.isNull()) {
+		//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (8)."));
+   		return NULL;
 	}
 
+	if (root.tagName() != "module") {
+		//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (9)."));
+		return NULL;
+	}
+
+	moduleID = root.attribute("moduleId");
+	if (moduleID.isNull() || moduleID.isEmpty()) {
+		//QMessageBox::information(NULL, QObject::tr("Fritzing"), QObject::tr("The file is not a Fritzing file (10)."));
+		return NULL;
+	}
+
+	// check if it's a wire
+	QDomElement propertiesElement = root.firstChildElement("properties");
+	propertiesText = propertiesElement.text();
+
+	QDomElement t = root.firstChildElement("title");
+	TextUtils::findText(t, title);
+
+    //DebugDialog::debug("module ID " + moduleID);
+
 	// FIXME: properties is nested right now
 	if (moduleID.compare(ModuleIDNames::WireModuleIDName) == 0) {
 		type = ModelPart::Wire;
@@ -499,15 +306,21 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update, bool fastL
 	else if (moduleID.compare(ModuleIDNames::NoteModuleIDName) == 0) {
 		type = ModelPart::Note;
 	}
-	else if (moduleID.compare(ModuleIDNames::JustPowerModuleIDName) == 0) {
-		type = ModelPart::Symbol;
+	else if (moduleID.endsWith(ModuleIDNames::TwoPowerModuleIDName)) {
+		type = ModelPart::Part;
 	}
-	else if (moduleID.compare(ModuleIDNames::PowerModuleIDName) == 0) {
+	else if (moduleID.endsWith(ModuleIDNames::PowerModuleIDName)) {
 		type = ModelPart::Symbol;
 	}
 	else if (moduleID.compare(ModuleIDNames::GroundModuleIDName) == 0) {
 		type = ModelPart::Symbol;
 	}
+	else if (moduleID.endsWith(ModuleIDNames::NetLabelModuleIDName)) {
+		type = ModelPart::Symbol;
+	}
+	else if (moduleID.compare(ModuleIDNames::PowerLabelModuleIDName) == 0) {
+		type = ModelPart::Symbol;
+	}
 	else if (moduleID.compare(ModuleIDNames::RulerModuleIDName) == 0) {
 		type = ModelPart::Ruler;
 	}
@@ -538,9 +351,24 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update, bool fastL
 	ModelPart * modelPart = new ModelPart(domDocument, path, type);
 	if (modelPart == NULL) return NULL;
 
-	modelPart->setCore(m_loadingCore);
+    if (path.startsWith(ResourcePath)) {
+	    modelPart->setCore(true);
+    }
+    else if (onCoreList(moduleID)) {
+        // for database entries which have existing fzp files.
+	    modelPart->setCore(true);
+    }
+
 	modelPart->setContrib(m_loadingContrib);
 
+    QDomElement subparts = root.firstChildElement("schematic-subparts");
+    QDomElement subpart = subparts.firstChildElement("subpart");
+    while (!subpart.isNull()) {
+        ModelPart * subModelPart = makeSubpart(modelPart, subpart);
+        m_partHash.insert(subModelPart->moduleID(), subModelPart);
+        subpart = subpart.nextSiblingElement("subpart");
+    }
+
 	if (m_partHash.value(moduleID, NULL)) {
 		if(!update) {
 			QMessageBox::warning(NULL, QObject::tr("Fritzing"),
@@ -563,62 +391,12 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update, bool fastL
     	modelPart->setParent(m_root);
    	}
 
-	if (fastLoad) {
-		ModelPartShared * modelPartShared = modelPart->modelPartShared();
-		modelPartShared->setModuleID(moduleID);
-		modelPartShared->setPartlyLoaded(true);
-		modelPartShared->setTitle(title);
-		modelPartShared->setDate(date);
-		modelPartShared->setAuthor(author);
-		//if (label.isEmpty()) {
-			//DebugDialog::debug(QString("empty label %1").arg(path));
-		//}
-		modelPartShared->setLabel(label);
-		modelPartShared->setDescription(description);
-		modelPartShared->setUrl(url);
-		modelPartShared->setTaxonomy(taxonomy);
-		modelPartShared->setVersion(version);
-		modelPartShared->setReplacedby(replacedby);
-		modelPartShared->setTags(tags);
-		modelPartShared->setProperties(properties);
-		modelPartShared->setDisplayKeys(displayKeys);
-		foreach (ViewIdentifierClass::ViewIdentifier viewIdentifier, hasViewFor.uniqueKeys()) {
-			foreach (ViewLayer::ViewLayerID viewLayerID, hasViewFor.values(viewIdentifier)) {
-				modelPartShared->setHasViewFor(viewIdentifier, viewLayerID);
-			}
-		}
-		foreach (ViewIdentifierClass::ViewIdentifier viewIdentifier, hasBaseNameFor.keys()) {
-			modelPartShared->setHasBaseNameFor(viewIdentifier, hasBaseNameFor.value(viewIdentifier));
-		}
-	}
-	else {
-		modelPart->modelPartShared()->flipSMDAnd();
-	}
-
-	if (FirstTime) {
-		// make sure saving to the common bin takes place after fastLoad
-		//DebugDialog::debug(QString("all parts %1").arg(JustAppendAllPartsInstances));
-		writeCommonBinInstance(moduleID,path,CreateAllPartsBinFile,AllPartsBinFilePath);
-
-		if (modelPart->isContrib()) {
-			//if (path.startsWith(FritzingContribPath, Qt::CaseInsensitive)) {
-				writeCommonBinInstance(moduleID,path,CreateContribPartsBinFile,ContribPartsBinFilePath);
-			//}
-		}
-
-		//DebugDialog::debug(QString("non core parts %1").arg(JustAppendAllPartsInstances));
-		if (!modelPart->isCore() && !modelPart->isObsolete()) {
-			writeCommonBinInstance(moduleID,path,CreateNonCorePartsBinFile,NonCorePartsBinFilePath);
-		}
-	}
-
     return modelPart;
 }
 
-
-bool PaletteModel::load(const QString & fileName, ModelBase * refModel) {
+bool PaletteModel::loadFromFile(const QString & fileName, ModelBase * referenceModel, bool checkViews) {
 	QList<ModelPart *> modelParts;
-	bool result = ModelBase::load(fileName, refModel, modelParts);
+	bool result = ModelBase::loadFromFile(fileName, referenceModel, modelParts, checkViews);
 	if (result) {
 		m_loadedFromFile = true;
 		m_loadedFrom = fileName;
@@ -647,7 +425,7 @@ ModelPart * PaletteModel::addPart(QString newPartPath, bool addToReference, bool
 		}
 	}*/
 
-	ModelPart * modelPart = loadPart(newPartPath, updateIdAlreadyExists, false);
+	ModelPart * modelPart = loadPart(newPartPath, updateIdAlreadyExists);
 	if (m_referenceModel && addToReference) {
 		m_referenceModel->addPart(modelPart,updateIdAlreadyExists);
 	}
@@ -696,6 +474,12 @@ void PaletteModel::removeParts() {
 
 void PaletteModel::clearPartHash() {
 	foreach (ModelPart * modelPart, m_partHash.values()) {
+        ModelPartShared * modelPartShared = modelPart->modelPartShared();
+        if (modelPartShared) {
+            modelPart->setModelPartShared(NULL);
+            delete modelPartShared;
+        }
+
 		delete modelPart;
 	}
 	m_partHash.clear();
@@ -780,3 +564,101 @@ void PaletteModel::search(ModelPart * modelPart, const QStringList & searchStrin
 	}
 }
 
+QList<ModelPart *> PaletteModel::findContribNoBin() {
+    QList<ModelPart *> modelParts;
+    foreach (ModelPart * modelPart, m_partHash.values()) {
+        if (modelPart->isContrib()) {
+            if (!modelPart->isInBin()) {
+                modelParts << modelPart;
+            }
+        }
+    }
+    return modelParts;
+}
+
+ModelPart * PaletteModel::makeSubpart(ModelPart * originalModelPart, const QDomElement & originalSubpart) {
+    QString newLabel = originalSubpart.attribute("label");
+    QString newID = originalSubpart.attribute("id");
+    QDomElement originalRoot = originalSubpart.ownerDocument().documentElement();
+    QString moduleID = originalRoot.attribute("moduleId") + "_" + newID;
+    ModelPart * modelPart = retrieveModelPart(moduleID);
+    if (modelPart) {
+        return modelPart;
+    }
+
+    QDomDocument subdoc = originalSubpart.ownerDocument().cloneNode(true).toDocument();
+    QDomElement root = subdoc.documentElement();
+    QDomElement subparts = root.firstChildElement("schematic-subparts");
+    root.removeChild(subparts);
+    root.setAttribute("moduleId", moduleID);
+    QDomElement label = root.firstChildElement("label");
+    if (!label.isNull()) {
+        TextUtils::replaceChildText(label, newLabel);
+    }
+    QDomElement views = root.firstChildElement("views");
+    QDomElement schematicView = views.firstChildElement(ViewLayer::viewIDXmlName(ViewLayer::SchematicView));
+    QDomElement schematicLayers = schematicView.firstChildElement("layers");
+    QString image = schematicLayers.attribute("image");
+    image.replace(".svg", "_" + newID + ".svg");
+    schematicLayers.setAttribute("image", image);
+    views.removeChild(schematicView);
+    QDomElement view = views.firstChildElement();
+    while (!view.isNull()) {
+        QDomElement layers = view.firstChildElement("layers");
+        view.removeChild(layers);
+        layers = schematicLayers.cloneNode(true).toElement();
+        view.appendChild(layers);
+        view = view.nextSiblingElement();
+    }
+    views.appendChild(schematicView);
+
+    QDomElement connectors = root.firstChildElement("connectors");
+    QDomElement connector = connectors.firstChildElement("connector");
+    QHash<QString, QDomElement> connectorHash;
+    while (!connector.isNull()) {
+        QDomElement next = connector.nextSiblingElement("connector");
+        connectors.removeChild(connector);
+        connectorHash.insert(connector.attribute("id"), connector);
+        connector = next;
+    }
+    QDomElement originalConnectors = originalSubpart.firstChildElement("connectors");
+    QDomElement originalConnector = originalConnectors.firstChildElement("connector");
+    QString schematicLayerName = ViewLayer::viewLayerXmlNameFromID(ViewLayer::Schematic);
+    while (!originalConnector.isNull()) {
+        QString id = originalConnector.attribute("id");
+        QDomElement connector = connectorHash.value(id);
+        connectors.appendChild(connector);
+        QDomElement cviews = connector.firstChildElement("views");
+        QDomElement view = cviews.firstChildElement();
+        while (!view.isNull()) {
+            QDomElement p = view.firstChildElement("p");
+            bool firstTime = true;
+            while (!p.isNull()) {
+                QDomElement next = p.nextSiblingElement("p");
+                if (firstTime) {
+                    p.setAttribute("layer", schematicLayerName);
+                    firstTime = false;
+                }
+                else {
+                    view.removeChild(p);
+                }
+                p = next;
+            }
+            view = view.nextSiblingElement();
+        }
+
+        originalConnector = originalConnector.nextSiblingElement("connector");
+    }
+
+    QString path = PartFactory::fzpPath() + moduleID + ".fzp";
+    QString fzp = subdoc.toString(4);
+    TextUtils::writeUtf8(path, fzp);
+
+    modelPart = new ModelPart(subdoc, path, ModelPart::SchematicSubpart);
+    modelPart->setSubpartID(newID);
+    originalModelPart->modelPartShared()->addSubpart(modelPart->modelPartShared());
+    m_partHash.insert(moduleID, modelPart);
+    return modelPart;
+}
+
+
