@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6265 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
+$Revision: 6962 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-14 00:08:36 +0200 (So, 14. Apr 2013) $
 
 ********************************************************************/
 
@@ -31,21 +31,24 @@ $Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 #include <QSettings>
 #include <QtDebug>
 #include <QInputDialog>
+#include <QDropEvent>
+#include <QMimeData>
 
 #include "binmanager.h"
 #include "stacktabwidget.h"
 #include "stacktabbar.h"
 
 #include "../../model/modelpart.h"
-#include "../../mainwindow.h"
+#include "../../mainwindow/mainwindow.h"
 #include "../../model/palettemodel.h"
 #include "../../waitpushundostack.h"
 #include "../../debugdialog.h"
-#include "../../partseditor/partseditormainwindow.h"
 #include "../../utils/folderutils.h"
+#include "../../utils/textutils.h"
 #include "../../utils/fileprogressdialog.h"
 #include "../../referencemodel/referencemodel.h"
 #include "../partsbinpalettewidget.h"
+#include "../partsbinview.h"
 
 ///////////////////////////////////////////////////////////
 
@@ -93,7 +96,6 @@ QString BinManager::MyPartsBinTemplateLocation;
 QString BinManager::SearchBinLocation;
 QString BinManager::SearchBinTemplateLocation;
 QString BinManager::ContribPartsBinLocation;
-QString BinManager::ContribPartsBinTemplateLocation;
 QString BinManager::TempPartsBinTemplateLocation;
 QString BinManager::CorePartsBinLocation;
 
@@ -102,12 +104,15 @@ QString BinManager::CurrentBinStyle = "background-color: black;";
 
 QHash<QString, QString> BinManager::StandardBinIcons;
 
-BinManager::BinManager(class ReferenceModel *refModel, class HtmlInfoView *infoView, WaitPushUndoStack *undoStack, MainWindow* parent)
+BinManager::BinManager(class ReferenceModel *referenceModel, class HtmlInfoView *infoView, WaitPushUndoStack *undoStack, MainWindow* parent)
 	: QFrame(parent)
 {
 	BinManager::Title = tr("Parts");
 
-	m_refModel = refModel;
+    m_combinedMenu = NULL;
+    m_showListViewAction = m_showIconViewAction = NULL;
+
+	m_referenceModel = referenceModel;
 	m_infoView = infoView;
 	m_undoStack = undoStack;
 	m_defaultSaveFolder = FolderUtils::getUserDataStorePath("bins");
@@ -127,13 +132,22 @@ BinManager::BinManager(class ReferenceModel *refModel, class HtmlInfoView *infoV
 	lo->setMargin(0);
 	lo->setSpacing(0);
 	setMaximumHeight(500);
+}
 
+BinManager::~BinManager() {
+}
+
+void BinManager::initStandardBins()
+{
 	createCombinedMenu();
 	createContextMenus();
 
-	DebugDialog::debug("init bin manager");
+	//DebugDialog::debug("init bin manager");
 	QList<BinLocation *> actualLocations;
 	findAllBins(actualLocations);
+
+    hackLocalContrib(actualLocations);
+
 	restoreStateAndGeometry(actualLocations);
 	foreach (BinLocation * location, actualLocations) {
 		PartsBinPaletteWidget* bin = newBin();
@@ -143,17 +157,16 @@ BinManager::BinManager(class ReferenceModel *refModel, class HtmlInfoView *infoV
 		registerBin(bin);
 		delete location;
 	}
-	DebugDialog::debug("open core bin");
+    actualLocations.clear();
+
+	//DebugDialog::debug("open core bin");
 	openCoreBinIn();
 		
-	DebugDialog::debug("after core bin");
+	//DebugDialog::debug("after core bin");
 
 	connectTabWidget();
 }
 
-BinManager::~BinManager() {
-}
-
 void BinManager::addBin(PartsBinPaletteWidget* bin) {
 	m_stackTabWidget->addTab(bin, bin->icon(), bin->title());
 	registerBin(bin);
@@ -203,7 +216,7 @@ void BinManager::insertBin(PartsBinPaletteWidget* bin, int index) {
 bool BinManager::beforeClosing() {
 	bool retval = true;
 
-	for(int j=0; j < m_stackTabWidget->count(); j++) {
+	for(int j = 0; j < m_stackTabWidget->count(); j++) {
 		PartsBinPaletteWidget *bin = qobject_cast<PartsBinPaletteWidget*>(m_stackTabWidget->widget(j));
 		if (bin && !bin->fastLoaded()) {
 			setAsCurrentTab(bin);
@@ -294,23 +307,15 @@ QString BinManager::createIfBinNotExists(const QString & dest, const QString & s
 	return binPath;
 }
 
-void BinManager::addPart(ModelPart *modelPart, int position) {
+void BinManager::addPartToBin(ModelPart *modelPart, int position) {
 	PartsBinPaletteWidget *bin = m_currentBin? m_currentBin: getOrOpenMyPartsBin();
-	addPartAux(bin,modelPart,position);
+	addPartToBinAux(bin,modelPart,position);
 }
 
-void BinManager::addToMyPart(ModelPart *modelPart) {
+void BinManager::addToMyParts(ModelPart *modelPart) {
 	PartsBinPaletteWidget *bin = getOrOpenMyPartsBin();
 	if (bin) {
-		addPartAux(bin,modelPart);
-		setAsCurrentTab(bin);
-	}
-}
-
-void BinManager::addToContrib(ModelPart *modelPart) {
-	PartsBinPaletteWidget *bin = getOrOpenBin(ContribPartsBinLocation, ContribPartsBinTemplateLocation);
-	if (bin) {
-		addPartAux(bin,modelPart);
+		addPartToBinAux(bin,modelPart);
 		setAsCurrentTab(bin);
 	}
 }
@@ -318,7 +323,7 @@ void BinManager::addToContrib(ModelPart *modelPart) {
 void BinManager::addToTempPartsBin(ModelPart *modelPart) {
 	PartsBinPaletteWidget *bin = getOrOpenBin(m_tempPartsBinLocation, TempPartsBinTemplateLocation);
 	if (bin) {
-		addPartAux(bin,modelPart);
+		addPartToBinAux(bin,modelPart);
 		setAsCurrentTab(bin);
 		bin->setDirty(false);
 	}
@@ -334,7 +339,7 @@ void BinManager::hideTempPartsBin() {
 	}
 }
 
-void BinManager::addPartAux(PartsBinPaletteWidget *bin, ModelPart *modelPart, int position) {
+void BinManager::addPartToBinAux(PartsBinPaletteWidget *bin, ModelPart *modelPart, int position) {
 	if(bin) {
 		if (bin->fastLoaded()) {
 			bin->load(bin->fileName(), bin, false);
@@ -380,7 +385,7 @@ void BinManager::updateTitle(PartsBinPaletteWidget* w, const QString& newTitle)
 
 PartsBinPaletteWidget* BinManager::newBinIn() {
 	PartsBinPaletteWidget* bin = newBin();
-	bin->setPaletteModel(new PaletteModel(true, false, false), true);
+	bin->setPaletteModel(new PaletteModel(true, false), true);
 	bin->setTitle(tr("New bin (%1)").arg(++m_unsavedBinsCount));
 	insertBin(bin, m_stackTabWidget->count());
     bin->setReadOnly(false);
@@ -447,7 +452,7 @@ PartsBinPaletteWidget* BinManager::openCoreBinIn() {
 }
 
 PartsBinPaletteWidget* BinManager::newBin() {
-	PartsBinPaletteWidget* bin = new PartsBinPaletteWidget(m_refModel,m_infoView,m_undoStack,this);
+	PartsBinPaletteWidget* bin = new PartsBinPaletteWidget(m_referenceModel, m_infoView, m_undoStack,this);
 	connect(
 		bin, SIGNAL(fileNameUpdated(PartsBinPaletteWidget*, const QString&, const QString&)),
 		this, SLOT(updateFileName(PartsBinPaletteWidget*, const QString&, const QString&))
@@ -484,16 +489,15 @@ void BinManager::setAsCurrentBin(PartsBinPaletteWidget* bin) {
 	}
 
 	QString style = m_mainWindow->styleSheet();
-	StackTabBar *currTabBar = NULL;
 	if(m_currentBin && m_stackTabWidget) {
-		currTabBar = m_stackTabWidget->stackTabBar();
+		StackTabBar *currTabBar = m_stackTabWidget->stackTabBar();
 		currTabBar->setProperty("current","false");
 		currTabBar->setStyleSheet("");
 		currTabBar->setStyleSheet(style);
 	}
 	if(m_stackTabWidget) {
 		m_currentBin = bin;
-		currTabBar = m_stackTabWidget->stackTabBar();
+		StackTabBar *currTabBar = m_stackTabWidget->stackTabBar();
 		currTabBar->setProperty("current","true");
 		currTabBar->setStyleSheet("");
 		currTabBar->setStyleSheet(style);
@@ -581,8 +585,8 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 	actualLocations.clear();
 
 	foreach (BinLocation * tLocation, theoreticalLocations) {
-                tLocation->marked = false;
-                bool gotOne = false;
+        tLocation->marked = false;
+        bool gotOne = false;
 
 		for (int ix = 0; ix < tempLocations.count(); ix++) {
 			BinLocation  * aLocation = tempLocations[ix];
@@ -590,6 +594,7 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 				gotOne = true;
 				actualLocations.append(aLocation);
 				tempLocations.removeAt(ix);
+                //DebugDialog::debug("adding loc 1 " + aLocation->path);
 				break;
 			}
 		}
@@ -602,7 +607,8 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 					gotOne = true;
 					actualLocations.append(aLocation);
 					tempLocations.removeAt(ix);
-					break;
+                    //DebugDialog::debug("adding loc 2 " + aLocation->path);
+                    break;
 				}
 			}
 		}
@@ -611,8 +617,9 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 		if (!tLocation->path.isEmpty()) {
 			QFileInfo info(tLocation->path);
 			if (info.exists()) {
+                //DebugDialog::debug("adding loc 3 " + tLocation->path);
 				actualLocations.append(tLocation);
-                                tLocation->marked = true;
+                tLocation->marked = true;
 			}		
 		}
 	}
@@ -625,6 +632,10 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 
 	// catch the leftovers
 	actualLocations.append(tempLocations);
+    //foreach (BinLocation * aLocation, tempLocations) {
+        //DebugDialog::debug("adding loc 4 " + aLocation->path);
+    //}
+
 
 }
 
@@ -651,6 +662,93 @@ void BinManager::readTheoreticalLocations(QList<BinLocation *> & theoreticalLoca
 	}
 }
 
+void BinManager::hackLocalContrib(QList<BinLocation *> & locations) 
+{
+    // with release 0.7.12, there is no more local contrib bin
+    // so clear out existing local contrib bins by copying parts to mine bin
+
+    BinLocation * localContrib = NULL;
+    BinLocation * myParts = NULL;
+    foreach (BinLocation * location, locations) {
+        if (location->location == BinLocation::User) {
+            if (location->title == "Contributed Parts") {
+                localContrib = location;
+            }
+            else if (location->title == "My Parts") {
+                myParts = location;
+            }
+        }
+    }
+
+    if (localContrib == NULL) return;
+
+    if (myParts == NULL) {
+        createIfBinNotExists(MyPartsBinLocation, MyPartsBinTemplateLocation);
+	    myParts = new BinLocation;
+	    myParts->location = BinLocation::User;
+	    myParts->path = MyPartsBinLocation;
+	    QString icon;
+	    getBinTitle(myParts->path, myParts->title, icon);
+	    locations.append(myParts);
+    }
+
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+
+    QFile contribFile(localContrib->path);
+    QDomDocument contribDoc;
+    bool result = contribDoc.setContent(&contribFile, true, &errorStr, &errorLine, &errorColumn);
+    locations.removeOne(localContrib);
+    contribFile.close();
+    bool removed = contribFile.remove();
+    if (!removed) {
+        DebugDialog::debug("failed to remove contrib bin");
+    }
+
+    if (!result) return;
+
+    QFile myPartsFile(myParts->path);
+    QDomDocument myPartsDoc;
+    if (!myPartsDoc.setContent(&myPartsFile, true, &errorStr, &errorLine, &errorColumn)) {
+        return;
+    }
+
+    QDomElement myPartsRoot = myPartsDoc.documentElement();
+    QDomElement myPartsInstances = myPartsRoot.firstChildElement("instances");
+
+    bool changed = false;
+
+    QDomElement contribRoot = contribDoc.documentElement();
+    QDomElement contribInstances = contribRoot.firstChildElement("instances");
+    QDomElement contribInstance = contribInstances.firstChildElement("instance");
+    while (!contribInstance.isNull()) {
+        QString moduleIDRef = contribInstance.attribute("moduleIdRef");
+        QDomElement myPartsInstance = myPartsInstances.firstChildElement("instance");
+        bool already = false;
+        while (!myPartsInstance.isNull()) {
+            QString midr = myPartsInstance.attribute("moduleIdRef");
+            if (midr == moduleIDRef) {
+                already = true;
+                break;
+            }
+            myPartsInstance = myPartsInstance.nextSiblingElement("instance");
+        }
+
+        if (!already) {
+            QDomNode node = contribInstance.cloneNode(true);
+            myPartsInstances.appendChild(node);
+            changed = true;
+        }
+
+        contribInstance = contribInstance.nextSiblingElement("instance");
+    }
+
+    if (changed) {
+        TextUtils::writeUtf8(myParts->path, myPartsDoc.toString(0));
+    }
+}
+
 void BinManager::findAllBins(QList<BinLocation *> & locations) 
 {
 	BinLocation * location = new BinLocation;
@@ -660,6 +758,12 @@ void BinManager::findAllBins(QList<BinLocation *> & locations)
 	getBinTitle(location->path, location->title, icon);
 	locations.append(location);
 
+	location = new BinLocation;
+	location->location = BinLocation::App;
+	location->path = ContribPartsBinLocation;
+	getBinTitle(location->path, location->title, icon);
+	locations.append(location);
+
 	QDir userBinsDir(FolderUtils::getUserDataStorePath("bins"));
 	findBins(userBinsDir, locations, BinLocation::User);
 
@@ -723,25 +827,9 @@ void BinManager::addPartTo(PartsBinPaletteWidget* bin, ModelPart* mp, bool setDi
 	}
 }
 
-void BinManager::newPartTo(PartsBinPaletteWidget* bin) {
-	PartsEditorMainWindow *partsEditor = m_mainWindow->getPartsEditor(NULL, -1, NULL, bin);
-	if (partsEditor == NULL) return;
-
-	partsEditor->show();
-	partsEditor->raise();
-}
-
-void BinManager::editSelectedPartFrom(PartsBinPaletteWidget* bin) {
-	ModelPart *selectedMP = bin->selected();
-	PartsEditorMainWindow *partsEditor = m_mainWindow->getPartsEditor(selectedMP, -1, NULL, bin);
-	if (partsEditor == NULL) return;
-
-	partsEditor->show();
-	partsEditor->raise();
-}
-
-void BinManager::dockedInto(FDockWidget* dock) {
-	Q_UNUSED(dock);
+void BinManager::editSelectedPartNewFrom(PartsBinPaletteWidget* bin) {
+	ItemBase * itemBase = bin->selectedItemBase();
+	m_mainWindow->getPartsEditorNewAnd(itemBase);
 }
 
 bool BinManager::isTabReorderingEvent(QDropEvent* event) {
@@ -759,6 +847,8 @@ QList<QAction*> BinManager::openedBinsActions(const QString &moduleId) {
 
 	for (int i = 0; i < m_stackTabWidget->count(); i++) {
 		PartsBinPaletteWidget* pppw = (PartsBinPaletteWidget *) m_stackTabWidget->widget(i);
+        if (pppw->readOnly()) continue;
+
 		QAction *act = pppw->addPartToMeAction();
 		act->setEnabled(!pppw->contains(moduleId));
 		titlesAndActions[pppw->title()] = act;
@@ -780,7 +870,7 @@ void BinManager::initNames() {
     BinManager::MyPartsBinTemplateLocation =":/resources/bins/my_parts.fzb";
     BinManager::SearchBinLocation = FolderUtils::getUserDataStorePath("bins")+"/search.fzb";
     BinManager::SearchBinTemplateLocation =":/resources/bins/search.fzb";
-	BinManager::ContribPartsBinLocation = FolderUtils::getUserDataStorePath("bins")+"/contribParts.fzb";
+	BinManager::ContribPartsBinLocation = FolderUtils::getApplicationSubFolderPath("bins")+"/contribParts.fzb";
     BinManager::CorePartsBinLocation = FolderUtils::getApplicationSubFolderPath("bins")+"/core.fzb";
     BinManager::TempPartsBinTemplateLocation =":/resources/bins/temp.fzb";
 
@@ -796,16 +886,18 @@ void BinManager::search(const QString & searchText) {
 
     FileProgressDialog progress(tr("Searching..."), 0, this);
     progress.setIncValueMod(10);
-    connect(m_refModel, SIGNAL(addSearchMaximum(int)), &progress, SLOT(addMaximum(int)));
-    connect(m_refModel, SIGNAL(incSearch()), &progress, SLOT(incValue()));
+    connect(m_referenceModel, SIGNAL(addSearchMaximum(int)), &progress, SLOT(addMaximum(int)));
+    connect(m_referenceModel, SIGNAL(incSearch()), &progress, SLOT(incValue()));
 
-    QList<ModelPart *> modelParts = m_refModel->search(searchText, false);
+    QList<ModelPart *> modelParts = m_referenceModel->search(searchText, false);
 
     progress.setIncValueMod(1);
     searchBin->removeParts();
     foreach (ModelPart * modelPart, modelParts) {
         //DebugDialog::debug(modelPart->title());
-        this->addPartTo(searchBin, modelPart, false);
+        if (modelPart->itemType() != ModelPart::SchematicSubpart) {
+            this->addPartTo(searchBin, modelPart, false);
+        }
         progress.incValue();
     }
  
@@ -841,15 +933,23 @@ void BinManager::updateBinCombinedMenuCurrent() {
 }
 
 void BinManager::updateBinCombinedMenu(PartsBinPaletteWidget * bin) {
+    if (m_combinedMenu == NULL) return;
+
 	m_saveBinAction->setEnabled(bin->allowsChanges());
 	m_renameBinAction->setEnabled(bin->canClose());
 	m_closeBinAction->setEnabled(bin->canClose());
 	m_deleteBinAction->setEnabled(bin->canClose());
-	ModelPart *mp = bin->selected();
-	bool enabled = (mp != NULL);
-	m_editPartAction->setEnabled(enabled);
-	m_exportPartAction->setEnabled(enabled && !mp->isCore());
+	ItemBase *itemBase = bin->selectedItemBase();
+	bool enabled = (itemBase != NULL);
+	m_editPartNewAction->setEnabled(enabled && itemBase->canEditPart());
+
+    bool enableAnyway = false;
+#ifndef QT_NO_DEBUG
+    enableAnyway = true;
+#endif
+	m_exportPartAction->setEnabled(enabled && (!itemBase->modelPart()->isCore() || enableAnyway));
 	m_removePartAction->setEnabled(enabled && bin->allowsChanges());
+	m_findPartAction->setEnabled(enabled);
 }
 
 void BinManager::createCombinedMenu() 
@@ -912,23 +1012,23 @@ void BinManager::createCombinedMenu()
 	m_combinedMenu->addAction(m_showIconViewAction);
 	m_combinedMenu->addAction(m_showListViewAction);
 
-	m_newPartAction = new QAction(tr("New Part..."), this);
-	m_editPartAction = new QAction(tr("Edit Part..."),this);
+	m_editPartNewAction = new QAction(tr("Edit Part (new parts editor)..."),this);
 	m_exportPartAction = new QAction(tr("Export Part..."),this);
 	m_removePartAction = new QAction(tr("Remove Part"),this);
+	m_findPartAction = new QAction(tr("Find Part in Sketch"),this);
 
-	connect(m_newPartAction, SIGNAL(triggered()),this, SLOT(newPart()));
-	connect(m_editPartAction, SIGNAL(triggered()),this, SLOT(editSelected()));
+	connect(m_editPartNewAction, SIGNAL(triggered()),this, SLOT(editSelectedNew()));
 	connect(m_exportPartAction, SIGNAL(triggered()),this, SLOT(exportSelected()));
 	connect(m_removePartAction, SIGNAL(triggered()),this, SLOT(removeSelected()));
+	connect(m_findPartAction, SIGNAL(triggered()),this, SLOT(findSelected()));
 
 	connect(m_combinedMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinCombinedMenuCurrent()));
 
 	m_combinedMenu->addSeparator();
-	m_combinedMenu->addAction(m_newPartAction);
-	m_combinedMenu->addAction(m_editPartAction);
+	m_combinedMenu->addAction(m_editPartNewAction);
 	m_combinedMenu->addAction(m_exportPartAction);
 	m_combinedMenu->addAction(m_removePartAction);
+	m_combinedMenu->addAction(m_findPartAction);
 
 }
 
@@ -942,9 +1042,10 @@ void BinManager::createContextMenus() {
 	m_binContextMenu->addAction(m_renameBinAction);
 
 	m_partContextMenu = new QMenu(this);
-	m_partContextMenu->addAction(m_editPartAction);
+	m_partContextMenu->addAction(m_editPartNewAction);
 	m_partContextMenu->addAction(m_exportPartAction);
 	m_partContextMenu->addAction(m_removePartAction);
+	m_partContextMenu->addAction(m_findPartAction);
 }
 
 void BinManager::closeBin() {
@@ -970,16 +1071,13 @@ void BinManager::deleteBin() {
 	QFile::remove(filename);
 }
 
-void BinManager::newPart() {
-	newPartTo(currentBin());
-}
-
-void BinManager::importPartToContribBin(const QString & filename) {
+void BinManager::importPartToMineBin(const QString & filename) {
 
 	if (!filename.isEmpty() && !filename.isNull()) {
-        PartsBinPaletteWidget * bin = getOrOpenBin(ContribPartsBinLocation, ContribPartsBinTemplateLocation);
+        PartsBinPaletteWidget * bin = getOrOpenBin(MyPartsBinLocation, MyPartsBinTemplateLocation);
         if (bin == NULL) return;
 
+        setAsCurrentTab(bin);
         importPart(filename, bin);
     }
 }
@@ -1001,8 +1099,8 @@ void BinManager::importPart(const QString & filename, PartsBinPaletteWidget * bi
 	}
 }
 
-void BinManager::editSelected() {
-	editSelectedPartFrom(currentBin());
+void BinManager::editSelectedNew() {
+	editSelectedPartNewFrom(currentBin());
 }
 
 void BinManager::renameBin() {
@@ -1011,7 +1109,7 @@ void BinManager::renameBin() {
 
 	if (!currentBin()->allowsChanges()) {
 		// TODO: disable menu item instead
-		QMessageBox::warning(NULL, tr("Read-only bin"), tr("This bin cannot be renamed."));
+		QMessageBox::warning(this, tr("Read-only bin"), tr("This bin cannot be renamed."));
 		return;
 	}
 
@@ -1049,6 +1147,9 @@ void BinManager::saveBinAs() {
 
 
 void BinManager::updateViewChecks(bool iconView) {
+    if (m_showListViewAction == NULL) return;
+    if (m_showIconViewAction == NULL) return;
+
 	if (iconView) {
 		m_showListViewAction->setChecked(false);
 		m_showIconViewAction->setChecked(true);
@@ -1082,32 +1183,48 @@ bool BinManager::removeSelected() {
 	PartsBinPaletteWidget * bin = currentBin();
 	if (bin == NULL) return false;
 
-	ModelPart * mp = bin->selected();
+	ModelPart * mp = bin->selectedModelPart();
 	if (mp == NULL) return false;
 
+    if (m_mainWindow->anyUsePart(mp->moduleID())) {
+        QMessageBox::warning(this, tr("Remove from Bin"), tr("Unable to remove part '%1'--it is in use in a sketch").arg(mp->title()));
+        return false;
+    }
+
 	QMessageBox::StandardButton answer = QMessageBox::question(
 		this,
 		tr("Remove from bin"),
-		tr("Do you really want to remove '%1' from the bin?").arg(mp->title()),
+		tr("Do you really want to remove '%1' from the bin? This operation cannot be undone.").arg(mp->title()),
 		QMessageBox::Yes | QMessageBox::No,
-		QMessageBox::Yes
+		QMessageBox::No
 	);
 	// TODO: make button texts translatable
 	if(answer != QMessageBox::Yes) return false;
 
 
 	m_undoStack->push(new QUndoCommand("Parts bin: part removed"));
-	bin->removePart(mp->moduleID());
+	bin->removePart(mp->moduleID(), mp->path());
 	bin->setDirty();
 
 	return true;
 }
 
+void BinManager::findSelected() {
+	PartsBinPaletteWidget * bin = currentBin();
+	if (bin == NULL) return;
+
+	ModelPart * mp = bin->selectedModelPart();
+	if (mp == NULL) return;
+
+    m_mainWindow->selectPartsWithModuleID(mp);
+}
+
+
 void BinManager::exportSelected() {
 	PartsBinPaletteWidget * bin = currentBin();
 	if (bin == NULL) return;
 
-	ModelPart * mp = bin->selected();
+	ModelPart * mp = bin->selectedModelPart();
 	if (mp == NULL) return;
 
 	emit savePartAsBundled(mp->moduleID());
@@ -1149,10 +1266,10 @@ void BinManager::copyFilesToContrib(ModelPart * mp, QWidget * originator) {
 	dir.cd("svg");
 	dir.cd("contrib");
 
-	QList<ViewIdentifierClass::ViewIdentifier> viewIdentifiers;
-	viewIdentifiers << ViewIdentifierClass::IconView << ViewIdentifierClass::BreadboardView << ViewIdentifierClass::SchematicView << ViewIdentifierClass::PCBView;
-	foreach (ViewIdentifierClass::ViewIdentifier viewIdentifier, viewIdentifiers) {
-		QString fn = mp->hasBaseNameFor(viewIdentifier);
+	QList<ViewLayer::ViewID> viewIDs;
+	viewIDs << ViewLayer::IconView << ViewLayer::BreadboardView << ViewLayer::SchematicView << ViewLayer::PCBView;
+	foreach (ViewLayer::ViewID viewID, viewIDs) {
+		QString fn = mp->hasBaseNameFor(viewID);
 		if (!fn.isEmpty()) {
 			QFile svg(dir.absoluteFilePath(fn));
 			svg.copy(prefix + fn);
@@ -1194,3 +1311,17 @@ void BinManager::mainLoad() {
     }
 }
 
+void BinManager::hideTabBar()
+{
+    m_stackTabWidget->stackTabBar()->hide();
+}
+
+void BinManager::reloadPart(const QString & moduleID) {
+	PartsBinView::removePartReference(moduleID);
+	for(int j = 0; j < m_stackTabWidget->count(); j++) {
+		PartsBinPaletteWidget *bin = qobject_cast<PartsBinPaletteWidget*>(m_stackTabWidget->widget(j));
+		if (bin == NULL) continue;
+
+		bin->reloadPart(moduleID);
+	}
+}
