@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6934 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-03-22 08:35:24 +0100 (Fr, 22. Mrz 2013) $
 
 ********************************************************************/
 
@@ -41,8 +41,8 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 static const QColor SectionHeaderBackgroundColor(128, 128, 128);
 static const QColor SectionHeaderForegroundColor(32, 32, 32);
 
-PartsBinListView::PartsBinListView(ReferenceModel* refModel, PartsBinPaletteWidget *parent)
-	: QListWidget((QWidget *) parent), PartsBinView(refModel, parent)
+PartsBinListView::PartsBinListView(ReferenceModel* referenceModel, PartsBinPaletteWidget *parent)
+	: QListWidget((QWidget *) parent), PartsBinView(referenceModel, parent)
 {
 	m_infoView = NULL;
 	m_hoverItem = NULL;
@@ -83,8 +83,7 @@ int PartsBinListView::setItemAux(ModelPart * modelPart, int position) {
 
 	emit settingItem();
 	QString moduleID = modelPart->moduleID();
-	if(contains(moduleID)) {
-		m_partHash[moduleID]->copy(modelPart);   // copies into the cached modelPart, but I don't know why
+	if (contains(moduleID)) {
 		return position;
 	}
 
@@ -97,28 +96,7 @@ int PartsBinListView::setItemAux(ModelPart * modelPart, int position) {
 		lwi->setText("        " + TranslatedCategoryNames.value(modelPart->instanceText(), modelPart->instanceText()));
 	}
 	else {
-        ItemBase * itemBase = ItemBaseHash.value(moduleID);
-        if (itemBase == NULL) {
-		    itemBase = PartFactory::createPart(modelPart, ViewLayer::ThroughHoleThroughTop_OneLayer, ViewIdentifierClass::IconView, ViewGeometry(), ItemBase::getNextID(), NULL, NULL, false);
-		    ItemBaseHash.insert(moduleID, itemBase);
-            QString error;
-		    LayerAttributes layerAttributes;
-		    FSvgRenderer * renderer = itemBase->setUpImage(modelPart, ViewIdentifierClass::IconView, ViewLayer::Icon, itemBase->viewLayerSpec(), layerAttributes, error);
-		    if (renderer != NULL) {
-			    if (itemBase) {
-				    itemBase->setFilename(renderer->filename());
-			    }
-                itemBase->setSharedRendererEx(renderer);
-            }
-        }
-		lwi->setData(Qt::UserRole, qVariantFromValue( itemBase ) );
-		QSize size(HtmlInfoView::STANDARD_ICON_IMG_WIDTH, HtmlInfoView::STANDARD_ICON_IMG_HEIGHT);
-		QPixmap * pixmap = FSvgRenderer::getPixmap(itemBase->renderer(), size);
-		lwi->setIcon(QIcon(*pixmap));
-		delete pixmap;
-		lwi->setData(Qt::UserRole + 1, itemBase->renderer()->defaultSize());
-
-		m_partHash[moduleID] = modelPart;
+		loadImage(lwi, moduleID);
 	}
 
 	if(position > -1 && position < count()) {
@@ -133,8 +111,6 @@ int PartsBinListView::setItemAux(ModelPart * modelPart, int position) {
 }
 
 void PartsBinListView::mouseMoveEvent(QMouseEvent *event) {
-	if (m_infoView == NULL) return;
-
 	if(m_infoViewOnHover) {
 		QListWidgetItem * item = itemAt(event->pos());
 		if (item != NULL) {
@@ -154,8 +130,8 @@ void PartsBinListView::showInfo(QListWidgetItem * item) {
 		return;
 	}
 
-	if (m_hoverItem != NULL) {
-		ItemBase * itemBase = m_hoverItem->data(Qt::UserRole).value<ItemBase *>();
+	if (m_hoverItem != NULL && m_infoView != NULL) {
+		ItemBase * itemBase = itemItemBase(m_hoverItem);
 		if (itemBase != NULL) {
 			m_infoView->hoverLeaveItem(NULL, NULL, itemBase);
 		}
@@ -166,11 +142,12 @@ void PartsBinListView::showInfo(QListWidgetItem * item) {
 	}
 
 	m_hoverItem = item;
-
-	ItemBase * itemBase = item->data(Qt::UserRole).value<ItemBase *>();
-	if (itemBase == NULL) return;
-
-	m_infoView->hoverEnterItem(NULL, NULL, itemBase, swappingEnabled());
+    if (m_infoView != NULL) {
+	    ItemBase * itemBase = itemItemBase(item);
+	    if (itemBase != NULL) {
+	        m_infoView->hoverEnterItem(NULL, NULL, itemBase, swappingEnabled());
+        }
+    }
 }
 
 
@@ -181,12 +158,12 @@ void PartsBinListView::mousePressEvent(QMouseEvent *event) {
 	QListWidgetItem * current = currentItem();
 	if (current == NULL) {
 		m_hoverItem = NULL;
-		m_infoView->viewItemInfo(NULL, NULL, false);
+        if (m_infoView != NULL) m_infoView->viewItemInfo(NULL, NULL, false);
 		return;
 	}
 
 	showInfo(current);
-	m_infoView->viewItemInfo(NULL, current->data(Qt::UserRole).value<ItemBase *>(), false);
+	if (m_infoView != NULL) m_infoView->viewItemInfo(NULL, itemItemBase(current), false);
 }
 
 void PartsBinListView::setInfoView(HtmlInfoView * infoView) {
@@ -197,14 +174,14 @@ void PartsBinListView::removePart(const QString &moduleID) {
 	m_hoverItem = NULL;
 	int idxToRemove = position(moduleID);
 	if(idxToRemove > -1) {
-		m_partHash.remove(moduleID);
+		m_itemBaseHash.remove(moduleID);
 		delete takeItem(idxToRemove);
 	}
 }
 
 void PartsBinListView::removeParts() {
 	m_hoverItem = NULL;
-    m_partHash.clear();
+    m_itemBaseHash.clear();
     while (count() > 0) {
         delete takeItem(0);
     }
@@ -219,8 +196,12 @@ int PartsBinListView::position(const QString &moduleID) {
 	return -1;
 }
 
+ItemBase * PartsBinListView::itemItemBase(const QListWidgetItem *item) const {
+    return item->data(Qt::UserRole).value<ItemBase *>();
+}
+
 ModelPart *PartsBinListView::itemModelPart(const QListWidgetItem *item) const {
-	ItemBase * itemBase = item->data(Qt::UserRole).value<ItemBase *>();
+	ItemBase * itemBase = itemItemBase(item);
 	if (itemBase == NULL) return NULL;
 
 	return itemBase->modelPart();
@@ -236,7 +217,14 @@ const QString &PartsBinListView::itemModuleID(const QListWidgetItem *item) {
 	return modelPart->moduleID();
 }
 
-ModelPart *PartsBinListView::selected() {
+ItemBase *PartsBinListView::selectedItemBase() {
+	if(selectedItems().size()==1) {
+		return itemItemBase(selectedItems()[0]);
+	}
+	return NULL;
+}
+
+ModelPart *PartsBinListView::selectedModelPart() {
 	if(selectedItems().size()==1) {
 		return itemModelPart(selectedItems()[0]);
 	}
@@ -371,3 +359,49 @@ void PartsBinListView::showContextMenu(const QPoint& pos) {
 	}
     menu->exec(mapToGlobal(pos));
 }
+
+void PartsBinListView::reloadPart(const QString & moduleID) {
+	if (!contains(moduleID)) return;
+
+	for(int i = 0; i < count(); i++) {
+		QListWidgetItem * lwi = item(i);
+		ItemBase * itemBase = itemItemBase(lwi);
+		if (itemBase == NULL) continue;
+
+		if (itemBase->moduleID().compare(moduleID) == 0) {
+			lwi->setText(itemBase->title());
+			loadImage(lwi, moduleID);
+			return;
+		}
+	}
+}
+
+void PartsBinListView::loadImage(QListWidgetItem * lwi, const QString & moduleID) 
+{
+    ItemBase * itemBase = ItemBaseHash.value(moduleID);
+    if (itemBase == NULL) {
+		ModelPart * modelPart = itemBase->modelPart();
+		itemBase = PartFactory::createPart(modelPart, ViewLayer::NewTop, ViewLayer::IconView, ViewGeometry(), ItemBase::getNextID(), NULL, NULL, false);
+		ItemBaseHash.insert(moduleID, itemBase);
+		LayerAttributes layerAttributes;
+        layerAttributes.viewID = ViewLayer::IconView;
+        layerAttributes.viewLayerID = ViewLayer::Icon;
+        layerAttributes.viewLayerPlacement = itemBase->viewLayerPlacement();
+        layerAttributes.createShape = layerAttributes.doConnectors = false;
+		FSvgRenderer * renderer = itemBase->setUpImage(modelPart, layerAttributes);
+		if (renderer != NULL) {
+			if (itemBase) {
+				itemBase->setFilename(renderer->filename());
+			}
+            itemBase->setSharedRendererEx(renderer);
+        }
+    }
+	lwi->setData(Qt::UserRole, qVariantFromValue( itemBase ) );
+	QSize size(HtmlInfoView::STANDARD_ICON_IMG_WIDTH, HtmlInfoView::STANDARD_ICON_IMG_HEIGHT);
+	QPixmap * pixmap = FSvgRenderer::getPixmap(itemBase->renderer(), size);
+	lwi->setIcon(QIcon(*pixmap));
+	delete pixmap;
+	lwi->setData(Qt::UserRole + 1, itemBase->renderer()->defaultSize());
+
+	m_itemBaseHash.insert(moduleID, itemBase);
+}
\ No newline at end of file
