@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6265 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
+$Revision: 6904 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
 
 ********************************************************************/
 
@@ -40,15 +40,14 @@ $Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 #include "partsbincommands.h"
 #include "partsbiniconview.h"
 #include "partsbinlistview.h"
-#include "simpleeditablelabelwidget.h"
 #include "searchlineedit.h"
 #include "../utils/misc.h"
 #include "../debugdialog.h"
-#include "../dockmanager.h"
 #include "../infoview/htmlinfoview.h"
 #include "../utils/fileprogressdialog.h"
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
+#include "../mainwindow/mainwindow.h"          // TODO: PartsBinPaletteWidget should not call MainWindow functions
 
 
 static QString CustomIconName = "Custom1.png";
@@ -62,9 +61,10 @@ static QHash<QString, PaletteModel *> PaletteBinModels;
 
 //////////////////////////////////////////////
 
-PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoView *infoView, WaitPushUndoStack *undoStack, BinManager* manager) :
+PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *referenceModel, HtmlInfoView *infoView, WaitPushUndoStack *undoStack, BinManager* manager) :
 	QFrame(manager)
 {
+    m_binLabel = NULL;
 	m_icon = NULL;
 	m_searchLineEdit = NULL;
 	m_saveQuietly = false;
@@ -78,7 +78,7 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 
 	m_manager = manager;
 
-	m_refModel = refModel;
+	m_referenceModel = referenceModel;
 	m_canDeleteModel = false;
 	m_orderHasChanged = false;
 
@@ -87,12 +87,10 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 	m_undoStack = new WaitPushUndoStack(this);
 	connect(m_undoStack, SIGNAL(cleanChanged(bool)), this, SLOT(undoStackCleanChanged(bool)) );
 
-	setupHeader();
-
-	m_iconView = new PartsBinIconView(m_refModel, this);
+	m_iconView = new PartsBinIconView(m_referenceModel, this);
 	m_iconView->setInfoView(infoView);
 
-	m_listView = new PartsBinListView(m_refModel, this);
+	m_listView = new PartsBinListView(m_referenceModel, this);
 	m_listView->setInfoView(infoView);
 
 	m_stackedWidget = new QStackedWidget(this);
@@ -102,14 +100,19 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 	QVBoxLayout * vbl = new QVBoxLayout(this);
     vbl->setMargin(3);
     vbl->setSpacing(0);
-	vbl->addWidget(m_header);
 
-	QFrame * separator = new QFrame();
-	separator->setMaximumHeight(1);
-	separator->setObjectName("partsBinHeaderSeparator");
-    separator->setFrameShape(QFrame::HLine);
-    separator->setFrameShadow(QFrame::Plain);
-	vbl->addWidget(separator);
+    m_header = NULL;
+    setupHeader();
+    if (m_header) {
+	    vbl->addWidget(m_header);
+
+	    QFrame * separator = new QFrame();
+	    separator->setMaximumHeight(1);
+	    separator->setObjectName("partsBinHeaderSeparator");
+        separator->setFrameShape(QFrame::HLine);
+        separator->setFrameShadow(QFrame::Plain);
+	    vbl->addWidget(separator);
+    }
 
 	vbl->addWidget(m_stackedWidget);
 	this->setLayout(vbl);
@@ -131,7 +134,7 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 	connect(m_listView, SIGNAL(informItemMoved(int,int)), this, SLOT(itemMoved()));
 	connect(m_iconView, SIGNAL(informItemMoved(int,int)), this, SLOT(itemMoved()));
 
-	m_binLabel->setText(m_title);
+	if (m_binLabel) m_binLabel->setText(m_title);
 
 	m_addPartToMeAction = new QAction(m_title,this);
 	connect(m_addPartToMeAction, SIGNAL(triggered()),this, SLOT(addSketchPartToMe()));
@@ -157,7 +160,7 @@ void PartsBinPaletteWidget::cleanup() {
 
 
 QSize PartsBinPaletteWidget::sizeHint() const {
-	return QSize(DockManager::DockDefaultWidth, DockManager::PartsBinDefaultHeight);
+	return QSize(DockWidthDefault, PartsBinHeightDefault);
 }
 
 QString PartsBinPaletteWidget::title() const {
@@ -167,14 +170,17 @@ QString PartsBinPaletteWidget::title() const {
 void PartsBinPaletteWidget::setTitle(const QString &title) {
 	if(m_title != title) {
 		m_title = title;
-		m_binLabel->setText(title);
+		if (m_binLabel) m_binLabel->setText(title);
 	}
 }
 
 void PartsBinPaletteWidget::setupHeader()
 {
-	m_combinedBinMenuButton = newToolButton("partsBinCombinedMenuButton");
-	m_combinedBinMenuButton->setMenu(m_manager->combinedMenu());
+    QMenu * combinedMenu = m_manager->combinedMenu();
+    if (combinedMenu == NULL) return;
+
+	m_combinedBinMenuButton = newToolButton("partsBinCombinedMenuButton", ___emptyString___, ___emptyString___);
+	m_combinedBinMenuButton->setMenu(combinedMenu);
 
 	m_binLabel = new QLabel(this);
 	m_binLabel->setObjectName("partsBinLabel");
@@ -254,6 +260,14 @@ bool PartsBinPaletteWidget::saveAsAux(const QString &filename) {
 	}
 	emit saved(hasAlienParts());
 
+    foreach (QString path, m_removed) {
+        bool result = QFile::remove(path);
+        if (!result) {
+            DebugDialog::debug("unable to delete '" + path + "' from bin");
+        }
+    }
+    m_removed.clear();
+
 	return true;
 }
 
@@ -286,14 +300,14 @@ void PartsBinPaletteWidget::grabTitle(PaletteModel *model) {
 	grabTitle(root->title(), iconFilename);
 	root->setIcon(iconFilename);
 
-	m_searchLineEdit->setText(root->searchTerm());
+	if (m_searchLineEdit) m_searchLineEdit->setText(root->searchTerm());
 }
 
 void PartsBinPaletteWidget::grabTitle(const QString & title, QString & iconFilename)
 {
 	m_title = title;
 	m_addPartToMeAction->setText(m_title);
-	m_binLabel->setText(m_title);
+	if (m_binLabel) m_binLabel->setText(m_title);
 
 	QString temp = BinManager::StandardBinIcons.value(m_fileName, "");
 	if (!temp.isEmpty()) {
@@ -438,7 +452,7 @@ bool PartsBinPaletteWidget::open(QString fileName, QWidget * progressTarget, boo
     	load(fileName, progressTarget, fastLoad);
     	m_isDirty = false;
     } else if(fileName.endsWith(FritzingBundledBinExtension)) {
-    	m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false, false);
+    	return m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false, false);
     }
 
     return true;
@@ -465,15 +479,15 @@ void PartsBinPaletteWidget::load(const QString &filename, QWidget * progressTarg
 	m_fastLoaded = false;
     PaletteModel * paletteBinModel = PaletteBinModels.value(filename);
     if (paletteBinModel == NULL) {
-	    paletteBinModel = new PaletteModel(true, false, false);
-	    DebugDialog::debug("after palette model");
+	    paletteBinModel = new PaletteModel(true, false);
+	    //DebugDialog::debug("after palette model");
 
 	    QString name = m_title;
-	    if (name.isEmpty()) name = QFileInfo(filename).baseName();
+	    if (name.isEmpty()) name = QFileInfo(filename).completeBaseName();
 
 	    bool deleteWhenDone = false;
         if (progressTarget != NULL) {
-            DebugDialog::debug("open progress " + filename);
+            //DebugDialog::debug("open progress " + filename);
 		    deleteWhenDone = true;
             progressTarget = m_loadingProgressDialog = new FileProgressDialog(tr("Loading..."), 200, progressTarget);
 		    m_loadingProgressDialog->setBinLoadingChunk(200);
@@ -489,11 +503,11 @@ void PartsBinPaletteWidget::load(const QString &filename, QWidget * progressTarg
 		    connect(m_listView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
 	    }
 	    DebugDialog::debug(QString("loading bin '%1'").arg(name));
-	    bool result = paletteBinModel->load(filename, m_refModel);
-	    DebugDialog::debug(QString("done loading bin '%1'").arg(name));
+	    bool result = paletteBinModel->loadFromFile(filename, m_referenceModel, false);
+	    //DebugDialog::debug(QString("done loading bin '%1'").arg(name));
 
 	    if (!result) {
-		    QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("Friting cannot load the parts bin"));
+		    QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("Fritzing cannot load the parts bin"));
 	    }
 	    else {
 		    m_fileName = filename;
@@ -502,7 +516,7 @@ void PartsBinPaletteWidget::load(const QString &filename, QWidget * progressTarg
 	    }
 
 	    if (progressTarget) {
-            DebugDialog::debug("close progress " + filename);
+            //DebugDialog::debug("close progress " + filename);
 		    disconnect(paletteBinModel, SIGNAL(loadingInstances(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstancesSlot(ModelBase *, QDomElement &)));
 		    disconnect(paletteBinModel, SIGNAL(loadingInstance(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstanceSlot(ModelBase *, QDomElement &)));
 		    disconnect(m_iconView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
@@ -582,8 +596,12 @@ void PartsBinPaletteWidget::mousePressEvent(QMouseEvent* event) {
 	QFrame::mousePressEvent(event);
 }
 
-ModelPart * PartsBinPaletteWidget::selected() {
-	return m_currentView->selected();
+ModelPart * PartsBinPaletteWidget::selectedModelPart() {
+	return m_currentView->selectedModelPart();
+}
+
+ItemBase * PartsBinPaletteWidget::selectedItemBase() {
+	return m_currentView->selectedItemBase();
 }
 
 bool PartsBinPaletteWidget::contains(const QString &moduleID) {
@@ -595,18 +613,20 @@ bool PartsBinPaletteWidget::hasAlienParts() {
 }
 
 void PartsBinPaletteWidget::addPart(const QString& moduleID, int position) {
-	ModelPart *modelPart = m_refModel->retrieveModelPart(moduleID);
+	ModelPart *modelPart = m_referenceModel->retrieveModelPart(moduleID);
 	addPart(modelPart, position);
 }
 
-void PartsBinPaletteWidget::removePart(const QString& moduleID) {
+void PartsBinPaletteWidget::removePart(const QString & moduleID, const QString & path) {
 	m_iconView->removePart(moduleID);
 	m_listView->removePart(moduleID);
 
 	// remove the model part from the model last, as this deletes it,
 	// and the removePart calls above still need the modelpart
 	m_model->removePart(moduleID);
-
+    if (path.contains(FolderUtils::getUserDataStorePath())) {
+        m_removed << path;
+    }
 }
 
 
@@ -621,7 +641,7 @@ void PartsBinPaletteWidget::removeParts() {
 
 void PartsBinPaletteWidget::removeAlienParts() {
 	foreach(QString moduleID, m_alienParts) {
-		removePart(moduleID);
+		removePart(moduleID, "");
 	}
 	m_alienParts.clear();
 }
@@ -634,7 +654,7 @@ void PartsBinPaletteWidget::setInfoViewOnHover(bool infoViewOnHover) {
 void PartsBinPaletteWidget::addPartCommand(const QString& moduleID) {
 	/*bool updating = alreadyIn(moduleID);
 
-	QString partTitle = m_refModel->partTitle(moduleID);
+	QString partTitle = m_referenceModel->partTitle(moduleID);
 	if(partTitle.isEmpty()) partTitle = moduleID;
 
 	QString undoStackMsg;
@@ -720,6 +740,8 @@ QAction *PartsBinPaletteWidget::addPartToMeAction() {
 void PartsBinPaletteWidget::addSketchPartToMe() {
     m_manager->openBinIn(this->m_fileName, false);
 	QString moduleID = m_manager->getSelectedModuleIDFromSketch();
+    if (moduleID.isEmpty()) return;
+
 	bool wasAlreadyIn = contains(moduleID);
 	addPart(moduleID, -1);
 	if(!wasAlreadyIn) {
@@ -822,7 +844,7 @@ QMenu * PartsBinPaletteWidget::binContextMenu()
 
 	// TODO: need to enable/disable actions based on this bin
 
-	ModelPartSharedRoot * root = m_model->rootModelPartShared();
+	ModelPartSharedRoot * root = (m_model == NULL) ? NULL : m_model->rootModelPartShared();
 	if (root) {
 		QString iconFilename = root->icon();
 		if (iconFilename.compare(CustomIconName) == 0 || isCustomSvg(iconFilename)) {
@@ -915,3 +937,8 @@ ModelPart * PartsBinPaletteWidget::root() {
 bool PartsBinPaletteWidget::isTempPartsBin() {
     return m_manager->isTempPartsBin(this);
 }
+
+void PartsBinPaletteWidget::reloadPart(const QString & moduleID) {
+	m_iconView->reloadPart(moduleID);
+	m_listView->reloadPart(moduleID);
+}
