@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6149 $:
-$Author: andre.knoerig@gmail.com $:
-$Date: 2012-07-07 15:46:37 +0200 (Sat, 07 Jul 2012) $
+$Revision: 6976 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-21 09:50:09 +0200 (So, 21. Apr 2013) $
 
 ********************************************************************/
 
@@ -37,8 +37,8 @@ $Date: 2012-07-07 15:46:37 +0200 (Sat, 07 Jul 2012) $
 
 static const double WireHoverStrokeFactor = 4.0;
 
-BreadboardSketchWidget::BreadboardSketchWidget(ViewIdentifierClass::ViewIdentifier viewIdentifier, QWidget *parent)
-    : SketchWidget(viewIdentifier, parent)
+BreadboardSketchWidget::BreadboardSketchWidget(ViewLayer::ViewID viewID, QWidget *parent)
+    : SketchWidget(viewID, parent)
 {
 	m_shortName = QObject::tr("bb");
 	m_viewName = QObject::tr("Breadboard View");
@@ -66,7 +66,8 @@ void BreadboardSketchWidget::findConnectorsUnder(ItemBase * item) {
 }
 
 void BreadboardSketchWidget::addViewLayers() {
-	addBreadboardViewLayers();
+	setViewLayerIDs(ViewLayer::Breadboard, ViewLayer::BreadboardWire, ViewLayer::Breadboard, ViewLayer::BreadboardRuler, ViewLayer::BreadboardNote);
+	addViewLayersAux(ViewLayer::layersForView(ViewLayer::BreadboardView), ViewLayer::layersForViewFromBelow(ViewLayer::BreadboardView));
 }
 
 bool BreadboardSketchWidget::disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash & connectorHash, bool doCommand, bool rubberBandLegEnabled, QUndoCommand * parentCommand)
@@ -92,7 +93,7 @@ bool BreadboardSketchWidget::disconnectFromFemale(ItemBase * item, QHash<long, I
 				fromConnectorItem->tempRemove(toConnectorItem, true);
 				toConnectorItem->tempRemove(fromConnectorItem, true);
 				if (doCommand) {
-					extendChangeConnectionCommand(BaseCommand::CrossView, fromConnectorItem, toConnectorItem, ViewLayer::Bottom, false, parentCommand);
+					extendChangeConnectionCommand(BaseCommand::CrossView, fromConnectorItem, toConnectorItem, ViewLayer::NewBottom, false, parentCommand);
 				}
 				connectorHash.insert(fromConnectorItem, toConnectorItem);
 
@@ -110,13 +111,17 @@ BaseCommand::CrossViewType BreadboardSketchWidget::wireSplitCrossView()
 }
 
 bool BreadboardSketchWidget::canDropModelPart(ModelPart * modelPart) {	
+	if (!SketchWidget::canDropModelPart(modelPart)) return false;
+
     if (Board::isBoard(modelPart)) {
         return matchesLayer(modelPart);
     }
 
 	switch (modelPart->itemType()) {
 		case ModelPart::Logo:
+             if (modelPart->moduleID().contains("breadboard", Qt::CaseInsensitive)) return true;
 		case ModelPart::Symbol:
+		case ModelPart::SchematicSubpart:
 		case ModelPart::Jumper:
 		case ModelPart::CopperFill:
 		case ModelPart::Hole:
@@ -139,7 +144,7 @@ void BreadboardSketchWidget::initWire(Wire * wire, int penWidth) {
 	wire->setColorString("blue", 1.0);
 }
 
-const QString & BreadboardSketchWidget::traceColor(ViewLayer::ViewLayerSpec) {
+const QString & BreadboardSketchWidget::traceColor(ViewLayer::ViewLayerPlacement) {
 	if (!m_lastColorSelected.isEmpty()) return m_lastColorSelected;
 
 	static QString color = "blue";
@@ -151,7 +156,7 @@ double BreadboardSketchWidget::getTraceWidth() {
 	return 2.0;
 }
 
-void BreadboardSketchWidget::getLabelFont(QFont & font, QColor & color, ViewLayer::ViewLayerSpec) {
+void BreadboardSketchWidget::getLabelFont(QFont & font, QColor & color, ItemBase *) {
 	font.setFamily("Droid Sans");
 	font.setPointSize(9);
 	font.setBold(false);
@@ -163,9 +168,11 @@ void BreadboardSketchWidget::getLabelFont(QFont & font, QColor & color, ViewLaye
 void BreadboardSketchWidget::setNewPartVisible(ItemBase * itemBase) {
 	switch (itemBase->itemType()) {
 		case ModelPart::Symbol:
+		case ModelPart::SchematicSubpart:
 		case ModelPart::Jumper:
 		case ModelPart::CopperFill:
 		case ModelPart::Logo:
+            if (itemBase->moduleID().contains("breadboard", Qt::CaseInsensitive)) return;
 		case ModelPart::Hole:
 		case ModelPart::Via:
 			itemBase->setVisible(false);
@@ -173,7 +180,8 @@ void BreadboardSketchWidget::setNewPartVisible(ItemBase * itemBase) {
 			return;
 		default:
 			if (itemBase->moduleID().endsWith(ModuleIDNames::SchematicFrameModuleIDName) || 
-				itemBase->moduleID().endsWith(ModuleIDNames::PadModuleIDName)) 
+				itemBase->moduleID().endsWith(ModuleIDNames::PadModuleIDName) ||
+                itemBase->moduleID().endsWith(ModuleIDNames::PowerModuleIDName)) 
 			{
 				itemBase->setVisible(false);
 				itemBase->setEverVisible(false);
@@ -195,7 +203,7 @@ double BreadboardSketchWidget::defaultGridSizeInches() {
 	return 0.1;
 }
 
-ViewLayer::ViewLayerID BreadboardSketchWidget::getLabelViewLayerID(ViewLayer::ViewLayerSpec) {
+ViewLayer::ViewLayerID BreadboardSketchWidget::getLabelViewLayerID(ItemBase *) {
 	return ViewLayer::BreadboardLabel;
 }
 
@@ -203,7 +211,8 @@ void BreadboardSketchWidget::addDefaultParts() {
 	long newID = ItemBase::getNextID();
 	ViewGeometry viewGeometry;
 	viewGeometry.setLoc(QPointF(0, 0));
-	m_addedDefaultPart = addItem(paletteModel()->retrieveModelPart(ModuleIDNames::FullPlusBreadboardModuleIDName), defaultViewLayerSpec(), BaseCommand::CrossView, viewGeometry, newID, -1, NULL, NULL);
+    ModelPart * modelPart = referenceModel()->retrieveModelPart(ModuleIDNames::FullPlusBreadboardModuleIDName);
+	m_addedDefaultPart = addItem(modelPart, defaultViewLayerPlacement(modelPart), BaseCommand::CrossView, viewGeometry, newID, -1, NULL);
 	m_addDefaultParts = true;
 	// have to put this off until later, because positioning the item doesn't work correctly until the view is visible
 }
@@ -229,6 +238,7 @@ void BreadboardSketchWidget::showEvent(QShowEvent * event) {
             p += QPointF(4, 50);
 			QPointF q = mapToScene(p.toPoint());
 			m_addedDefaultPart->setPos(q);
+			alignOneToGrid(m_addedDefaultPart);
 			QTimer::singleShot(10, this, SLOT(vScrollToZero()));
 		}
 	}
