@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6912 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 
 ********************************************************************/
 
@@ -36,7 +36,7 @@ class BreadboardSketchWidget : public SketchWidget
 	Q_OBJECT
 
 public:
-    BreadboardSketchWidget(ViewIdentifierClass::ViewIdentifier, QWidget *parent=0);
+    BreadboardSketchWidget(ViewLayer::ViewID, QWidget *parent=0);
 
 	void addViewLayers();
 	void initWire(Wire *, int penWidth);
@@ -55,13 +55,13 @@ protected:
 	bool disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash &, bool doCommand, bool rubberBandLegEnabled, QUndoCommand * parentCommand);
 	BaseCommand::CrossViewType wireSplitCrossView();
 	bool canDropModelPart(ModelPart * modelPart); 
-	void getLabelFont(QFont &, QColor &, ViewLayer::ViewLayerSpec);
+	void getLabelFont(QFont &, QColor &, ItemBase *);
 	void setNewPartVisible(ItemBase *);
 	double defaultGridSizeInches();
-	ViewLayer::ViewLayerID getLabelViewLayerID(ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerID getLabelViewLayerID(ItemBase *);
 	QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	double getTraceWidth();
-	const QString & traceColor(ViewLayer::ViewLayerSpec);
+	const QString & traceColor(ViewLayer::ViewLayerPlacement);
 	double getRatsnestOpacity();
 	double getRatsnestWidth();
 };
