@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6912 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 
 ********************************************************************/
 
@@ -38,6 +38,7 @@ InfoGraphicsView::InfoGraphicsView( QWidget * parent )
 	m_infoView = NULL;
 	m_boardLayers = 1;
 	m_hoverEnterMode = m_hoverEnterConnectorMode = false;
+    m_smdOrientation = Qt::Vertical;
 }
 
 void InfoGraphicsView::viewItemInfo(ItemBase * item) {
@@ -49,7 +50,7 @@ void InfoGraphicsView::viewItemInfo(ItemBase * item) {
 void InfoGraphicsView::hoverEnterItem(QGraphicsSceneHoverEvent * event, ItemBase * itemBase) {
 	if (m_infoView == NULL) return;
 
-	if (event->modifiers() & Qt::ShiftModifier || itemBase->viewIdentifier() == ViewIdentifierClass::IconView) {
+	if (event->modifiers() & Qt::ShiftModifier || itemBase->viewID() == ViewLayer::IconView) {
 		m_hoverEnterMode = true;
 		m_infoView->hoverEnterItem(this, event, itemBase ? itemBase->layerKinChief() : itemBase, swappingEnabled(itemBase));
 	}
@@ -167,7 +168,7 @@ void InfoGraphicsView::getBendpointWidths(class Wire *, double w, double & w1, d
 	Q_UNUSED(negativeOffsetRect);
 }
 
-void InfoGraphicsView::getLabelFont(QFont &, QColor &, ViewLayer::ViewLayerSpec) {
+void InfoGraphicsView::getLabelFont(QFont &, QColor &, ItemBase *) {
 }
 
 double InfoGraphicsView::getLabelFontSizeTiny() {
@@ -276,11 +277,12 @@ int InfoGraphicsView::boardLayers() {
 	return m_boardLayers;
 }
 
-VirtualWire * InfoGraphicsView::makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor color) {
+VirtualWire * InfoGraphicsView::makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor color, bool force) {
 	Q_UNUSED(source);
 	Q_UNUSED(dest);
 	Q_UNUSED(routed);
 	Q_UNUSED(color);
+	Q_UNUSED(force);
 	return NULL;
 }
 
@@ -388,3 +390,45 @@ void InfoGraphicsView::setActiveConnectorItem(ConnectorItem * connectorItem)
 void InfoGraphicsView::resolveTemporary(bool, ItemBase *)
 {
 }
+void InfoGraphicsView::newWire(Wire * wire) 
+{
+	bool succeeded = connect(wire, SIGNAL(wireChangedSignal(Wire*, const QLineF & , const QLineF & , QPointF, QPointF, ConnectorItem *, ConnectorItem *)	),
+			this, SLOT(wireChangedSlot(Wire*, const QLineF & , const QLineF & , QPointF, QPointF, ConnectorItem *, ConnectorItem *)),
+			Qt::DirectConnection);		// DirectConnection means call the slot directly like a subroutine, without waiting for a thread or queue
+	succeeded = connect(wire, SIGNAL(wireChangedCurveSignal(Wire*, const Bezier *, const Bezier *, bool)),
+			this, SLOT(wireChangedCurveSlot(Wire*, const Bezier *, const Bezier *, bool)),
+			Qt::DirectConnection);		// DirectConnection means call the slot directly like a subroutine, without waiting for a thread or queue
+	succeeded = succeeded && connect(wire, SIGNAL(wireSplitSignal(Wire*, QPointF, QPointF, const QLineF & )),
+			this, SLOT(wireSplitSlot(Wire*, QPointF, QPointF, const QLineF & )));
+	succeeded = succeeded && connect(wire, SIGNAL(wireJoinSignal(Wire*, ConnectorItem *)),
+			this, SLOT(wireJoinSlot(Wire*, ConnectorItem*)));
+	if (!succeeded) {
+		DebugDialog::debug("wire signal connect failed");
+	}
+
+	emit newWireSignal(wire);
+}
+
+void InfoGraphicsView::setSMDOrientation(Qt::Orientations orientation) {
+    m_smdOrientation = orientation;
+}
+
+Qt::Orientations InfoGraphicsView::smdOrientation() {
+    return m_smdOrientation;
+}
+
+void InfoGraphicsView::moveItem(ItemBase *, double x, double y) {
+    Q_UNUSED(x);
+    Q_UNUSED(y);
+}
+
+void InfoGraphicsView::updateRotation(ItemBase * itemBase) {
+    if (m_infoView) m_infoView->updateRotation(itemBase);
+}
+
+void InfoGraphicsView::rotateX(double degrees, bool rubberBandLegEnabled, ItemBase * originatingItem) 
+{
+    Q_UNUSED(degrees);
+    Q_UNUSED(rubberBandLegEnabled);
+    Q_UNUSED(originatingItem);
+}
