@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Revision: 6715 $:
+$Author: irascibl@gmail.com $:
+$Date: 2012-12-17 11:27:32 +0100 (Mo, 17. Dez 2012) $
 
 ********************************************************************/
 
@@ -42,14 +42,15 @@ class InfoGraphicsView : public ZoomableGraphicsView
 public:
 	InfoGraphicsView(QWidget* parent = 0);
 
-	void viewItemInfo(ItemBase * item);
-	virtual void hoverEnterItem(QGraphicsSceneHoverEvent * event, ItemBase * item);
-	virtual void hoverLeaveItem(QGraphicsSceneHoverEvent * event, ItemBase * item);
+	virtual void viewItemInfo(ItemBase *);
+	virtual void hoverEnterItem(QGraphicsSceneHoverEvent * event, ItemBase *);
+	virtual void hoverLeaveItem(QGraphicsSceneHoverEvent * event, ItemBase *);
+    void updateRotation(ItemBase *);
 
 	virtual bool swappingEnabled(ItemBase *) = 0;
 
-	virtual void hoverEnterConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
-	virtual void hoverLeaveConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
+	virtual void hoverEnterConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem *);
+	virtual void hoverLeaveConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem *);
 
 	void setInfoView(class HtmlInfoView *);
 	class HtmlInfoView * infoView();
@@ -67,7 +68,7 @@ public:
 
 	virtual void setIgnoreSelectionChangeEvents(bool) {}
 	virtual void getBendpointWidths(class Wire *, double w, double & w1, double & w2, bool & negativeOffsetRect);
-	virtual void getLabelFont(QFont &, QColor &, ViewLayer::ViewLayerSpec);
+	virtual void getLabelFont(QFont &, QColor &, ItemBase *);
     virtual double getLabelFontSizeTiny();
 	virtual double getLabelFontSizeSmall();
 	virtual double getLabelFontSizeMedium();
@@ -81,7 +82,7 @@ public:
 
 	int boardLayers();
 	virtual void setBoardLayers(int, bool redraw);
-	virtual class VirtualWire * makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor);
+	virtual class VirtualWire * makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor, bool force);
 	virtual void getRatsnestColor(QColor &);
 
 	virtual void changeBus(ItemBase *, bool connec, const QString & oldBust, const QString & newBus, QList<ConnectorItem *> &, const QString & message);
@@ -108,10 +109,14 @@ public:
 	virtual void changeWireColor(const QString newColor);
 	virtual void swap(const QString & family, const QString & prop, QMap<QString, QString> & propsMap, ItemBase *);
     virtual void resolveTemporary(bool, ItemBase *);
+	virtual void newWire(Wire *);
 
 	void setActiveWire(Wire *);
 	void setActiveConnectorItem(ConnectorItem *);
-
+    void setSMDOrientation(Qt::Orientations);
+    Qt::Orientations smdOrientation();
+    virtual void moveItem(ItemBase *, double x, double y);
+	virtual void rotateX(double degrees, bool rubberBandLegEnabled, ItemBase * originatingItem);
 
 public slots:
 	virtual void setVoltage(double, bool doEmit);
@@ -124,6 +129,7 @@ signals:
 	void changePinLabelsSignal(ItemBase *, bool singleRow);
 	void setActiveWireSignal(Wire *);
 	void setActiveConnectorItemSignal(ConnectorItem *);
+	void newWireSignal(Wire *);
 
 public:
 	static InfoGraphicsView * getInfoGraphicsView(QGraphicsItem *);
@@ -134,7 +140,7 @@ protected:
 	int m_boardLayers;
 	bool m_hoverEnterMode;
 	bool m_hoverEnterConnectorMode;
-
+    Qt::Orientations m_smdOrientation;
 };
 
 #endif
