@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6255 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-06 19:45:20 +0200 (Mon, 06 Aug 2012) $
+$Revision: 6979 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-21 23:20:35 +0200 (So, 21. Apr 2013) $
 
 ********************************************************************/
 
@@ -28,19 +28,24 @@ $Date: 2012-08-06 19:45:20 +0200 (Mon, 06 Aug 2012) $
 #define PCBSKETCHWIDGET_H
 
 #include "sketchwidget.h"
+#include "../dialogs/quotedialog.h"
 #include <QVector>
+#include <QNetworkReply>
+#include <QDialog>
+
+///////////////////////////////////////////////
 
 class PCBSketchWidget : public SketchWidget
 {
 	Q_OBJECT
 
 public:
-    PCBSketchWidget(ViewIdentifierClass::ViewIdentifier, QWidget *parent=0);
+    PCBSketchWidget(ViewLayer::ViewID, QWidget *parent=0);
 
 	void addViewLayers();
 	bool canDeleteItem(QGraphicsItem * item, int count);
 	bool canCopyItem(QGraphicsItem * item, int count);
-	void createTrace(Wire *);
+	void createTrace(Wire *, bool useLastWireColor);
 	void excludeFromAutoroute(bool exclude);
 	void selectAllExcludedTraces();
 	void selectAllIncludedTraces();
@@ -52,19 +57,18 @@ public:
 	virtual bool autorouteTypePCB();
 	virtual double getKeepout();
 	virtual const QString & traceColor(ConnectorItem *);
-	const QString & traceColor(ViewLayer::ViewLayerSpec);
+	const QString & traceColor(ViewLayer::ViewLayerPlacement);
 	virtual void ensureTraceLayersVisible();
 	virtual void ensureTraceLayerVisible();
 	bool canChainMultiple();
 	void setNewPartVisible(ItemBase *);
-	virtual bool usesJumperItem();
 	void setClipEnds(class ClipableWire *, bool);
 	void showGroundTraces(QList<ConnectorItem *> & seeds, bool show);
     virtual double getLabelFontSizeTiny();
 	virtual double getLabelFontSizeSmall();
 	virtual double getLabelFontSizeMedium();
 	virtual double getLabelFontSizeLarge();
-	ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	QList<ItemBase *> findBoard();
     ItemBase * findSelectedBoard(int & boardCount);
     ItemBase * findBoardBeneath(ItemBase *);
@@ -72,17 +76,16 @@ public:
 	double getRatsnestWidth();
 
 	void setBoardLayers(int, bool redraw);
-    void swapLayers(ItemBase * itemBase, int newLayers, bool flip, QUndoCommand * parentCommand);
+    void swapLayers(ItemBase * itemBase, int newLayers, QUndoCommand * parentCommand);
 	void loadFromModelParts(QList<ModelPart *> & modelParts, BaseCommand::CrossViewType, QUndoCommand * parentCommand, 
 							bool offsetPaste, const QRectF * boundingRect, bool seekOutsideConnections, QList<long> & newIDs);
-	virtual bool isInLayers(ConnectorItem *, ViewLayer::ViewLayerSpec);
+	virtual bool isInLayers(ConnectorItem *, ViewLayer::ViewLayerPlacement);
 	bool routeBothSides();
 	bool sameElectricalLayer2(ViewLayer::ViewLayerID, ViewLayer::ViewLayerID);
-	void changeTraceLayer();
+	void changeTraceLayer(ItemBase *, bool force, QUndoCommand * parentCommand);
 	void changeLayer(long id, double z, ViewLayer::ViewLayerID viewLayerID);
-	void updateNet(Wire*);
 	bool acceptsTrace(const ViewGeometry & viewGeometry);
-	ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
+	ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerPlacement, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
 	void autorouterSettings();
 	void getViaSize(double & ringThickness, double & holeSize);
     void deleteItem(ItemBase *, bool deleteModelPart, bool doEmit, bool later);
@@ -95,8 +98,7 @@ public:
 	void clearGroundFillSeeds();
 	QString generateCopperFillUnit(ItemBase * itemBase, QPointF whereToStart);
 	double getWireStrokeWidth(Wire *, double wireWidth);
-	bool curvyWiresIndicated(Qt::KeyboardModifiers);
-	ItemBase * addCopperLogoItem(ViewLayer::ViewLayerSpec viewLayerSpec);
+	ItemBase * addCopperLogoItem(ViewLayer::ViewLayerPlacement viewLayerPlacement);
 	QString characterizeGroundFill(ViewLayer::ViewLayerID);
 	ViewGeometry::WireFlag getTraceFlag();
 	void hideCopperLogoItems(QList<ItemBase *> &);
@@ -108,15 +110,27 @@ public:
 	void convertToVia(ConnectorItem * lastHoverEnterConnectorItem);
 	void convertToBendpoint();
     int checkLoadedTraces();
-
-public:
-	static QSizeF jumperItemSize();
-	static void getDefaultViaSize(QString & ringThickness, QString & holeSize);
+	bool hasCustomBoardShape();
+    virtual QSizeF jumperItemSize();
+    virtual LayerList routingLayers(ViewLayer::ViewLayerPlacement);
+    virtual bool attachedToTopLayer(ConnectorItem *);
+    virtual bool attachedToBottomLayer(ConnectorItem *);
+    QHash<QString, QString> getAutorouterSettings();
+    void setAutorouterSettings(QHash<QString, QString> &);
+	void getDefaultViaSize(QString & ringThickness, QString & holeSize);
+    void hidePartSilkscreen();
+    void fabQuote();
+    QDialog * quoteDialog(QWidget * parent);
+    void setGroundFillKeepout();
+    void setViewFromBelow(bool);
+    bool dropOnBottom();
+    ViewLayer::ViewLayerPlacement defaultViewLayerPlacement(ModelPart *);
 
 public slots:
 	void resizeBoard(double w, double h, bool doEmit);
 	void showLabelFirstTime(long itemID, bool show, bool doEmit);
 	void changeBoardLayers(int layers, bool doEmit);
+	ItemBase * resizeBoard(long id, double w, double h);
 
 
 public:
@@ -130,7 +144,7 @@ public:
 protected:
 	void setWireVisible(Wire * wire);
 	// void checkAutorouted();
-	ViewLayer::ViewLayerID multiLayerGetViewLayerID(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerSpec, QDomElement & layers, QString & layerName);
+	ViewLayer::ViewLayerID multiLayerGetViewLayerID(ModelPart * modelPart, ViewLayer::ViewID, ViewLayer::ViewLayerPlacement, LayerList &);
 	bool canChainWire(Wire *);
 	bool canDragWire(Wire * wire);
 	const QString & hoverEnterPartConnectorMessage(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
@@ -142,22 +156,18 @@ protected:
 	void setUpColor(ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, Wire * wire, QUndoCommand * parentCommand);
 	ConnectorItem * findNearestPartConnectorItem(ConnectorItem * fromConnectorItem);
 	bool bothEndsConnectedAux(Wire * wire, ViewGeometry::WireFlags flag, ConnectorItem * oneEnd, QList<Wire *> & wires, QList<ConnectorItem *> & partConnectorItems, QList<Wire *> & visited);
-	void getLabelFont(QFont &, QColor &, ViewLayer::ViewLayerSpec);
+	void getLabelFont(QFont &, QColor &, ItemBase *);
 	double defaultGridSizeInches();
-	ViewLayer::ViewLayerID getLabelViewLayerID(ViewLayer::ViewLayerSpec);
-	ViewLayer::ViewLayerSpec wireViewLayerSpec(ConnectorItem *);
-	bool resizingJumperItemPress(QGraphicsItem * item);
+	ViewLayer::ViewLayerID getLabelViewLayerID(ItemBase *);
+	ViewLayer::ViewLayerPlacement wireViewLayerPlacement(ConnectorItem *);
+	bool resizingJumperItemPress(ItemBase *);
 	bool resizingJumperItemRelease();
-	bool resizingBoardPress(QGraphicsItem * item);
-	bool resizingBoardRelease();
-	void resizeBoard();
 	void resizeJumperItem();
 	QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	void dealWithDefaultParts();
-	void clearSmdTraces(QList<ItemBase *> & smds, 	QList<Wire *> & already, QUndoCommand * parentCommand);
-	bool connectorItemHasSpec(ConnectorItem * connectorItem, ViewLayer::ViewLayerSpec spec);
-	ViewLayer::ViewLayerSpec createWireViewLayerSpec(ConnectorItem * from, ConnectorItem * to);
-	Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	bool connectorItemHasSpec(ConnectorItem * connectorItem, ViewLayer::ViewLayerPlacement spec);
+	ViewLayer::ViewLayerPlacement createWireViewLayerPlacement(ConnectorItem * from, ConnectorItem * to);
+	Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	void prereleaseTempWireForDragging(Wire*);
 	void rotatePartLabels(double degrees, QTransform &, QPointF center, QUndoCommand * parentCommand);
 	bool hasNeighbor(ConnectorItem * connectorItem, ViewLayer::ViewLayerID viewLayerID, const QRectF & r);
@@ -170,9 +180,15 @@ protected:
     QString checkDroppedModuleID(const QString & moduleID);
     bool canConnect(Wire * from, ItemBase * to);
 	void collectThroughHole(QList<ConnectorItem *> & th, QList<ConnectorItem *> & pads, const LayerList &);
+	ViewLayer::ViewLayerPlacement getViewLayerPlacement(ModelPart *, QDomElement & instance, QDomElement & view, ViewGeometry &);
+    void requestQuote(bool byUser);
+    double calcBoardArea(int & boardCount);
+    PaletteItem* addPartItem(ModelPart * modelPart, ViewLayer::ViewLayerPlacement, PaletteItem * paletteItem, bool doConnectors, bool & ok, ViewLayer::ViewID, bool temporary);
+    void requestQuoteSoon();
+    double getKeepoutMils();
 
 signals:
-	void subSwapSignal(SketchWidget *, ItemBase *, const QString & newModuleID, ViewLayer::ViewLayerSpec, long & newID, QUndoCommand * parentCommand);
+	void subSwapSignal(SketchWidget *, ItemBase *, const QString & newModuleID, ViewLayer::ViewLayerPlacement, long & newID, QUndoCommand * parentCommand);
 	void boardDeletedSignal();
 	void groundFillSignal();
 	void copperFillSignal();
@@ -186,18 +202,26 @@ protected:
 protected slots:
 	void alignJumperItem(class JumperItem *, QPointF &);
 	void wireSplitSlot(class Wire*, QPointF newPos, QPointF oldPos, QLineF oldLine);
-	void postImageSlot(class GroundPlaneGenerator *, QImage * image, QGraphicsItem * board);
+	void postImageSlot(class GroundPlaneGenerator *, QImage * copperImage, QImage * boardImage, QGraphicsItem * board, QList<QRectF> *);
+    void gotFabQuote(QNetworkReply *);
+    void requestQuoteNow();
+    void getDroppedItemViewLayerPlacement(ModelPart * modelPart, ViewLayer::ViewLayerPlacement &);
+
 
 protected:
 	CleanType m_cleanType;
 	QPointF m_jumperDragOffset;
 	QPointer<class JumperItem> m_resizingJumperItem;
-	QPointer<class ResizableBoard> m_resizingBoard;
 	QList<ConnectorItem *> * m_groundFillSeeds;
+    QHash<QString, QString> m_autorouterSettings;
+    QPointer<class QuoteDialog> m_quoteDialog;
+    QPointer<class QuoteDialog> m_rolloverQuoteDialog;
+    QTimer m_requestQuoteTimer;
 
 protected:
 	static QSizeF m_jumperItemSize;
 	static const char * FakeTraceProperty;
 };
 
+
 #endif
