@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6160 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-11 06:53:18 +0200 (Wed, 11 Jul 2012) $
+$Revision: 6979 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-21 23:20:35 +0200 (So, 21. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,10 +35,10 @@ class SchematicSketchWidget : public PCBSketchWidget
 	Q_OBJECT
 
 public:
-    SchematicSketchWidget(ViewIdentifierClass::ViewIdentifier, QWidget *parent=0);
+    SchematicSketchWidget(ViewLayer::ViewID, QWidget *parent=0);
 
 	void addViewLayers();
-	ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	ViewLayer::ViewLayerID getDragWireViewLayerID(ConnectorItem *);
 	void initWire(Wire *, int penWidth);
 	bool autorouteTypePCB();
@@ -46,10 +46,9 @@ public:
 	void tidyWires();
 	void ensureTraceLayersVisible();
 	void ensureTraceLayerVisible();
-	bool usesJumperItem();
 	void setClipEnds(ClipableWire * vw, bool);
 	void getBendpointWidths(class Wire *, double w, double & w1, double & w2, bool & negativeOffsetRect);
-	void getLabelFont(QFont &, QColor &, ViewLayer::ViewLayerSpec);
+	void getLabelFont(QFont &, QColor &, ItemBase *);
 	void setNewPartVisible(ItemBase *);
 	bool canDropModelPart(ModelPart * modelPart); 
 	bool includeSymbols();
@@ -57,13 +56,13 @@ public:
 	void changeConnection(long fromID,
 						  const QString & fromConnectorID,
 						  long toID, const QString & toConnectorID,
-						  ViewLayer::ViewLayerSpec,
+						  ViewLayer::ViewLayerPlacement,
 						  bool connect, bool doEmit, 
 						  bool updateConnections);
 	double defaultGridSizeInches();
 	const QString & traceColor(ConnectorItem * forColor);
-	const QString & traceColor(ViewLayer::ViewLayerSpec);
-	bool isInLayers(ConnectorItem *, ViewLayer::ViewLayerSpec);
+	const QString & traceColor(ViewLayer::ViewLayerPlacement);
+	bool isInLayers(ConnectorItem *, ViewLayer::ViewLayerPlacement);
 	bool routeBothSides();
 	void addDefaultParts();
 	bool sameElectricalLayer2(ViewLayer::ViewLayerID, ViewLayer::ViewLayerID);
@@ -73,31 +72,44 @@ public:
 	double getAutorouterTraceWidth();
 	QString generateCopperFillUnit(ItemBase * itemBase, QPointF whereToStart);
 	double getWireStrokeWidth(Wire *, double wireWidth);
-	Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	void rotatePartLabels(double degrees, QTransform &, QPointF center, QUndoCommand * parentCommand);
 	void loadFromModelParts(QList<ModelPart *> & modelParts, BaseCommand::CrossViewType, QUndoCommand * parentCommand, 
 							bool offsetPaste, const QRectF * boundingRect, bool seekOutsideConnections, QList<long> & newIDs);
+    LayerList routingLayers(ViewLayer::ViewLayerPlacement);
+    bool attachedToTopLayer(ConnectorItem *);
+    bool attachedToBottomLayer(ConnectorItem *);
+    QSizeF jumperItemSize();
+    QHash<QString, QString> getAutorouterSettings();
+    void setAutorouterSettings(QHash<QString, QString> &);
+	ViewLayer::ViewLayerPlacement getViewLayerPlacement(ModelPart *, QDomElement & instance, QDomElement & view, ViewGeometry &);
 
 public slots:
 	void setVoltage(double voltage, bool doEmit);
+	void setProp(ItemBase *, const QString & propName, const QString & translatedPropName, const QString & oldValue, const QString & newValue, bool redraw);
+	void setInstanceTitle(long id, const QString & oldTitle, const QString & newTitle, bool isUndoable, bool doEmit);
 
 protected slots:
 	void updateBigDots();
+    void getDroppedItemViewLayerPlacement(ModelPart * modelPart, ViewLayer::ViewLayerPlacement &);
 
 protected:
 	double getRatsnestOpacity();
 	double getRatsnestWidth();
-	ViewLayer::ViewLayerID getLabelViewLayerID(ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerID getLabelViewLayerID(ItemBase *);
 	QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	void extraRenderSvgStep(ItemBase *, QPointF offset, double dpi, double printerScale, QString & outputSvg);
 	QString makeCircleSVG(QPointF p, double r, QPointF offset, double dpi, double printerScale);
-	ViewLayer::ViewLayerSpec createWireViewLayerSpec(ConnectorItem * from, ConnectorItem * to);
+	ViewLayer::ViewLayerPlacement createWireViewLayerPlacement(ConnectorItem * from, ConnectorItem * to);
     void selectAllWires(ViewGeometry::WireFlag flag);
     bool canConnect(Wire * from, ItemBase * to);
+    QString checkDroppedModuleID(const QString & moduleID);
 
 protected:
 	QTimer m_updateDotsTimer;
 
+    static QSizeF m_jumperItemSize;
+
 };
 
 #endif
