@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
-$Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Revision: 6904 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
 
 ********************************************************************/
 
@@ -30,7 +30,7 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include <QTextStream>
 #include <qmath.h>
 
-static const qreal MaskClearance = 0.003;  // 3 mils clearance
+static const double MaskClearance = 0.005;  // 5 mils clearance
 
 bool fillNotStroke(QDomElement & element, SVG2gerber::ForWhy forWhy) {
 	if (forWhy == SVG2gerber::ForOutline) return false;
@@ -45,7 +45,9 @@ bool fillNotStroke(QDomElement & element, SVG2gerber::ForWhy forWhy) {
 	if (stroke.compare("none") == 0) return true;
 
 	// both fill and stroke
-	return false;
+
+	// note: this originally returned 'false' which suggests there may be some parts that will surprise us next time they are exported to gerber
+	return true;
 }
 
 //TODO: currently only supports one board per sketch (i.e. multiple board outlines will mess you up)
@@ -63,7 +65,7 @@ int SVG2gerber::convert(const QString & svgStr, bool doubleSided, const QString
     int errorColumn;
     bool result = m_SVGDom.setContent(svgStr, &errorStr, &errorLine, &errorColumn);
     if (!result) {
-        DebugDialog::debug(QString("gerber svg failed %1 %2 %3 %4").arg(svgStr).arg(errorStr).arg(errorLine).arg(errorColumn));
+        DebugDialog::debug(QString("gerber svg failed %2 %3 %4 %1").arg(svgStr).arg(errorStr).arg(errorLine).arg(errorColumn));
     }
 
 #ifndef QT_NO_DEBUG
@@ -112,7 +114,7 @@ int SVG2gerber::renderGerber(bool doubleSided, const QString & mainLayerName, Fo
 		// setup drill file header
 		m_gerber_header = "M48\n";
 		// set to english (inches) units, with trailing zeros
-		m_gerber_header += "M72,TZ\n";
+		m_gerber_header += "INCH\n";
 
 	}
 
@@ -142,7 +144,13 @@ int SVG2gerber::renderGerber(bool doubleSided, const QString & mainLayerName, Fo
 				continue;
 			}
 
-			apertures.insert(current, string);
+            QStringList values = apertures.values(current);
+            if (values.contains(string)) {
+                //DebugDialog::debug("already have this value");
+            }
+            else {
+			    apertures.insert(current, string);
+            }
 		}
 
 		foreach (QString T, apertures.uniqueKeys()) {
@@ -203,6 +211,9 @@ void SVG2gerber::convertShapes2paths(QDomNode node){
         if(tag=="polygon"){
             path = element;
         }
+        else if(tag=="polyline"){
+            path = element;
+        }
         else if(tag=="rect"){
             path = element;
         }
@@ -218,6 +229,15 @@ void SVG2gerber::convertShapes2paths(QDomNode node){
         else if((tag=="path") || (tag=="svg:path")){
             path = element;
         }
+		else if (tag == "g") {
+			// no op
+		}
+		else if (tag == "#comment") {
+			// no op
+		}
+		else if (tag == "#text") {
+			// no op
+		}
         else {
             DebugDialog::debug("svg2gerber ignoring SVG element: " + tag);
         }
@@ -282,14 +302,15 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
     QDomNodeList polyList = m_SVGDom.elementsByTagName("polygon");
     //DebugDialog::debug("polygons to gerber: " + QString::number(polyList.length()));
 
+    QDomNodeList polyLineList = m_SVGDom.elementsByTagName("polyline");
+    //DebugDialog::debug("polylines to gerber: " + QString::number(polyLineList.length()));
+
 	QDomNodeList lineList = m_SVGDom.elementsByTagName("line");
     //DebugDialog::debug("lines to gerber: " + QString::number(lineList.length()));
 
 	QDomNodeList pathList = m_SVGDom.elementsByTagName("path");
     //DebugDialog::debug("paths to gerber: " + QString::number(pathList.length()));
 
-	int totalCount = circleList.count() + rectList.count() + polyList.count() + lineList.count() + pathList.count();
-
     // if this is the board outline, use it as the contour
     if (forWhy == ForOutline) {
         //DebugDialog::debug("drawing board outline");
@@ -306,21 +327,17 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			continue;
 		}
 
-		if (forWhy == ForOutline && totalCount > 1) {
-			if (circle.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-		}
-
-        qreal centerx = circle.attribute("cx").toDouble();
-        qreal centery = circle.attribute("cy").toDouble();
-        qreal r = circle.attribute("r").toDouble();
+        double centerx = circle.attribute("cx").toDouble();
+        double centery = circle.attribute("cy").toDouble();
+        double r = circle.attribute("r").toDouble();
 		if (r == 0) continue;
 
-        qreal stroke_width = circle.attribute("stroke-width").toDouble();
-		qreal hole = ((2*r) - stroke_width)/1000;
+        double stroke_width = circle.attribute("stroke-width").toDouble();
+		double hole = ((2*r) - stroke_width) / 1000;  // convert mils (standard fritzing resolution) to inches
 
 		if (forWhy == ForDrill) {
-			QString drill_cx = QString::number(flipxNoRound(centerx) / 1000, 'f');				// drill file seems to be in inches
-			QString drill_cy = QString::number(flipyNoRound(centery) / 1000, 'f');				// drill file seems to be in inches
+			QString drill_cx = QString("%1").arg((int) (flipxNoRound(centerx) * 10), 6, 10, QChar('0'));				// drill file is in inches 00.0000, converting mils to 10000ths
+			QString drill_cy = QString("%1").arg((int) (flipyNoRound(centery) * 10), 6, 10, QChar('0'));				// drill file is in inches 00.0000, converting mils to 10000ths
             QString aperture = QString("C%1").arg(hole, 0, 'f');
 			if(!apertureMap.contains(aperture)) {
 				apertureMap[aperture] = QString::number(dcode_index);
@@ -344,12 +361,12 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 
         QString fill = circle.attribute("fill");
 
-        qreal diam = ((2*r) + stroke_width)/1000;
+        double diam = ((2*r) + stroke_width)/1000;
 		if (forWhy == ForMask) {
 			diam += 2 * MaskClearance;
 		}
 
-        if(fill=="none" && forWhy != ForMask){
+        if (forWhy != ForCopper && fill=="none" && forWhy != ForMask){
 			aperture = QString("C,%1X%2").arg(diam, 0, 'f').arg(hole);
         }
         else {
@@ -392,31 +409,36 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 		// rects
 		for(uint j = 0; j < rectList.length(); j++){
 			QDomElement rect = rectList.item(j).toElement();
-			if (forWhy == ForOutline && totalCount > 1) {
-				if (rect.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-			}
 
 			QString aperture;
 
-			qreal width = rect.attribute("width").toDouble();
-			qreal height = rect.attribute("height").toDouble();
+			double width = rect.attribute("width").toDouble();
+			double height = rect.attribute("height").toDouble();
+
+			double rx = rect.attribute("rx", "0").toDouble();
+			double ry = rect.attribute("ry", "0").toDouble();
+			if (rx != 0 || ry != 0) {
+				// not sure how to do rounded rects in gerber
+				invalidPathsCount++;
+				continue;
+			}
 
 			if (width == 0) continue;
 			if (height == 0) continue;
 
-			qreal x = rect.attribute("x").toDouble();
-			qreal y = rect.attribute("y").toDouble();
-			qreal centerx = x + (width/2);
-			qreal centery = y + (height/2);
+			double x = rect.attribute("x").toDouble();
+			double y = rect.attribute("y").toDouble();
+			double centerx = x + (width/2);
+			double centery = y + (height/2);
 			QString cx = QString::number(flipx(centerx));
 			QString cy = QString::number(flipy(centery));
 			QString fill = rect.attribute("fill");
-			qreal stroke_width = rect.attribute("stroke-width").toDouble();
+			double stroke_width = rect.attribute("stroke-width").toDouble();
 
-			qreal totalx = (width + stroke_width)/1000;
-			qreal totaly = (height + stroke_width)/1000;
-			qreal holex = (width - stroke_width)/1000;
-			qreal holey = (height - stroke_width)/1000;
+			double totalx = (width + stroke_width)/1000;
+			double totaly = (height + stroke_width)/1000;
+			double holex = (width - stroke_width)/1000;
+			double holey = (height - stroke_width)/1000;
 
 			if (forWhy == ForMask) {
 				totalx += 2 * MaskClearance;
@@ -424,7 +446,7 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			}
 
 
-			if(fill=="none" && forWhy != ForMask) {
+			if(forWhy != ForCopper && fill=="none" && forWhy != ForMask) {
 				aperture = QString("R,%1X%2X%3X%4").arg(totalx, 0, 'f').arg(totaly, 0, 'f').arg(holex, 0, 'f').arg(holey, 0, 'f');
 			}
 			else {
@@ -438,7 +460,11 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 				dcode_index++;
 			}
 
-			if (forWhy != ForOutline) {
+			bool doLines = false;
+			if (forWhy == ForOutline) doLines = true;
+			else if (forWhy == ForSilk && fill == "none") doLines = true;
+
+			if (!doLines) {
 				QString dcode = apertureMap[aperture];
 				if(current_dcode != dcode){
 					//switch to correct aperture
@@ -464,16 +490,13 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 		// lines - NOTE: this assumes a circular aperture
 		for(uint k = 0; k < lineList.length(); k++){
 			QDomElement line = lineList.item(k).toElement();
-			if (forWhy == ForOutline && totalCount > 1) {
-				if (line.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-			}
 
 			// Note: should be no forWhy == ForMask cases 
 
-			qreal x1 = line.attribute("x1").toDouble();
-			qreal y1 = line.attribute("y1").toDouble();
-			qreal x2 = line.attribute("x2").toDouble();
-			qreal y2 = line.attribute("y2").toDouble();
+			double x1 = line.attribute("x1").toDouble();
+			double y1 = line.attribute("y1").toDouble();
+			double x2 = line.attribute("x2").toDouble();
+			double y2 = line.attribute("y2").toDouble();
 
 			standardAperture(line, apertureMap, current_dcode, dcode_index, 0);
 
@@ -495,72 +518,19 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 		}
 
 		// polys - NOTE: assumes comma- or space- separated formatting
-		for(uint p = 0; p < polyList.length(); p++){
-			QDomElement polygon = polyList.item(p).toElement();
-			if (forWhy == ForOutline && totalCount > 1) {
-				if (polygon.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-			}
-
-			QString temp;
-			QTextStream tempStream(&temp);
-			polygon.save(tempStream, 1);
-
-			QString points = polygon.attribute("points");
-			QStringList pointList = points.split(QRegExp("\\s+|,"), QString::SkipEmptyParts);
-
-			QString aperture;
-
-			QString pointString;
-
-			qreal startx = pointList.at(0).toDouble();
-			qreal starty = pointList.at(1).toDouble();
-			// move to start - light off
-			pointString += "X" + QString::number(flipx(startx)) + "Y" + QString::number(flipy(starty)) + "D02*\n";
-
-			// iterate through all other points - light on
-			for(int pt = 2; pt < pointList.length(); pt +=2){
-				qreal ptx = pointList.at(pt).toDouble();
-				qreal pty = pointList.at(pt+1).toDouble();
-				pointString += "X" + QString::number(flipx(ptx)) + "Y" + QString::number(flipy(pty)) + "D01*\n";
-			}
-
-			// move back to start point
-			pointString += "X" + QString::number(flipx(startx)) + "Y" + QString::number(flipy(starty)) + "D01*\n";
-
-			standardAperture(polygon, apertureMap, current_dcode, dcode_index, 0);		
-
-			bool polyFill = fillNotStroke(polygon, forWhy);
-			// set poly fill if this is actually a filled in shape
-			if (polyFill) {							
-				// start poly fill
-				m_gerber_paths += "G36*\n";
-			}
-
-			m_gerber_paths += pointString;
-
-			// stop poly fill if this is actually a filled in shape
-			if(polyFill){
-				// stop poly fill
-				m_gerber_paths += "G37*\n";
-			}
-
-			if (forWhy == ForMask) {
-				// draw the outline, G36 only does the fill
-				standardAperture(polygon, apertureMap, current_dcode, dcode_index,  polygon.attribute("stroke-width").toDouble() + (MaskClearance * 2 * 1000));
-				m_gerber_paths += pointString;
-			}
-
-			// light off
-			m_gerber_paths += "D02*\n";
+		for(uint p = 0; p < polyList.length(); p++) {
+                        QDomElement polygon = polyList.item(p).toElement();
+                        doPoly(polygon, forWhy, true, apertureMap, current_dcode, dcode_index);
+		}
+                for(uint p = 0; p < polyLineList.length(); p++) {
+                        QDomElement polygon = polyLineList.item(p).toElement();
+                        doPoly(polygon, forWhy, false, apertureMap, current_dcode, dcode_index);
 		}
 	}
 
     // paths - NOTE: this assumes circular aperture
     for(uint n = 0; n < pathList.length(); n++){
         QDomElement path = pathList.item(n).toElement();
-		if (forWhy == ForOutline && totalCount > 1) {
-			if (path.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-		}
 
 		if (forWhy == ForDrill) {
 			handleOblongPath(path, dcode_index);
@@ -578,18 +548,54 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
         pathUserData.string = "";
 
         SvgFlattener flattener;
-        flattener.parsePath(data, slot, pathUserData, this, true);
-
+		bool invalid = false;
+		try {
+			flattener.parsePath(data, slot, pathUserData, this, true);
+		}
+		catch (const QString & msg) {
+			DebugDialog::debug("flattener.parsePath failed " + msg);
+			invalid = true;
+		}
+		catch (char const *str) {
+			DebugDialog::debug("flattener.parsePath failed " + QString(str));
+			invalid = true;
+		}
+		catch (...) {
+			DebugDialog::debug("flattener.parsePath failed");
+			invalid = true;
+		}
 		
 
         // only add paths if they contained gerber-izable path commands (NO CURVES!)
         // TODO: display some informative error for the user
-        if(pathUserData.string.contains("INVALID")) {
+        if (invalid || pathUserData.string.contains("INVALID")) {
 			invalidPathsCount++;
             continue;
 		}
 
-		standardAperture(path, apertureMap, current_dcode, dcode_index, 0);
+        if (path.attribute("stroke-linecap") == "square") {
+	        double stroke_width = path.attribute("stroke-width").toDouble();
+	        if (stroke_width != 0) {
+	            QString aperture = QString("R,%1X%1").arg(stroke_width/1000, 0, 'f');
+
+                    // add aperture to defs if we don't have it yet
+                if (!apertureMap.contains(aperture)) {
+                    apertureMap[aperture] = QString::number(dcode_index);
+                    m_gerber_header += "%ADD" + QString::number(dcode_index) + aperture + "*%\n";
+                    dcode_index++;
+                }
+
+	            QString dcode = apertureMap[aperture];
+	            if (current_dcode != dcode) {
+		            //switch to correct aperture
+		            m_gerber_paths += "G54D" + dcode + "*\n";
+		            current_dcode = dcode;
+	            }
+            }
+        }
+        else {
+		    standardAperture(path, apertureMap, current_dcode, dcode_index, 0);
+        }
 
         // set poly fill if this is actually a filled in shape
 		bool polyFill = fillNotStroke(path, forWhy);
@@ -627,7 +633,67 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 	return invalidPathsCount;
 }
 
-QString SVG2gerber::standardAperture(QDomElement & element, QHash<QString, QString> & apertureMap, QString & current_dcode, int & dcode_index, qreal stroke_width) {
+void SVG2gerber::doPoly(QDomElement & polygon, ForWhy forWhy, bool closedCurve,
+					QHash<QString, QString> & apertureMap, QString & current_dcode, int & dcode_index) 
+{
+
+	//QString temp;
+	//QTextStream tempStream(&temp);
+	//polygon.save(tempStream, 1);
+
+	QString points = polygon.attribute("points");
+	QStringList pointList = points.split(QRegExp("\\s+|,"), QString::SkipEmptyParts);
+
+	QString aperture;
+
+	QString pointString;
+
+	double startx = pointList.at(0).toDouble();
+	double starty = pointList.at(1).toDouble();
+	// move to start - light off
+	pointString += "X" + QString::number(flipx(startx)) + "Y" + QString::number(flipy(starty)) + "D02*\n";
+
+	// iterate through all other points - light on
+	for(int pt = 2; pt < pointList.length(); pt +=2){
+		double ptx = pointList.at(pt).toDouble();
+		double pty = pointList.at(pt+1).toDouble();
+		pointString += "X" + QString::number(flipx(ptx)) + "Y" + QString::number(flipy(pty)) + "D01*\n";
+	}
+
+	if (closedCurve) {
+		// move back to start point
+		pointString += "X" + QString::number(flipx(startx)) + "Y" + QString::number(flipy(starty)) + "D01*\n";
+	}
+
+	standardAperture(polygon, apertureMap, current_dcode, dcode_index, 0);		
+
+	bool polyFill = fillNotStroke(polygon, forWhy);
+	// set poly fill if this is actually a filled in shape
+	if (polyFill) {							
+		// start poly fill
+		m_gerber_paths += "G36*\n";
+	}
+
+	m_gerber_paths += pointString;
+
+	// stop poly fill if this is actually a filled in shape
+	if(polyFill){
+		// stop poly fill
+		m_gerber_paths += "G37*\n";
+	}
+
+	if (forWhy == ForMask) {
+		// draw the outline, G36 only does the fill
+		standardAperture(polygon, apertureMap, current_dcode, dcode_index,  polygon.attribute("stroke-width").toDouble() + (MaskClearance * 2 * 1000));
+		m_gerber_paths += pointString;
+	}
+
+	// light off
+	m_gerber_paths += "D02*\n";
+
+}
+
+QString SVG2gerber::standardAperture(QDomElement & element, QHash<QString, QString> & apertureMap, QString & current_dcode, int & dcode_index, double stroke_width) {
 	if (stroke_width == 0) {
 		stroke_width = element.attribute("stroke-width").toDouble();
 	}
@@ -665,13 +731,13 @@ void SVG2gerber::handleOblongPath(QDomElement & path, int & dcode_index) {
 	QDomElement nextLine = nextPath.nextSiblingElement("line");
 	if (nextLine.isNull()) return;
 				
-	qreal diameter = parent.attribute("stroke-width").toDouble();
-	qreal cx1 = nextLine.attribute("x1").toDouble();
-	qreal cy1 = nextLine.attribute("y1").toDouble();
-	qreal cx2 = nextLine.attribute("x2").toDouble();
-	qreal cy2 = nextLine.attribute("y2").toDouble();
+	double diameter = parent.attribute("stroke-width").toDouble();
+	double cx1 = nextLine.attribute("x1").toDouble();
+	double cy1 = nextLine.attribute("y1").toDouble();
+	double cx2 = nextLine.attribute("x2").toDouble();
+	double cy2 = nextLine.attribute("y2").toDouble();
 
-	QString drill_aperture = QString("C%1").arg(diameter / 1000, 0, 'f') + "\n";
+	QString drill_aperture = QString("C%1").arg(diameter / 1000, 0, 'f') + "\n";   // convert mils to inches
 	if (!m_gerber_header.contains(drill_aperture)) {
 		m_gerber_header += "T" + QString::number(dcode_index++) + drill_aperture;
 	}
@@ -679,10 +745,10 @@ void SVG2gerber::handleOblongPath(QDomElement & path, int & dcode_index) {
 	int it = m_gerber_header.lastIndexOf("T", ix);
 	m_drill_slots += QString("%1\nX%2Y%3G85X%4Y%5\nG05\n")
 		.arg(m_gerber_header.mid(it, ix - it), 0, 'f')
-		.arg(flipxNoRound(cx1) / 1000, 0, 'f')
-		.arg(flipyNoRound(cy1) / 1000, 0, 'f')
-		.arg(flipxNoRound(cx2) / 1000, 0, 'f')
-		.arg(flipyNoRound(cy2) / 1000, 0, 'f');
+        .arg((int) (flipxNoRound(cx1) * 10), 6, 10, QChar('0'))
+		.arg((int) (flipyNoRound(cy1) * 10), 6, 10, QChar('0'))
+		.arg((int) (flipxNoRound(cx2) * 10), 6, 10, QChar('0'))
+		.arg((int) (flipyNoRound(cy2) * 10), 6, 10, QChar('0'));
 }
 
 QDomElement SVG2gerber::ellipse2path(QDomElement ellipseElement){
@@ -712,10 +778,13 @@ void SVG2gerber::path2gerbCommandSlot(QChar command, bool relative, QList<double
 					case 'C':
 					case 'q':
 					case 'Q':
+					case 's':
+					case 'S':
 					case 't':
 					case 'T':
 						// TODO: implement elliptical arc, etc.
 						pathUserData->string.append("INVALID");
+						argIndex = args.count();
 						break;
 					case 'm':
 					case 'M':
@@ -772,6 +841,7 @@ void SVG2gerber::path2gerbCommandSlot(QChar command, bool relative, QList<double
 						pathUserData->pathStarting = true;
 						break;
 					default:
+						argIndex = args.count();
 						pathUserData->string.append("INVALID");
 						break;
 		}
@@ -779,22 +849,22 @@ void SVG2gerber::path2gerbCommandSlot(QChar command, bool relative, QList<double
 }
 
 
-int SVG2gerber::flipx(qreal x)
+int SVG2gerber::flipx(double x)
 {
 	return qRound(x);
 }
 
-int SVG2gerber::flipy(qreal y)
+int SVG2gerber::flipy(double y)
 {
 	return qRound(m_boardSize.height() - y);
 }
 
-qreal SVG2gerber::flipxNoRound(qreal x)
+double SVG2gerber::flipxNoRound(double x)
 {
 	return x;
 }
 
-qreal SVG2gerber::flipyNoRound(qreal y)
+double SVG2gerber::flipyNoRound(double y)
 {
 	return m_boardSize.height() - y;
 }
