@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6112 $:
+$Revision: 6435 $:
 $Author: cohen@irascible.com $:
-$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
+$Date: 2012-09-16 23:31:00 +0200 (So, 16. Sep 2012) $
 
 ********************************************************************/
 
@@ -82,15 +82,14 @@ void SvgFlattener::unRotateChild(QDomElement & element, QMatrix transform) {
 
     if(!element.hasChildNodes()) {
 
-		double scale = qMin(qAbs(transform.m11()), qAbs(transform.m22()));
-		if (scale != 1 && transform.m21() == 0 && transform.m12() == 0) {
-			QString sw = element.attribute("stroke-width");
-			if (!sw.isEmpty()) {
-				bool ok;
-				double strokeWidth = sw.toDouble(&ok);
-				if (ok) {
-					element.setAttribute("stroke-width", QString::number(strokeWidth * scale));
-				}
+		QString sw = element.attribute("stroke-width");
+		if (!sw.isEmpty()) {
+			bool ok;
+			double strokeWidth = sw.toDouble(&ok);
+			if (ok) {
+                QLineF line(0, 0, strokeWidth, 0);
+                QLineF newLine = transform.map(line);
+				element.setAttribute("stroke-width", newLine.length());
 			}
 		}
 
@@ -147,9 +146,13 @@ void SvgFlattener::unRotateChild(QDomElement & element, QMatrix transform) {
         else if(tag == "circle"){
             float cx = element.attribute("cx").toFloat();
             float cy = element.attribute("cy").toFloat();
+            float r = element.attribute("r").toFloat();
             QPointF point = transform.map(QPointF(cx,cy));
             element.setAttribute("cx", point.x());
             element.setAttribute("cy", point.y());
+            QLineF line(0, 0, r, 0);
+            QLineF newLine = transform.map(line);
+            element.setAttribute("r", newLine.length());
         }
         else if(tag == "line") {
             float x1 = element.attribute("x1").toFloat();
@@ -277,22 +280,20 @@ void SvgFlattener::rotateCommandSlot(QChar command, bool relative, QList<double>
 	}
 }
 
-void SvgFlattener::flipSMDSvg(const QString & filename, const QString & svg, QDomDocument & domDocument, const QString & elementID, const QString & altElementID, double printerScale) {
-	QString errorStr;
-	int errorLine;
-	int errorColumn;
-	bool result;
-	if (filename.isEmpty()) {
-		result = domDocument.setContent(svg, &errorStr, &errorLine, &errorColumn);
-	}
-	else {
-		QFile file(filename);
-		result = domDocument.setContent(&file, &errorStr, &errorLine, &errorColumn);
-	}
-	if (!result) {
-		domDocument.clear();			// probably redundant
-		return;
-	}
+void SvgFlattener::replaceElementID(const QString & filename, const QString & svg, QDomDocument & domDocument, const QString & elementID, const QString & altElementID) 
+{
+    if (!loadDocIf(filename, svg, domDocument)) return;
+
+    QDomElement root = domDocument.documentElement();
+	QDomElement element = TextUtils::findElementWithAttribute(root, "id", elementID);
+	if (!element.isNull()) {
+        element.setAttribute("id", altElementID);
+    }
+}
+
+void SvgFlattener::flipSMDSvg(const QString & filename, const QString & svg, QDomDocument & domDocument, const QString & elementID, const QString & altElementID, double printerScale, Qt::Orientations orientation) 
+{
+    if (!loadDocIf(filename, svg, domDocument)) return;
 
     QDomElement root = domDocument.documentElement();
 
@@ -310,7 +311,7 @@ void SvgFlattener::flipSMDSvg(const QString & filename, const QString & svg, QDo
 	QDomElement element = TextUtils::findElementWithAttribute(root, "id", elementID);
 	if (!element.isNull()) {
 		QDomElement altElement = TextUtils::findElementWithAttribute(root, "id", altElementID);
-		flipSMDElement(domDocument, renderer, element, elementID, altElement, altElementID, printerScale);
+		flipSMDElement(domDocument, renderer, element, elementID, altElement, altElementID, printerScale,  orientation);
 	}
 
 #ifndef QT_NO_DEBUG
@@ -319,26 +320,53 @@ void SvgFlattener::flipSMDSvg(const QString & filename, const QString & svg, QDo
 #endif
 }
 
-void SvgFlattener::flipSMDElement(QDomDocument & domDocument, QSvgRenderer & renderer, QDomElement & element, const QString & att, QDomElement altElement, const QString & altAtt, double printerScale) 
+void SvgFlattener::flipSMDElement(QDomDocument & domDocument, QSvgRenderer & renderer, QDomElement & element, const QString & att, QDomElement altElement, const QString & altAtt, double printerScale, Qt::Orientations orientation) 
 {
 	Q_UNUSED(printerScale);
 	Q_UNUSED(att);
 
 	QMatrix m = renderer.matrixForElement(att) * TextUtils::elementToMatrix(element);
-	QRectF bounds = renderer.boundsOnElement(att);
+	//QRectF bounds = renderer.boundsOnElement(att);
+    QRectF bounds;   // want part bounds, not layer bounds
+    double w, h;
+    TextUtils::ensureViewBox(domDocument, 1, bounds, false, w, h, false);
 	m.translate(bounds.center().x(), bounds.center().y());
 	QMatrix mMinus = m.inverted();
-    QMatrix cm = mMinus * QMatrix().scale(1, -1) * m;
+    QMatrix cm = mMinus * ((orientation & Qt::Vertical) ? QMatrix().scale(-1, 1) : QMatrix().scale(1, -1)) * m;
 	QDomElement newElement = element.cloneNode(true).toElement();
 	newElement.removeAttribute("id");
 	QDomElement pElement = domDocument.createElement("g");
-	pElement.appendChild(newElement);
-	TextUtils::setSVGTransform(pElement, cm);
 	pElement.setAttribute("id", altAtt);
 	pElement.setAttribute("flipped", true);
+    QDomElement mElement = domDocument.createElement("g");
+	TextUtils::setSVGTransform(mElement, cm);
+    pElement.appendChild(mElement);
+    mElement.appendChild(newElement);
 	if (!altElement.isNull()) {
-		pElement.appendChild(altElement);
+		mElement.appendChild(altElement);
 		altElement.removeAttribute("id");
 	}
 	element.parentNode().appendChild(pElement);
 }
+
+bool SvgFlattener::loadDocIf(const QString & filename, const QString & svg, QDomDocument & domDocument) {
+	if (domDocument.isNull()) {
+        QString errorStr;
+	    int errorLine;
+	    int errorColumn;
+	    bool result;
+	    if (filename.isEmpty()) {
+		    result = domDocument.setContent(svg, &errorStr, &errorLine, &errorColumn);
+	    }
+	    else {
+		    QFile file(filename);
+		    result = domDocument.setContent(&file, &errorStr, &errorLine, &errorColumn);
+	    }
+	    if (!result) {
+		    domDocument.clear();			// probably redundant
+		    return false;
+	    }
+    }
+
+    return true;
+}
