@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2010 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 4183 $:
-$Author: cohen@irascible.com $:
-$Date: 2010-05-06 22:30:19 +0200 (Thu, 06 May 2010) $
+$Revision: 6904 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
 
 ********************************************************************/
 
@@ -28,7 +28,9 @@ $Date: 2010-05-06 22:30:19 +0200 (Thu, 06 May 2010) $
 #define FOCUSOUTCOMBOBOX_H
 
 #include <QComboBox>
+#include <QLineEdit>
 #include <QFocusEvent>
+#include <QMouseEvent>
 
 class FocusOutComboBox : public QComboBox {
 Q_OBJECT
@@ -38,7 +40,10 @@ public:
 	~FocusOutComboBox();
 
 protected:
-	void focusOutEvent(QFocusEvent *);
+    void focusInEvent(QFocusEvent *);
+    void focusOutEvent(QFocusEvent *);
+    bool eventFilter( QObject *target, QEvent *event );
+    void checkSelectAll();
 
 };
 
