@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,14 +18,15 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6265 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
+$Revision: 7004 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-29 13:10:59 +0200 (Mo, 29. Apr 2013) $
 
 ********************************************************************/
 
 #include "folderutils.h"
 #include "lockmanager.h"
+#include "textutils.h"
 #include <QDesktopServices>
 #include <QCoreApplication>
 #include <QSettings>
@@ -34,8 +35,13 @@ $Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 #include <QCryptographicHash>
 
 #include "../debugdialog.h"
-#include "../lib/quazip/quazip.h"
-#include "../lib/quazip/quazipfile.h"
+#ifdef QUAZIP_INSTALLED
+    #include <quazip/quazip.h>
+    #include <quazip/quazipfile.h>
+#else
+    #include "../lib/quazip/quazip.h"
+    #include "../lib/quazip/quazipfile.h"
+#endif
 #include "../lib/qtsysteminfo/QtSystemInfo.h"
 
 
@@ -154,9 +160,20 @@ const QString FolderUtils::applicationDirPath() {
 #ifdef Q_WS_WIN
 		return QCoreApplication::applicationDirPath();
 #else
-		// look in standard Fritzing location (applicationDirPath) then in standard linux locations
+		// look in standard Fritzing location (applicationDirPath and parent folders) then in standard linux locations
 		QStringList candidates;
 		candidates.append(QCoreApplication::applicationDirPath());
+		QDir dir(QCoreApplication::applicationDirPath());
+		if (dir.cdUp()) {
+			candidates.append(dir.absolutePath());
+			if (dir.cdUp()) {
+				candidates.append(dir.absolutePath());
+				if (dir.cdUp()) {
+					candidates.append(dir.absolutePath());
+				}
+			}
+		}
+		
 #ifdef PKGDATADIR
 		candidates.append(QLatin1String(PKGDATADIR));
 #else
@@ -165,11 +182,11 @@ const QString FolderUtils::applicationDirPath() {
 #endif
 		candidates.append(QDir::homePath() + "/.local/share/fritzing");
 		foreach (QString candidate, candidates) {
-                        //DebugDialog::debug(QString("candidate:%1").arg(candidate));
+            //DebugDialog::debug(QString("candidate:%1").arg(candidate));
 			QDir dir(candidate);
-                        if (!dir.exists("parts")) continue;
+            if (!dir.exists("parts")) continue;
 
-                        if (dir.exists("bins")) {
+            if (dir.exists("bins")) {
 				m_appPath = candidate;
 				return m_appPath;
 			}
@@ -177,7 +194,7 @@ const QString FolderUtils::applicationDirPath() {
 		}
 
 		m_appPath = QCoreApplication::applicationDirPath();
-                DebugDialog::debug("data folders not found");
+        DebugDialog::debug("data folders not found");
 
 #endif
 	}
@@ -291,18 +308,6 @@ void FolderUtils::replicateDir(QDir srcDir, QDir targDir) {
 	}
 }
 
-QString FolderUtils::getRandText() {
-	QString rand = QUuid::createUuid().toString();
-	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
-	return randext;
-}
-
-/*QString FolderUtils::getBase64RandText() {
-	QString rand = QUuid::createUuid().toString();
-	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
-	return randext;
-}*/
-
 // NOTE: This function cannot remove directories that have non-empty name filters set on it.
 void FolderUtils::rmdir(const QString &dirPath) {
 	QDir dir = QDir(dirPath);
@@ -329,7 +334,7 @@ void FolderUtils::rmdir(QDir & dir) {
 bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &filepath, const QStringList & skipSuffixes) {
 	DebugDialog::debug("zipping "+dirToCompress.path()+" into "+filepath);
 
-	QString tempZipFile = QDir::temp().path()+"/"+getRandText()+".zip";
+	QString tempZipFile = QDir::temp().path()+"/"+TextUtils::getRandText()+".zip";
 	DebugDialog::debug("temp file: "+tempZipFile);
 	QuaZip zip(tempZipFile);
 	if(!zip.open(QuaZip::mdCreate)) {
@@ -403,12 +408,17 @@ bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &f
 }
 
 
-bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompress) {
+bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompress, QString & error) {
+    static QChar badCharacters[] = { '\\', '/', ':', '*', '?', '"', '<', '>', '|' };
+    static QChar underscore('_');
+
 	QuaZip zip(filepath);
 	if(!zip.open(QuaZip::mdUnzip)) {
-		qWarning("zip.open(): %d", zip.getZipError());
+        error = QString("zip.open(): %d").arg(zip.getZipError());
+		DebugDialog::debug(error);
 		return false;
 	}
+
 	zip.setFileNameCodec("IBM866");
 	DebugDialog::debug(QString("unzipping %1 entries from %2").arg(zip.getEntriesCount()).arg(filepath));
 	QuaZipFileInfo info;
@@ -418,25 +428,43 @@ bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompres
 	char c;
 	for(bool more=zip.goToFirstFile(); more; more=zip.goToNextFile()) {
 		if(!zip.getCurrentFileInfo(&info)) {
-			qWarning("getCurrentFileInfo(): %d\n", zip.getZipError());
+			error = QString("getCurrentFileInfo(): %d\n").arg(zip.getZipError());
+			DebugDialog::debug(error);
 			return false;
 		}
 
 		if(!file.open(QIODevice::ReadOnly)) {
-			qWarning("file.open(): %d", file.getZipError());
+			error = QString("file.open(): %d").arg(file.getZipError());
+			DebugDialog::debug(error);
 			return false;
 		}
 		name=file.getActualFileName();
 		if(file.getZipError()!=UNZ_OK) {
-			qWarning("file.getFileName(): %d", file.getZipError());
+			error = QString("file.getFileName(): %d").arg(file.getZipError());
+			DebugDialog::debug(error);
 			return false;
 		}
 
 		out.setFileName(dirToDecompress+"/"+name);
 		// this will fail if "name" contains subdirectories, but we don't mind that
 		if(!out.open(QIODevice::WriteOnly)) {
-			qWarning("out.open(): %s", out.errorString().toLocal8Bit().constData());
-			return false;
+            for (int i = 0; i < name.length(); i++) {
+                if (name[i].unicode() < 32) {
+                    name.replace(i, 1, &underscore, 1);
+                }
+                else for (unsigned int j = 0; j < (sizeof(badCharacters) / sizeof(QChar)); j++) {
+                    if (name[i] == badCharacters[j]) {
+                        name.replace(i, 1, &underscore, 1);
+                        break;
+                    }
+                }
+            }
+            out.setFileName(dirToDecompress+"/"+name);
+            if(!out.open(QIODevice::WriteOnly)) {
+                error = QString("out.open(): %s").arg(out.errorString().toLocal8Bit().constData());
+			    DebugDialog::debug(error);
+			    return false;
+            }
 		}
 
 		// Slow like hell (on GNU/Linux at least), but it is not my fault.
@@ -460,41 +488,47 @@ bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompres
 
 		out.close();
 		if(file.getZipError()!=UNZ_OK) {
-			qWarning("file.getFileName(): %d", file.getZipError());
+			error = QString("file.getFileName(): %d").arg(file.getZipError());
+			DebugDialog::debug(error);
 			return false;
 		}
 		if(!file.atEnd()) {
-			qWarning("read all but not EOF");
+			error = "read all but not EOF";
+			DebugDialog::debug(error);
 			return false;
 		}
 		file.close();
 		if(file.getZipError()!=UNZ_OK) {
-			qWarning("file.close(): %d", file.getZipError());
+			error = QString("file.close(): %d").arg(file.getZipError());
+			DebugDialog::debug(error);
 			return false;
 		}
 	}
 	zip.close();
 	if(zip.getZipError()!=UNZ_OK) {
-		qWarning("zip.close(): %d", zip.getZipError());
+		error = QString("zip.close(): %d").arg(zip.getZipError());
+		DebugDialog::debug(error);
 		return false;
 	}
 	return true;
 }
 
-void FolderUtils::collectFiles(const QDir & parent, QStringList & filters, QStringList & files)
+void FolderUtils::collectFiles(const QDir & parent, QStringList & filters, QStringList & files, bool recursive)
 {
 	QFileInfoList fileInfoList = parent.entryInfoList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
 	foreach (QFileInfo fileInfo, fileInfoList) {
 		files.append(fileInfo.absoluteFilePath());
 	}
 
-	QFileInfoList dirList = parent.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::NoSymLinks);
-	foreach (QFileInfo dirInfo, dirList) {
-		QDir dir(dirInfo.filePath());
-		//DebugDialog::debug(QString("looking in backup dir %1").arg(dir.absolutePath()));
+    if (recursive) {
+        QFileInfoList dirList = parent.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::NoSymLinks);
+        foreach (QFileInfo dirInfo, dirList) {
+            QDir dir(dirInfo.filePath());
+            //DebugDialog::debug(QString("looking in backup dir %1").arg(dir.absolutePath()));
 
-		collectFiles(dir, filters, files);
-	}
+            collectFiles(dir, filters, files, recursive);
+        }
+    }
 }
 
 
@@ -541,4 +575,4 @@ void FolderUtils::copyBin(const QString & dest, const QString & source) {
 #endif
 
     );
-}
\ No newline at end of file
+}
