@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6218 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
+$Revision: 6912 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 
 ********************************************************************/
 
@@ -50,6 +50,7 @@ public:
 	static double pixels2mils(double p, double dpi);
 	static double pixels2ins(double p, double dpi);
 	static double distanceSqd(QPointF p1, QPointF p2);
+	static double distanceSqd(QPoint p1, QPoint p2);
 	static double getNearestOrdinate(double ordinate, double units);
 
 	static double mm2mils(double mm);
@@ -68,6 +69,9 @@ public:
 	static QPainterPath shapeFromPath(const QPainterPath &path, const QPen &pen, double shapeStrokeWidth, bool includeOriginalPath);
 	static void qt_graphicsItem_highlightSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRectF & boundingRect, const QPainterPath & path);
 	static QPointF calcRotation(QTransform & rotation, QPointF rCenter, QPointF p, QPointF pCenter); 
+    static void drawBorder(QImage * image, int border);
+    static bool isFlipped(const QMatrix & matrix, double & rotation);
+    static void renderOne(QDomDocument *, QImage *, const QRectF & renderRect);
 
 public:
 	static const double IllustratorDPI;
