@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6145 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-07-05 19:17:11 +0200 (Thu, 05 Jul 2012) $
+$Revision: 6954 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-05 10:22:00 +0200 (Fr, 05. Apr 2013) $
 
 ********************************************************************/
 
@@ -29,12 +29,13 @@ $Date: 2012-07-05 19:17:11 +0200 (Thu, 05 Jul 2012) $
 #include "../items/wire.h"
 #include "../items/jumperitem.h"
 #include "../sketch/sketchwidget.h"
+#include "../debugdialog.h"
 
 #ifdef _MSC_VER 
 #pragma warning(push) 
+#pragma warning(disable:4503)
 #pragma warning(disable:4100)			// disable scary-looking compiler warnings in Boost library
 #pragma warning(disable:4181)	
-#pragma warning(disable:4503)
 #endif
 
 #include <boost/config.hpp>
@@ -85,11 +86,11 @@ void GraphUtils::minCut(QList<ConnectorItem *> & connectorItems, QList<SketchWid
 	using namespace boost;
 
 	typedef adjacency_list_traits < vecS, vecS, directedS > Traits;
-	typedef property < vertex_index_t, long,
-			property < vertex_color_t, boost::default_color_type > > VERTEX;
-	typedef property < edge_capacity_t, long,										
-			property < edge_residual_capacity_t, long,								
-			property < edge_reverse_t, Traits::edge_descriptor > > > EDGE;
+    typedef property < vertex_color_t, boost::default_color_type > COLOR;
+	typedef property < vertex_index_t, long, COLOR > VERTEX;
+    typedef property < edge_reverse_t, Traits::edge_descriptor > REVERSE;
+    typedef property < edge_residual_capacity_t, long, REVERSE > RESIDUAL;
+	typedef property < edge_capacity_t, long, RESIDUAL > EDGE;
 	typedef adjacency_list < listS, vecS, directedS, VERTEX, EDGE > Graph;
 
 	Graph g;
@@ -229,7 +230,7 @@ void GraphUtils::minCut(QList<ConnectorItem *> & connectorItems, QList<SketchWid
 
 		int ix = vertices.value(key);
 		QList<ConnectorItem *> bcis;
-		key->attachedTo()->busConnectorItems(key->bus(), bcis);
+		key->attachedTo()->busConnectorItems(key->bus(), key, bcis);
 		foreach (ConnectorItem * bci, bcis) {
 			if (bci == key) continue;
 
@@ -293,7 +294,8 @@ void GraphUtils::minCut(QList<ConnectorItem *> & connectorItems, QList<SketchWid
 
 	// if color_map parameter not specified, colors are not set
     long flow = edmonds_karp_max_flow(g, s, t, color_map(color)); 
-	DebugDialog::debug(QString("flow %1, s%2, t%3").arg(flow).arg(index(s)).arg(index(t)));
+	Q_UNUSED(flow);
+	//DebugDialog::debug(QString("flow %1, s%2, t%3").arg(flow).arg(index(s)).arg(index(t)));
 	//for (int i = 0; i < verts.count(); ++i) {
 	//	DebugDialog::debug(QString("index %1 %2").arg(index(verts[i])).arg(color(verts[i])));
 	//}
@@ -461,6 +463,18 @@ bool GraphUtils::scoreOneNet(QList<ConnectorItem *> & partConnectorItems, ViewGe
 				continue;
 			}
 
+            if (from->attachedToItemType() == ModelPart::Symbol && 
+                to->attachedToItemType() == ModelPart::Symbol && 
+                from->attachedTo()->isEverVisible() && 
+                to->attachedTo()->isEverVisible()) 
+           {
+                // equipotential symbols are treated as if they were connected by wires
+				add_edge_d(i, j, G);
+				add_edge_d(j, i, G);
+				gotUserConnection = true;
+				continue;
+			}
+
 			if (to->attachedTo() != from->attachedTo()) {
 				gotUserConnection = true;
 				continue;
@@ -480,6 +494,8 @@ bool GraphUtils::scoreOneNet(QList<ConnectorItem *> & partConnectorItems, ViewGe
 		return false;
 	}
 
+
+
 	routingStatus.m_netCount++;
 
 	for (int i = 0; i < num_nodes; i++) {
@@ -506,8 +522,6 @@ bool GraphUtils::scoreOneNet(QList<ConnectorItem *> & partConnectorItems, ViewGe
 			            }
                     }
                     continue;
-                default:
-                    continue;
             }
 
 			Wire * wire = qobject_cast<Wire *>(toConnectorItem->attachedTo());
@@ -588,7 +602,7 @@ void GraphUtils::collectBreadboard(ConnectorItem * connectorItem, QList<Connecto
         Bus * bus = candidate->bus();
         if (bus) {
             QList<ConnectorItem *> busConnectorItems;
-            candidate->attachedTo()->busConnectorItems(bus, busConnectorItems);
+            candidate->attachedTo()->busConnectorItems(bus, candidate, busConnectorItems);
             foreach (ConnectorItem * bci, busConnectorItems) {
                 if (!itemsToGo.contains(bci)) {
                     itemsToGo.append(bci);
