@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -29,28 +29,39 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 
 #include "textutils.h"
 #include "misc.h"
+#include "../installedfonts.h"
+
 //#include "../debugdialog.h"
 
 #include <QRegExp>
 #include <QBuffer>
 #include <QFile>
+#include <QSettings>
+#include <QUrl>
+#include <QUuid>
+#include <QCryptographicHash>
 
 #include <qmath.h>
 #include <qnumeric.h>
 
+QSet<QString> InstalledFonts::InstalledFontsList;
+QMultiHash<QString, QString> InstalledFonts::InstalledFontsNameMapper;   // family name to filename; SVG files seem to have to use filename
+
 const QString TextUtils::CreatedWithFritzingString("Created with Fritzing (http://www.fritzing.org/)");
 const QString TextUtils::CreatedWithFritzingXmlComment("<!-- " + CreatedWithFritzingString + " -->\n");
 
 const QRegExp TextUtils::FindWhitespace("[\\s]+");
-const QRegExp TextUtils::SodipodiDetector("((inkscape)|(sodipodi)):[^=\\s]+=\"([^\"\\\\]*(\\\\.[^\"\\\\]*)*)\"");
+static const QRegExp SodipodiAttributeDetector("(inkscape|sodipodi):[^=\\s]+=\"([^\"\\\\]*(\\\\.[^\"\\\\]*)*)\"");
+static const QRegExp SodipodiElementDetector("</{0,1}(inkscape|sodipodi):[^>]+>");
 const QString TextUtils::SMDFlipSuffix("___");
 
 const QString TextUtils::RegexFloatDetector = "[-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?";
 const QRegExp TextUtils::floatingPointMatcher(RegexFloatDetector);		
 
-const QRegExp HexExpr("&#x[0-9a-fA-F];");   // &#x9; &#xa; &#xd;
+static const QRegExp HexExpr("&#x[0-9a-fA-F];");   // &#x9; &#xa; &#xd;
+static const QRegExp Xmlns("xmlns=([\"|'])[^\"']*\\1");
 
-static const ushort MicroSymbolCode = 181;
+const ushort TextUtils::MicroSymbolCode = 181;
 const QString TextUtils::MicroSymbol = QString::fromUtf16(&MicroSymbolCode, 1);
 
 const QString TextUtils::AdobeIllustratorIdentifier = "Generator: Adobe Illustrator";
@@ -59,6 +70,103 @@ QList<QString> PowerPrefixes;
 QList<double> PowerPrefixValues;
 const QString TextUtils::PowerPrefixesString = QString("pnmkMGTu\\x%1").arg(MicroSymbolCode, 4, 16, QChar('0'));
 
+typedef QHash<QString /*brokenFont*/, QString /*replacementFont*/> FixedFontsHash;
+
+//////////////////////////////////////////////
+
+QSet<QString> getAttrFontFamilies(const QString &fileContent) {
+	/*
+	 * font-family defined as attr example:
+
+<text xmlns="http://www.w3.org/2000/svg" font-family="Droid Sans"
+id="text2732" transform="matrix(1 0 0 1 32.2012 236.969)"
+font-size="9.9771" >A0</text>
+
+	 */
+
+	static QString pattern = "font-family\\s*=\\s*\"([^\"]*)\"";
+	return TextUtils::getRegexpCaptures(pattern,fileContent);
+}
+
+QSet<QString> getFontFamiliesInsideStyleTag(const QString &fileContent) {
+	/*
+	 * regexp: font-family\s*:\s*(.|[^;"]*).*"
+	 * font-family defined in a style attr example:
+
+style="font-size:9;-inkscape-font-specification:Droid Sans;font-family:Droid Sans;font-weight:normal;font-style:normal;font-stretch:normal;font-variant:normal"
+
+style="font-size:144px;font-style:normal;font-weight:normal;line-height:100%;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans" x="18.000002"
+
+	 */
+
+	static QString pattern = "font-family\\s*:\\s*([^;\"]*)";
+	return TextUtils::getRegexpCaptures(pattern,fileContent);
+}
+
+
+FixedFontsHash fixFontsMapping(const QSet<QString> fontsTofix, const QString & destFont) {
+	FixedFontsHash retval;
+
+	QSet<QString> tempFontsToFix;
+	tempFontsToFix = fontsTofix - InstalledFonts::InstalledFontsList;
+	foreach (QString fontFileName, InstalledFonts::InstalledFontsNameMapper.values()) {
+		// SVG uses filename which may not match family name (e.g. "Droid Sans" and "Droid Sans")
+		tempFontsToFix.remove(fontFileName);
+	}
+
+	foreach (QString broken, tempFontsToFix) {
+		retval.insert(broken, destFont);
+	}
+
+	return retval;
+}
+
+bool removeFontFamilySingleQuotes(QString &fileContent) {
+	static QString pattern = "font-family=\"('[^']*')\"";
+	QSet<QString> wrongFontFamilies = TextUtils::getRegexpCaptures(pattern, fileContent);
+
+	foreach(QString ff, wrongFontFamilies) {
+		QString wrongFF = ff;
+		QString fixedFF = ff.remove('\'');
+		fileContent.replace(wrongFF,fixedFF);
+	}
+
+	return wrongFontFamilies.size() > 0;
+}
+
+bool fixUnavailableFontFamilies(QString &fileContent, const QString & destFont) {
+	QSet<QString> definedFFs;
+	definedFFs.unite(getAttrFontFamilies(fileContent));
+	definedFFs.unite(getFontFamiliesInsideStyleTag(fileContent));
+
+	FixedFontsHash fixedFonts = fixFontsMapping(definedFFs, destFont);
+	foreach(QString oldF, fixedFonts.keys()) {
+		QString newF = fixedFonts[oldF];
+		fileContent.replace(oldF,newF);
+	}
+
+	return fixedFonts.size() > 0;
+}
+
+QString  makeDashString(bool dashed, const QVector<qreal> & pattern, double dpi, double printerScale)
+{
+    QString dash;
+    if (dashed && pattern.count() > 0) {
+        dash = "stroke-dasharray='";
+        foreach (qreal p, pattern) {
+            dash += QString::number(p * dpi / printerScale);
+            dash += ",";
+        }
+        dash.chop(1);
+        dash.append("'");
+    }
+
+    return dash;
+}
+
+///////////////////////////////////////
+
+
 void TextUtils::findElementsWithAttribute(QDomElement & element, const QString & att, QList<QDomElement> & elements) {
 	if (!element.attribute(att).isEmpty()) {
 		elements.append(element);
@@ -102,30 +210,37 @@ QSet<QString> TextUtils::getRegexpCaptures(const QString &pattern, const QString
 double TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrator) {
 	QString string = s;
 	double divisor = 1.0;
+	int chop = 2;
 	if (string.endsWith("cm", Qt::CaseInsensitive)) {
 		divisor = 2.54;
-		string.chop(2);
 	}
 	else if (string.endsWith("mm", Qt::CaseInsensitive)) {
 		divisor = 25.4;
-		string.chop(2);
 	}
 	else if (string.endsWith("in", Qt::CaseInsensitive)) {
 		divisor = 1.0;
-		string.chop(2);
 	}
 	else if (string.endsWith("px", Qt::CaseInsensitive)) {
 		divisor = isIllustrator? 72.0: 90.0;
-		string.chop(2);
 	}
 	else if (string.endsWith("mil", Qt::CaseInsensitive)) {
 		divisor = 1000.0;
-		string.chop(3);
+		chop = 3;
+	}
+	else if (string.endsWith("pt", Qt::CaseInsensitive)) {
+		divisor = 72.0;
+	}
+	else if (string.endsWith("pc", Qt::CaseInsensitive)) {
+		divisor = 6.0;
 	}
 	else {
+        chopNotDigits(string);
 		divisor = 90.0;			// default to Qt's standard internal units if all else fails
+		chop = 0;
 	}
 
+	if (chop) string.chop(chop);
+
 	bool fine;
 	double result = string.toDouble(&fine);
 	if (!fine) {
@@ -137,6 +252,16 @@ double TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrat
 	return result / divisor;
 }
 
+void TextUtils::chopNotDigits(QString & string) {
+	for (int ix = string.count() - 1; ix >= 0; ix--) {
+        QChar ch = string.at(ix);
+		if (ch.isDigit()) return;
+        if (ch == '.') return;
+			
+		string.chop(1);
+	}
+}
+
 bool TextUtils::squashElement(QDomDocument & doc, const QString & elementName, const QString &attName, const QRegExp &matchContent) {
     bool result = false;
     QDomElement root = doc.documentElement();
@@ -176,7 +301,7 @@ QString TextUtils::replaceTextElement(const QString & svg, const QString & id, c
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		replaceChildText(doc, node, newValue);
+		replaceChildText(node, newValue);
 		return doc.toString();
 	}
 		
@@ -198,7 +323,7 @@ QByteArray TextUtils::replaceTextElement(const QByteArray & svg, const QString &
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		replaceChildText(doc, node, newValue);
+		replaceChildText(node, newValue);
 		return doc.toByteArray();
 	}
 		
@@ -221,7 +346,7 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 		foreach (QString id, hash.keys()) {
 			if (node.attribute("id").compare(id) != 0) continue;
 
-			replaceChildText(doc, node, hash.value(id));
+			replaceChildText(node, hash.value(id));
 			changed = true;
 			break;
 		}
@@ -233,18 +358,19 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 }
 
 
-void TextUtils::replaceElementChildText(QDomDocument & doc, QDomElement & root, const QString & elementName, const QString & text) {
+void TextUtils::replaceElementChildText(QDomElement & root, const QString & elementName, const QString & text) {
 	QDomElement element = root.firstChildElement(elementName);
 	if (element.isNull()) {
-		doc.createElement(elementName);
+		element = root.ownerDocument().createElement(elementName);
 		root.appendChild(element);
 	}
-	replaceChildText(doc, element, text);
+	replaceChildText(element, text);
 }
 
-void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QString & text) {
+void TextUtils::replaceChildText(QDomNode & node, const QString & text) {
 	node.normalize();
 
+
 	QDomNodeList childList = node.childNodes();
 	for (int j = 0; j < childList.count(); j++) {
 		QDomNode child = childList.item(j);
@@ -254,7 +380,7 @@ void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QStr
 		}
 	}
 
-	QDomText t = doc.createTextNode(text);
+	QDomText t = node.ownerDocument().createTextNode(text);
 	node.appendChild(t);
 }
 
@@ -370,18 +496,23 @@ bool TextUtils::isIllustratorDoc(const QDomDocument & doc) {
 }
 
 QString TextUtils::removeXMLEntities(QString svgContent) {
-	return removeXMLNS(svgContent.remove(HexExpr));
+	return svgNSOnly(svgContent.remove(HexExpr));
 }
 
-QString TextUtils::removeXMLNS(QString svgContent) {
+QString TextUtils::killXMLNS(QString svgContent) {
 	// TODO: this is a bug in Qt, it would be nice to fix it there
-	// but as a stopgap, it would be nice if this function removed all repetitious xmlns attributes, not just the svg one
-	QString svgNS = "xmlns=\"http://www.w3.org/2000/svg\"";
-	QStringList strings = svgContent.split(svgNS);
-	if (strings.count() < 3) return svgContent;
 
-	QString first = strings.takeFirst();
-	return first + svgNS + strings.join("");
+    svgContent.remove(Xmlns);
+    return svgContent;
+}
+
+QString TextUtils::svgNSOnly(QString svgContent) {
+    svgContent.remove(Xmlns);
+    int ix = svgContent.indexOf("<svg");
+    if (ix >= 0) {
+        svgContent.insert(ix + 5, "xmlns=\"http://www.w3.org/2000/svg\" ");
+    }
+    return svgContent;
 }
 
 bool TextUtils::cleanSodipodi(QString &content)
@@ -389,22 +520,9 @@ bool TextUtils::cleanSodipodi(QString &content)
 	// clean out sodipodi stuff
 	// TODO: don't bother with the core parts
 	int l1 = content.length();
-	content.remove(SodipodiDetector);
+	content.remove(SodipodiAttributeDetector);          // remove attributes first
+	content.remove(SodipodiElementDetector);
 	if (content.length() != l1) {
-		/*
-		QFileInfo f(filename);
-		QString p = f.absoluteFilePath();
-		p.remove(':');
-		p.remove('/');
-		p.remove('\\');
-		QFile fi(QCoreApplication::applicationDirPath() + p);
-		bool ok = fi.open(QFile::WriteOnly);
-		if (ok) {
-			QTextStream out(&fi);
-   			out << str;
-			fi.close();
-		}
-		*/
 		return true;
 	}
 	return false;
@@ -429,41 +547,6 @@ bool TextUtils::cleanSodipodi(QString &content)
 	*/
 }
 
-bool TextUtils::fixViewboxOrigin(QString &fileContent) {
-	QDomDocument svgDom;
-
-	bool fileHasChanged = false;
-	if(isIllustratorFile(fileContent)) {
-		QString errorMsg;
-		int errorLine;
-		int errorCol;
-		if(!svgDom.setContent(fileContent, true, &errorMsg, &errorLine, &errorCol)) {
-			return false;
-		}
-
-		QDomElement elem = svgDom.firstChildElement("svg");
-
-		fileHasChanged = moveViewboxToTopLeftCorner(elem);
-
-		if(fileHasChanged) {
-			fileContent = svgDom.toString();
-		}
-	}
-
-	return fileHasChanged;
-}
-
-bool TextUtils::moveViewboxToTopLeftCorner(QDomElement &elem) {
-	QString attrName = elem.hasAttribute("viewbox")? "viewbox": "viewBox";
-	QStringList vals = elem.attribute(attrName).split(" ");
-	if(vals.length() == 4 && (vals[0] != "0" || vals[1] != "0")) {
-		QString newValue = QString("0 0 %1 %2").arg(vals[2]).arg(vals[3]);
-		elem.setAttribute(attrName,newValue);
-		return true;
-	}
-	return false;
-}
-
 bool TextUtils::fixPixelDimensionsIn(QString &fileContent) {
 	bool isIllustrator = isIllustratorFile(fileContent);
 	if (!isIllustrator) return false;
@@ -876,44 +959,232 @@ void TextUtils::gWrap(QDomDocument & domDocument, const QHash<QString, QString>
 	}
 }
 
+bool TextUtils::fixInternalUnits(QString & svg) 
+{
+	// float detector is a little weak
+	static QRegExp findInternalUnits("[\"']([\\d,\\.]+)(px|mm|cm|in|pt|pc)[\"']");
+	static QRegExp findStrokeWidth("stroke-width:([\\d,\\.]+)(px|mm|cm|in|pt|pc)");
 
-bool TextUtils::noPattern(QString &svg) {
-	if (!svg.contains("<pattern")) return false;
+	int sw = 0;
+	int iu = 0;
+	int jx = svg.indexOf("<svg");
+	if (jx >= 0) {
+		sw = iu = svg.indexOf(">", jx + 4);
+	}
+
+	bool firstTime = true;
+	QSizeF size;
+	QRectF viewBox;
+	bool result = false;
+	while (true) {
+		iu = findInternalUnits.indexIn(svg, iu);
+		if (iu < 0) break;
+
+		result = true;
+		if (firstTime) {
+			// assumes width dpi = height dpi
+			size = parseForWidthAndHeight(svg, viewBox, true);
+			if (size.width() == 0) {
+				// svg is messed up
+				return false;
+			}
+            firstTime = false;
+		}
+
+		QString old = findInternalUnits.cap(1) + findInternalUnits.cap(2); 
+		double in = convertToInches(old);
+		double replacement = in * viewBox.width() / size.width();
+		svg.replace(iu + 1, old.length(), QString::number(replacement));
+	}
+
+	while (true) {
+		sw = findStrokeWidth.indexIn(svg, sw);
+		if (sw < 0) break;
+
+		result = true;
+		if (firstTime) {
+			// assumes width dpi = height dpi
+			size = parseForWidthAndHeight(svg, viewBox, true);
+			if (size.width() == 0) {
+				// svg is messed up
+				return false;
+			}
+		}
+
+		QString old = findStrokeWidth.cap(1) + findStrokeWidth.cap(2); 
+		double in = convertToInches(old);
+		double replacement = in * viewBox.width() / size.width();
+		svg.replace(sw + 13, old.length(), QString::number(replacement));
+	}
+
+	return result;
+}
+
+bool TextUtils::fixMuch(QString &svg, bool fixStrokeWidthFlag)
+{
+    bool result = cleanSodipodi(svg);
+	result |= fixInternalUnits(svg);
 
 	QDomDocument svgDom;
 	QString errorMsg;
 	int errorLine;
 	int errorCol;
 	if(!svgDom.setContent(svg, true, &errorMsg, &errorLine, &errorCol)) {
-		return false;
+		return result;
 	}
 
-	QDomNodeList nodeList = svgDom.elementsByTagName("pattern");
-    for (int i = 0; i < nodeList.count(); i++) {
-        QDomNode pattern = nodeList.at(i);
+    QDomElement root = svgDom.documentElement();
+    result |= fixViewBox(root);
 
-        pattern.parentNode().removeChild(pattern);
-	}
+    QStringList strings;
+    strings << "pattern" << "marker" << "clipPath";
+    foreach (QString string, strings) {
+        if (svg.contains("<" + string)) {
+            result |= noPatternAux(svgDom, string);
+        }
+    }
+
+    if (svg.contains("<use")) {
+        result |= noUseAux(svgDom);
+    }
+
+    if (svg.contains("<tspan")) {
+        result |= tspanRemoveAux(svgDom);
+    }
+
+    if (fixStrokeWidthFlag) {
+        result |= fixStrokeWidth(svgDom);
+    }
+
+    result |= elevateTransform(root);
+
+    if (result) {
+        svg = removeXMLEntities(svgDom.toString());
+    }
+
+    return result;
+}
+
+bool TextUtils::fixViewBox(QDomElement & root) {
+	QString viewBox = root.attribute("viewBox");
+    if (viewBox.isEmpty()) return false;
+
+	QStringList coords = viewBox.split(QRegExp(" |,"));
+	if (coords.length() != 4) return false;
+    
+    if (coords[0] == "0" && coords[1] == "0") return false;
+
+    bool ok;
+    double x = coords.at(0).toDouble(&ok);
+    if (!ok) return false;
+
+    double y = coords.at(1).toDouble(&ok);
+    if (!ok) return false;
+
+	QString newValue = QString("0 0 %1 %2").arg(coords[2]).arg(coords[3]);
+	root.setAttribute("viewBox", newValue);
+
+    QDomElement transformElement = root.ownerDocument().createElement("g");
+    transformElement.setAttribute("transform", QString("translate(%1,%2)").arg(-x).arg(-y));
+    transformElement = root.insertBefore(transformElement, QDomElement()).toElement();
+    QDomElement nextElement = transformElement.nextSiblingElement();
+    while (!nextElement.isNull()) {
+        transformElement.appendChild(nextElement);
+        nextElement = transformElement.nextSiblingElement();
+    }
 
-    svg = removeXMLEntities(svgDom.toString());
 	return true;
 }
 
+double TextUtils::getStrokeWidth(QDomElement & element, double defaultValue)
+{
+    bool ok;
+	double sw = element.attribute("stroke-width").toDouble(&ok);
+    if (ok) return sw;
 
-bool TextUtils::tspanRemove(QString &svg) {
-	if (!svg.contains("<tspan")) return false;
+    QString stroke = element.attribute("stroke");
+    if (stroke == "none") return 0;
 
-	// TODO: if the <text> element has its own text node, this will be ordered first, even if the text node occurs after one or more <tspan> elements
-	// this is a very unlikely occurrence, so fixing it is very low priority.
+    QDomElement parent = element.parentNode().toElement();
+    while (!parent.isNull()) {
+        sw = parent.attribute("stroke-width").toDouble(&ok);
+        if (ok) return sw;
 
-	QDomDocument svgDom;
-	QString errorMsg;
-	int errorLine;
-	int errorCol;
-	if(!svgDom.setContent(svg, true, &errorMsg, &errorLine, &errorCol)) {
-		return false;
+        stroke = parent.attribute("stroke");
+        if (stroke == "none") return 0;
+
+        parent = parent.parentNode().toElement();
+    }
+
+    //QString text;
+    //QTextStream stream(&text);
+    //element.save(stream, 0);
+    //DebugDialog::debug(QString("no circle stroke width set in %1: %2").arg(filename).arg(text));
+    
+    // default if there is no value to inherit
+    element.setAttribute("stroke-width", defaultValue);
+    return defaultValue;
+
+	//QString strokewidth("stroke-width");
+	//QString s = element.attribute("style");
+	//SvgFileSplitter::fixStyleAttribute(connectorElement, s, strokewidth);
+	//sw = connectorElement.attribute("stroke-width").toDouble(&ok);
+	//if (!ok) {
+		//return false;
+	//}
+}
+
+bool TextUtils::fixStrokeWidth(QDomDocument & svgDoc) {
+    bool result = false;
+
+    QList<QDomElement> todo;
+    todo << svgDoc.documentElement();
+    while (todo.count() > 0) {
+        QDomElement element = todo.takeFirst();
+        QDomElement child = element.firstChildElement();
+        while (!child.isNull()) {
+            todo.append(child);
+            child = child.nextSiblingElement();
+        }
+
+        QString stroke, strokeWidth;
+        stroke = element.attribute("stroke");
+        if (stroke.isEmpty()) {
+            QString style = element.attribute("style");
+            if (style.contains("stroke")) {
+                fixStyleAttribute(element);
+                stroke = element.attribute("stroke");
+            }
+        }
+        strokeWidth = element.attribute("stroke-width");
+
+        if (stroke.isEmpty()) continue;
+        if (stroke == "none") continue;
+        if (!strokeWidth.isEmpty()) continue;
+
+        getStrokeWidth(element, 1);
+        result = true;
+    }
+
+    return result;
+}
+
+
+bool TextUtils::noPatternAux(QDomDocument & svgDom, const QString & tag) 
+{
+    bool result = false;
+	QDomNodeList nodeList = svgDom.elementsByTagName(tag);
+    for (int i = 0; i < nodeList.count(); i++) {
+        QDomNode pattern = nodeList.at(i);
+        pattern.parentNode().removeChild(pattern);
+        result = true;
 	}
+    return result;
+}
 
+
+bool TextUtils::tspanRemoveAux(QDomDocument & svgDom)
+{
 	QList<QDomElement> texts;
 	QDomNodeList textNodeList = svgDom.elementsByTagName("text");
     for (int i = 0; i < textNodeList.count(); i++) {
@@ -924,6 +1195,8 @@ bool TextUtils::tspanRemove(QString &svg) {
 		texts.append(text);
 	}
 
+    if (texts.count() == 0) return false;
+
 	foreach (QDomElement text, texts) {
 		QDomElement g = svgDom.createElement("g");
 		text.parentNode().replaceChild(g, text);
@@ -946,20 +1219,11 @@ bool TextUtils::tspanRemove(QString &svg) {
 		}
 	}
 
-	svg = removeXMLEntities(svgDom.toString());
 	return true;
 }
 
-bool TextUtils::noUse(QString &svg) {
-	if (!svg.contains("<use")) return false;
-
-	QDomDocument svgDom;
-	QString errorMsg;
-	int errorLine;
-	int errorCol;
-	if (!svgDom.setContent(svg, true, &errorMsg, &errorLine, &errorCol)) {
-		return false;
-	}
+bool TextUtils::noUseAux(QDomDocument & svgDom)
+{
 
     QDomElement root = svgDom.documentElement();
     if (root.isNull()) return false;
@@ -977,6 +1241,8 @@ bool TextUtils::noUse(QString &svg) {
 		uses.append(use);
 	}
 
+    if (uses.count() == 0) return false;
+
 	foreach (QDomElement use, uses) {
 		QString transform = use.attribute("transform");
         QString refid = use.attribute("href");
@@ -999,7 +1265,6 @@ bool TextUtils::noUse(QString &svg) {
         copy.setAttribute("id", id);
 	}
 
-	svg = removeXMLEntities(svgDom.toString());
 	return true;
 }
 
@@ -1157,27 +1422,32 @@ double TextUtils::getViewBoxCoord(const QString & svg, int coord)
 	return c.toDouble();
 }
 
-QString TextUtils::makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+QString TextUtils::makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> & pattern) 
 {
 	p1.setX(p1.x() * dpi / printerScale);
 	p1.setY(p1.y() * dpi / printerScale);
 	p2.setX(p2.x() * dpi / printerScale);
 	p2.setY(p2.y() * dpi / printerScale);
 
+    QString dash = makeDashString(dashed, pattern, dpi, printerScale);
+
 	QString stroke = (blackOnly) ? "black" : colorString;
-	return QString("<line stroke-linecap='round' stroke='%6' x1='%1' y1='%2' x2='%3' y2='%4' stroke-width='%5' />")
+	return QString("<line stroke-linecap='round' stroke='%6' x1='%1' y1='%2' x2='%3' y2='%4' stroke-width='%5' %7/>")
 					.arg(p1.x())
 					.arg(p1.y())
 					.arg(p2.x())
 					.arg(p2.y())
 					.arg(width * dpi / printerScale)
-					.arg(stroke);
+					.arg(stroke)
+                    .arg(dash)                
+                   ;
 }
 
-QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> & pattern) 
 {
-	QString stroke = (blackOnly) ? "black" : colorString;
-	return QString("<path stroke-linecap='round' fill='none' stroke-width='%1' stroke='%2' d='M%3,%4C%5,%6, %7,%8 %9,%10' />")
+    QString dash = makeDashString(dashed, pattern, dpi, printerScale);
+    QString stroke = (blackOnly) ? "black" : colorString;
+	return QString("<path stroke-linecap='round' fill='none' stroke-width='%1' stroke='%2' d='M%3,%4C%5,%6, %7,%8 %9,%10' %11/>")
 					.arg(width * dpi / printerScale)
 					.arg(stroke)
 					.arg(poly.at(0).x() * dpi / printerScale)
@@ -1188,6 +1458,7 @@ QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QStr
 					.arg(poly.at(2).y() * dpi / printerScale)
 					.arg(poly.at(3).x() * dpi / printerScale)
 					.arg(poly.at(3).y() * dpi / printerScale)
+                    .arg(dash)
 					;
 }
 
@@ -1328,7 +1599,7 @@ void TextUtils::expandAndFillAux(QDomElement & element, const QString & color, d
 bool TextUtils::writeUtf8(const QString & fileName, const QString & text)
 {
 	QFile file(fileName);
-	if (file.open(QIODevice::WriteOnly)) {
+	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
 		QTextStream out(&file);
 		out.setCodec("UTF-8");
 		out << text;
@@ -1366,16 +1637,49 @@ int TextUtils::getPinsAndSpacing(const QString & expectedFileName, QString & spa
 }
 
 QSizeF TextUtils::parseForWidthAndHeight(const QString & svg)
+{
+	QRectF viewBox;
+    return parseForWidthAndHeight(svg, viewBox, false);
+}
+
+QString TextUtils::parseForModuleID(const QString & fzp)
+{
+	QXmlStreamReader streamReader(fzp);
+    streamReader.setNamespaceProcessing(false);
+
+	while (!streamReader.atEnd()) {
+        switch (streamReader.readNext()) {
+            case QXmlStreamReader::StartElement:
+			    if (streamReader.name().toString().compare("module") == 0) {
+				    return streamReader.attributes().value("moduleId").toString();
+                }
+                break;
+            default:
+                break;
+        }
+    }
+
+    return "";
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(const QString & svg, QRectF & viewBox, bool getViewBox)
 {
 	QXmlStreamReader streamReader(svg);
-    return parseForWidthAndHeight(streamReader);
+    return parseForWidthAndHeight(streamReader, viewBox, getViewBox);
 }
 
 QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg)
+{
+	QRectF viewBox;
+    return parseForWidthAndHeight(svg, viewBox, false);
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg, QRectF & viewBox, bool getViewBox)
 {
     svg.setNamespaceProcessing(false);
 
-	QSizeF size(0,0);
+	QSizeF size(0, 0);
+	viewBox.setRect(0, 0, 0, 0);
 
 	bool isIllustrator = false;
 	bool bad = false;
@@ -1401,6 +1705,31 @@ QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg)
 
 				size.setWidth(w);
 				size.setHeight(h);
+
+                if (getViewBox) {
+                    bool gotViewBox = false;
+				    QString vb = svg.attributes().value("viewBox").toString();
+				    QStringList vbs = vb.split(QRegExp(",| "));
+				    if (vbs.count() == 4) {
+					    bool ok = false;
+					    double d[4];
+					    for (int i = 0; i < 4; i++) {
+						    d[i] = vbs.at(i).toDouble(&ok);
+						    if (!ok) break;
+					    }
+					    if (ok) {
+                            gotViewBox = true;
+						    viewBox.setRect(d[0], d[1], d[2], d[3]);
+					    }
+				    }
+
+                    if (!gotViewBox) {
+                        chopNotDigits(hs);
+                        chopNotDigits(ws);
+                        viewBox.setRect(0, 0, ws.toDouble(), hs.toDouble());
+                    }
+                }
+
 				return size;
 			}
 			break;		
@@ -1411,3 +1740,160 @@ QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg)
 
     return size;
 }
+
+void TextUtils::gornTree(QDomDocument & doc)
+{
+    QDomElement root = doc.documentElement();
+    QString oldid = root.attribute("id");
+    root.setAttribute("id", "0");
+    root.setAttribute("gorn", "0");
+    root.setAttribute("oldid", oldid);
+    gornTreeAux(root);
+}
+
+void TextUtils::gornTreeAux(QDomElement & root) {
+    QString prefix = root.attribute("id");
+    QDomElement child = root.firstChildElement();
+    int index = 0;
+    while (!child.isNull()) {
+        QString oldid = child.attribute("id");
+        QString newid = QString("%1.%2").arg(prefix).arg(index++);
+        child.setAttribute("id", newid);
+        child.setAttribute("gorn", newid);
+        child.setAttribute("oldid", oldid);
+        gornTreeAux(child);
+        child = child.nextSiblingElement();
+    }
+}
+
+bool TextUtils::elevateTransform(QDomElement & root) {
+    QList<QDomElement> transforms;
+    collectTransforms(root, transforms);
+    if (transforms.length() == 0) return false;
+
+    foreach (QDomElement element, transforms) {
+        QString transform = element.attribute("transform");
+        element.removeAttribute("transform");
+        QDomElement g = element.ownerDocument().createElement("g");
+        g.setAttribute("transform", transform);
+        element.parentNode().insertBefore(g, element);
+        g.appendChild(element);
+    }
+
+    return true;
+}
+
+void TextUtils::collectTransforms(QDomElement & root, QList<QDomElement> & transforms) {
+    QString transform = root.attribute("transform");
+    if (!transform.isEmpty()) {
+        transforms.append(root);
+    }
+
+    QDomElement child = root.firstChildElement();
+    while (!child.isNull()) {
+        collectTransforms(child, transforms);
+        child = child.nextSiblingElement();
+    }
+}
+
+bool TextUtils::fixFonts(QString & svg, const QString & destFont) {
+	bool changed = removeFontFamilySingleQuotes(svg);
+	changed |= fixUnavailableFontFamilies(svg, destFont);
+
+	return changed;
+}
+
+void TextUtils::fixStyleAttribute(QDomElement & element)
+{
+	QString style = element.attribute("style");
+	if (style.isEmpty()) return;
+
+	fixStyleAttribute(element, style, "stroke-width");
+	fixStyleAttribute(element, style, "stroke");
+	fixStyleAttribute(element, style, "fill");
+	fixStyleAttribute(element, style, "fill-opacity");
+	fixStyleAttribute(element, style, "stroke-opacity");
+	fixStyleAttribute(element, style, "font-size");
+
+	if (style.trimmed().isEmpty()) {
+		element.removeAttribute("style");
+	}
+	else {
+		element.setAttribute("style", style);
+	}
+
+	//QString deleteMe;
+	//QTextStream stream(&deleteMe);
+	//stream << element;
+	//DebugDialog::debug(deleteMe);
+}
+
+void TextUtils::fixStyleAttribute(QDomElement & element, QString & style, const QString & attributeName)
+{
+    static const QString findStyle("%1[\\s]*:[\\s]*([^;]*)[;]?");
+
+	QString str = findStyle.arg(attributeName);
+	QRegExp sw(str);
+	if (sw.indexIn(style) >= 0) {
+		QString value = sw.cap(1);
+		style.remove(sw);
+		element.setAttribute(attributeName, value);
+	}
+}
+
+QString TextUtils::getRandText() {
+	QString rand = QUuid::createUuid().toString();
+	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
+	return randext;
+}
+
+/*QString TextUtils::getBase64RandText() {
+	QString rand = QUuid::createUuid().toString();
+	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
+	return randext;
+}*/
+
+
+bool TextUtils::ensureViewBox(QDomDocument doc, double dpi, QRectF & rect, bool convert, double & w, double & h, bool getwh) {
+    bool isIllustrator = TextUtils::isIllustratorDoc(doc);
+
+    QDomElement root = doc.documentElement();
+	QString viewBox = root.attribute("viewBox");
+	if (viewBox.isEmpty() || getwh) {
+		bool ok1, ok2;
+        if (convert) {
+		    w = TextUtils::convertToInches(root.attribute("width"), &ok1, isIllustrator) * dpi;
+		    h = TextUtils::convertToInches(root.attribute("height"), &ok2, isIllustrator) * dpi;
+        }
+        else {
+		    w = root.attribute("width").toDouble(&ok1);
+		    h = root.attribute("height").toDouble(&ok2);
+        }
+		if (!ok1 || !ok2) {
+			return false;
+		}
+    }
+
+	if (viewBox.isEmpty()) {
+		root.setAttribute("viewBox", QString("0 0 %1 %2").arg(w).arg(h));
+        rect.setRect(0, 0, w, h);
+        return true;
+	}
+
+    QStringList coords = viewBox.split(QRegExp(" |,"));
+    if (coords.count() != 4) return false;
+
+    rect.setRect(coords.at(0).toDouble(), coords.at(1).toDouble(), coords.at(2).toDouble(), coords.at(3).toDouble());
+    return true;
+}
+
+QString TextUtils::findAnchor(const QDomElement & text) {
+    if (text.isNull()) return "start";
+
+    QString anchor = text.attribute("text-anchor");
+    if (!anchor.isEmpty()) return anchor;
+
+    return findAnchor(text.parentNode().toElement());
+}
+
+
