@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6268 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
+$Revision: 6984 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -37,7 +37,6 @@ $Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 typedef QString (*CopyPinFunction)(int pin, const QString & argString, void * userData);
 typedef QString (*MultiplyPinFunction)(int pin, double increment, double value);
 
-
 class TextUtils
 {
 
@@ -64,7 +63,6 @@ public:
 	static bool isIllustratorDoc(const QDomDocument & doc);
 	static QString removeXMLEntities(QString svgContent);
 	static bool cleanSodipodi(QString &bytes);
-	static bool fixViewboxOrigin(QString &fileContent);
 	static bool fixPixelDimensionsIn(QString &fileContent);
 	static bool addCopper1(const QString & filename, QDomDocument & doc, const QString & srcAtt, const QString & destAtt);
 	static void setSVGTransform(QDomElement &, QMatrix &);
@@ -79,11 +77,9 @@ public:
 	static QMatrix transformStringToMatrix(const QString & transform);
     static QList<double> getTransformFloats(QDomElement & element);
 	static QList<double> getTransformFloats(const QString & transform);
-	static QString removeXMLNS(QString svgContent);
+	static QString svgNSOnly(QString svgContent);
+	static QString killXMLNS(QString svgContent);
 	static void gWrap(QDomDocument & domDocument, const QHash<QString, QString> & attributes);
-	static bool tspanRemove(QString &svg);
-	static bool noUse(QString &svg);
-	static bool noPattern(QString &svg);
 	static void slamStrokeAndFill(QDomElement &, const QString & stroke, const QString & strokeWidth, const QString & fill);
 	static QString slamStrokeAndFill(const QString & svg, const QString & stroke, const QString & strokeWidth, const QString & fill);
 	static QString incrementTemplate(const QString & filename, int pins, double unitIncrement, MultiplyPinFunction, CopyPinFunction, void * userData);
@@ -95,28 +91,41 @@ public:
 	static QString incCopyPinFunction(int pin, const QString & argString, void * userData);
 	static QString negIncCopyPinFunction(int pin, const QString & argString, void * userData);
 	static double getViewBoxCoord(const QString & svg, int coord);
-	static QString makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly);
-	static QString makeCubicBezierSVG(const QPolygonF & controlPoints, double width, QString colorString, double dpi, double printerScale, bool blackOnly);
+	static QString makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> &);
+	static QString makeCubicBezierSVG(const QPolygonF & controlPoints, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> &);
 	static QString makeRectSVG(QRectF r, QPointF offset, double dpi, double printerscale);
 	static QString makePolySVG(const QPolygonF & poly, QPointF offset, double width, QString colorString, double dpi, double printerScale, bool blackOnly);
 	static QPolygonF polygonFromElement(QDomElement & element);
 	static QString pointToSvgString(QPointF p, QPointF offset, double dpi, double printerScale);
-	static void replaceChildText(QDomDocument & doc, QDomNode & node, const QString & text);
-	static void replaceElementChildText(QDomDocument & doc, QDomElement & root, const QString & elementName, const QString & text);
+	static void replaceChildText(QDomNode & node, const QString & text);
+	static void replaceElementChildText(QDomElement & root, const QString & elementName, const QString & text);
 	static QString removeSVGHeader(QString & string);
 	static QString getMacAddress();
 	static QString expandAndFill(const QString & svg, const QString & color, double expandBy);
 	static void expandAndFillAux(QDomElement &, const QString & color, double expandBy);
     static bool writeUtf8(const QString & fileName, const QString & text);
     static int getPinsAndSpacing(const QString & expectedFileName, QString & spacingString);
+	static QSizeF parseForWidthAndHeight(QXmlStreamReader &, QRectF & viewBox, bool getViewBox);
 	static QSizeF parseForWidthAndHeight(QXmlStreamReader &);
+	static QSizeF parseForWidthAndHeight(const QString & svg, QRectF & viewBox, bool getViewBox);
 	static QSizeF parseForWidthAndHeight(const QString & svg);
+    static void gornTree(QDomDocument &);
+    static bool elevateTransform(QDomElement &);
+    static bool fixMuch(QString &svg, bool fixStrokeWidth);
+	static bool fixInternalUnits(QString & svg);
+	static bool fixFonts(QString & svg, const QString & destFont);
+	static void fixStyleAttribute(QDomElement & element);
+    static QString parseForModuleID(const QString & fzp);
+	static QString getRandText();
+    static bool ensureViewBox(QDomDocument doc, double dpi, QRectF & rect, bool toInches, double & w, double & h, bool getwh);
+    static QString findAnchor(const QDomElement & text);
+    static double getStrokeWidth(QDomElement &, double defaultValue);
 
 public:
 	static const QRegExp FindWhitespace;
-	static const QRegExp SodipodiDetector;
 	static const QString SMDFlipSuffix;
 	static const QString MicroSymbol;
+    static const ushort MicroSymbolCode;
 	static const QString PowerPrefixesString;
 	static const QString CreatedWithFritzingString;
 	static const QString CreatedWithFritzingXmlComment;
@@ -127,11 +136,20 @@ public:
 	static const QString AdobeIllustratorIdentifier;
 
 protected:
-	static bool moveViewboxToTopLeftCorner(QDomElement &elem);
 	static bool pxToInches(QDomElement &elem, const QString &attrName, bool isIllustrator);
     static void squashNotElement(QDomElement & element, const QString & elementName, const QString & attName, const QRegExp & matchContent, bool & result);
 	static void initPowerPrefixes();
 	static QDomElement copyText(QDomDocument & svgDom, QDomElement & parent, QDomElement & text, const QString & defaultX, const QString & defaultY, bool copyAttributes);
+    static void gornTreeAux(QDomElement &);
+    static bool noPatternAux(QDomDocument & svgDom, const QString & tag);
+    static bool noUseAux(QDomDocument & svgDom);
+    static bool tspanRemoveAux(QDomDocument & svgDom);
+    static void fixStyleAttribute(QDomElement & element, QString & style, const QString & attributeName);
+    static bool fixStrokeWidth(QDomDocument & svgDoc);
+    static bool fixViewBox(QDomElement & root);
+    static void chopNotDigits(QString &);
+    static void collectTransforms(QDomElement & root, QList<QDomElement> & transforms);
+
 };
 
 #endif
