@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,36 +18,39 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 	 			
 *********************************************************************
 							                    					
-$Revision: 6293 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-14 05:17:17 +0200 (Tue, 14 Aug 2012) $
+$Revision: 9268 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-05-10 17:47:21 +0200 (Fr, 10. Mai 2013) $
 
 ********************************************************************/
-					   					
-#include "version.h"	            										
+					
+#include "version.h"
 											
 #include <QString>
-#include <QStringList>			
+#include <QStringList>
+#include <QSettings>
+#include <QUrl>
 			
-#include "../debugdialog.h"		                	
+#include "../debugdialog.h"                
+#include "../utils/textutils.h"                
+#include "../lib/qtsysteminfo/QtSystemInfo.h"
 
-	
-QString Version::m_majorVersion("0");				
-QString Version::m_minorVersion("7");
-QString Version::m_minorSubVersion("7");
+QString Version::m_majorVersion("0");       
+QString Version::m_minorVersion("8");
+QString Version::m_minorSubVersion("0");
 QString Version::m_modifier("b");
-QString Version::m_svnRevision("$Revision: 6293 $:");
-QString Version::m_svnDate("$Date: 2012-08-14 05:17:17 +0200 (Tue, 14 Aug 2012) $");
+QString Version::m_svnRevision("$Revision: 08d5 $:");
+QString Version::m_svnDate("$Date: 2013-06-12 07:35:21 +0200 (Mi, 12. Jun 2013) $");
 QString Version::m_revision;
 QString Version::m_date;
 QString Version::m_shortDate;
-QString Version::m_versionString;			
+QString Version::m_versionString;
 QString Version::m_year;
-QStringList Version::m_modifiers;	
+QStringList Version::m_modifiers;      
 
 Version * Version::m_singleton = new Version();		
 
-QString Version::FirstVersionWithDetachedUserData = "0.3.1b.05.26.3016";
+QString Version::FirstVersionWithDetachedUserData = "0.3.1b.05.26.3016";    
 			
 Version::Version() {
 	if (m_modifiers.count() == 0) {
@@ -191,3 +194,30 @@ void Version::cleanup() {
 		m_singleton = NULL;
 	}
 }
+
+QString Version::makeRequestParamsString(bool withID) {
+	QSettings settings;
+	if (settings.value("pid").isNull()) {
+		settings.setValue("pid", TextUtils::getRandText());
+	}
+
+    QString id;
+    if (withID) id = QString("&pid=%1").arg(settings.value("pid").toString());
+	QtSystemInfo systemInfo(NULL);
+	QString siVersion(QUrl::toPercentEncoding(Version::versionString()));
+	QString siSystemName(QUrl::toPercentEncoding(systemInfo.systemName()));
+	QString siSystemVersion(QUrl::toPercentEncoding(systemInfo.systemVersion()));
+	QString siKernelName(QUrl::toPercentEncoding(systemInfo.kernelName()));
+	QString siKernelVersion(QUrl::toPercentEncoding(systemInfo.kernelVersion()));
+	QString siArchitecture(QUrl::toPercentEncoding(systemInfo.architectureName()));
+    QString string = QString("?version=%2&sysname=%3&kernname=%4&kernversion=%5arch=%6&sysversion=%7%8")
+		.arg(siVersion)
+		.arg(siSystemName)
+		.arg(siKernelName)
+		.arg(siKernelVersion)
+		.arg(siArchitecture)
+		.arg(siSystemVersion)
+        .arg(id)
+        ;
+	return string;
+}
