@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6289 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
+$Revision: 6976 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-21 09:50:09 +0200 (So, 21. Apr 2013) $
 
 ********************************************************************/
 
@@ -31,8 +31,14 @@ $Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 #include <QAction>
 #include <QHash>
 #include <QMultiHash>
+#include <QDomElement>
 
-#include "utils/misc.h"
+struct NamePair {
+    QString xmlName;
+    QString displayName;
+
+    NamePair(QString xml, QString display);
+};
 
 class ViewLayer : public QObject
 {
@@ -50,6 +56,7 @@ public:
 		BreadboardRuler,
 		SchematicFrame,
 		Schematic,
+        SchematicText,
 		SchematicWire,
 		SchematicTrace,
 		SchematicLabel,
@@ -74,24 +81,23 @@ public:
 		ViewLayerCount
 	};
 
-	enum ViewLayerSpec {
-		ThroughHoleThroughTop_OneLayer,
-		ThroughHoleThroughTop_TwoLayers,
-		ThroughHoleThroughBottom_TwoLayers,
-		SMDOnTop_TwoLayers,
-		SMDOnBottom_OneLayer,
-		SMDOnBottom_TwoLayers,
-		WireOnTop_TwoLayers,
-		WireOnBottom_OneLayer,
-		WireOnBottom_TwoLayers,
-		GroundPlane_Top,
-		GroundPlane_Bottom,
-		Top,
-		Bottom,
-		TopAndBottom,
-		UnknownSpec
+	enum ViewLayerPlacement {
+        NewTop,
+        NewBottom,
+		NewTopAndBottom,
+		UnknownPlacement
 	};
 
+   enum ViewID {
+    	IconView,
+    	BreadboardView,
+    	SchematicView,
+    	PCBView,
+    	AllViews,
+		UnknownView,
+    	ViewCount
+   	};
+
 public:
 	static const QString HolesColor;
 	static const QString Copper0Color;
@@ -102,7 +108,7 @@ public:
 
 protected:
 	static double zIncrement;
-	static QHash<ViewLayerID, StringPair *> names;
+	static QHash<ViewLayerID, NamePair *> names;
 	static QMultiHash<ViewLayerID, ViewLayerID> alternatives;
 	static QMultiHash<ViewLayerID, ViewLayerID> unconnectables;
 	static QHash<QString, ViewLayerID> xmlHash;
@@ -128,6 +134,10 @@ public:
 	bool isActive();
 	bool includeChildLayers();
 	void setIncludeChildLayers(bool);
+    void setFromBelow(bool);
+    bool fromBelow();
+    void setInitialZFromBelow(double);
+    double getZFromBelow(double currentZ, bool fromBelow);
 
 public:
 	static ViewLayerID viewLayerIDFromXmlString(const QString &);
@@ -138,15 +148,28 @@ public:
 	static void cleanup();
 	static QList<ViewLayerID> findAlternativeLayers(ViewLayerID);
 	static bool canConnect(ViewLayerID, ViewLayerID);
-	static ViewLayer::ViewLayerSpec specFromID(ViewLayer::ViewLayerID);
-	static const QList<ViewLayer::ViewLayerID> & copperLayers(ViewLayer::ViewLayerSpec);
-	static const QList<ViewLayer::ViewLayerID> & maskLayers(ViewLayer::ViewLayerSpec);
-	static const QList<ViewLayer::ViewLayerID> & silkLayers(ViewLayer::ViewLayerSpec);
+	static ViewLayer::ViewLayerPlacement specFromID(ViewLayer::ViewLayerID);
+	static const QList<ViewLayer::ViewLayerID> & copperLayers(ViewLayer::ViewLayerPlacement);
+	static const QList<ViewLayer::ViewLayerID> & maskLayers(ViewLayer::ViewLayerPlacement);
+	static const QList<ViewLayer::ViewLayerID> & silkLayers(ViewLayer::ViewLayerPlacement);
 	static const QList<ViewLayer::ViewLayerID> & outlineLayers();
 	static const QList<ViewLayer::ViewLayerID> & drillLayers();
+	static const QList<ViewLayer::ViewLayerID> & topLayers();
+	static const QList<ViewLayer::ViewLayerID> & bottomLayers();
+	static const QList<ViewLayer::ViewLayerID> & silkLayers();
 	static bool isCopperLayer(ViewLayer::ViewLayerID);
 	static bool isNonCopperLayer(ViewLayer::ViewLayerID viewLayerID);  // for pcb view layers only
 
+    static QString & viewIDName(ViewLayer::ViewID);
+	static QString & viewIDXmlName(ViewLayer::ViewID);
+	static QString & viewIDNaturalName(ViewLayer::ViewID);
+	static ViewID idFromXmlName(const QString & name);
+	static const QList<ViewLayer::ViewLayerID> & layersForView(ViewLayer::ViewID);
+	static const QList<ViewLayer::ViewLayerID> & layersForViewFromBelow(ViewLayer::ViewID);
+	static bool viewHasLayer(ViewID, ViewLayer::ViewLayerID);
+
+    static bool getConnectorSvgIDs(QDomElement & connector, ViewLayer::ViewID, QString & id, QString & terminalID);
+    static QDomElement getConnectorPElement(const QDomElement & connector, ViewLayer::ViewID);
 
 protected:
 	bool m_visible;
@@ -154,10 +177,14 @@ protected:
 	QAction* m_action;
 	double m_nextZ;
 	double m_initialZ;
+	double m_initialZFromBelow;
 	QList<ViewLayer *> m_childLayers;
 	ViewLayer * m_parentLayer;
 	bool m_active;
 	bool m_includeChildLayers;
+    bool m_fromBelow;
+
+    static QHash <ViewID, class NameTriple * > ViewIDNames;
 
 };
 
